/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 9, 2003
 * Time: 4:30:17 PM
 */

package EVolve;

import java.util.ArrayList;

import EVolve.data.DataManager;
import EVolve.data.DataSource;

public class DataSourceManager {
    private ArrayList dataManagerList;
    private ArrayList filterList;
    private int currentDataSource;
    private DataManager currentDataManager;
    private Filter currentFilter;

    public DataSourceManager(ArrayList dsList) {
        dataManagerList = new ArrayList();
        for (int i=0; i<dsList.size(); i++) {
            DataSource dataSource = (DataSource)dsList.get(i);
            dataManagerList.add(new DataManager(dataSource));
        }
        filterList = new ArrayList();
        currentDataSource = -1;
    }

    public void init() {
        for (int i=0; i<dataManagerList.size(); i++) {
            filterList.add(new Filter());
        }
        switchDataSource(0);
    }

    public int getCurrentDataSourceId() {
        return currentDataSource;
    }

    public void switchDataSource(int selectedDataSourceIndex) {
        if (currentDataSource == selectedDataSourceIndex) return;

        currentDataSource = selectedDataSourceIndex;
        currentDataManager = (DataManager)dataManagerList.get(selectedDataSourceIndex);
        currentFilter = (Filter)filterList.get(selectedDataSourceIndex);
        Scene.selectDataSource(currentDataManager, currentFilter);
    }

    public ArrayList getDataManagerList() {
        return dataManagerList;
    }

    public boolean addDataSource(String name) {
        boolean success = false;
        Object dsource = EVolve.Main.instanciateDataSource(name);

        if (dsource == null) {
            Scene.showErrorMessage("EVolve failed to instanciate data source: \"" + name + "\"");
        } else if (dsource instanceof DataSource) {
            dataManagerList.add(new DataManager((DataSource)dsource));
            filterList.add(new Filter());
            success = true;
        } else {
            Scene.showErrorMessage("EVolve> \"" + name + "\" is not a DataSource");
        }
        return success;
    }

    public String getUsedDataSourceName(int dataSourceId) {
        DataManager dm = (DataManager)dataManagerList.get(dataSourceId);
        return dm.getDatasourceName();
    }

    public String getCurrentDataSourceName() {
        return currentDataManager.getDatasourceName();
    }

    public DataManager getDataManager(int index) {
        if (index >= dataManagerList.size()) return null;

        return (DataManager)dataManagerList.get(index);
    }

    public void removeDataSource(int[] tobeMoved) {
        ArrayList newDMList = new ArrayList();
        ArrayList newFilterList = new ArrayList();

        for (int i=0; i<dataManagerList.size(); i++) {
            boolean remove = false;
            for (int j=0; j<tobeMoved.length; j++) {
                if (i == tobeMoved[j]) {
                    remove = true;
                    break;
                }
            }
            if (!remove) {
                newDMList.add(dataManagerList.get(i));
                newFilterList.add(filterList.get(i));
            }
        }

        dataManagerList = newDMList;
        filterList = newFilterList;
        currentDataSource = 0;
        currentDataManager = (DataManager)dataManagerList.get(0);
        currentFilter = (Filter)filterList.get(0);
        Scene.selectDataSource(currentDataManager, currentFilter);
    }
}
