package EVolve;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Stack;
import it.unimi.dsi.fastUtil.*;

public class DemoData {
	private RandomAccessFile dataInFile, dataOutFile, insInFile, insOutFile;

	private ArrayList classList, threadList, locationList;
	private ArrayList methodUsed;

	private Object2ObjectHashMap methodDefined;
	private Object2ObjectHashMap classIdToClass;
	private Int2ObjectHashMap methodIdToClassId;

	private Object2ObjectHashMap stackList;
	private Stack methodsStack;

	private int allocationEvents = 0;
	private int invocationEvents = 0;

	private DemoData(String dataInFileName, String dataOutFileName, String insInFileName, String insOutFileName) {
		try {
			dataInFile = new RandomAccessFile(dataInFileName, "r");
			dataOutFile = new RandomAccessFile(dataOutFileName, "rw");
			//insInFile = new RandomAccessFile(insInFileName,"r");
			//insOutFile = new RandomAccessFile(insOutFileName,"rw");
		} catch (IOException e) {
			System.out.println("I/O error.");
		}

		classList = new ArrayList();
		threadList = new ArrayList();
		locationList = new ArrayList();    
		methodDefined = new Object2ObjectHashMap();
		classIdToClass = new Object2ObjectHashMap();
		methodUsed = new ArrayList();
		methodIdToClassId = new Int2ObjectHashMap();
		methodsStack = new Stack();
		stackList = new Object2ObjectHashMap();
		//locationList.add("Unknown_location");
		//classList.add("Unknown_type");
	}

	private void process() {
		try {
			step1();
			step2();

			dataInFile.close();
			dataOutFile.close();
		} catch (IOException e) {
			System.out.println("I/O error.");
		}
	}

	private String getSub(String line, int part) {
		int start = line.indexOf(' ') + 1;
		int i = 1;
		while (i < part) {
			start = line.indexOf(' ', start) + 1;
			i++;
		}
		int end = line.indexOf(' ', start);
		if (end == -1) {
			return line.substring(start);
		} else {
			return line.substring(start, end);
		}
	}

	private void methodPush(String thread, String methodIndex) {
		methodsStack = (Stack)stackList.get(thread);
		if (threadList.indexOf(thread) == -1) {
			threadList.add(thread);
			methodsStack = new Stack();
			stackList.put(thread,methodsStack);
		}
		methodsStack.push(methodIndex);
	}

	private int methodPop(String thread, String methodIndex) {
		int stackSize = 0;
		methodsStack = (Stack)stackList.get(thread);
		if (threadList.indexOf(thread) == -1) {
			threadList.add(thread);
			methodsStack = new Stack();
			stackList.put(thread,methodsStack);
		}
		stackSize = methodsStack.size();

		if ((methodsStack.size()>0) && (methodIndex.equals(methodsStack.get(methodsStack.size()-1)))) {
			methodsStack.pop();
		}

		return stackSize;
	}

	private String getStackTop(String thread) {
		String top = null;
		methodsStack = (Stack)stackList.get(thread);

		if ((methodsStack!=null)&&(methodsStack.size()>0)) {
			//top = (String)methodsStack.get(methodsStack.size()-1);
			top = (String)methodsStack.peek();
		}

		return top;
	}

	private void clearStack() {
		for (int i=0; i<threadList.size(); i++) {
			methodsStack =(Stack) stackList.get(threadList.get(i));
			methodsStack.clear();
		}
		methodsStack = null;
	}

	private String getClassName(String method) {
		String strClassId = (String) methodIdToClassId.get(Integer.parseInt(method));
		return (String) classIdToClass.get(strClassId);
	}

	private String getClassNamebyClassId(String classId) {
		String className = (String)classIdToClass.get(classId);

		if (className == null)
			return "";

		return className;
	}

	private void step1() throws IOException {
		int unknownCounter = 0;
		System.out.println("Step 1...");

		classIdToClass.put("boolean","boolean[]");
		classIdToClass.put("byte","byte[]");
		classIdToClass.put("char","char[]");
		classIdToClass.put("short","short[]");
		classIdToClass.put("int","int[]");
		classIdToClass.put("long","long[]");
		classIdToClass.put("float","float[]");
		classIdToClass.put("double","double[]");


		String line = dataInFile.readLine();
		while (line != null) {
			if (line.lastIndexOf(": ") == 1) {
				line = line.trim();
				char ch = line.charAt(0);
				if (ch == 'C') {
					classIdToClass.put(getSub(line,2),getSub(line,1));
					classList.add(getSub(line,1));
				} else if (ch == 'M') {
					String classId = getSub(line,3);
					methodIdToClassId.put(Integer.parseInt(getSub(line,2)),classId);
					methodDefined.put(getSub(line, 2),getSub(line,1));
				} else if (ch == '+') {
					String methodId = getSub(line, 4);
					String thread = getSub(line,2);
					methodPush(thread,methodId);
					if (!methodDefined.containsKey(methodId)) {
						methodDefined.put(methodId,"Unknown_method_"+unknownCounter);
						unknownCounter++;
					}
					if (methodUsed.indexOf(methodId)==-1) {
						methodUsed.add(methodId);
					}
					invocationEvents++;
				} else if (ch == '-') {
					String methodId = getSub(line, 4);
					String thread = getSub(line,2);
					methodPop(thread,methodId);
					if (methodUsed.indexOf(methodId)==-1) {
						methodUsed.add(methodId);
					}
				} else if (ch == 'O') {
					allocationEvents++;
				}
			}
			line = dataInFile.readLine();
		}
	}

	private void step2() throws IOException {
		ArrayList unknowMethodList = new ArrayList();
		Object2ObjectHashMap dupMethods = new Object2ObjectHashMap();
		int unknownCounter = 0;
		int unknownReturn = 0, unknownEnter = 0;
		int Enter = 0, Return = 0;
		System.out.println("Step 2...");

		dataOutFile.setLength(0);

		clearStack();

		//write("H: invocation " + invocationEvents);
		//write("H: allocation " + allocationEvents);

		for (int i = 0; i < threadList.size(); i++) {
			write("T: Thread" + i + " " + i);
		}

		//write("C: Unknown_type 0");
		//write("L: Unknown_location 0");

		int firstBytecode = -1;

		dataInFile.seek(0);
		String line = dataInFile.readLine();
		int lastClass = 0;
		String lastClassName = "";

		while (line != null) {
			if (line.lastIndexOf(": ") == 1) {
				line = line.trim();
				char ch = line.charAt(0);
				if (ch == 'C') {
					int index = classList.indexOf(getSub(line, 1));
					if (index != -1) {
						write("C: " + getSub(line, 1) + " " + index);
						lastClass = index;
						lastClassName = getSub(line, 1);
					} else {
						write("C: " + getSub(line,1) + " " + classList.size());
						classList.add(getSub(line,1));
					}
				} else if (ch == 'M') {
					String methodId = getSub(line,2);
					int index = methodUsed.indexOf(methodId);
					if (index != -1) {
						String fullName = lastClassName+"."+restoreMethod((String)methodDefined.get(methodId));
						//if (!dupMethods.containsKey(fullName)) {
						write("M: " + fullName + " " + index + " " + lastClass);
						dupMethods.put(fullName,methodId);
						/*} else {
							int ii = 0;
							ii++;
						}*/
					}

				} else if (ch == '+') {
					String thread = getSub(line,2);
					int threadIndex = threadList.indexOf(thread);
					if (threadIndex != -1) {
						Enter ++;

						String method = getSub(line, 4);
						String caller = getStackTop(thread);
						methodPush(thread,method); // in this method we set methodsStack to the current using method stack
						int methodIndex = methodUsed.indexOf(method);
						String className = "";

						className = getClassName(method);

						if (((String)methodDefined.get(method)).indexOf("Unknown_method_")==-1) {
							String fullName = className+"."+restoreMethod((String)methodDefined.get(method));
							methodIndex = methodUsed.indexOf(dupMethods.get(fullName));
							int index = unknowMethodList.indexOf(method);
							if (index != -1) {
								unknowMethodList.remove(index);
							}
						}
						else {
							unknownEnter++;
							if (unknowMethodList.indexOf(method) ==-1) unknowMethodList.add(method);
						}

						if (methodsStack.size() > 0) {
							if (firstBytecode == -1) {
								firstBytecode = Integer.parseInt(getSub(line, 1)) + 1;
							}
							String location = "Unknown_location";
							if ((caller != null) && methodDefined.containsKey(caller)) {
								String callerName = (String) methodDefined.get(caller);
								className = getClassName(caller);
								if (className.length() > 0)
									location = className + "." + restoreMethod(callerName) + "_#" + getSub(line, 3);
								else
									location = callerName + "_#" + getSub(line,3);
							}
							int locationIndex = locationList.indexOf(location);
							if (locationIndex == -1) {
								locationList.add(location);
								locationIndex = locationList.indexOf(location);
								write("L: " + location + " " + locationIndex);
							}
							if (methodIndex != -1)
								write("+: " + (Integer.parseInt(getSub(line, 1)) -  firstBytecode + 1) + " " + threadIndex + " " + methodIndex + " " + locationIndex);

						}
					}
				} else if (ch == '-') {
					String thread = getSub(line,2);
					int threadIndex = threadList.indexOf(thread);
					if (threadIndex != -1) {
						Return++;

						String method = getSub(line, 4);
						int stackSize = methodPop(thread,method); // in this method we set methodsStack to the current using method stack
						String caller = getStackTop(thread);

						int methodIndex = methodUsed.indexOf(method);
						String className = "";
						if (!methodDefined.containsKey(method)) {
							methodDefined.put(method,"Unknown_method_"+unknownCounter);
							unknownCounter++;
						}

						className = getClassName(method);
						if (((String)methodDefined.get(method)).indexOf("Unknown_method_")==-1) {
							String fullName = className+"."+restoreMethod((String)methodDefined.get(method));
							methodIndex = methodUsed.indexOf(dupMethods.get(fullName));
							int index = unknowMethodList.indexOf(method);
							if (index != -1)
								unknowMethodList.remove(index);
						}
						else {
							unknownReturn++;
							if (unknowMethodList.indexOf(method) ==-1) unknowMethodList.add(method);
						}

						if (stackSize - methodsStack.size() == 1) {
							if (firstBytecode == -1) {
								firstBytecode = Integer.parseInt(getSub(line, 1));
							}
							String location = "Unknown_location";
							if ((caller != null) && methodDefined.containsKey(caller)) {
								String callerName = (String) methodDefined.get(caller);
								className = getClassName(caller);
								if (className.length() > 0)
									location = className + "." + restoreMethod(callerName) + "_#" + getSub(line, 3);
								else
									location = callerName + "_#" + getSub(line,3);
							}
							int locationIndex = locationList.indexOf(location);
							if (locationIndex == -1) {
								locationList.add(location);
								locationIndex = locationList.indexOf(location);
								write("L: " + location + " " + locationIndex);
							}
							if (methodIndex != -1) {
								write("-: " + (Integer.parseInt(getSub(line, 1)) -  firstBytecode) + " " + threadIndex + " " + methodIndex + " " + locationIndex);
							}
						}
					}
				} else if (ch == 'O') {
					String thread = getSub(line,2);
					String typeId = getSub(line,4);
					int threadIndex = threadList.indexOf(thread);
					String method = getStackTop(thread);

					if ((threadIndex != -1)&&(methodsStack.size()>0)) {

						int classIndex = classList.indexOf(getClassNamebyClassId(typeId));
						int methodIndex = methodUsed.indexOf(method);

						if ((methodIndex == -1)&&(methodDefined.containsKey(method))) {
							methodIndex = methodUsed.size();
							methodUsed.add(method);
						}

						if ((classIndex == -1)&&(classIdToClass.containsKey(typeId))) {
							classIndex = classList.size();
							String className = getClassNamebyClassId(typeId);
							write("C: " + className + " " + classIndex);
							classList.add(className);
						}

						if ((classIndex != -1)) {

							if (firstBytecode == -1) {
								firstBytecode = Integer.parseInt(getSub(line, 1));
							}

							if (methodIndex != -1) {
								String callerName = (String) methodDefined.get(method);
								String className = (String)classIdToClass.get(methodIdToClassId.get(Integer.parseInt(method)));//"";
								String location;
								if (className != null)
									location = className+"."+restoreMethod(callerName) + "_#" + getSub(line, 3);
								else
									location = callerName + "_#" + getSub(line, 3);
								int locationIndex = locationList.indexOf(location);
								if (locationIndex == -1) {
									locationList.add(location);
									locationIndex = locationList.indexOf(location);
									write("L: " + location + " " + locationIndex);
								}

								write("O: " + (Integer.parseInt(getSub(line, 1)) - firstBytecode + 1)+ " " + threadIndex + " " +
									  methodIndex + " " + locationIndex + " " + classIndex + " " + getSub(line, 5));
							} else {
								System.out.println("ooops..., method unknown");
								write("O: " + (Integer.parseInt(getSub(line, 1)) - firstBytecode )+ " " + threadIndex + " " +
									  0 + " " + 0 + " " + classIndex + " " + getSub(line, 5));
							}
						} else {
							System.out.println("ooops..., object unknown!");
						}

					}
				}
			}
			line = dataInFile.readLine();
		}

		for (int i = 0; i< unknowMethodList.size(); i++) {
			String method = (String)unknowMethodList.get(i);
			write("M: " + (String)methodDefined.get(method) + " " + methodUsed.indexOf(method) + " 0");
		}


		System.out.println("Method Defined: "+ methodDefined.size()+", Method Used: "+methodUsed.size());
		System.out.println("Method NOT Resolved: "+ unknowMethodList.size());
		System.out.println("Resolve Percert: "+(1-(float)unknowMethodList.size()/(float)methodUsed.size())*100+"%");

		System.out.println("Method enter events: "+ Enter + ", which contains " + unknownEnter + " unknown enter events.");
		System.out.println("Method exit events: "+ Return + ", which contains " + unknownReturn + " unknown exit events.");
		System.out.println("Unknown Percert: "+((float)(unknownEnter+unknownReturn)/(float)(Enter+Return))*100+"%");

		// process instruction file
		/*System.out.println("processing instruction file...");
		insInFile.seek(0);
		insOutFile.setLength(0);
		int index = 0,start =0;
		String num,origId,offset,header,threadId;

		do {
			line = insInFile.readLine();
		} while (line.indexOf('+')==-1);

		while (line != null) {

			header = "";
			index = line.lastIndexOf(':');
			if (index != -1) {
				header = line.substring(0,index+1);
				line = line.substring(index+1);
			}

			index = line.indexOf(' ');
			num = line.substring(0,index);

			start = index+1;
			index = line.indexOf(' ',start);
			origId = line.substring(start,index);
			start = index+1;
			index = line.indexOf(' ',start);
			offset = line.substring(start, index);
			start = index +1;
			threadId = line.substring(start);

			int threadIndex =threadList.indexOf(threadId);
			if (threadIndex == -1) continue;
			int methodIndex = methodUsed.indexOf(origId);
			if ((methodIndex != -1)&&(((String)methodDefined.get(origId)).indexOf("Unknown_method_")==-1)) {
				writeInstruction(header + (Integer.parseInt(num)-firstBytecode) + " " + methodIndex + " " + offset + " " + threadIndex);
			} else {
				//writeInstruction(header+(Integer.parseInt(num)-firstBytecode) + " " + 0 + " " + offset + " " + threadIndex);
			}
			line = insInFile.readLine();
		}*/
		System.out.println("done!");
	}

	private void writeInstruction(String string) throws IOException {
		insOutFile.writeBytes(string + "\n");
	}

	private void write(String string) throws IOException {
		dataOutFile.writeBytes(string + "\n");
	}

	private String restoreMethod(String method) {
		String returnVal = method.substring(0, method.indexOf('(') + 1);
		String str = method.substring(method.indexOf('(') + 1,method.indexOf(')'));
		int arrayNum = 0;
		while (str.length() > 0) {
			String type;
			while (str.charAt(0) == '[') {
				arrayNum++;
				str = str.substring(1);
			}
			if (str.charAt(0) == 'L') {
				type = str.substring(1, str.indexOf(';'));
				str = str.substring(str.indexOf(';') + 1);
			} else {
				type = str.substring(0, 1);
				str = str.substring(1);
			}
			returnVal = returnVal + restoreType(type);
			for (int i = 0; i < arrayNum; i++) {
				returnVal = returnVal + "[]";
			}
			arrayNum = 0;
			returnVal = returnVal + ",";
		}
		if (returnVal.charAt(returnVal.length() - 1) == ',') {
			returnVal = returnVal.substring(0, returnVal.length() - 1);
		}
		returnVal = returnVal + ")";
		returnVal = returnVal.replace('<', '[');
		returnVal = returnVal.replace('>', ']');
		return returnVal;
	}

	private String restoreType(String type) {
		if (type.equals("Z")) {
			return "boolean";
		}
		if (type.equals("B")) {
			return "byte";
		}
		if (type.equals("C")) {
			return "char";
		}
		if (type.equals("S")) {
			return "short";
		}
		if (type.equals("I")) {
			return "int";
		}
		if (type.equals("J")) {
			return "long";
		}
		if (type.equals("F")) {
			return "float";
		}
		if (type.equals("D")) {
			return "double";
		}
		return type.replace('/', '.');
	}

	public static void main(String[] args) {
		//DemoData data = new DemoData(args[0], args[1], args[2], args[3]);
		DemoData data = new DemoData(args[0], args[1], "","");
		data.process();
	}
}
