/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import EVolve.data.Entity;
import EVolve.data.ReferenceLink;
import EVolve.data.Selection;
import EVolve.exceptions.EVolveException;
import EVolve.visualization.Visualization;
import EVolve.util.xmlutils.datastructures.SerializedSelection;

public class Filter {
    private ArrayList selection;
    private ArrayList field;

    private final JDialog dialog;
    private JDialog timeFrameChangeDialog;
    private JSplitPane splitMain, splitLeft, splitLeftLower;
    private JList listSelection, listField, listEntity;
    private DefaultListModel modelSelection, modelField, modelEntity;
    private JTextArea timeframeArea;
    private JTextField textStartTime, textEndTime;

    private JPopupMenu popup;
    private JMenuItem itemRename, itemCloneSelection, itemChangeTimeFrame;
    private JMenuItem itemRemoveItems, itemKeepItems;
    private int activateFieldIndex;

    public Filter() {
        selection = new ArrayList();
        field = new ArrayList();

        dialog = new JDialog(Scene.getFrame(), "Filter", false);

        final CardLayout cardButton = new CardLayout();
        final JPanel panelButton = new JPanel(cardButton);
        dialog.getContentPane().add(panelButton, BorderLayout.SOUTH);

        // selection
        modelSelection = new DefaultListModel();
        listSelection = new JList(modelSelection);
        listSelection.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listSelection.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                updateSelection();
                cardButton.show(panelButton, "Selection");
            }
        });

        JScrollPane scrollSelection = new JScrollPane(listSelection, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollSelection.setBackground(Color.white);
        scrollSelection.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Selections"));

        JPanel panelSelection = new JPanel(new FlowLayout());
        panelButton.add(panelSelection, "Selection");

        JButton buttonDeselect = new JButton("Deselect");
        buttonDeselect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                listSelection.clearSelection();
            }
        });
        panelSelection.add(buttonDeselect);

        JButton buttonColor = new JButton("Coloring");
        buttonColor.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                color();
            }
        });
        panelSelection.add(buttonColor);

        JButton buttonApplyNow = new JButton("Visualize Now");
        buttonApplyNow.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
                Scene.getVisualizationManager().visualizeNow();
            }
        });
        panelSelection.add(buttonApplyNow);

        JButton buttonFilter = new JButton("Decoloring");
        buttonFilter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filter();
            }
        });
        panelSelection.add(buttonFilter);

        JButton buttonRemove = new JButton("Remove");
        buttonRemove.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                remove();
            }
        });
        panelSelection.add(buttonRemove);

        JButton buttonSelectionClose = new JButton("Close");
        buttonSelectionClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
            }
        });
        panelSelection.add(buttonSelectionClose);

        // field
        modelField = new DefaultListModel();
        listField = new JList(modelField);
        listField.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listField.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                updateField();
                cardButton.show(panelButton, "Field");
            }
        });
        listField.setEnabled(false);

        JScrollPane scrollField = new JScrollPane(listField, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollField.setBackground(Color.white);
        scrollField.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Fields"));

        JPanel panelField = new JPanel(new FlowLayout());
        panelButton.add(panelField, "Field");

        JButton buttonSelectAll = new JButton("Select All");
        buttonSelectAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectAll();
            }
        });
        panelField.add(buttonSelectAll);

        JButton buttonClearAll = new JButton("Clear All");
        buttonClearAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clearAll();
            }
        });
        panelField.add(buttonClearAll);

        JButton buttonFieldClose = new JButton("Close");
        buttonFieldClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
            }
        });
        panelField.add(buttonFieldClose);

        // entity
        modelEntity = new DefaultListModel();
        listEntity = new JList(modelEntity);
        listEntity.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listEntity.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                cardButton.show(panelButton, "Entity");
            }
        });

        JScrollPane scrollEntity = new JScrollPane(listEntity, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollEntity.setBackground(Color.white);
        scrollEntity.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Selected Entities"));

        timeframeArea = new JTextArea();
        timeframeArea.setBackground(Color.white);
        timeframeArea.append("   ");
        timeframeArea.setEditable(false);

        JScrollPane scrollTimeFrame = new JScrollPane(timeframeArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollTimeFrame.setBackground(Color.white);
        scrollTimeFrame.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Selected Time Frame"));

        JPanel panelEntity = new JPanel(new FlowLayout());
        panelButton.add(panelEntity, "Entity");

        JButton buttonRemoveSelected = new JButton("Remove Selected");
        buttonRemoveSelected.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                removeSelected();
            }
        });
        panelEntity.add(buttonRemoveSelected);

        JButton buttonRemoveUnselected = new JButton("Remove Unselected");
        buttonRemoveUnselected.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                removeUnselected();
            }
        });
        panelEntity.add(buttonRemoveUnselected);

        JButton buttonEntityClose = new JButton("Close");
        buttonEntityClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                dialog.setVisible(false);
            }
        });
        panelEntity.add(buttonEntityClose);

        splitLeftLower = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollField, scrollTimeFrame);
        splitLeftLower.setDividerSize(2);
        splitLeft = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollSelection, splitLeftLower);
        splitLeft.setDividerSize(2);

        splitMain = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, splitLeft, scrollEntity);
        splitMain.setDividerSize(2);
        dialog.getContentPane().add(splitMain, BorderLayout.CENTER);

        createMenu();
        addPopupTrigger(listSelection);
    }

    public void load() {
        dialog.pack();
        Scene.getUIManager().showDialog(dialog, Toolkit.getDefaultToolkit().getScreenSize().width / 2, Toolkit.getDefaultToolkit().getScreenSize().height * 3 / 4);
        splitMain.setDividerLocation(0.5);
        splitLeft.setDividerLocation(0.5);
        splitLeftLower.setDividerLocation(0.5);
        updateSelection();
    }

    public void addSelection(Selection selection) {
        this.selection.add(selection);
        update();

        listSelection.setSelectedIndex(this.selection.size() - 1);
        updateSelection();

        if (!dialog.isVisible()) {
            load();
        }
    }

    private void update() {
        modelSelection.removeAllElements();
        for (int i = 0; i < selection.size(); i++) {
            Selection s = (Selection)(selection.get(i));
            if (s.getColor() == null) {
                modelSelection.addElement("* " + s.getName() + " (" + Scene.getDataManager().getElementDefinition()[((Selection)(selection.get(i))).getEntityType()].getName() + ")");
            } else {
                modelSelection.addElement("<html><font color=#" + getColorHex(s.getColor()) + ">" + s.getName() + " (" + Scene.getDataManager().getElementDefinition()[((Selection)(selection.get(i))).getEntityType()].getName() + ")</font></html>");
            }
        }
    }

    private void updateTimeFrameInfo() {
        timeframeArea.replaceRange(null,0,timeframeArea.getText().length());
        Selection selected = getActiveSelection();
        if (selected != null) {
            String info = "Start Event no.(overall): " + selected.getStart() + "\n";
            info = info + "End Event no.(overall): " + selected.getEnd() + "\n";
            //String info = "";
            if (selected.getStartTime() != -1) {
                info = info + "Start Time: " + selected.getStartTime() + "\n";
                info = info + "End Time: " + selected.getEndTime() + "\n";
            } else {
                info = info + "Start Time: begin of the trace\n";
                info = info + "End Time: end of the trace\n";
            }
            timeframeArea.append(info);
        }

    }

    private String getColorHex(Color color) {
        String returnVal = Integer.toHexString(color.getBlue());
        if (returnVal.length() < 2) {
            returnVal = "0" + returnVal;
        }
        returnVal = Integer.toHexString(color.getGreen()) + returnVal;
        if (returnVal.length() < 4) {
            returnVal = "0" + returnVal;
        }
        returnVal = Integer.toHexString(color.getRed()) + returnVal;
        if (returnVal.length() < 6) {
            returnVal = "0" + returnVal;
        }

        return returnVal;
    }

    public void updateSelection() {
        if (listSelection.getSelectedIndex() != -1) {
            modelField.removeAllElements();
            modelEntity.removeAllElements();

            Selection s = (Selection)(selection.get(listSelection.getSelectedIndex()));

            for (int i = 0; i < s.getSelected().length; i++) {
                modelEntity.addElement(s.getSelected()[i].getName());
            }

            ArrayList vizType = Scene.getVisualizationManager().getVisualizationType();
            field.clear();
            ReferenceLink[] link = Scene.getDataManager().getReferenceLink();

            activateFieldIndex = -1;
            Selection selection = getActiveSelection();
            for (int i = 0; i < link.length; i++) {
                /*if ((vizType.indexOf("" + link[i].getSourceType() + "") != -1) && (link[i].getTargetType() == s.getEntityType())) {
                    field.add(link[i]);
                }*/
                if (vizType.indexOf("" + link[i].getSourceType() + "") != -1) {
                    if (Scene.getDataManager().getElementDefinition()[link[i].getSourceType()].getType()==selection.getSourceType()) {
                        /*if ((link[i].getTargetType() == s.getEntityType())&&(activateFieldIndex == -1))
                            activateFieldIndex = field.size();*/
                        if (link[i].getName().equals(s.getEntityCategory()))
                            activateFieldIndex = field.size();
                        field.add(link[i]);
                    }
                }
            }

            for (int i = 0; i < field.size(); i++) {
                modelField.addElement(((ReferenceLink)(field.get(i))).getName() + " (" + Scene.getDataManager().getElementDefinition()[((ReferenceLink)(field.get(i))).getSourceType()].getName() + ")");
            }

            /*ArrayList selected = new ArrayList();
            for (int i = 0; i < s.getLink().length; i++) {
                if (field.indexOf(s.getLink()[i]) != -1) {
                    selected.add(new Integer(field.indexOf(s.getLink()[i])));
                }
            }
            selected.add(new Integer(activateFieldIndex));

            int[] selectedLink = new int[selected.size()];
            for (int i = 0; i < selectedLink.length; i++) {
                selectedLink[i] = ((Integer)(selected.get(i))).intValue();
            }
            listField.setSelectedIndices(selectedLink);*/
            listField.setSelectedIndex(activateFieldIndex);

            updateTimeFrameInfo();
            updateField();
        }
    }

    private void updateField() {
        if (listSelection.getSelectedIndex() != -1) {
            ReferenceLink[] newLink = new ReferenceLink[listField.getSelectedIndices().length];
            for (int i = 0; i < newLink.length; i++) {
                newLink[i] = (ReferenceLink)(field.get(listField.getSelectedIndices()[i]));
            }

            Selection s = (Selection)(selection.get(listSelection.getSelectedIndex()));
            s.setLink(newLink);
        }
    }

    private void updateEntity() {
        if (listSelection.getSelectedIndex() != -1) {
            modelEntity.removeAllElements();

            Selection s = (Selection)(selection.get(listSelection.getSelectedIndex()));
            for (int i = 0; i < s.getSelected().length; i++) {
                modelEntity.addElement(s.getSelected()[i].getName());
            }
        }
    }

    private void color() {
        if (listSelection.getSelectedIndex() != -1) {
            Color newColor = JColorChooser.showDialog(Scene.getFrame(), "Choose a color", Color.black);
            if (newColor != null) {
                int selected = listSelection.getSelectedIndex();
                ((Selection)(selection.get(selected))).setColor(newColor);
                update();

                listSelection.setSelectedIndex(selected);
                updateSelection();
            }
        }
    }

    private void filter() {
        if (listSelection.getSelectedIndex() != -1) {
            int selected = listSelection.getSelectedIndex();
            ((Selection)(selection.get(selected))).setColor(null);
            update();

            listSelection.setSelectedIndex(selected);
            updateSelection();
        }
    }

    private void remove() {
        if (listSelection.getSelectedIndex() != -1) {
            selection.remove(listSelection.getSelectedIndex());
            update();
        }
    }

    private void selectAll() {
        /*if (listSelection.getSelectedIndex() != -1) {
            int[] all = new int[modelField.size()];
            for (int i = 0; i < all.length; i++) {
                all[i] = i;
            }
            listField.setSelectedIndices(all);
            updateField();
        }*/
    }

    private void clearAll() {
        if (listSelection.getSelectedIndex() != -1) {
            listField.setSelectedIndices(new int[0]);
            updateField();
        }
    }

    private void removeSelected() {
        if (listSelection.getSelectedIndex() != -1) {
            Selection s = (Selection)(selection.get(listSelection.getSelectedIndex()));
            int[] index = listEntity.getSelectedIndices();
            Entity[] newEntity = new Entity[s.getSelected().length - index.length];

            int j = 0;
            int k = 0;
            for (int i = 0; i < newEntity.length; i++) {
                while ((j < index.length) && (index[j] == k)) {
                    j++;
                    k++;
                }
                newEntity[i] = s.getSelected()[k];
                k++;
            }
            s.setSelected(newEntity);
            updateEntity();
        }
    }

    private void removeUnselected() {
        if (listSelection.getSelectedIndex() != -1) {
            Selection s = (Selection)(selection.get(listSelection.getSelectedIndex()));
            int[] index = listEntity.getSelectedIndices();
            Entity[] newEntity = new Entity[index.length];

            for (int i = 0; i < newEntity.length; i++) {
                newEntity[i] = s.getSelected()[index[i]];
            }
            s.setSelected(newEntity);
            updateEntity();
        }
    }

    public Selection[] getSelection() {
        Selection[] returnVal = new Selection[selection.size()];
        for (int i = 0; i < returnVal.length; i++) {
            returnVal[i] = (Selection)(selection.get(i));
        }

        return returnVal;
    }

    public Selection getActiveSelection() {
        int selectedIndex = listSelection.getSelectedIndex();

        if (selectedIndex == -1) return null;

        return (Selection)selection.get(selectedIndex);
    }

    public int getSelectedIndex() {
        return listSelection.getSelectedIndex();
    }

    public void loadSelection(ArrayList selections, int selectedIndex) throws EVolveException{
        Selection newSelection = null;
        long start, end, startTime, endTime;
        int sourceType, entityType;
        String entityCategory;
        long selected[];
        ArrayList IDs,timeMap;
        String currentName;
        StringTokenizer token;
        Color color;

        this.selection.clear();

        try {
            for (int i=0; i<selections.size(); i++) {
                SerializedSelection data = (SerializedSelection)selections.get(i);
                timeMap = null;

                currentName = data.CurrentSelectionName;
                start = Long.parseLong(data.StartEventNo);
                end = Long.parseLong(data.EndEventNo);
                startTime = Long.parseLong(data.StartTime);
                endTime = Long.parseLong(data.EndTime);

                token = new StringTokenizer(data.TimeMap,",");
                while (token.hasMoreTokens()) {
                    if (timeMap == null) timeMap = new ArrayList();
                    long[] value = new long[2];
                    value[0] = Long.parseLong(token.nextToken());
                    value[1] = Long.parseLong(token.nextToken());
                    timeMap.add(value);
                }
                sourceType = Integer.parseInt(data.SourceType);
                entityType = Integer.parseInt(data.EntityType);
                entityCategory = data.EntityCategory;
                color = null;
                if ((data.Color != null)&&(data.Color.length() > 0))
                    color = new Color(Integer.parseInt(data.Color));
                token = new StringTokenizer(data.SelectedEntities,",");

                IDs = new ArrayList();
                while (token.hasMoreTokens()) {
                    IDs.add(token.nextToken());
                }
                selected = new long[IDs.size()];
                for (int k=0; k<selected.length; k++)
                    selected[k] = Long.parseLong((String)IDs.get(k));

                newSelection = new Selection(currentName,sourceType,entityType,entityCategory,selected,start,end,timeMap);
                newSelection.setStartTime(startTime);
                newSelection.setEndTime(endTime);
                newSelection.setColor(color);

                this.selection.add(newSelection);
                update();
            }
            if (selectedIndex != -1)
                listSelection.setSelectedIndex(selectedIndex);
            updateSelection();
        } catch (NumberFormatException e) {
            throw new EVolveException("Selection data is invalid, expect:numerical, found:string.");
        }
    }

    public ArrayList getSelections() {
        ArrayList list = new ArrayList();
        Selection currentSelection;
        Entity[] entities;
        ArrayList timeMap;

        for (int i=0; i<selection.size(); i++) {
            SerializedSelection data = new SerializedSelection();
            timeMap = null;
            currentSelection = (Selection)selection.get(i);

            data.CurrentSelectionName = currentSelection.getName();
            data.StartEventNo = String.valueOf(currentSelection.getStart());
            data.EndEventNo = String.valueOf(currentSelection.getEnd());
            data.StartTime = String.valueOf(currentSelection.getStartTime());
            data.EndTime = String.valueOf(currentSelection.getEndTime());

            timeMap = currentSelection.getTimeMap();
            if (timeMap != null) {
                data.TimeMap = new String();
                for (int j=0; j<timeMap.size(); j++) {
                    long[] value = (long[])timeMap.get(j);
                    data.TimeMap = data.TimeMap + String.valueOf(value[0])+","+String.valueOf(value[1])+",";
                }
            }
            data.SourceType = String.valueOf(currentSelection.getSourceType());
            data.EntityType = String.valueOf(currentSelection.getEntityType());
            data.EntityCategory = currentSelection.getEntityCategory();
            if (currentSelection.getColor() != null)
                data.Color = String.valueOf(currentSelection.getColor().getRGB());
            entities = currentSelection.getSelected();
            data.SelectedEntities = new String();
            for (int j=0; j<entities.length; j++) {
                data.SelectedEntities = data.SelectedEntities + String.valueOf(entities[j].getId())+",";
            }
            list.add(data);
        }

        return list;
    }

    public Selection getNewestSelection() {
        return (Selection)selection.get(selection.size()-1);
    }

    public void selectWithRegExp() {
        int selectionNumber = selection.size();

        Visualization viz = Scene.getVisualizationManager().getActiveVisualization();

        if (viz == null) return;

        viz.makeSelection();

        if (selection.size() > selectionNumber) {
            for (int i=selectionNumber; i<selection.size(); i++) {
                String patterns = JOptionPane.showInputDialog(Scene.getFrame(),
                                  "Input pattern expression(s).\n(seperated with \";\")");
                if (patterns != null) {
                    patterns = generateRegExp(patterns);
                    Selection added = (Selection)selection.get(i);
                    added.filterEntities(patterns,true);
                }
            }
            updateSelection();
        }
    }

    private String generateRegExp(String inputs) {
        String returnVal = "";
        StringTokenizer token = new StringTokenizer(inputs+";",";");
        while (token.hasMoreTokens()) {
            String input = token.nextToken();
            for (int i=0; i<input.length(); i++) {
                char aChar = input.charAt(i);
                switch (aChar) {
                    case '*':
                        returnVal = returnVal + ".{1,}";
                        break;
                    case '.':
                        returnVal += "/{1}";
                        break;
                    case '?':
                        returnVal += '.';
                        break;
                    default:
                        returnVal += aChar;
                        break;
                }
            }
            returnVal += ';';
        }
        return returnVal;
    }

    private void createMenu() {
        popup = new JPopupMenu();

        itemRename = new JMenuItem("Rename Selection ...");
        itemRename.setMnemonic(KeyEvent.VK_R);
        itemRename.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Selection selected = getActiveSelection();
                if (selected != null) {
                    String newName = JOptionPane.showInputDialog(selected.getName());
                    selected.setName(newName);
                    update();
                }
            }
        });
        popup.add(itemRename);

        itemCloneSelection = new JMenuItem("Clone Current Selection");
        itemCloneSelection.setMnemonic(KeyEvent.VK_C);
        itemCloneSelection.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Selection selected = getActiveSelection();
                if (selected != null) {
                    Selection newSelection = (Selection)selected.clone();
                    addSelection(newSelection);
                }
            }
        });
        popup.add(itemCloneSelection);

        itemChangeTimeFrame = new JMenuItem("Change Time Frame");
        itemChangeTimeFrame.setMnemonic(KeyEvent.VK_C);
        itemChangeTimeFrame.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Selection selected = getActiveSelection();
                if (selected != null) {
                    createChangeTimeDialog(selected);
                }
            }
        });
        popup.add(itemChangeTimeFrame);

        itemRemoveItems = new JMenuItem("Remove Entities ...");
        itemRemoveItems.setMnemonic(KeyEvent.VK_R);
        itemRemoveItems.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filterEntities(false);
            }
        });
        popup.add(itemRemoveItems);

        itemKeepItems = new JMenuItem("Remove All Entities Except ...");
        itemKeepItems.setMnemonic(KeyEvent.VK_E);
        itemKeepItems.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filterEntities(true);
            }
        });
        popup.add(itemKeepItems);
    }

    private void addPopupTrigger(Component component) {

        component.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }
        });
    }

    private void showPopup(MouseEvent e) {
        int mouseX = e.getX();
        int mouseY = e.getY();
        Rectangle rect = e.getComponent().getBounds();
        Rectangle rect2 = popup.getBounds();
        int posX= mouseX, posY = mouseY;
        if (mouseY+rect2.height > rect.height)
            posY = posY - rect2.height;
        if (mouseX+rect2.width>rect.width)
            posX = posX - rect2.width;
            popup.show(e.getComponent(), posX, posY);
    }

    private void createChangeTimeDialog(Selection selected) {

        if (timeFrameChangeDialog == null) {
            textStartTime = new JTextField(String.valueOf(selected.getStartTime()));
            textEndTime = new JTextField(String.valueOf(selected.getEndTime()));
            timeFrameChangeDialog = new JDialog(dialog,"Change Time Frame",true);

            Box boxMain = new Box(BoxLayout.Y_AXIS);
            boxMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                    "Choose range"));
            timeFrameChangeDialog.getContentPane().add(boxMain,BorderLayout.CENTER);

            boxMain.add(new JLabel("Start time:"));
            boxMain.add(textStartTime);
            boxMain.add(new JLabel("End time:"));
            boxMain.add(textEndTime);

            timeFrameChangeDialog.getContentPane().add(boxMain, BorderLayout.CENTER);

            JPanel panelButton = new JPanel(new FlowLayout());
            timeFrameChangeDialog.getContentPane().add(panelButton, BorderLayout.SOUTH);

            JButton buttonApply = new JButton("Ok");
            buttonApply.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Selection selected = getActiveSelection();
                    if (selected != null) {
                        try {
                            long start = Long.parseLong(textStartTime.getText());
                            long end = Long.parseLong(textEndTime.getText());
                            selected.changeTimeFrame(start,end);
                            updateTimeFrameInfo();
                            timeFrameChangeDialog.setVisible(false);
                        } catch (NumberFormatException e1) {
                            Scene.showErrorMessage("Only numbers are allowed.");
                        }
                    }
                }
            });
            panelButton.add(buttonApply);

            JButton buttonCancel = new JButton("Cancel");
            buttonCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    timeFrameChangeDialog.setVisible(false);
                }
            });
            panelButton.add(buttonCancel);
            timeFrameChangeDialog.pack();
        } else {
            textStartTime.setText(String.valueOf(selected.getStartTime()));
            textEndTime.setText(String.valueOf(selected.getEndTime()));
        }

        Scene.getUIManager().showDialog(timeFrameChangeDialog, timeFrameChangeDialog.getWidth(), timeFrameChangeDialog.getHeight());
    }

    private void filterEntities(boolean keepMatched) {
        int selected = listSelection.getSelectedIndex();
        if (selected == -1) return;
        String patterns = JOptionPane.showInputDialog(Scene.getFrame(),
                          "Input pattern expression(s).\n(seperated with \";\")");
        if (patterns != null) {
            patterns = generateRegExp(patterns);
            Selection tobeChanged = (Selection)selection.get(selected);
            tobeChanged.filterEntities(patterns, keepMatched);
        }
        updateSelection();
    }

}