/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/evolve/
 */

package EVolve;

import java.util.*;
import EVolve.data.DataSource;
import EVolve.visualization.DefaultPredictorFactory;
import EVolve.visualization.VizFactory.*;

public class Main {
    private static VisualizationFactory[] factory;
    private static ArrayList dsList;
    public static ArrayList availableDataSource;

    public static void main(String[] args) {
        // initialize XML driver
        System.setProperty("org.xml.sax.driver", "org.apache.crimson.parser.XMLReaderImpl");

        dsList = new ArrayList();
        availableDataSource = new ArrayList();

        availableDataSource.add("EVolve.DemoSource");
        availableDataSource.add("EVolve.PMCPentiumSource");
        parseArgs(args);

        /* This code has been moved to 'addVizLibrary' */
        /*
        VisualizationFactory[] factory = new VisualizationFactory[10];
        factory[0] = new TableVizFactory();
        factory[1] = new BarChartVizFactory();
        factory[2] = new ThreadHotspotVizFactory();
        factory[3] = new HotspotVizFactory();
        factory[4] = new StackHotspotVizFactory();
        factory[5] = new PredictionVizFactory();
        factory[6] = new CorrelationVizFactory();
        factory[7] = new StackVizFactory();
        factory[8] = new DotPlotVizFactory();
        factory[9] = new MissPredictionMetricVizFactory();
        //factory[9] = new EscObjectVizFactory();
        ((PredictionVizFactory)(factory[5])).addPredictorFactory(new DefaultPredictorFactory());
        ((MissPredictionMetricVizFactory)(factory[9])).addPredictorFactory(new DefaultPredictorFactory());
        */

        //Scene.start(dataSource, factory);
        Scene.start(dsList, factory);
    }

    public static void parseArgs(String[] args) {
        factory = null;

        HashMap vizFactories = new HashMap();

        addVizLibrary(vizFactories);

        try {
            for (int i = 0; i < args.length; i++) {
                if (args[i].equals("-d") || args[i].equals("--datasource")) {
                    StringTokenizer token = new StringTokenizer(args[++i],",");
                    while (token.hasMoreTokens()) {
                        String name = token.nextToken();
                        Object dsource = instanciateDataSource(name);

                        if (dsource == null) {
                            System.err.println("EVolve> Failed to instanciate class: \"" + name + "\"");
                            System.exit(1);
                        } else if (dsource instanceof DataSource) {
                            dsList.add(dsource);
                        } else {
                            System.err.println("EVolve> \"" + args[i] + "\" is not a DataSource");
                            System.exit(2);
                        }
                    }
                } else if (args[i].equals("+v") || args[i].equals("--addviz")) {
                    Object viz = instanciateFromName(args[++i]);
                    if (viz == null) {
                        System.err.println("EVolve> Failed to instanciate class: \"" + args[i] + "\"");
                        System.exit(1);
                    } else if (viz instanceof VisualizationFactory) {
                        vizFactories.put(args[i], viz);
                    } else {
                        System.err.println("EVolve> \"" + args[i] + "\" is not a VisualizationFactory");
                        System.exit(2);
                    }
                } else if (args[i].equals("-v") || args[i].equals("--removeviz")) {
                    if (vizFactories.containsKey(args[++i])) {
                        vizFactories.remove(args[i]);
                    } else {
                        System.err.println("EVolve> Unknown visualization factory: \"" + args[i] + "\"");
                        System.exit(1);
                    }
                } else if (args[i].equals("-h") || args[i].equals("--help")) {
                    System.out.println("Usage: java evolve.Main [options]");
                    System.out.println("  where options can be:");
                    System.out.println("  -h | --help");
                    System.out.println("      Prints this help message");
                    System.out.println("  -d | --datasource <data_source_1,data_source_2,...>");
                    System.out.println("      Specifies the class name of the data source to be used");
                    System.out.println("  +v | --addviz <viz factory>");
                    System.out.println("      Specifies the name of a visualization factory to be registered with EVolve");
                    System.out.println("  -v | --removeviz <viz factory>");
                    System.out.println("      Specifies the name of a visualization factory to be unregistered with EVolve");
                    System.out.println("");
                    System.out.println("Visit the EVolve page at http://www.sable.mcgill.ca/evolve");
                    System.exit(0);
                } else {
                    System.err.println("EVolve> Unrecognized option: \"" + args[i] + "\"");
                    System.exit(1);
                }
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            System.err.println("EVolve> Missing argument!!");
            System.exit(3);
        }

        if (dsList.size() == 0) {
            dsList.add(new DemoSource()); // provide a default data source
        }

        Collection factories = vizFactories.values();
        factory = new VisualizationFactory[factories.size()];
        int j = 0;
        Iterator it = factories.iterator();
        while (it.hasNext()) {
            factory[j++] = (VisualizationFactory) it.next();
        }
    }

    private static void addVizLibrary(HashMap vizFactories) {
        vizFactories.put("TableVizFactory", new TableVizFactory());
        vizFactories.put("BarChartVizFactory", new BarChartVizFactory());
        vizFactories.put("ThreadHotspotVizFactory", new ThreadHotspotVizFactory());
        vizFactories.put("HotspotVizFactory", new HotspotVizFactory());
        vizFactories.put("StackHotspotVizFactory", new StackHotspotVizFactory());
        PredictionVizFactory predViz = new PredictionVizFactory();
        predViz.addPredictorFactory(new DefaultPredictorFactory());
        vizFactories.put("PredictionVizFactory", predViz);
        vizFactories.put("CorrelationVizFactory", new CorrelationVizFactory());
        vizFactories.put("StackVizFactory", new StackVizFactory());
        vizFactories.put("DotPlotVizFactory", new DotPlotVizFactory());
        MissPredictionMetricVizFactory missPredViz = new MissPredictionMetricVizFactory();
        missPredViz.addPredictorFactory(new DefaultPredictorFactory());
        vizFactories.put("MissPredictionMetricVizFactory", missPredViz);
        EventVizFactory eventViz = new EventVizFactory();
        eventViz.addPredictorFactory(new DefaultPredictorFactory());
        vizFactories.put("EventVizFactory", eventViz);
        vizFactories.put("RelationshipVizFactory", new RelationshipVizFactory());
        vizFactories.put("HardwareCounterVizFactory", new HardwareCounterVizFactory());
    }

    private static Object instanciateFromName(String className) {
        /* Get a Class object corresponding to the given
           class name using Java's Reflection API */
        try {
            Class klass = null;
            try {
                klass = Class.forName(className);
            } catch (ClassNotFoundException e) {
                System.err.println("EVolve> Class not found: \"" + className + "\"");
                return null;
            }

            return klass.newInstance();
        } catch (InstantiationException e) {
        } catch (IllegalAccessException e) {
        }

        return null;
    }

    public static Object instanciateDataSource(String className) {
        Object retObj = instanciateFromName(className);
        if ((retObj != null)&&(!availableDataSource.contains(className))) {
        	if (retObj instanceof DataSource) {
				availableDataSource.add(className);
        	}
        }

		return retObj;
    }
}

