package EVolve;

/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.TreeMap;

import javax.swing.JFileChooser;

import EVolve.data.DataSource;
import EVolve.data.ElementDefinition;
import EVolve.data.Entity;
import EVolve.data.EntityBuilder;
import EVolve.data.Event;
import EVolve.data.EventBuilder;
import EVolve.data.FieldDefinition;
import EVolve.exceptions.DataProcessingException;
import EVolve.exceptions.EVolveException;

public class PMCPentiumSource implements DataSource {

    private final String dsourceName = "PMCPentiumSource";
     private String fn = null; // data file name
    private long LastBytecode;
    private long eventCounter;


    private int PMC_COUNTER_NUM = 2; // For Pentinum, Change to 4 for Athlon
    private static int TAIL_LEN = 45; // The length of the last line of the trace file, where the amount of the events is recorded
    private RandomAccessFile file;

    private EntityBuilder EVolveEventTypeBuilder;       // Give the type of an event for Evolve: GC or Ordinary  interval


    private EntityBuilder hardwareEventNameBuilder;       // Give the names of the events
    private EntityBuilder hardwareEventDescBuilder;       // Give the descriptions of the events
    private EntityBuilder hardwareEventIDBuilder;         // Give the IDs of the events
    private EntityBuilder byteCodeIntervalBuilder;        // Give the default interval

    private EventBuilder hardwareEvent_1;
    private EventBuilder hardwareEvent_2;
    private EventBuilder cycleEvent;

    private FieldDefinition[] EVolveEventType = new FieldDefinition[PMC_COUNTER_NUM+1];                // EVolve Type, GC or Interval
    private FieldDefinition[] intervalBytecodeNumStart= new FieldDefinition[PMC_COUNTER_NUM+1];      // Start point
    private FieldDefinition[] intervalBytecodeNumEnd= new FieldDefinition[PMC_COUNTER_NUM+1];        // End point
    private FieldDefinition[] intervalEventsVal= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles


    /*
    private FieldDefinition[] intervalEventsDes = new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    private FieldDefinition[] intervalEventsName= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    private FieldDefinition[] intervalEventsID= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    */
    private EventBuilder semiGCBuilder;
    private FieldDefinition GCBytecodeNum;
    private FieldDefinition[] GCEventsID= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    private FieldDefinition[] GCEventsName= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    private FieldDefinition[] GCEventsDes = new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles
    private FieldDefinition[] GCEventsAllGCVal= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles for all GC phases
    private FieldDefinition[] GCEventsRootScanGCVal= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles for GC phase 1
    private FieldDefinition[] GCEventsCopyGCVal= new FieldDefinition[PMC_COUNTER_NUM+1];  // Events plus Cycles for GC phase 2

    private int[] countedEvents = new int[PMC_COUNTER_NUM+1];




    private ElementDefinition[] definition;
    private int definitionCounter;

    private TreeMap EVTypeMap;
    private TreeMap IDMap;
    private TreeMap nameMap;
    private TreeMap descMap;
    //private TreeMap defIntMap;
    private TreeMap intervalMap;
    private TreeMap GCMap;
    private ArrayList TotalList;
    private Iterator  ItTotalList;



    private String[] eventName;
    private String[] eventDescription;
    private String[] hardEventType;

    private int totalEventNum, interEventNum,gcEventNum;

//    private InstructionAnalyzeRunner runner;

    public void init() throws EVolveException {
        String lastName = fn;
        LastBytecode = 0;
        eventCounter = 0;
        fn = Scene.getDataFileName();
        //runner = Scene.getToolsManager().getInstructionAnalyzeRunner();
        if (fn == null) {
            JFileChooser fc = new JFileChooser(Scene.getUIManager().getLastDataDir());
            if (fc.showOpenDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
                try {
                    file = new RandomAccessFile(fc.getSelectedFile(), "r");
                    fn = fc.getSelectedFile().getName();
                    Scene.setDataFilename(fc.getSelectedFile().getPath());
                    Scene.getUIManager().setLastDataDir(fc.getSelectedFile().getPath());
                    Scene.setDataFilename(null);
                } catch (IOException e) {
                    throw new EVolveException("File loading failed.");
                }
            } else {
                fn = lastName;
                throw new EVolveException("File loading canceled.");
            }
        } else {
            try {
                file = new RandomAccessFile(fn, "r");
            } catch (IOException e) {
                throw new EVolveException("File loading failed.");
            }
        }

        // Get the length of the file
        readLength();

        initEventNameDesc();
    }

    public String getFileName() {
        return fn;
    }

    public String getName() {
        return dsourceName;
    }

    // Read the length of the file to get the event number
    private void readLength()throws EVolveException {
        // Skip to the end of the file
        int trials = 1;

        try {
            long lenOfFile = file.length();
            file.seek(lenOfFile-TAIL_LEN);
            String eventNumStr = file.readLine();
            if (eventNumStr.length() < TAIL_LEN-2 ) {
                eventNumStr = file.readLine();
            }

            // Try to skip all the illegal letters
            while (eventNumStr.indexOf("ALL") != 0) {
                trials++;
                file.seek(lenOfFile-TAIL_LEN*trials);
                while (eventNumStr.indexOf("ALL") != 0) {
                    eventNumStr = file.readLine();
                    if (eventNumStr == null) break;
                }

            }


            // Read the number of events
            int start = eventNumStr.indexOf(' ')+1;
            eventNumStr = eventNumStr.substring(start);
            //start= eventNumStr.indexOf(' ') ;
            //String InterStr= eventNumStr.substring(0,start);
            String InterStr= eventNumStr.substring(0,20);
            InterStr = InterStr.substring(2);
            InterStr = InterStr.trim();
            interEventNum = Integer.valueOf(InterStr).intValue();
            eventNumStr = eventNumStr.substring(23);
            eventNumStr= eventNumStr.trim();
            gcEventNum = Integer.valueOf(eventNumStr).intValue();
            totalEventNum = interEventNum+gcEventNum;
            //System.out.println("total="+totalEventNum);
        } catch (IOException e) {
             throw new EVolveException("FILE CHECKING FAILED");
        }
    }

    // Initlize the Event names and description, The XX item of these array is for the No XX event in PMc
    // Event 257 is saved for the cycles
    private void initEventNameDesc(){
        eventName = new String[258];
        eventDescription = new String[258];


        int i;
        for (i=0; i<258;i++) {
            eventName[i] = "UNDEFINED PENTIUM EVENT";
            eventDescription[i] = "UNDEFINED PENTIUM EVENT";
        }

        // Set the value

        eventName[40] = "L2_IFETCH";
        eventDescription[40] = "L2 cache instruction fetches";

        eventName[41] = "L2_LD";
        eventDescription[41] = "L2 cache data load";

        eventName[42] = "L2_ST";
        eventDescription[42] = "L2 cache data store";

        eventName[44] = "L2_LINES_IN";
        eventDescription[44] = "Number of Lines allocated in L2";

        eventName[67] = "DATA_MEM_REFS";
        eventDescription[67] = "All loads from and store to any memory type";

        eventName[69] = "DCU_LINE_IN";
        eventDescription[69] = "L1 Data Cache Miss";

        eventName[72] = "DCU_MISS_OUTSTANDING";
        eventDescription[72] = "Weighted num. of cycles whiel DCU miss is outstanding";

        eventName[121] = "CPU_CLK_UNHALTED";
        eventDescription[121] = "Num. of cycles during the processor us not halted";

        eventName[128] = "IFU_IFETCH";
        eventDescription[128] = "Num. of instruction fetched";

        eventName[129] = "IFU_IFETCH_MISS";
        eventDescription[128] = "Num. of instruction fetch miss";

        eventName[133] = "ITLB_MISS";
        eventDescription[133] = "Num. of ITLB miss";

        eventName[134] = "IFU_MEM_STALL";
        eventDescription[134] = "Num. of cycles instruction fetch is stalled";

        eventName[162] = "RESOURCE_STALLS";
        eventDescription[162] = "Num. of cycles during ehich there is a resource related stall";

        eventName[192] = "INSTRUCTION_RETIRED";
        eventDescription[192] = "Instructions have been executed to the end";

        eventName[196] = "BR_INSTRUCTION_RETIRED";
        eventDescription[196] =  "Number of branch Instructions retired";

        eventName[197] = "BR_MISS_PRED_RETIRED";
        eventDescription[197] =  "Number of mispredict branch Instructions retired";

        eventName[201] = "BR_TAKEN_RETIRED";
        eventDescription[201] =  "Number of taken branch Instructions retired";

        eventName[202] = "BR_MISS_PRED_TAKEN_RETIRED";
        eventDescription[202] =  "Number of taken mispredict branch Instructions retired";

        eventName[224] = "BR_IN_DECODED";
        eventDescription[224] =  "Number of branch Instructions decoded";

        eventName[226] = "BTB_MISS";
        eventDescription[226] =  "Number of branch for which BTB did not produce a prediction";

        eventName[257] = "CYCLE";
        eventDescription[257] = "Executed Cycles";


        hardEventType = new String[2];
        hardEventType[0] = "Interval";
        hardEventType[1] = "GC_Cycle";
    }


    public void startBuildDefinition() throws DataProcessingException {
        String[] propertySum = {"time","sum"};
        String[] propertyCount = {"time","count","Dotplot time"};
        String[] propertyAmount = {"amount"};
        String[] propertyCoordinate = {"coordinate","time"};
        String[] propertyIndicator = {"indicator"};
        String[] propertyBytecode = {"bytecode" };
        String[] propertyEventValue = {"eventvalue"};
        int i;
        int Id1, Id2;

        definition = new ElementDefinition[5];

        EVolveEventTypeBuilder = new EntityBuilder("EventType", "Type of the Event for EVovle");
        definition[0] = EVolveEventTypeBuilder.buildDefinition();

        byteCodeIntervalBuilder = new EntityBuilder("DefaultInterval", "The default bytecode Interval");
        definition[1] = byteCodeIntervalBuilder.buildDefinition();

         try {
             file.seek(0);

             String line;
            // Skip the beginning text
            for (i=0; i<5;i++ ) {
                file.readLine();
            }
             line = file.readLine();
             line = line.trim();
             Id1 = Integer.valueOf(line).intValue();
             line = file.readLine();
             line = line.trim();
             Id2 = Integer.valueOf(line).intValue();

        } catch (IOException e) {
            throw new DataProcessingException("File processing failed.");
        }


        // Build Interval Event for event 1
        hardwareEvent_1 = new EventBuilder(eventName[Id1], eventDescription[Id1]);
        EVolveEventType[0] = hardwareEvent_1.buildReferenceDefinition("Type",EVolveEventTypeBuilder,null,"Type of this record");
        intervalBytecodeNumStart[0] = hardwareEvent_1.buildValueDefinition("Bytecode Start", propertyBytecode, "Bytecode sequence startpoint");
        intervalBytecodeNumEnd[0] = hardwareEvent_1.buildValueDefinition("Bytecode End", propertyBytecode, "Bytecode sequence endpoint");
        intervalEventsVal[0] = hardwareEvent_1.buildValueDefinition("Number of Event "+eventName[Id1], propertyEventValue, "Occurence Number of Event"+eventName[Id1]);
        definition[2] = hardwareEvent_1.buildDefinition();

        // Build Interval Event for event 2
        hardwareEvent_2 = new EventBuilder(eventName[Id2], eventDescription[Id2]);
        EVolveEventType[1] = hardwareEvent_2.buildReferenceDefinition("Type",EVolveEventTypeBuilder,null,"Type of this record");
        intervalBytecodeNumStart[1] = hardwareEvent_2.buildValueDefinition("Bytecode Start", propertyBytecode, "Bytecode sequence startpoint");
        intervalBytecodeNumEnd[1] = hardwareEvent_2.buildValueDefinition("Bytecode End", propertyBytecode, "Bytecode sequence endpoint");
        intervalEventsVal[1] = hardwareEvent_2.buildValueDefinition("Number of Event "+eventName[Id2], propertyEventValue, "Occurence Number of Event"+eventName[Id2]);
        definition[3] = hardwareEvent_2.buildDefinition();

        // Build Interval Event for cycle
        cycleEvent = new EventBuilder(eventName[257], eventDescription[257]);
        EVolveEventType[2] = cycleEvent.buildReferenceDefinition("Type",EVolveEventTypeBuilder,null,"Type of this record");
        intervalBytecodeNumStart[2] = cycleEvent.buildValueDefinition("Bytecode Start", propertyBytecode, "Bytecode sequence startpoint");
        intervalBytecodeNumEnd[2] = cycleEvent.buildValueDefinition("Bytecode End", propertyBytecode, "Bytecode sequence endpoint");
        intervalEventsVal[2] = cycleEvent.buildValueDefinition("Number of Event "+eventName[257], propertyEventValue, "Occurence Number of Event"+eventName[257]);
        definition[4] = cycleEvent.buildDefinition();
        definitionCounter = -1;
    }

    public ElementDefinition getNextDefinition() throws DataProcessingException {
        definitionCounter++;
        if (definitionCounter < definition.length) {
            return definition[definitionCounter];
        } else {
            return null;
        }
    }

    private String getSub(String line, int part) {
        int start = line.indexOf('\t') + 1;
        int i = 1;
        while (i < part) {
            start = line.indexOf('\t', start) + 1;
            i++;
        }
        int end = line.indexOf('\t', start);
        if (end == -1) {
            return line.substring(start);
        } else {
            return line.substring(start, end);
        }
    }

    public void startBuildEntity() throws DataProcessingException {
        String line;
        try {
            file.seek(0);
            // Skip the beginning text
            for (int i=0; i<5;i++ ) {
                file.readLine();
            }

            EVTypeMap = new TreeMap();
            IDMap = new TreeMap();
            descMap = new TreeMap();
            nameMap = new TreeMap();
            intervalMap = new TreeMap();
            GCMap = new TreeMap();
            TotalList = new ArrayList();

        } catch (IOException e) {
            throw new DataProcessingException("File processing failed.");
        }

        // Read the entities here
        int  Id;
        try {
                Entity newVal = null;
                for (int i=0; i<PMC_COUNTER_NUM;i++ ) {

                        line = file.readLine();
                        line = line.trim();
                        Id = Integer.valueOf(line).intValue();
                        countedEvents[i] = Id;

                }
                line="257";
                countedEvents[PMC_COUNTER_NUM] = 257;
                String str = "CYCLE";
                String str2 = "Executed Cycles";


                EVolveEventTypeBuilder.newEntity(hardEventType[0]);
                //EVolveEventTypeBuilder.newEntity("Interval");
                newVal = EVolveEventTypeBuilder.buildEntity();
                EVTypeMap.put(hardEventType[0],newVal);
                TotalList.add(newVal);

                EVolveEventTypeBuilder.newEntity(hardEventType[1]);
                //EVolveEventTypeBuilder.newEntity("GarbageCollection");
                newVal = EVolveEventTypeBuilder.buildEntity();
                EVTypeMap.put(hardEventType[1],newVal);
                TotalList.add(newVal);

                // For the interval
                line = file.readLine();
                line = line.trim();
                byteCodeIntervalBuilder.newEntity(line);
                newVal = byteCodeIntervalBuilder.buildEntity();
                TotalList.add(newVal);

        } catch (IOException e) {
              throw new DataProcessingException("File processing failed.");
        }

        ItTotalList = TotalList.iterator();
    }


    public Entity getNextEntity() throws DataProcessingException {
        Entity returnVal = null;

        try {
                returnVal = (Entity)ItTotalList.next();
                return returnVal;

        } catch (NoSuchElementException e) {
                returnVal = null;
                return returnVal;
        }
    }

    public long getTotalNumberOfEvents() {
        return totalEventNum * 3 ;
    }

    public long getNumberOfEvents(String definitionName) {

       // if (definitionName.equalsIgnoreCase("HardwareEventCount"))
            return totalEventNum;
        //else
        //    return 0;
    }

    public void startBuildEvent() throws DataProcessingException {
        eventCounter = 0;
        LastBytecode = 0;
        try {
            file.seek(0);
            // Skip the beginning text
            for (int i=0; i<5;i++ ) {
                file.readLine();
            }
            // Skip the first three lines for the entities
            for (int i=0; i<PMC_COUNTER_NUM+1;i++ ) {
                file.readLine();
            }
        } catch (IOException e) {
            throw new DataProcessingException("File processing failed.");
        }
    }

    public Event getNextEvent() throws DataProcessingException {

        eventCounter++;
        try {


            if ((eventCounter%3) == 2) return hardwareEvent_2.buildEvent();
            else if ((eventCounter%3) == 0) return cycleEvent.buildEvent();

            Event returnVal = null;
            String line = file.readLine();

            /* Now Event Counter % 3 == 1 */
            while ((returnVal == null) && (line != null)) {
                line = line.trim();
                if (line.length() < 2)
                {
                   line = file.readLine();
                   continue;
                }
                char ch = line.charAt(0);

                if (ch == 'I') {  // This is an Interval Event
                    hardwareEvent_1.newEvent();
                    hardwareEvent_2.newEvent();
                    cycleEvent.newEvent();

                    hardwareEvent_1.addReferenceField(EVolveEventType[0],(Entity)EVTypeMap.get(hardEventType[0])); // Inteval
                    hardwareEvent_2.addReferenceField(EVolveEventType[1],(Entity)EVTypeMap.get(hardEventType[0])); // Inteval
                    cycleEvent.addReferenceField(EVolveEventType[2],(Entity)EVTypeMap.get(hardEventType[0])); // Inteval

                    String bytecodeStr = getSub(line, 1);
                    bytecodeStr = bytecodeStr.trim();
                    int val = Integer.valueOf(bytecodeStr).intValue(); // Currently use integer, should be changed into long long int later
                    hardwareEvent_1.addValueField(intervalBytecodeNumStart[0],LastBytecode);
                    hardwareEvent_1.addValueField(intervalBytecodeNumEnd[0],val);
                    hardwareEvent_2.addValueField(intervalBytecodeNumStart[1],LastBytecode);
                    hardwareEvent_2.addValueField(intervalBytecodeNumEnd[1],val);
                    cycleEvent.addValueField(intervalBytecodeNumStart[2],LastBytecode);
                    cycleEvent.addValueField(intervalBytecodeNumEnd[2],val);

                    LastBytecode = val;

                    String evtValStr = getSub(line, 2);
                    evtValStr = evtValStr.trim();
                    long evtVal = Long.valueOf(evtValStr).longValue();
                    hardwareEvent_1.addValueField(intervalEventsVal[0],evtVal);

                    evtValStr = getSub(line, 3);
                    evtValStr = evtValStr.trim();
                    evtVal = Long.valueOf(evtValStr).longValue();
                    hardwareEvent_2.addValueField(intervalEventsVal[1],evtVal);

                    evtValStr = getSub(line, 4);
                    evtValStr = evtValStr.trim();
                    evtVal = Long.valueOf(evtValStr).longValue();
                    cycleEvent.addValueField(intervalEventsVal[2],evtVal);
                    returnVal = hardwareEvent_1.buildEvent();

                } else if (ch == 'G') { // This is a GC Event

                    hardwareEvent_1.newEvent();
                    hardwareEvent_2.newEvent();
                    cycleEvent.newEvent();

                    hardwareEvent_1.addReferenceField(EVolveEventType[0],(Entity)EVTypeMap.get(hardEventType[1])); // GC
                    hardwareEvent_2.addReferenceField(EVolveEventType[1],(Entity)EVTypeMap.get(hardEventType[1])); // GC
                    cycleEvent.addReferenceField(EVolveEventType[2],(Entity)EVTypeMap.get(hardEventType[1])); // GC

                    String bytecodeStr = getSub(line, 1);
                    bytecodeStr = bytecodeStr.trim();
                    int val = Integer.valueOf(bytecodeStr).intValue(); // Currently use integer, should be changed into long long int later
                    hardwareEvent_1.addValueField(intervalBytecodeNumStart[0],LastBytecode);
                    hardwareEvent_1.addValueField(intervalBytecodeNumEnd[0],val);
                    hardwareEvent_2.addValueField(intervalBytecodeNumStart[1],LastBytecode);
                    hardwareEvent_2.addValueField(intervalBytecodeNumEnd[1],val);
                    cycleEvent.addValueField(intervalBytecodeNumStart[2],LastBytecode);
                    cycleEvent.addValueField(intervalBytecodeNumEnd[2],val);

                    LastBytecode = val;

                    String evtValStr = getSub(line, 2);
                    evtValStr = evtValStr.trim();
                    long evtVal = Long.valueOf(evtValStr).longValue();
                    hardwareEvent_1.addValueField(intervalEventsVal[0],evtVal);

                    evtValStr = getSub(line, 3);
                    evtValStr = evtValStr.trim();
                    evtVal = Long.valueOf(evtValStr).longValue();
                    hardwareEvent_2.addValueField(intervalEventsVal[1],evtVal);

                    evtValStr = getSub(line, 4);
                    evtValStr = evtValStr.trim();
                    evtVal = Long.valueOf(evtValStr).longValue();
                    cycleEvent.addValueField(intervalEventsVal[2],evtVal);
                    returnVal = hardwareEvent_1.buildEvent();
                }

                if (returnVal == null) {
                    line = file.readLine();
                }
            }
            return returnVal;
        } catch (IOException e) {
            throw new DataProcessingException("File processing failed.");
        }
    }

}