/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import javax.swing.*;
import javax.swing.event.*;
import EVolve.data.DataManager;
import EVolve.util.HelperFuncs;
import EVolve.util.predefinedutils.VisualizationSerializer;
import EVolve.util.predefinedutils.*;
import EVolve.util.overlappers.OverlapVisualization;
import EVolve.util.phasedetectors.phasedetectorUI.PhaseDetectorToolBar;
import EVolve.util.settings.SceneSetting;
import EVolve.visualization.Visualization;

public class UIManager {
    private JFrame frame;
    private JDesktopPane desktop;
    private boolean doNotRemoveWindow;

    private ArrayList windowList;

    private JPanel panelStatus;
    private JLabel labelStatus;

    private JMenuBar menuBar;

    private JMenu menuFile;
    private JMenuItem itemLoad;
    private JMenuItem itemAddDataSource;
    private JMenuItem itemExit;
    private JMenuItem itemRemoveDataSource;

    private JMenu menuVisualization;
    private JMenu menuNew;
    private JMenuItem[] itemNew;
    private JMenuItem itemVisualize;

    private JMenu menuWindow;
    private JMenuItem itemArrange;

    /** added functions */
    private JMenu menuTool;

    private JMenu menuBatch;
    private JMenuItem itemBuildBatch;
    private JMenuItem itemRunBatch;

    private JMenuItem itemReadSetting;
    private JMenuItem itemSaveSetting;

    private JMenu menuOverlapViz;

    private JCheckBoxMenuItem itemDebug;

    private JMenuItem[] itemSettings;
    private JMenuItem itemSynOverlap;
    private JMenuItem itemAsynOverlap;

    private JMenuItem itemEnablePhaseDetector;

    private JMenu menuOptions;
    private JMenuItem itemPath;

    private JMenu menuHelp;
    private JMenuItem itemAbout;

    private JDialog dialog;
    private JComboBox comboColumn;
    private JList listVisualization;
    private DefaultListModel model;

    private JDialog dialogAbout;

    private String lastConfigDir, lastDataDir, lastResultDir;

    private JToolBar toolbar;
    private EVolve.util.phasedetectors.phasedetectorUI.PhaseDetectorToolBar toolbarPhaseDetector;
    private JButton tblVizNow, tblConfigure;
    private JComboBox tblComboDataSource;

    private JList listDataSource;
    private JDialog dialogDataSource;

    public UIManager() {
        try {
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) {}

        frame = new JFrame(Scene.VERSION);

        ImageIcon icon = new ImageIcon(this.getClass().getResource("img/logo_icon.gif"));
        frame.setIconImage(icon.getImage());

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        frame.setBounds(0, 0, Toolkit.getDefaultToolkit().getScreenSize().width, Toolkit.getDefaultToolkit().getScreenSize().height);

        windowList = new ArrayList();

        createMenu();

        createToolBar();
        frame.getContentPane().add(toolbar, BorderLayout.NORTH);


        desktop = new JDesktopPane();
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        frame.getContentPane().add(desktop, BorderLayout.CENTER);

        panelStatus = new JPanel(new BorderLayout());
        panelStatus.setBackground(new Color(240, 240, 240));
        panelStatus.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLoweredBevelBorder(), BorderFactory.createLineBorder(new Color(240, 240, 240), 2)));
        frame.getContentPane().add(panelStatus, BorderLayout.SOUTH);

        labelStatus = new JLabel(Scene.VERSION);
        panelStatus.add(labelStatus, BorderLayout.WEST);

        frame.setVisible(true);

        createDialog();
        createAbout();
        doNotRemoveWindow = false;

        lastDataDir = "";
        lastConfigDir = "";
        lastResultDir="";
    }

    private void createMenu() {
        menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);

        menuFile = new JMenu("File");
        menuFile.setMnemonic(KeyEvent.VK_F);
        menuFile.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}
            public void menuSelected(MenuEvent e) {
                menuFile.removeAll();
                menuFile.add(itemLoad);
                menuFile.addSeparator();
                for (int i=0; i<Main.availableDataSource.size(); i++) {
                    JMenuItem item = new JMenuItem(((char)(i + 48)) + " - Load a " + Main.availableDataSource.get(i) + " Trace");
                    item.setMnemonic(i + 48);
                    item.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            String menuSelected = ((JMenuItem)e.getSource()).getText();
                            String name = menuSelected.substring(11,menuSelected.length()-6);
                            boolean success = addDataSource(name);
                            if (success)
                                loadDataSource();
                        }
                    });
                    menuFile.add(item);
                }
                menuFile.addSeparator();
                menuFile.add(itemAddDataSource);
                menuFile.add(itemRemoveDataSource);
                for (int i=0; i<Main.availableDataSource.size(); i++) {
                    JMenuItem item = new JMenuItem(((char)(i + 65)) + " - Add " + Main.availableDataSource.get(i));
                    item.setMnemonic(i + 65);
                    item.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            String name = ((JMenuItem)(e.getSource())).getText().substring(8);
                            addDataSource(name);
                        }
                    });
                    menuFile.add(item);
                }

                menuFile.addSeparator();
                menuFile.add(itemExit);
            }
        });

        menuBar.add(menuFile);

        itemLoad = new JMenuItem("Load Data...");
        itemLoad.setMnemonic(KeyEvent.VK_L);
        itemLoad.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                loadDataSource();
            }
        });

        itemAddDataSource = new JMenuItem("Add Data Source...");
        itemAddDataSource.setMnemonic(KeyEvent.VK_A);
        itemAddDataSource.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = JOptionPane.showInputDialog(getDesktop(),"Name of the new data source:");
                if (name != null) {
                    addDataSource(name);
                }
            }
        });

        itemRemoveDataSource = new JMenuItem("Remove Data Source...");
        itemRemoveDataSource.setMnemonic(KeyEvent.VK_R);
        itemRemoveDataSource.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                removeDataSource();
            }
        });

        itemExit = new JMenuItem("Exit");
        itemExit.setMnemonic(KeyEvent.VK_X);
        itemExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });

        menuVisualization = new JMenu("Visualization");
        menuVisualization.setMnemonic(KeyEvent.VK_V);
        menuBar.add(menuVisualization);
        menuVisualization.addMenuListener(new MenuListener() {
            public void menuSelected(MenuEvent e) {
                menuVisualization.removeAll();
                menuVisualization.add(menuNew);
                menuVisualization.add(itemVisualize);
                menuVisualization.add(itemReadSetting);
                menuVisualization.add(itemSaveSetting);

                itemSettings = new JMenuItem[Scene.getToolsManager().getPredefinedVisualizationRunner().getPredefinedVisualizationNumber()];
                for (int i = 0; i < itemSettings.length; i++) {
                    String addConfigureName = Scene.getToolsManager().getPredefinedVisualizationRunner().getConfigureName(i);
                    itemSettings[i] = new JMenuItem(addConfigureName);
                    itemSettings[i].addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            PredefinedVisualizationRunner runner = Scene.getToolsManager().getPredefinedVisualizationRunner();
                            int selected = runner.getIdFromName(((JMenuItem)(e.getSource())).getText());
                            runner.setSelected(selected);
                            runner.runPredefinedVisualization();
                        }
                    });
                }
                if (itemSettings.length > 0) {
                    menuVisualization.addSeparator();
                    for (int i=0; i<itemSettings.length; i++)
                        menuVisualization.add(itemSettings[i]);
                }
                enablePredefinedMenu();
            }

            public void menuDeselected(MenuEvent e) {};
            public void menuCanceled(MenuEvent e) {};
        });

        itemVisualize = new JMenuItem("Visualize...");
        itemVisualize.setMnemonic(KeyEvent.VK_V);
        itemVisualize.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().preVisualize();
            }
        });
        menuVisualization.add(itemVisualize);

        itemReadSetting = new JMenuItem("Load Predefined Viz Setting");
        itemReadSetting.setMnemonic(KeyEvent.VK_L);
        itemReadSetting.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                VisualizationSerializer.v().loadConfiguration();
                enableFileMenus();
            }
        });
        itemSaveSetting = new JMenuItem("Save Current Viz Setting");
        itemSaveSetting.setMnemonic(KeyEvent.VK_S);
        itemSaveSetting.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ArrayList vizList = Scene.getVisualizationManager().getVisualizationList();
                if (vizList.size() == 0) return;

                VisualizationSerializer.v().showSavingDialog();
            }
        });

        menuNew = new JMenu("New");
        menuNew.setMnemonic(KeyEvent.VK_N);
        menuVisualization.add(menuNew);

        menuWindow = new JMenu("Window");
        menuWindow.setMnemonic(KeyEvent.VK_W);
        menuWindow.addMenuListener(new MenuListener() {
            public void menuSelected(MenuEvent e) {
                menuWindow.removeAll();
                menuWindow.add(itemArrange);
                for (int i=0; i<windowList.size(); i++){
                    JMenuItem item = new JMenuItem(((char)(i + 65)) + " - " + ((Window)windowList.get(i)).getTitle());
                    item.setMnemonic(i + 65);
                    item.addActionListener(new ActionListener() {
                       public void actionPerformed(ActionEvent e) {
                           Window selected = null;
                           for (int i=0; i<windowList.size(); i++) {
                               selected = (Window)windowList.get(i);
                               if (((JMenuItem)(e.getSource())).getText().substring(4).equals(selected.getTitle())) {
                                   break;
                               }
                           }
                           if (selected != null) {
                               try {
                                selected.setIcon(false);
                               } catch (Exception exp) {}
                               selected.moveToFront();
                               selected.setVisible(true);
                           }
                       }
                    });
                    menuWindow.add(item);
                }
            }

            public void menuDeselected(MenuEvent e) {};
            public void menuCanceled(MenuEvent e) {};
        });
        menuBar.add(menuWindow);

        itemArrange = new JMenuItem("Arrange...");
        itemArrange.setMnemonic(KeyEvent.VK_A);
        itemArrange.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                arrange();
            }
        });
        menuWindow.add(itemArrange);

        menuTool = new JMenu("Tools");
        menuTool.setMnemonic(KeyEvent.VK_T);
        menuBar.add(menuTool);

        itemBuildBatch = new JMenuItem("Build Batch");
        itemBuildBatch.setMnemonic(KeyEvent.VK_B);
        itemBuildBatch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getToolsManager().getBatchRunner().showWindow();
             }
        });
        itemRunBatch = new JMenuItem("Run Batch");
        itemRunBatch.setMnemonic(KeyEvent.VK_R);
        itemRunBatch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (Scene.getToolsManager().getBatchRunner() != null)
                    Scene.getToolsManager().getBatchRunner().runBatch();
            }
        });
        menuBatch = new JMenu("Auto Generate Visualizations");
        menuBatch.setMnemonic(KeyEvent.VK_A);
        menuBatch.add(itemBuildBatch);
        menuBatch.add(itemRunBatch);

        itemSynOverlap = new JMenuItem("Overlap Visualizations...");
        itemSynOverlap.setMnemonic(KeyEvent.VK_O);
        itemSynOverlap.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getToolsManager().getOverlapVisualizationRunner().getSynOverlapper().showWindow();
            }
        });
        itemAsynOverlap = new JMenuItem("Auto Overlap Visualizations ...");
        itemAsynOverlap.setMnemonic(KeyEvent.VK_A);
        itemAsynOverlap.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getToolsManager().getOverlapVisualizationRunner().getAsynOverlapper().showWindow();
            }
        });
        menuOverlapViz = new JMenu("Overlap Visualizations");
        menuOverlapViz.setMnemonic(KeyEvent.VK_O);
        menuOverlapViz.add(itemSynOverlap);
        menuOverlapViz.add(itemAsynOverlap);

        itemEnablePhaseDetector = new JMenuItem("Enable Phase Detector");
        itemEnablePhaseDetector.setMnemonic(KeyEvent.VK_E);
        itemEnablePhaseDetector.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                toolbarPhaseDetector.setVisible(true);
                refreshToolBar();
                Scene.getVisualizationManager().enablePhaseDetector(true);
            }
        });

        itemDebug = new JCheckBoxMenuItem("In debugging");
        itemDebug.setMnemonic(KeyEvent.VK_D);
        itemDebug.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                HelperFuncs.inDebug = itemDebug.isSelected();
            }
        });
        itemDebug.setSelected(false);

        menuTool.add(menuBatch);
        menuTool.add(menuOverlapViz);
        //menuTool.add(itemLinkViz);
        menuTool.add(itemEnablePhaseDetector);
        //menuTool.add(itemDebug);

        menuOptions = new JMenu("Options");
        menuOptions.setMnemonic(KeyEvent.VK_O);
        menuBar.add(menuOptions);

        itemPath = new JMenuItem("Setting Path...");
        itemPath.setMnemonic(KeyEvent.VK_S);
        itemPath.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SceneSetting.v().showSettingDialog();
            }
        });

        menuOptions.add(itemPath);

        menuHelp = new JMenu("Help");
        menuHelp.setMnemonic(KeyEvent.VK_H);
        menuBar.add(menuHelp);

        itemAbout = new JMenuItem("About EVolve...");
        itemAbout.setMnemonic(KeyEvent.VK_A);
        itemAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                showDialog(dialogAbout, 300, 150);
            }
        });
        menuHelp.add(itemAbout);
    }

    private void createToolBar() {
        toolbar = new JToolBar();
        toolbar.setPreferredSize(new java.awt.Dimension(frame.getWidth(),31));
        FlowLayout layout = new FlowLayout(FlowLayout.LEFT,2,0);
        layout.layoutContainer(toolbar);
        toolbar.setLayout(layout);
        toolbar.setFloatable(false);
        toolbar.setRollover(true);
        toolbar.setBackground(new Color(207,207,207));

        tblComboDataSource = new JComboBox();
        ArrayList dmList = Scene.getDataSourceManager().getDataManagerList();
        for (int i=0; i<dmList.size(); i++) {
            tblComboDataSource.addItem(((DataManager)dmList.get(i)).getDatasourceName());
        }
        tblComboDataSource.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DataSourceManager dsm = Scene.getDataSourceManager();
                dsm.switchDataSource(tblComboDataSource.getSelectedIndex());
                if (Scene.getDataManager().isDataLoaded() == false) {
                    disableFunctionMenus();
                } else {
                    enableMenu();
                }
            }
        });

        tblConfigure = new JButton(new ImageIcon(Scene.getGifURL("config.gif")));
        tblConfigure.setToolTipText("Configure Current Visualization");
        tblConfigure.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Visualization visual = Scene.getVisualizationManager().getActiveVisualization();
                if (visual != null) visual.configure();
            }
        });
        tblConfigure.setPreferredSize(new java.awt.Dimension(28,28));

        tblVizNow = new JButton(new ImageIcon(Scene.getGifURL("vizNow.gif")));
        tblVizNow.setToolTipText("Visualize Current Visualization Now");
        tblVizNow.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().visualizeNow();
            }
        });
        tblVizNow.setPreferredSize(new java.awt.Dimension(28,28));

        toolbarPhaseDetector = new EVolve.util.phasedetectors.phasedetectorUI.PhaseDetectorToolBar();
        toolbarPhaseDetector.setVisible(false);

        toolbar.add(new JLabel("Data Source: "));
        toolbar.add(tblComboDataSource);
        toolbar.add(tblConfigure);
        toolbar.add(tblVizNow);
        toolbar.add(toolbarPhaseDetector);

        toolbar.addContainerListener(new ContainerAdapter() {
            public void componentAdded(ContainerEvent e) {
                refreshToolBar();
            }

            public void componentRemoved(ContainerEvent e) {
                refreshToolBar();
            }
        });
    }

    private void createDialog() {
        dialog = new JDialog(frame, "Select Windows", true);
        model = new DefaultListModel();
        listVisualization = new JList(model);
        listVisualization.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        JPanel panelColumn = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelColumn, BorderLayout.NORTH);

        panelColumn.add(new JLabel("Number of column: "));

        comboColumn = new JComboBox();
        comboColumn.addItem("   1   ");
        comboColumn.addItem("   2   ");
        comboColumn.addItem("   3   ");
        comboColumn.addItem("   4   ");
        panelColumn.add(comboColumn);

        JScrollPane scrollPane = new JScrollPane(listVisualization);
        scrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Choose windows"));
        dialog.getContentPane().add(scrollPane, BorderLayout.CENTER);

        JPanel panelButton = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelButton, BorderLayout.SOUTH);

        JButton buttonApply = new JButton("Apply");
        buttonApply.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogApply();
            }
        });
        panelButton.add(buttonApply);

        JButton buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogCancel();
            }
        });
        panelButton.add(buttonCancel);
    }

    public void createAbout() {
        dialogAbout = new JDialog(frame, "About EVolve", true);
        dialogAbout.getContentPane().setBackground(new Color(240, 240, 240));
        dialogAbout.getContentPane().add(new JLabel(new ImageIcon(this.getClass().getResource("img/logo_about.gif"))), BorderLayout.CENTER);
    }

    public void init() {
        if (doNotRemoveWindow) return;
        /*windowList.clear();
        JInternalFrame[] frame = desktop.getAllFrames();
        for (int i = 0; i < frame.length; i++) {
            frame[i].dispose();
        }*/
        ArrayList vizList = Scene.getVisualizationManager().getVisualizationList();
        ArrayList newWindowList = new ArrayList();
        int datasourceId = Scene.getDataSourceManager().getCurrentDataSourceId();
        for (int i=0; i<vizList.size(); i++) {
            Visualization visual = (Visualization)vizList.get(i);
            EVolve.Window window = visual.getWindow();
            if (visual.getDataSourceId() == datasourceId)
                window.dispose();
            else
                newWindowList.add(window);
        }
        windowList = newWindowList;

        disableFunctionMenus();
        menuWindow.setEnabled(false);
    }

    public JFrame getFrame() {
        return frame;
    }

    public void setStatus(String status) {
        labelStatus.setText(status);
    }

    public void enableMenu() {
        menuNew.removeAll();
        String[] name = Scene.getVisualizationManager().getFactoryName();
        itemNew = new JMenuItem[name.length];
        for (int i = 0; i < itemNew.length; i++) {
            itemNew[i] = new JMenuItem(((char)(i + 65)) + " - " + name[i]);
            itemNew[i].setMnemonic(i + 65);
            itemNew[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Scene.getVisualizationManager().newVisualization(((JMenuItem)(e.getSource())).getText().substring(4));
                }
            });
            menuNew.add(itemNew[i]);
        }

        //menuVisualization.setEnabled(true);
        menuNew.setEnabled(true);
        itemVisualize.setEnabled(true);
        menuWindow.setEnabled(true);
        enablePredefinedMenu();
        updateToolbarButtons(windowList.size() > 0);
    }

    public void enableFileMenus() {
        menuFile.setEnabled(true);
        menuTool.setEnabled(true);
    }

    public void showDialog(JDialog dialog, int width, int height) {
        dialog.setBounds((Toolkit.getDefaultToolkit().getScreenSize().width - width) / 2, (Toolkit.getDefaultToolkit().getScreenSize().height - height) / 2, width, height);
        dialog.setVisible(true);
    }

    public void addVisualization(Visualization visualization) {
        Window newWindow = new Window(visualization);
        newWindow.setBounds(0, 0, desktop.getSize().width, desktop.getSize().height);
        newWindow.setVisible(true);

        desktop.add(newWindow);
        newWindow.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameActivated(InternalFrameEvent e) {
            	Visualization visual = Scene.getVisualizationManager().getActiveVisualization();
            	if (visual != null) {
                    switchDataSource(visual.getDataSourceId());
            	}
                refreshToolBar();
            }

            public void internalFrameClosed(InternalFrameEvent e) {
                refreshToolBar();
            }

            public void internalFrameOpened(InternalFrameEvent e) {
                refreshToolBar();
            }
        });
        try {
            newWindow.setSelected(true);
        } catch (Exception e) {};

        windowList.add(newWindow);

        visualization.setWindow(newWindow);
        updateToolbarButtons(windowList.size() > 0);
        refreshToolBar();
    }

    public Window addOverlappedVisualization(OverlapVisualization visual) {
        Window newWindow = new Window(visual);
        newWindow.setBounds(0, 0, desktop.getSize().width/2, desktop.getSize().height/2);
        newWindow.setVisible(true);

        desktop.add(newWindow);
        newWindow.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameActivated(InternalFrameEvent e) {
                updateToolbarButtons(false);
                refreshToolBar();
            }

            public void internalFrameClosed(InternalFrameEvent e) {
                updateToolbarButtons(true);
                refreshToolBar();
            }

            public void internalFrameDeactivated(InternalFrameEvent e) {
                updateToolbarButtons(true);
                refreshToolBar();
            }
        });
        try {
            newWindow.setSelected(true);
        } catch (Exception e) {};

        windowList.add(newWindow);
        return newWindow;
    }

    public void removeWindow(Window window) {
        window.dispose();
        Scene.getVisualizationManager().removeVisualization(window.getVisualization());
        windowList.remove(window);
        updateToolbarButtons(windowList.size()>0);
    }

    private void arrange() {
        int selectedCounter = 0;
        for (int i = 0; i < windowList.size(); i++) {
            if (!((Window)(windowList.get(i))).isIcon()) {
                selectedCounter++;
            }
        }

        int[] selected = new int[selectedCounter];
        selectedCounter = 0;

        model.removeAllElements();

        for (int i = 0; i < windowList.size(); i++) {
            model.addElement(((Window)(windowList.get(i))).getTitle());
            if (!((Window)(windowList.get(i))).isIcon()) {
                selected[selectedCounter] = i;
                selectedCounter++;
            }
        }

        listVisualization.setSelectedIndices(selected);

        showDialog(dialog, 270, 360);
    }

    private void dialogApply() {
        dialog.setVisible(false);

        for (int i = 0; i < windowList.size(); i++) {
            try {
                ((Window)(windowList.get(i))).setIcon(true);
            } catch (Exception e) {}
        }

        int[] selected = listVisualization.getSelectedIndices();

        if (selected.length != 0) {
            boolean[] located = new boolean[selected.length];
            for (int i = 0; i < located.length; i++) {
                located[i] = false;
            }

            int columnNumber = comboColumn.getSelectedIndex() + 1;
            int width = desktop.getWidth() / columnNumber;
            int height = desktop.getHeight() / ((selected.length - 1) / columnNumber + 1);

            for (int i = 0; i < selected.length; i++) {
                int x = (i % columnNumber) * width;
                int y = (i / columnNumber) * height;

                int minDistance = Integer.MAX_VALUE;
                int minIndex = -1;

                for (int j = 0; j < selected.length; j++) {
                    if (!located[j]) {
                        int distance = (((Window)(windowList.get(selected[j]))).getX() - x) * (((Window)(windowList.get(selected[j]))).getX() - x) + (((Window)(windowList.get(selected[j]))).getY() - y) * (((Window)(windowList.get(selected[j]))).getY() - y);
                        if (distance < minDistance) {
                            minDistance = distance;
                            minIndex = j;
                        }
                    }
                }

                try {
                    ((Window)(windowList.get(selected[minIndex]))).setIcon(false);
                    ((Window)(windowList.get(selected[minIndex]))).setBounds(x, y, width, height);
                    located[minIndex] = true;
                } catch (Exception e) {}
            }
        }
    }

    private void dialogCancel() {
        dialog.setVisible(false);
    }

    private void refreshMenu(String filename,String configureName) {
        int selIndex = 0;
        /*if (Scene.getToolsManager().getPredefinedVisualizationRunner().containsConfigure(configureName)) {
            Scene.showErrorMessage("Visualization configuration for this name already exists!");
            return;
        }*/

        Scene.getToolsManager().getPredefinedVisualizationRunner().addPredefinedVisualization(filename,configureName);
        itemSettings = new JMenuItem[Scene.getToolsManager().getPredefinedVisualizationRunner().getPredefinedVisualizationNumber()];
        for (int i = 0; i < itemSettings.length; i++) {
            String addConfigureName = Scene.getToolsManager().getPredefinedVisualizationRunner().getConfigureName(i);
            if (addConfigureName.equals(configureName)) {
                Scene.getToolsManager().getPredefinedVisualizationRunner().setSelected(i);
                selIndex = i;
            }
            itemSettings[i] = new JMenuItem(addConfigureName);
            itemSettings[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    for (int i=0; i<itemSettings.length; i++) {
                        if (itemSettings[i].isSelected()) itemSettings[i].setSelected(false);
                    }
                    int selected = Scene.getToolsManager().getPredefinedVisualizationRunner().getIdFromName(
                                   ((JRadioButtonMenuItem)(e.getSource())).getText());
                    Scene.getToolsManager().getPredefinedVisualizationRunner().setSelected(selected);
                    itemSettings[selected].setSelected(true);
                }
            });
        }
        itemSettings[selIndex].setSelected(true);
    }

    public void clearWindowsList() {
        windowList.clear();
    }

    public void setDoNotRemoveWindow(boolean newValue) {
        doNotRemoveWindow = newValue;
    }

    public JDesktopPane getDesktop() {
        return desktop;
    }

    public String getLastDataDir() {
        return lastDataDir;
    }

    public void setLastDataDir(String dir) {
        lastDataDir = dir;
    }

    public String getLastConfigDir() {
        return lastConfigDir;
    }

    public void setLastConfigDir(String dir) {
        lastConfigDir = dir;
    }

    public String getLastResultDir() {
        return lastResultDir;
    }

    public void setLastResultDir(String dir) {
        lastResultDir = dir;
    }

    public void disableFunctionMenus() {
        //menuVisualization.setEnabled(false);
        menuNew.setEnabled(false);
        itemVisualize.setEnabled(false);
        enablePredefinedMenu();
        tblVizNow.setEnabled(false);
        tblConfigure.setEnabled(false);
    }

    public void disableFileMenus() {
        menuFile.setEnabled(false);
        menuTool.setEnabled(false);
    }

    public void updateDatasourceCombo() {
        int dsourceId = Scene.getDataSourceManager().getCurrentDataSourceId();

        tblComboDataSource.removeAllItems();
        ArrayList dmList = Scene.getDataSourceManager().getDataManagerList();
        for (int i=0; i<dmList.size(); i++) {
            tblComboDataSource.addItem(((DataManager)dmList.get(i)).getDatasourceName());
        }
        tblComboDataSource.setSelectedIndex(dsourceId);
        refreshToolBar();
    }

    public void updateToolbarButtons(boolean flag) {
        tblVizNow.setEnabled(flag);
        tblConfigure.setEnabled(flag);
    }

    public void refreshToolBar() {
        int toolbarWidth = 0;

        toolbarPhaseDetector.refresh();
        for (int i=0; i<toolbar.getComponents().length; i++) {
            if (!toolbar.getComponent(i).isVisible()) continue;
            double width = toolbar.getComponent(i).getPreferredSize().getWidth();

            if (width < toolbar.getComponent(i).getWidth())
                width = toolbar.getComponent(i).getWidth();

            toolbarWidth += width;
        }
        if (toolbarWidth>=frame.getWidth()-20) {
            toolbar.setBounds(0,0,frame.getWidth(),60);
            toolbar.setPreferredSize(new Dimension(frame.getWidth(),60));
        } else {
            toolbar.setPreferredSize(new Dimension(frame.getWidth(),30));
            toolbar.setBounds(0,0,frame.getWidth(),30);
        }
        toolbar.revalidate();
        desktop.revalidate();

    }

    public void enablePhaseDetectorButton(boolean flag) {
        toolbarPhaseDetector.enableButton(flag);
    }

    public void updateFunctionControl(boolean flag) {
        //menuVisualization.setEnabled(flag);
        menuNew.setEnabled(flag);
        itemVisualize.setEnabled(flag);
        enablePredefinedMenu();
        tblVizNow.setEnabled(flag);
        tblConfigure.setEnabled(flag);

        menuFile.setEnabled(flag);
        menuTool.setEnabled(flag);

        for (int i=0; i<toolbar.getComponents().length; i++) {
            toolbar.getComponent(i).setEnabled(flag);
        }
        toolbarPhaseDetector.enableButton(flag);
    }

    public PhaseDetectorToolBar getPhaseDetectorToolBar() {
        return toolbarPhaseDetector;
    }

    public void resetDebugMenu(boolean flag) {
        itemDebug.setSelected(flag);
    }

    public void switchDataSource(int datasourceId) {
    	if (!tblComboDataSource.isEnabled()) return;
        DataSourceManager dsm = Scene.getDataSourceManager();
        tblComboDataSource.setSelectedIndex(datasourceId);
        dsm.switchDataSource(datasourceId);
        refreshToolBar();
    }

    public boolean addDataSource(String name) {
        boolean success = Scene.getDataSourceManager().addDataSource(name);
        if (success) {
            updateDatasourceCombo();
            switchDataSource(Scene.getDataSourceManager().getDataManagerList().size()-1);
        }

        return success;
    }

    private void loadDataSource() {
        Scene.loadDataSource();
        for (int i = 0; i<Scene.getToolsManager().getPredefinedVisualizationRunner().getPredefinedVisualizationNumber(); i++) {
            Scene.getToolsManager().getPredefinedVisualizationRunner().getPredefinedVizByIndex(i).setTranslated(false);
        }
    }

    private void removeDataSource() {
        dialogDataSource = new JDialog(Scene.getFrame(), "Select Data Source to be reomved", true);
        model = new DefaultListModel();
        listDataSource = new JList(model);
        listDataSource.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        JScrollPane scrollPane = new JScrollPane(listDataSource);
        scrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Choose Data Source"));
        dialogDataSource.getContentPane().add(scrollPane, BorderLayout.CENTER);

        JPanel panelButton = new JPanel(new FlowLayout());
        dialogDataSource.getContentPane().add(panelButton, BorderLayout.SOUTH);

        JButton buttonRemove = new JButton("Remove");
        buttonRemove.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                remove();
                dialogDataSource.setVisible(false);
            }
        });
        panelButton.add(buttonRemove);

        JButton buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogDataSource.setVisible(false);
            }
        });
        panelButton.add(buttonCancel);

        ArrayList dmList = Scene.getDataSourceManager().getDataManagerList();
        for (int i=0; i<dmList.size(); i++)
            model.addElement(((DataManager)dmList.get(i)).getDatasourceName());

        Scene.getUIManager().showDialog(dialogDataSource, 270, 360);
    }

    private void remove() {
        int [] indices = listDataSource.getSelectedIndices();

        if (indices.length == model.getSize()) {
            Scene.showErrorMessage("At least one data source should be left!");
            return;
        }

        windowList = Scene.getVisualizationManager().removeDataSource(indices);
        Scene.getDataSourceManager().removeDataSource(indices);
        Scene.setCurrentDataFilename(Scene.getDataManager().getDataSource().getFileName());
        updateDatasourceCombo();
        createDialog();
    }

    private void enablePredefinedMenu() {
        if (itemSettings == null) return;

        PredefinedVisualizationRunner runner = Scene.getToolsManager().getPredefinedVisualizationRunner();
        ArrayList list = runner.getPredefinedVizList();
        boolean flag = Scene.getDataManager().isDataLoaded();

        for (int i=0; i<itemSettings.length; i++) {
            PredefinedVisualization pViz = (PredefinedVisualization)list.get(i);
            itemSettings[i].setEnabled(pViz.isAutoLoadTrace()|(flag&&
                   pViz.getUsedDataSourceName().equals(Scene.getDataManager().getDataSource().getClass().getName())));
        }
    }
}
