/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve;

import java.awt.BorderLayout;

import javax.swing.JInternalFrame;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import EVolve.util.overlappers.OverlapVisualization;
import EVolve.visualization.Visualization;

public class Window extends JInternalFrame {
    private Visualization visualization;
    private OverlapVisualization oVisual;

    public Window(OverlapVisualization oVisual) {
        super("Overlapped visualization",true,true,true,true);
        this.visualization = null;
        this.oVisual = oVisual;
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(oVisual.getPanel(), BorderLayout.CENTER);

        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(InternalFrameEvent e) {
                cleanup();
            }
        });
    }

    public Window(Visualization visualization) {
        super(visualization.getName() +  " - " + visualization.getSubjectDefinition().getName(), true, true, true, true);

        this.visualization = visualization;
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(visualization.getPanel(), BorderLayout.CENTER);

        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(InternalFrameEvent e) {
                unregisterLinkedViz();
                cleanup();
            }
        });
    }

    public Visualization getVisualization() {
        return visualization;
    }

    private void cleanup() {
        Scene.getUIManager().removeWindow(this);
        if (visualization!=null)
            visualization.cleanup();
        else
            oVisual.cleanup();
    }

    private void unregisterLinkedViz() {
        Scene.getToolsManager().getLinkedVisualizationRunner().unregisterLinkedViz(visualization);
    }

    private void unregisterOverlappedViz() {
        Scene.getToolsManager().getOverlapVisualizationRunner().unregisterOverlappedViz(visualization);
    }

}