/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;


public class DataFilter implements Cloneable{
    private String name; // name of the filter
    private int fieldIndex; // index of the field that the filter needs to get
    private ReferenceLink referenceLink; // reference link
    private String description; // description of the filter
    private String[] property; // property names of viz dimension

    /**
     * Creates a data filter.
     *
     * @param  name  name of the filter
     * @param  fieldIndex  index of the field that the filter needs to get
     * @param  referenceLink  reference link, null if the field is a value
     * @param  descri  description of the filter
     */
    public DataFilter(String name, int fieldIndex, ReferenceLink referenceLink, String description, String[] property) {
        this.name = name;
        this.fieldIndex = fieldIndex;
        this.referenceLink = referenceLink;
        this.description = description;
        this.property = property;
    }

    /**
     * Gets the name of the filter.
     *
     * @return  name of the filter
     */
    public String getName() {
        return name;
    }

    /**
     * Gets the type of the refenrence link.
     *
     * @return  type of the refenrence link, -1 if the field is a value
     */
    public int getTargetType() {
        if (referenceLink == null) {
            return -1;
        } else {
            return referenceLink.getTargetType();
        }
    }

    /**
     * Gets the data from the element.
     *
     * @return  id of the entity if the field is a reference, or the value if the field is a value
     */
    public long getData(Element element) {
        if (referenceLink == null) {
            return element.getField()[fieldIndex];
        } else {
            return referenceLink.getTarget(element.getField()[fieldIndex]);
        }
    }

    /**
     * Gets the description of the filter.
     *
     * @return  description of the filter
     */
    public String getDescription() {
        return description;
    }

    public String[] getProperty() {
        return property;
    }

    public Object clone() {
        DataFilter o = null;
        try {
            o = (DataFilter)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.name = name;
        o.referenceLink = (referenceLink == null) ? null : (ReferenceLink)referenceLink.clone();
        return o;
    }
}