/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;

import java.awt.Color;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import EVolve.Scene;
import EVolve.exceptions.DataProcessingException;
import EVolve.exceptions.EVolveException;
import EVolve.visualization.VisualizationDefinition;

public class DataManager implements Worker{
    private DataSource dataSource;
    private ElementDefinition[] definition;
    private ReferenceLink[] link;
    private HashMap[] entity;
    private ElementFilter[][] elementFilter;
    private Color color;
    private int colorRGB;
    private long eventCounter;
    private long[] eventInterval;
    private int currentInterval;
    private boolean dataProcessed;
    private boolean dataLoaded;
    private int state;


    public DataManager(DataSource dataSource) {
        this.dataSource = dataSource;
        this.eventCounter = 0;
        dataLoaded = false;
    }

    public void init() throws EVolveException {
        ElementBuilder.init();

        dataSource.init();

        dataSource.startBuildDefinition();

        ArrayList definitionList = new ArrayList();
        ElementDefinition nextDefinition = dataSource.getNextDefinition();
        while (nextDefinition != null) {
            definitionList.add(nextDefinition);
            nextDefinition = dataSource.getNextDefinition();
        }

        definition = new ElementDefinition[definitionList.size()];
        for (int i = 0; i < definition.length; i++) {
            definition[((ElementDefinition)(definitionList.get(i))).getType()] = (ElementDefinition)(definitionList.get(i));
        }

        /*ArrayList[] entityList = new ArrayList[definition.length];
        for (int i = 0; i < entityList.length; i++) {
            entityList[i] = new ArrayList();
        }*/

        entity = new HashMap[definition.length];
        for (int i = 0; i < entity.length; i++) {
            entity[i] = new HashMap();
        }

        dataSource.startBuildEntity();

        /*Entity nextEntity = dataSource.getNextEntity();
        while (nextEntity != null) {
            entityList[nextEntity.getType()].add(nextEntity);
            nextEntity = dataSource.getNextEntity();
        }*/

        Entity nextEntity = dataSource.getNextEntity();
        while (nextEntity != null) {
            int type = nextEntity.getType();
            entity[type].put(new Long(nextEntity.getId()),nextEntity);
            nextEntity = dataSource.getNextEntity();
        }

        /*entity = new Entity[entityList.length][];

        for (int i = 0; i < entity.length; i++) {
            entity[i] = new Entity[entityList[i].size()];
            for (int j = 0; j < entity[i].length; j++) {
                entity[i][((Entity)(entityList[i].get(j))).getId()] = (Entity)(entityList[i].get(j));
            }
        }*/

        // creates links
        ArrayList list = new ArrayList();
        for (int i = 0; i < definition.length; i++) {
            for (int j = 0; j < definition[i].getFieldDefinition().length; j++) {
                if ((definition[i].getFieldDefinition()[j].getReference() != -1) &&
                    (definition[i].getFieldDefinition()[j].getReference() != i)) {
                    list.add(new ReferenceLink(definition[i].getFieldDefinition()[j].getName(), i,
                             j, definition[i].getFieldDefinition()[j].getReference(),
                             definition[i].getFieldDefinition()[j].getDescription(),
                             definition[i].getFieldDefinition()[j].getProperties()));
                }
            }
        }

        int head = 0;
        int tail = list.size();
        int initSize = list.size();

        while (head < tail) {
            for (int i = head; i < tail; i++) {
                for (int j = 0; j < initSize; j++) {
                    ReferenceLink from = (ReferenceLink)(list.get(j));
                    ReferenceLink to = (ReferenceLink)(list.get(i));
                    if ((from.getTargetType() == to.getSourceType()) && (from.getSourceType() != to.getTargetType())) {
                        list.add(new ReferenceLink(from, to));
                    }
                }
            }
            head = tail;
            tail = list.size();
        }

        link = new ReferenceLink[list.size()];
        for (int i = 0; i < link.length; i++) {
            link[i] = (ReferenceLink)(list.get(i));
        }
    }

    public void sendEvents() throws DataProcessingException {

        state = Worker.STATE_RUNNING;

        Selection[] selection = Scene.getFilter().getSelection();
        Selection activeSelection = Scene.getFilter().getActiveSelection();
        dataProcessed = false;
        if (activeSelection != null) {
            eventInterval = new long[4];
            eventInterval[0] = 0;
            eventInterval[1] = activeSelection.getStart();
            eventInterval[2] = activeSelection.getEnd();
            eventInterval[3] = Integer.MAX_VALUE;
        } else {
            eventInterval = new long[2];
            eventInterval[0] = 0;
            eventInterval[1] = Integer.MAX_VALUE;
        }

        elementFilter = new ElementFilter[eventInterval.length - 1][definition.length];
        for (int i = 0; i < elementFilter.length; i++) {
            boolean isIncluded = activeSelection == null ? true : false;
            for (int j = 0; j < selection.length; j++) {
                if ((selection[j] == activeSelection)&&
                    (((selection[j].getStart() >= eventInterval[i])&&(selection[j].getEnd() <= eventInterval[i+1]))
                     ||((selection[j].getStart()==eventInterval[i])&&(eventInterval[i]==0)))) {
                    isIncluded = true;
                    break;
                }
            }

            if (isIncluded)
            {
                ArrayList list = new ArrayList();
                if (activeSelection != null) list.add(activeSelection);
                for (int j = 0; j < selection.length; j++) {
                    if ((selection[j].getColor()!=null) && (selection[j] != activeSelection)) {
                        list.add(selection[j].specialClone());
                    }
                }
                Selection[] tempSelection = new Selection[list.size()];
                for (int j = 0; j < tempSelection.length; j++) {
                    tempSelection[j] = (Selection)(list.get(j));
                }

                for (int j = 0; j < definition.length; j++) {
                    if (Scene.getVisualizationManager().getVisualizationType().indexOf("" + j + "") != -1) {
                        elementFilter[i][j] = new ElementFilter(definition[j], tempSelection);
                    } else {
                        elementFilter[i][j] = null;
                    }
                }
            } else {
                elementFilter[i] = null;
            }

        }

        currentInterval = 0;

        dataSource.startBuildEvent();
        eventCounter = 0;

        Event nextEvent = dataSource.getNextEvent();
        while (nextEvent != null) {

            if (state == STATE_PAUSING) {
                try {
                    synchronized(this) {
                        this.state = STATE_PAUSED;
                        wait();
                    }
                    state = STATE_RUNNING;
                } catch (InterruptedException e) {
                    if (state == STATE_STOPPING) {
                        state = STATE_STOPPED;
                        throw new DataProcessingException("Data processing cancelled by the user.");
                    }
                }
            } else if (state == STATE_STOPPING) {
                state = STATE_STOPPED;
                throw new DataProcessingException("Data processing cancelled by the user.");
            }


            if ((elementFilter[currentInterval] != null) && (elementFilter[currentInterval][nextEvent.getType()] != null)) {
                color = elementFilter[currentInterval][nextEvent.getType()].getColor(nextEvent);
                if (color != null) {
                    colorRGB = color.getRGB();
                    Scene.getVisualizationManager().receiveEvent(nextEvent);
                }
            }

            if (eventCounter == eventInterval[currentInterval + 1]) {
                currentInterval++;
            }

            eventCounter++;
            nextEvent = dataSource.getNextEvent();
        }
        dataProcessed = true;
        state = STATE_FINISHED;
    }

    public HashMap[] getEntity() {
        return entity;
    }

    public ElementDefinition[] getElementDefinition() {
        return definition;
    }

    public ElementDefinition[] getElementDefinition(VisualizationDefinition visualizationDefinition) {
        ArrayList list = new ArrayList();
        for (int i = 0; i < definition.length; i++) {
            boolean add = true;
            for (int j = 0; j < visualizationDefinition.getDimensionDefinition().length; j++) {
                DataFilter[] temp = getDataFilter(definition[i], visualizationDefinition.getDimensionDefinition()[j].getProperty());
                if (temp.length == 0) {
                    add = false;
                    break;
                }
            }
            if (add) {
                list.add(definition[i]);
            }
        }

        ElementDefinition[] returnVal = new ElementDefinition[list.size()];
        for (int i = 0; i < returnVal.length; i++) {
            returnVal[i] = (ElementDefinition)(list.get(i));
        }

        return returnVal;
    }

    public DataFilter[] getDataFilter(ElementDefinition subjectDefinition, String property) {
        ArrayList list = new ArrayList();
        FieldDefinition fd;
        boolean nonLinkField = true;

        for (int i = 0; i<link.length; i++) {
            if (link[i].hasProperty(property)) {
                if (link[i].getSourceType() == subjectDefinition.getType()) {
                    list.add(new DataFilter(link[i].getName(), link[i].getSourceIndex(), link[i], link[i].getDescription(),null));
                }
                nonLinkField = false;
            }
        }

        if (nonLinkField) {
            for (int i = 0; i < subjectDefinition.getFieldDefinition().length; i++) {
                fd = subjectDefinition.getFieldDefinition()[i];
                if (subjectDefinition.getFieldDefinition()[i].hasProperty(property)) {
                    list.add(new DataFilter(subjectDefinition.getFieldDefinition()[i].getName(), i,
                                            null, subjectDefinition.getFieldDefinition()[i].getDescription(),fd.getProperties()));
                }
            }
        }

        DataFilter[] returnVal = new DataFilter[list.size()];
        for (int i = 0; i < returnVal.length; i++) {
            returnVal[i] = (DataFilter)(list.get(i));
        }

        return returnVal;
    }

    public Color getColor() {
        return color;
    }

    public int getColorRGB() {
        return colorRGB;
    }

    public long getEventCounter() {
        return eventCounter;
    }

    public ReferenceLink[] getReferenceLink() {
        return link;
    }

    public boolean isCompleted() {
        return dataProcessed;
    }

    public String getDatasourceName() {
        String fileName = dataSource.getFileName();

        if (fileName == null)
            fileName = "No Data";
        else
            fileName = fileName.substring(fileName.lastIndexOf(File.separatorChar)+1);

        return dataSource.getName()+" - "+fileName;
    }

    public boolean isDataLoaded() {
        return dataLoaded;
    }

    public void setDataLoaded(boolean loaded) {
        dataLoaded = loaded;
    }

    public DataSource getDataSource() {
        return dataSource;
    }

    public synchronized void resume() {
        notifyAll();
    }

    public synchronized void pause() {
        if (state == STATE_RUNNING) {
            state = STATE_PAUSING;
        }
    }

    public synchronized void stop() {
        switch (state) {
            case STATE_NOT_STARTED:
            case STATE_STOPPED:
            case STATE_FINISHED:
                break;
            default:
                state = STATE_STOPPING;
                break;
        }
    }

    public synchronized long getCurrentProgress() {
        return eventCounter;
    }

    public synchronized long getMaxProgress() {
        return dataSource.getTotalNumberOfEvents();
    }

    public synchronized int getCurrentState() {
        return state;
    }

    public synchronized void start() {
        state = STATE_RUNNING;
    }

    public void join() throws InterruptedException {
    }

    public void resetState() {
        state = Worker.STATE_NOT_STARTED;
    }
}