/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;

import EVolve.exceptions.DataProcessingException;
import EVolve.exceptions.EVolveException;

/**
 * EVolve data source.
 */
public interface DataSource {
    /**
     * Initializes the data source.
     */
    public void init() throws EVolveException;

    /**
     * Starts building the element definitions.
     */
    public void startBuildDefinition() throws DataProcessingException;

    /**
     * Gets the next element definition.
     *
     * @return  next element definition, null if all the definitions are sent.
     */
    public ElementDefinition getNextDefinition() throws DataProcessingException;

    /**
     * Starts building the entities.
     */
    public void startBuildEntity() throws DataProcessingException;

    /**
     * Gets the next entity.
     *
     * @return  next entity, null if all the entities are sent.
     */
    public Entity getNextEntity() throws DataProcessingException;

    /**
     * Starts building the events.
     */
    public void startBuildEvent() throws DataProcessingException;

    /**
     * Gets the next event.
     *
     * @return  next event, null if all the events are sent.
     */
    public Event getNextEvent() throws DataProcessingException;

    /**
     * Gets the name of the datasource.
     *
     * @return  data sourse name
     */
    public String getName();

    /**
     * Gets the name of the data trace file.
     *
     * @return  trace file name or null
     */
    public String getFileName();

    /**
     * Gets total number of events in the data trace
     *
     * @return  a long type number containing total number of events
     */
    public long getTotalNumberOfEvents();

    /**
     * Gets number of events corresponding event definition name
     *
     * @return  event number
     */
    public long getNumberOfEvents(String definitionName);

}