/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;

import java.util.ArrayList;

/**
 * Entity builder.
 */
public class EntityBuilder extends ElementBuilder {
    private ArrayList comparatorList;

    /**
     * Creates an entity builder.
     *
     * @param   entityName name of the entities
     * @param   entityDescription description of the entities
     */
    public EntityBuilder(String entityName, String entityDescription) {
        super(entityName, entityDescription);
        comparatorList = new ArrayList();
    }

    /**
     * Builds the element definition.
     *
     * @return  the element definition
     */
    public EntityDefinition buildDefinition() {
        EntityDefinition entityDefinition = super.buildEntityDefinition();
        entityDefinition.addComparators(comparatorList);
        return entityDefinition;
    }

    /**
     * Starts building new entity.
     *
     * @param   entityName name of the entity
     */
    public void newEntity(String entityName) {
        super.newEntity(entityName);
    }

    /**
     * Builds the entity.
     *
     * @return  the entity
     */
    public Entity buildEntity() {
        return super.buildEntity();
    }

    public void addComparator(EntityComparator entityComparator) {
        comparatorList.add(entityComparator);
    }
}