/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import EVolve.Scene;
import EVolve.util.HelperFuncs;

public class ReferenceLink implements Cloneable{
    private String name; // name of the link
    private int sourceType; // type of the source element
    private int sourceIndex; // field index of the source element
    private int targetType; // type of the target entity
    private HashMap target; // id of the targets
    private String description; // description of the link
    private HashSet properties;

    /**
     * Creates a reference link.
     *
     * @param  name  name of the link
     * @param  sourceType  type of the source element
     * @param  sourceIndex  field index of the source element
     * @param  targetType  type of the target entity
     * @param  description  description of the link
     * @param  properties properties of the link
     */
    public ReferenceLink(String name, int sourceType, int sourceIndex, int targetType, String description, String[] properties) {
        this.name = name;
        this.sourceType = sourceType;
        this.sourceIndex = sourceIndex;
        this.targetType = targetType;
        this.description = description;
        this.properties = new HashSet();
        for (int i=0; i<properties.length; i++)
            this.properties.add(properties[i]);
        if (properties.length == 0)
            this.properties.add("reference");

        target = Scene.getDataManager().getEntity()[targetType];
        /*target = new int[Scene.getDataManager().getEntity()[targetType].length];
        for (int i = 0; i < target.length; i++) {
            target[i] = i;
        }*/
    }

    /**
     * Creates a reference link by combining two reference links.
     *
     * @param  from  where the link starts
     * @param  to  where the link ends
     */
    public ReferenceLink(ReferenceLink from, ReferenceLink to) {
        this.name = to.name;
        this.sourceType = from.sourceType;
        this.sourceIndex = from.sourceIndex;
        this.targetType = to.targetType;
        this.description = from.description + "  >>>  " + to.description;

        /*target = new int[from.target.length];
        for (int i = 0; i < target.length; i++) {
            target[i] = to.target[Scene.getDataManager().getEntity()[from.targetType][from.target[i]].getField()[to.sourceIndex]];
        }*/
        target = new HashMap();
        Iterator it = from.target.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object fromKey = new Long(((Entity)from.target.get(key)).getField()[to.sourceIndex]);
            Entity entity = (Entity)Scene.getDataManager().getEntity()[from.targetType].get(fromKey);
            target.put(key,entity);
        }
        this.properties = HelperFuncs.cloneHashSet(from.properties);
    }

    /**
     * Gets the name of the link.
     *
     * @return  name of the link
     */
    public String getName() {
        return name;
    }

    /**
     * Gets the type of the source element.
     *
     * @return  type of the source element
     */
    public int getSourceType() {
        return sourceType;
    }

    /**
     * Gets the field index of the source element.
     *
     * @return  field index of the source element
     */
    public int getSourceIndex() {
        return sourceIndex;
    }

    /**
     * Gets the type of the target entity.
     *
     * @return  type of the target entity
     */
    public int getTargetType() {
        return targetType;
    }

    /**
     * Gets the target.
     *
     * @param  source reference
     * @return  target
     */
    public long getTarget(long source) {
        return ((Entity)target.get(new Long(source))).getId();
    }

    /**
     * Gets the description of the link.
     *
     * @return  description of the link
     */
    public String getDescription() {
        return description;
    }

    public boolean hasProperty(String property) {
        return properties.contains(property);
    }

    public Object clone() {
        ReferenceLink o = null;
        try {
            o = (ReferenceLink)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.name = name;
        o.target = HelperFuncs.cloneHashMap(target) ;
        return o;
    }
}