/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.data;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import EVolve.Scene;

public class Selection implements Cloneable{
    public static int total = 0;
    private String name;
    private int sourceType, entityType;
    private String entityCategory;
    private Entity[] selected;
    private long start, end;
    private ReferenceLink[] link;
    private Color color;
    private long startTime = -1, endTime = -1;
    private ArrayList timeMap;

    public Selection(String name, int sourceType, int entityType, String entityCategory, long[] selected, long start, long end, ArrayList timemap) {
        this.entityType = entityType;
        this.entityCategory = entityCategory;
        this.sourceType = sourceType;
        this.selected = new Entity[selected.length];
        for (int i = 0; i < selected.length; i++) {
            try {
            this.selected[i] = (Entity)Scene.getDataManager().getEntity()[entityType].get(new Long(selected[i]));
            } catch (Exception e) {
                int qq = 0;
                qq++;
            }
        }
        this.start = start;
        this.end = end;
        this.color = null;
        this.link = new ReferenceLink[0];

        this.name = name;
        total++;
        timeMap = new ArrayList();
        if (timemap != null) {
            for (int i=0; i<timemap.size(); i++) {
                long[] newValue = new long[2];
                long[] oldValue = (long[])timemap.get(i);
                newValue[0] = oldValue[0];
                newValue[1] = oldValue[1];
                timeMap.add(newValue);
            }
            findTimeInterval(start, end);
        }
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public Color getColor() {
        return color;
    }

    public void setLink(ReferenceLink[] link) {
        this.link = link;
    }

    public ReferenceLink[] getLink() {
        return link;
    }

    public int getEntityType() {
        return entityType;
    }

    public Entity[] getSelected() {
        return selected;
    }

    public void setSelected(Entity[] selected) {
        this.selected = selected;
    }

    public long getStart() {
        return start;
    }

    public long getEnd() {
        return end;
    }

    public long getStartTime() {
        return startTime;
    }

    public void setStartTime(long startTime) {
        this.startTime = startTime;
    }

    public void setEndTime(long endTime) {
        this.endTime = endTime;
    }

    public long getEndTime() {
        return endTime;
    }

    public void setStartEvent(int start) {
        startTime = start;
    }

    public void setEndEvent(int end) {
        endTime = end;
    }

    public void setTimeInterval(long startTime, long endTime) {
        this.startTime = startTime;
        this.endTime = endTime;
        findEventInterval();
    }

    public Selection specialClone() {
        long ids[] = new long[selected.length];

        for (int i=0; i<ids.length; i++) {
            ids[i] = selected[i].getId();
        }

        //Selection returnVal = new Selection(entityType,ids,0,Integer.MAX_VALUE,-1,-1,null);
        Selection returnVal = new Selection(name, sourceType,entityType,entityCategory,ids,0,Long.MAX_VALUE,null);
        returnVal.setColor(color);
        returnVal.setName(name);
        returnVal.setLink(link);

        return returnVal;
    }

    public ArrayList getTimeMap() {
        return timeMap;
    }

    public Object clone() {
        Selection o = null;

        try {
            o = (Selection)super.clone();
        } catch (CloneNotSupportedException e) {
            System.out.println("Clone not supported in selection");
            return o;
        }

        o.name = name;
        o.selected = new Entity[selected.length];
        for (int i=0; i<selected.length; i++) {
            o.selected[i] = selected[i];
        }
        o.link = new ReferenceLink[link.length];
        for (int i=0; i<link.length; i++) {
            o.link[i] = (ReferenceLink)link[i].clone();
        }
        o.color = color == null ? null : new Color(color.getRGB());
        return o;
    }

    private void findTimeInterval(long start, long end) {
        int found = 0;
        if (timeMap != null) {
            for (int i = 0; i< timeMap.size(); i++) {
                long[] map = (long[])timeMap.get(i);
                if ((startTime == -1) && (map[1] == start)) {
                    startTime = map[0];
                    found++;
                }
                if (map[1] == end) {
                    endTime = map[0];
                    found++;
                }
                if (found == 2) break;
            }
            if (endTime == -1) endTime = Long.MAX_VALUE;
        }
    }

    private void findEventInterval() {
        long time[] = new long[2], event[] = new long[2];
        time[0] = startTime;
        time[1] = endTime;
        event[0] = 0;
        event[1] = Long.MAX_VALUE;

        if (timeMap != null) {
            for (int i=0; i<time.length; i++) {
                long lastTarget = -1;
                for (int j = 0; j< timeMap.size(); j++) {
                    long[] map = (long[])timeMap.get(j);
                    if ((lastTarget<= time[i])&&(map[0]>=time[i])) {
                        event[i] = map[1];
                        break;
                    }
                }
            }
            start = event[0];
            end = event[1];
        }
    }

    public void changeTimeFrame(long startTime, long endTime) {
        setTimeInterval(startTime, endTime);
        findEventInterval();
    }

    public int getSourceType() {
        return sourceType;
    }

    public String getEntityCategory() {
        return entityCategory;
    }

    public void filterEntities(String regExps, boolean keepMatched) {
        HashSet result = new HashSet();

        for (int i=0; i<selected.length; i++) {
            String entityName = selected[i].getName().replace('.','/');
            StringTokenizer token = new StringTokenizer(regExps,";");
            boolean found = false;
            while (token.hasMoreTokens()) {
                String regExp = token.nextToken();
                Pattern pattern = Pattern.compile(regExp);
                Matcher matcher = pattern.matcher("");
                matcher.reset(entityName);

                found = matcher.lookingAt();
                if (found) break;

            }

            if ((found==keepMatched)&&(!result.contains(new Integer(i))))
                result.add(new Integer(i));
        }

        Entity[] newSelected = new Entity[result.size()];
        int i = 0;
        for (int j=0; j<selected.length; j++) {
            if (result.contains(new Integer(j)))
                newSelected[i++] = selected[j];
        }
        selected = null;
        selected = newSelected;
    }
}