/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: 2002-9-12
 * Time: 13:58:40
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package EVolve.util;

import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import EVolve.Scene;
import EVolve.visualization.AxesPanel;


public class Magnifier implements Cloneable{
    private JInternalFrame window;
    private JDesktopPane desktop;
    private AxesPanel magnifiedPanel;
    private Point2D.Double mousePosition;
    //private Rectangle imageBounds;
    private boolean flipImage;

    public Magnifier(boolean flipImage) {
        window = null;
        desktop = Scene.getUIManager().getDesktop();
        mousePosition = new Point2D.Double(-1,-1);
        this.flipImage = flipImage;
    }

    private void createWindow() {
        if (window == null) {
            window = new JInternalFrame("Zoomed View");
            window.addInternalFrameListener(new InternalFrameAdapter() {
                public void internalFrameClosing(InternalFrameEvent e) {
                    window = null;
                }
            });
            /*window.addInternalFrameListener(new InternalFrameAdapter() {
                public void internalFrameDeactivated(InternalFrameEvent e) {
                    cleanup();
                }
            });*/
            window.setClosable(true);
            int flag = AxesPanel.Pointer_Enabled | AxesPanel.Scale_Image;
            if (flipImage)
                flag = flag | AxesPanel.Flip_Image;
            magnifiedPanel = new AxesPanel(flag);
            window.setBounds(desktop.getSize().width - 400, 0, 300, 300);
            window.setResizable(true);
            window.getContentPane().add(magnifiedPanel);
            desktop.add(window);
            mousePosition.setLocation(magnifiedPanel.getWidth()/2,magnifiedPanel.getHeight()/2);
        }
        window.moveToFront();
        window.setVisible(true);
    }

    public void showWindow(BufferedImage subImage) {
        createWindow();

        int w = magnifiedPanel.getWidth();
        int h = magnifiedPanel.getHeight();
        if (w*h != 0) {
            //imageBounds = new Rectangle(0,0,subImage.getWidth(),subImage.getHeight());

            magnifiedPanel.setImage(subImage);
            magnifiedPanel.setPointerPosition((int)Math.round(mousePosition.x), (int)Math.round(mousePosition.y));
            magnifiedPanel.repaint();
            if (!window.isVisible()) {
                try {
                    window.show();
                    window.setVisible(true);
                } catch (Exception e) {}
            }
        }
    }

    public void convertMousePosition(AxesPanel zoomed) {
        if ((magnifiedPanel == null) || (magnifiedPanel.getWidth() == 0)) {
            return;
        }
        int w = magnifiedPanel.getWidth(), h = magnifiedPanel.getHeight();


        Point2D.Double mousePos = zoomed.getMouseMovement();
        //double pixel_x_size = (double)zoomed.getWidth()/imageBounds.getWidth();
        //double pixel_y_size = (double)zoomed.getHeight()/imageBounds.getHeight();

        int guessX = 0, guessY = 0;
        while (guessX<w) {
            if (magnifiedPanel.getImageX(guessX) == mousePos.x) break;
            guessX++;
        }
        while (guessY<h){
            if (magnifiedPanel.getImageY(guessY) == mousePos.y) break;
            guessY++;
        }

        mousePosition.setLocation(guessX+3,guessY+3);
    }

    public void cleanup() {
        try {
            window.dispose();
            window.setVisible(false);
            window = null;
        } catch (Exception e) {}
    }

    public Object clone() {
        Magnifier o = null;
        try {
            o = (Magnifier)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.magnifiedPanel = (AxesPanel)magnifiedPanel.clone();
        return o;
    }
}
