/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 16, 2003
 * Time: 11:31:25 PM
 */

package EVolve.util;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import javax.swing.*;
import EVolve.ProcessingThread;
import EVolve.Scene;
import EVolve.data.Worker;

public class ProgressIndicator extends JDialog implements Runnable{
    private JProgressBar progressBar;
    private JButton buttonPause, buttonResume, buttonStop;
    private Thread self = null;
    private Worker worker;
    private boolean mustStop;
    private ProcessingThread workerTd;

    public ProgressIndicator(JFrame owner, Worker worker, ProcessingThread workerTd) {
        super(owner,"Process progress");

        mustStop = false;
        this.worker = worker;
        this.workerTd = workerTd;
        createUI();
        repaint();
        self = new Thread(this);
        self.start();
    }

    private void createUI() {
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        progressBar = new JProgressBar();
        JPanel panelButton = new JPanel(new GridLayout(1,3));

        buttonResume = new JButton(new ImageIcon(Scene.getGifURL("play.gif")));
        buttonResume.setToolTipText("Resume Processing");
        buttonResume.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                resume();
            }
        });
        buttonResume.setPreferredSize(new java.awt.Dimension(30,25));
        buttonResume.setEnabled(false);

        buttonPause = new JButton(new ImageIcon(Scene.getGifURL("pause.gif")));
        buttonPause.setToolTipText("Pause Processing");
        buttonPause.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pause();
            }
        });
        buttonPause.setPreferredSize(new java.awt.Dimension(30,28));

        buttonStop = new JButton(new ImageIcon(Scene.getGifURL("stop.gif")));
        buttonStop.setToolTipText("Stop Processing");
        buttonStop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stop();
            }
        });
        buttonStop.setPreferredSize(new java.awt.Dimension(30,28));

        panelButton.add(buttonResume);
        panelButton.add(buttonPause);
        panelButton.add(buttonStop);

        Box box = new Box(BoxLayout.Y_AXIS);
        progressBar.setPreferredSize(new Dimension(290,25));

        box.add(Box.createVerticalStrut(8));
        box.add(progressBar);
        box.add(Box.createVerticalStrut(5));
        box.add(panelButton);
        getContentPane().add(box);
        progressBar.setStringPainted(true);
        progressBar.setValue(0);
        progressBar.setBorderPainted(true);
        pack();
        Scene.getUIManager().showDialog(this,300,100);
        //Scene.getUIManager().showDialog(this,300,60);
    }

    public void run() {
        Scene.getUIManager().updateFunctionControl(false);

        long totalEvents = worker.getMaxProgress();
        while (worker.getCurrentState() == Worker.STATE_NOT_STARTED) {
            self.yield();
        }

        while (worker.getCurrentState() != Worker.STATE_FINISHED) {
            try {
                long current = worker.getCurrentProgress();
                double percent = (double)current/(double)totalEvents;
                if (totalEvents > 0L) {
                    progressBar.setValue((int)(percent*100));
                    progressBar.setString("Data processed: " + NumberFormat.getPercentInstance().format(percent));
                }
                self.sleep(100);
            } catch (InterruptedException e) {
                if (mustStop || worker.getCurrentState() >= Worker.STATE_STOPPING) {
                    break;
                }
            }
        }
        progressBar.setValue(100);
        progressBar.setString("Data processed: " + NumberFormat.getPercentInstance().format(1));
        try {
            workerTd.join();
        } catch (InterruptedException e1) {
        }
        setVisible(false);
        dispose();
        worker.resetState();
        Scene.getUIManager().updateFunctionControl(true);
    }

    public void stop(){
        worker.stop();
        if (self != null) {
            mustStop = true;
            self.interrupt();
        }
        Scene.getUIManager().updateFunctionControl(true);
        self = null;
        setVisible(false);
        dispose();
    }

    public void resume() {
        buttonResume.setEnabled(false);
        buttonPause.setEnabled(true);
        worker.resume();
    }

    public void pause() {
        buttonResume.setEnabled(true);
        buttonPause.setEnabled(false);
        worker.pause();
    }
}
