/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.util;

import EVolve.data.Entity;
import EVolve.data.EntityComparator;


public class Sorter implements Cloneable{
    private int[] source, target;
    private EntityComparator comparator;

    public Sorter(Entity[] array, EntityComparator comparator) {
        this.comparator = comparator;

        int size = array.length;
        Entity[] arr = new Entity[size];

        source = new int[size];
        target = new int[size];

        for (int i = 0; i < size; i++) {
            arr[i] = array[i];
            source[i] = i;
        }

        mergesort(arr);

        for (int i = 0; i < size; i++) {
            target[source[i]] = i;
        }
    }

    public int getTarget(int source) {
        if ((source >= target.length) || (source < 0))
            return -1;
        else
            return target[source];
    }

    public int getSource(int target) {
        if ((target >= source.length) || (target < 0))
            return -1;
        else
            return source[target];
    }

    private void mergesort(Entity[] array) {
        Entity[] temp = new Entity[array.length];
        int[] tempSource = new int[array.length];

        int step = 1;
        while (step<array.length) {
            int L1=0, L2, H1, H2, k=0;
            while (L1+step<array.length) {
                L2 = L1 + step;
                H1 = L2 - 1;
                if (L2+step-1>=array.length)
                    H2 = array.length-1;
                else
                    H2 = L2+step-1;
                k = mergepass(temp,array,tempSource,L1,L2,H1,H2,k);
                L1 = H2 + 1;
            }
            int i = L1;
            while ((k < array.length)&&(i<array.length)) {
                temp[k] = array[i];
                tempSource[k] = source[i];
                i++; k++;
            }
            for (i=0; i<array.length; i++) {
                array[i] = temp[i];
                source[i] = tempSource[i];
            }
            step = step * 2;
        }
    }

    private int mergepass(Entity[] temp, Entity[] array, int[] tempSource, int L1, int L2, int H1, int H2, int k) {
        int i = L1, j = L2;
        while ((i<=H1)&&(j<=H2)) {
            if (comparator.compare(array[i], array[j]) < 0) {
                tempSource[k] = source[i];
                temp[k] = array[i];
                i++;
            } else {
                tempSource[k] = source[j];
                temp[k] = array[j];
                j++;
            }
            k++;
        }
        while (i<=H1) {
            temp[k] = array[i];
            tempSource[k] = source[i];
            i++; k++;
        }
        while (j<=H2) {
            temp[k] = array[j];
            tempSource[k] = source[j];
            j++; k++;
        }
        return k;
    }

    public Object clone() {
        Sorter o = null;
        try {
            o = (Sorter)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }

        o.source = new int[source.length];
        for (int i=0; i<source.length; i++)
            o.source[i] = source[i];

        o.target = new int[target.length];
        for (int i=0; i<target.length; i++)
            o.target[i] = target[i];

        o.comparator = (EntityComparator)comparator.clone();
        return o;
    }
}