/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: 2002-8-22
 * Time: 20:04:38
 */
package EVolve.util.batchutils;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.FileSystemView;
import EVolve.Scene;
import EVolve.exceptions.EVolveException;
import EVolve.util.predefinedutils.VisualizationSerializer;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.xmlutils.datastructures.SerializedData;

public class BatchRunner {
    private JDialog dialog;
    private java.awt.List fileList;
    private java.awt.List procList;
    private Frame owner;
    private HashSet setFile;
    private HashSet setProcessFiles;
    private HashMap batches;
    private JTextField txtConfName;
    private SerializedVisualization vizInfo = null;

    public BatchRunner() {
        owner = Scene.getFrame();
        setFile = new HashSet();
        setProcessFiles = new HashSet();
        batches = new HashMap();
    }

    public void showWindow() {
        if (dialog == null) {
            createDialog();
            Scene.getUIManager().showDialog(dialog, dialog.getWidth(), dialog.getHeight());
        }
        else dialog.setVisible(true);
    }

    public void runBatch() {
        int iBatchNum = batches.keySet().size();
        Iterator it = batches.keySet().iterator();
        BatchInfo batchInfo;

        if (iBatchNum <= 0) {
            Scene.showErrorMessage("No batch available!");
            return;
        }

        while (it.hasNext()) {
            String outPath,dataFileName,batchName = (String) it.next();
            Iterator it2;
            batchInfo = (BatchInfo) batches.get(batchName);

            // read in configure file
            try {
                it2 = batchInfo.getTraceFileList().iterator();
                dataFileName = (String) it2.next();
                Scene.setDataFilename(dataFileName);
                Scene.autoLoadDataSource();

                SerializedData data = VisualizationSerializer.v().getVizInfoFromDisk(batchInfo.getConfigureFile());
                vizInfo = (SerializedVisualization)data.SerializedVisualizations.get(0);
                outPath = data.PathForResult;

                execCmd(outPath,dataFileName);
            } catch (EVolveException e) {
                Scene.showErrorMessage(e.getMessage());
                continue;
            }

            while (it2.hasNext()) {
                // load data file first
                dataFileName = (String)it2.next();
                Scene.setDataFilename(dataFileName);
                Scene.autoLoadDataSource();
                execCmd(outPath,dataFileName);
            }
        }  // end of while(it)
        batches.clear();
        Scene.setDataFilename(null);
    }

    private void execCmd(String path,String dataFn) {
        Scene.getVisualizationManager().newVisualization(vizInfo.FactoryName);

        Scene.getVisualizationManager().autoPreVisualize(vizInfo);
        Scene.autoVisualize();
        Scene.getVisualizationManager().autoSaveResult(path,dataFn);

    }

    private void createDialog(){
        dialog = new JDialog(owner,"Create Batch",true);
        dialog.setBounds(new Rectangle(500,400));

        JPanel batchName = new JPanel(new FlowLayout());
        dialog.getContentPane().add(batchName,BorderLayout.NORTH);

        Box boxMain = new Box(BoxLayout.Y_AXIS);
        boxMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                "Choose file(s) to be processed & Processing configuration"));
        dialog.getContentPane().add(boxMain,BorderLayout.CENTER);


        Box boxLabels = new Box(BoxLayout.X_AXIS);
        JButton buttonGetDir = new JButton("Select Directory ..");
        buttonGetDir.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                    fillFileList();
            }
        });
        boxLabels.add(buttonGetDir);
        boxLabels.add(Box.createHorizontalGlue());
        boxLabels.add(new JLabel("Processing List                          "));

        Box boxLists = new Box(BoxLayout.X_AXIS);
        fileList = new java.awt.List(8,true);

        Box boxAddRemove = new Box(BoxLayout.Y_AXIS);
        JButton buttonAdd = new JButton("  >  ");
        buttonAdd.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                addFiles();
            }
        });
        JButton buttonRemove = new JButton("  <  ");
        buttonRemove.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                removeFiles();
            }
        });
        boxAddRemove.add(Box.createVerticalStrut(40));
        boxAddRemove.add(buttonAdd);
        boxAddRemove.add(Box.createVerticalStrut(20));
        boxAddRemove.add(buttonRemove);

        procList = new java.awt.List(8,true);
        boxLists.add(fileList);
        boxLists.add(Box.createHorizontalStrut(40));
        boxLists.add(boxAddRemove);
        boxLists.add(procList);

        boxMain.add(boxLabels);
        boxMain.add(Box.createVerticalStrut(5));
        boxMain.add(boxLists);

        Box boxConfig = new Box(BoxLayout.X_AXIS);
        boxConfig.add(new JLabel("Choose configure file:"));
        txtConfName = new JTextField(12);
        boxConfig.add(Box.createHorizontalStrut(20));
        boxConfig.add(txtConfName,BorderLayout.CENTER);
        JButton buttonConfig = new JButton("...");
        buttonConfig.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                txtConfName.setText(chooseConfig());
            }
        });
        boxConfig.add(buttonConfig,BorderLayout.EAST);

        Box boxOkCancel = new Box(BoxLayout.X_AXIS);
        JButton buttonOK = new JButton("OK");
        JButton buttonCancel = new JButton("Cancel");

        buttonOK.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                onOK();
            }
        });
        buttonCancel.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                onCancel();
            }
        });
        boxOkCancel.add(Box.createHorizontalStrut(30));
        boxOkCancel.add(buttonOK);
        boxOkCancel.add(Box.createHorizontalStrut(20));
        boxOkCancel.add(buttonCancel);

        Box boxBottom = Box.createVerticalBox();
        boxBottom.add(Box.createVerticalStrut(12));
        boxBottom.add(boxConfig);
        boxBottom.add(Box.createVerticalStrut(40));
        boxBottom.add(boxOkCancel);


        dialog.getContentPane().add(boxBottom,BorderLayout.SOUTH);
        //dialog.setResizable(false);
    }

    private void clearList() {
        fileList.removeAll();
        procList.removeAll();
        setFile.clear();
        setProcessFiles.clear();
    }

    private void onOK() {
        BatchInfo batchInfo = new BatchInfo();

        if (txtConfName.getText().trim().length() == 0) {
            Scene.showErrorMessage("No configuration file selected!");
            return;
        }

        if (setProcessFiles.size() == 0) {
            Scene.showErrorMessage("Select file(s) to be processed please!");
            return;
        }

        batchInfo.setConfigureFile(txtConfName.getText().trim());
        batchInfo.setTraceFileList(setProcessFiles);
        batches.put("Batch "+batches.entrySet().size(),batchInfo);
        dialog.setVisible(false);
        clearList();
    }

    private void onCancel() {
        dialog.setVisible(false);
        clearList();
    }

    private String chooseConfig() {
        JFileChooser fc = new JFileChooser(Scene.getUIManager().getLastConfigDir());

        if(fc.showOpenDialog(owner) == JFileChooser.APPROVE_OPTION) {
            File f = fc.getSelectedFile();
            Scene.getUIManager().setLastConfigDir(f.getPath());
            return (f.getPath());
        }
        else return "";
    }

    private void addFiles() {
        String[] selItems = fileList.getSelectedItems();

        for (int i=0;i<selItems.length;i++) {
            if (setProcessFiles.contains(selItems[i])) continue;

            setProcessFiles.add(selItems[i]);
            procList.add(selItems[i]);
        }
    }

    private void removeFiles() {
        String[] selItems = procList.getSelectedItems();

        for (int i=0;i<selItems.length;i++) {
            setProcessFiles.remove(selItems[i]);
            procList.remove(selItems[i]);
        }
    }

    private void fillFileList() {
        JFileChooser fc = new JFileChooser();
        String path;

        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        if(fc.showOpenDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
            path = fc.getSelectedFile().getAbsolutePath();
            File dir = new File(path);
            FileSystemView fv = fc.getFileSystemView();
            File[] fl = fv.getFiles(dir,false);
            for (int i=0;i<fl.length;i++) {
                String fn = fl[i].getName();
                if (!fl[i].isFile() || !fn.endsWith(".dat") || setFile.contains(path+File.separator+fn))
                    continue;
                setFile.add(path+File.separator+fn);
                fileList.add(path+File.separator+fn);
            }
        }

    }

}