/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: 2002-9-6
 * Time: 15:00:07
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package EVolve.util.overlappers;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;

import EVolve.Scene;
import EVolve.data.Entity;
import EVolve.data.EntityComparator;
import EVolve.exceptions.NoDataPlotException;
import EVolve.util.Magnifier;
import EVolve.visualization.AutoImage;
import EVolve.visualization.AxesPanel;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.Visualization;

public abstract class OverlapVisualization {
    protected JDialog dialog;
    protected JScrollPane panel;
    protected AxesPanel canvas;
    protected ArrayList visualizationList;
    private String xAxisName,yAxisName;
    protected EVolve.Window window;
    protected long xMax;
    protected ArrayList colorList;
    protected HashSet[] fullEntitySet;
    protected boolean noEntityAvailable;

    private JPopupMenu popup;
    private JMenu menuSort;
    private JMenuItem[][] itemSort;
    private JMenu[] menuDimension;
    private JMenuItem itemSave, itemScale;
    private ArrayList refLocation;
    private Magnifier magnifier;
    private AutoImage overlappedImage;
    private boolean shift_pressed;


    public abstract void createDialog();

    public abstract boolean isOverlapable(Visualization visualToBeOverlapped);

    public abstract void overlappedVisualize();

    public OverlapVisualization() {
        visualizationList = new ArrayList();

        fullEntitySet = new HashSet[2];
        fullEntitySet[0] = null;
        fullEntitySet[1] = null;
        noEntityAvailable = false;
        shift_pressed = false;

        panel = createPanel();
        addPopupTrigger(canvas);
    }

    public void showWindow() {
        createDialog();
        Scene.getUIManager().showDialog(dialog, dialog.getWidth(), dialog.getHeight());
    }

    public void newOverlappedVisualization(OverlapVisualization oVisual) {
        Visualization visual = (Visualization)visualizationList.get(0);
        xAxisName = visual.getDimension()[0].getName();
        yAxisName = visual.getDimension()[1].getName();
        window = Scene.getUIManager().addOverlappedVisualization(oVisual);
        createMenu();
    }

    public JPanel getPanel() {
        return canvas;
    }

    private JScrollPane createPanel() {
        canvas = new AxesPanel(AxesPanel.Axis_Enabled | AxesPanel.Flip_Image | AxesPanel.Scale_Image);
        JScrollPane returnVal = new JScrollPane(canvas);

        magnifier = new Magnifier(true);
        canvas.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent e) {
                if (e.isShiftDown()) shift_pressed = true;
                else shift_pressed = false;

                if (e.isAltDown()) {
                    int x = canvas.getImageX(e.getX());
                    int y = canvas.getImageY(e.getY());
                    magnifier.convertMousePosition(canvas);
                    magnifier.showWindow(canvas.getSubImage(x,y));
                    canvas.drawZoomingArea(e.getX(),e.getY());
                }
                mMoved(e.getX(), e.getY());
            }
        });

        canvas.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                mMoved(e.getX(), e.getY());
            }
        });

        return returnVal;
    }

    private void mMoved(int x, int y) {
        int xIndex = canvas.getImageX(x);
        int yIndex = canvas.getImageY(y);

        if (noEntityAvailable) {
            Scene.setStatus("No entity information available when temporal viz overlapped.");
        } else {
            try {
                if (shift_pressed && (getOverlappedSortedColor(xIndex,yIndex)==null)) {
                    Scene.setStatus("  ");
                    return;
                }

                if (fullEntitySet[0].size() == 0) {
                    String hint = getEntityName(1,yIndex);
                    if (hint != null) {
                        Scene.setStatus(hint);
                    }
                } else {
                    String xHint = getEntityName(0,xIndex), yHint = getEntityName(1, yIndex);

                    if ((xHint != null) && (yHint != null)) {
                        Scene.setStatus("X: " + xHint + "   Y: " + yHint);
                    }
                }
            } catch (Exception e) {
                System.out.println("Exception");
            }
        }
    }

    private void createMenu() {
        popup = new JPopupMenu();

        menuSort = new JMenu("Sort");
        menuSort.setMnemonic(KeyEvent.VK_S);
        menuSort.setEnabled(false);
        popup.add(menuSort);

        ArrayList tempList = new ArrayList();
        Visualization visual = (Visualization)visualizationList.get(visualizationList.size()-1);
        EVolve.visualization.Dimension [] dimension = visual.getDimension();
        refLocation = new ArrayList();
        for (int i = 0; i < dimension.length; i++) {
            if (dimension[i] instanceof ReferenceDimension) {
                tempList.add(new Integer(i));
                refLocation.add(new Integer(i));
            }
        }

        ReferenceDimension[] referenceDimension = new ReferenceDimension[tempList.size()];
        menuDimension = new JMenu[tempList.size()];
        itemSort = new JMenuItem[tempList.size()][];
        for (int i = 0; i < tempList.size(); i++) {
            int j = ((Integer)(tempList.get(i))).intValue();
            referenceDimension[i] = (ReferenceDimension)(dimension[j]);
            menuDimension[i] = new JMenu(visual.getDefinition().getDimensionDefinition()[j].getName());
            menuSort.add(menuDimension[i]);
        }

        for (int i = 0; i < referenceDimension.length; i++) {
            ArrayList comparatorList = referenceDimension[i].getComparator();
            itemSort[i] = new JMenuItem[comparatorList.size()];
            menuDimension[i].removeAll();
            for (int j = 0; j < itemSort[i].length; j++) {
                itemSort[i][j] = new JMenuItem(((EntityComparator)(comparatorList.get(j))).getName());
                itemSort[i][j].addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        selectComparator(e);
                    }
                });
                menuDimension[i].add(itemSort[i][j]);
            }
        }

        itemScale = new JMenuItem("Restore");
        itemScale.setMnemonic(KeyEvent.VK_R);
        itemScale.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().charAt(0) == 'F') {
                    canvas.scaleImage(true);
                    itemScale.setText("Restore");
                    itemScale.setActionCommand("Restore");
                    itemScale.setMnemonic(KeyEvent.VK_R);
                } else {
                    canvas.scaleImage(false);
                    itemScale.setText("Fit Window");
                    itemScale.setActionCommand("Fit Window");
                    itemScale.setMnemonic(KeyEvent.VK_F);
                }
                canvas.repaint();
            }
        });
        itemScale.setSelected(true);
        popup.add(itemScale);

        itemSave = new JMenuItem("Save...");
        itemSave.setMnemonic(KeyEvent.VK_V);
        itemSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        });
        popup.add(itemSave);
    }

    protected void showPopup(MouseEvent e) {
        popup.show(e.getComponent(), e.getX(), e.getY());
    }

    protected void addPopupTrigger(Component component) {
        component.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }
        });
    }

    protected void enableSortMenu() {
        menuSort.setEnabled(true);
        Scene.getUIManager().enableFileMenus();
    }

    protected String getColorHex(Color color) {
        String returnVal = Integer.toHexString(color.getBlue());
        if (returnVal.length() < 2) {
            returnVal = "0" + returnVal;
        }
        returnVal = Integer.toHexString(color.getGreen()) + returnVal;
        if (returnVal.length() < 4) {
            returnVal = "0" + returnVal;
        }
        returnVal = Integer.toHexString(color.getRed()) + returnVal;
        if (returnVal.length() < 6) {
            returnVal = "0" + returnVal;
        }

        return returnVal;
    }

    private void save() {
        BufferedImage image = new BufferedImage(canvas.getWidth(), canvas.getHeight(), BufferedImage.TYPE_INT_RGB);
        canvas.paint(image.getGraphics());
        ImageWriter writer = (ImageWriter)(ImageIO.getImageWritersByFormatName("png").next());

        JFileChooser fc = new JFileChooser(Scene.getUIManager().getLastResultDir());

        if (fc.showSaveDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            Scene.getUIManager().setLastResultDir(file.getPath());
            try {
                writer.setOutput(ImageIO.createImageOutputStream(file));
                writer.write(image);
            } catch (IOException e) {}
        }

    }

    public AutoImage getSortImage() {
        Visualization visual = null;
        overlappedImage = new AutoImage();
        ReferenceDimension sortedDimension[] = new ReferenceDimension[2];

        xMax = Integer.MIN_VALUE;

        for (int i=0; i<visualizationList.size(); i++) {
            AutoImage image;
            sortedDimension[0] = null;
            sortedDimension[1] = null;

            visual = (Visualization)visualizationList.get(i);

            for (int j=0; j<2; j++) {
                Dimension dim = visual.getDimension()[j];
                if (dim instanceof ReferenceDimension)
                    sortedDimension[j] = (ReferenceDimension)dim;
            }
            image = visual.getImage().getSortedImage(sortedDimension[0],sortedDimension[1]);
            int w = image.getW();
            int h = image.getH();

            for (int j=0; j<w; j++) {
                for (int k=0; k<h; k++) {
                    if (image.getColor(j,k) == null) continue;

                    if (overlappedImage.getColor(j,k) == null )
                        overlappedImage.setColor(j,k,image.getColor(j,k));
                    else
                        overlappedImage.setColor(j,k,new Color(153,0,153));
                }
            }
            visual.visualize();
            if (xMax < visual.getxMax()) xMax = visual.getxMax();
        }

        String[] comparatorName = new String[2];
        for (int i=0; i<2; i++) {
            if (visual.getDimension()[i] instanceof ReferenceDimension) {
                comparatorName[i] = ((ReferenceDimension)visual.getDimension()[i]).getSelectedComparatorName();
            } else {
                comparatorName[i] = "";
            }
        }

        visual = (Visualization)visualizationList.get(0);

        canvas.setName(timeHeader(visual.getDimension()[0]) + xAxisName + " (" + ((fullEntitySet[0].size() == 0) ? xMax : fullEntitySet[0].size())+
                                   ((comparatorName[0].length()==0) ? "" : ", "+ comparatorName[0]) + ")",
                                   timeHeader(visual.getDimension()[1]) + yAxisName + " (" + fullEntitySet[1].size() +
                                   ((comparatorName[1].length()==0) ? "" : ", "+ comparatorName[1]) + ")");
        return overlappedImage;
    }

    public void sort() {
        try {
            canvas.setImage(getSortImage().getImage());
            canvas.repaint();
        } catch (NoDataPlotException e) {
            Scene.showErrorMessage(e.getMessage());
        }
    }

    private void selectComparator(ActionEvent e) {
        for (int i = 0; i < itemSort.length; i++) {
            for (int j = 0; j < itemSort[i].length; j++) {
                if (itemSort[i][j] == e.getSource()) {
                    if (itemSort[i][j].getText().equals("Temporal")) {
                        noEntityAvailable = true;
                    } else {
                        noEntityAvailable = false;
                    }
                    for (int k=0; k<visualizationList.size(); k++) {
                        Visualization visual = (Visualization)visualizationList.get(k);
                        ReferenceDimension ref = (ReferenceDimension)visual.getDimension()[((Integer)refLocation.get(i)).intValue()];
                        ref.selectComparator(j);
                    }
                    sort();
                    return;
                }
            }
        }
    }

    public void unregisterOverlappedVisualization(Visualization visual) {
        for (int i=0;i<visualizationList.size();i++) {
            if (((Visualization)visualizationList.get(i)).getVisualizationID() == visual.getVisualizationID()) {
                visualizationList.remove(i);
            }
        }
        if (visualizationList.size() ==1) visualizationList.clear();
    }

    private String getEntityName(int dim,int index) {
        Entity entity = null;
        for (int i=0; i<visualizationList.size(); i++) {
            Visualization visual = (Visualization)visualizationList.get(i);
            if (!(visual.getDimension()[dim] instanceof ReferenceDimension)) return null;
            ReferenceDimension ref = (ReferenceDimension)visual.getDimension()[dim];
            entity = ref.getEntity(index);
            if (entity != null) {
                break;
            }
        }

        return (entity == null) ? null : entity.getName();
    }

    private Color getOverlappedSortedColor(int x, int y) {
        Color returnVal = null;
        ReferenceDimension sortedDimension[] = new ReferenceDimension[2];
        for (int i=0; i<visualizationList.size(); i++) {
            Visualization visual = (Visualization)visualizationList.get(i);
            for (int j=0; j<2; j++) {
                Dimension dim = visual.getDimension()[j];
                if (dim instanceof ReferenceDimension)
                    sortedDimension[j] = (ReferenceDimension)dim;
            }
            returnVal = (Color)visual.getImage().getSortedColor(sortedDimension[0],sortedDimension[1],x,y);
            if (returnVal != null) break;
        }

        return returnVal;
    }

    private String timeHeader(Dimension dim) {
        String property[] = dim.getDataFilter().getProperty();

        if (property == null) return "";

        for (int i=0; i<property.length; i++) {
            if (property[i].equals("time"))
                return "Time - ";
        }
        return "";
    }

    public void cleanup() {
        fullEntitySet = null;
        itemSort = null;
        menuDimension = null;
        magnifier.cleanup();
    }
}
