/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: 2002-9-6
 * Time: 15:53:23
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package EVolve.util.overlappers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import EVolve.Scene;
import EVolve.exceptions.NoDataPlotException;
import EVolve.util.unifyutils.Unification;
import EVolve.visualization.AutoImage;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.Visualization;

public class SynchronousOverlapper extends OverlapVisualization{
    private final String name = "SynchronousOverlapper";
    private Visualization overlappedVisualization;
    private JList availableList,overlappableList;
    private DefaultListModel modelList;
    private ArrayList candidates;
    private boolean shift_ctrl_pressed;

    public SynchronousOverlapper() {
        super();
        colorList = new ArrayList();
        dialog = null;
        overlappedVisualization = null;
        candidates = new ArrayList();
        modelList = new DefaultListModel();
        xMax = 0;
    }

    public String getName() {
        return name;
    }

    public void createDialog() {
        shift_ctrl_pressed = false;
        colorList.clear();
        visualizationList.clear();
        modelList.removeAllElements();
        dialog = new JDialog(Scene.getFrame(), "Overlap Visualizations...", true);
        dialog.setBounds(new Rectangle(500,400));

        Box boxMain = new Box(BoxLayout.Y_AXIS);
        boxMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                "Choose Visualizations to be overlapped"));
        dialog.getContentPane().add(boxMain,BorderLayout.CENTER);

        Box boxLabels = new Box(BoxLayout.X_AXIS);
        boxLabels.add(new JLabel("Available visualizations:"));
        boxLabels.add(Box.createHorizontalStrut(130));
        boxLabels.add(new JLabel("Overlappable visualizations:"));

        JPanel boxLists = new JPanel(new GridLayout(1,2,25,2));
        DefaultListModel modelAvail = new DefaultListModel();
        availableList = new JList(modelAvail);
        ArrayList arr = Scene.getVisualizationManager().getVisualizationList();
        for (int i=0; i<arr.size(); i++) {
            modelAvail.addElement((((Visualization)arr.get(i)).getWindow().getTitle()));
        }
        availableList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e){
                updateOverlappableList();
            }
        });

        overlappableList = new JList(modelList);
        overlappableList.addKeyListener(new KeyListener(){
            public void keyPressed(KeyEvent e) {
                if ((e.getKeyCode() == KeyEvent.VK_SHIFT) || (e.getKeyCode()==KeyEvent.VK_CONTROL)) {
                    shift_ctrl_pressed = true;
                }
            }
            public void keyReleased(KeyEvent e) {
                if ((e.getKeyCode() == KeyEvent.VK_SHIFT) || (e.getKeyCode()==KeyEvent.VK_CONTROL)) {
                    shift_ctrl_pressed = false;
                }
            }
            public void keyTyped(KeyEvent e) { }
        });
        overlappableList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e){
                if ((!shift_ctrl_pressed)&&(!e.getValueIsAdjusting()) && overlappableList.isFocusOwner())
                    selectColor();
            }
        });
        JScrollPane scrollPane1 = new JScrollPane(availableList);
        JScrollPane scrollPane2 = new JScrollPane(overlappableList);
        boxLists.add(scrollPane1);
        boxLists.add(scrollPane2);

        boxMain.add(Box.createVerticalStrut(-65));
        boxMain.add(boxLabels);
        boxMain.add(Box.createVerticalStrut(-65));
        boxMain.add(boxLists);
        boxMain.add(Box.createVerticalStrut(10));

        Box boxOkCancel = new Box(BoxLayout.X_AXIS);
        JButton buttonOK = new JButton("OK");
        JButton buttonCancel = new JButton("Cancel");

        buttonOK.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                onOK();
            }
        });
        buttonCancel.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                onCancel();
            }
        });

        boxOkCancel.add(Box.createHorizontalStrut(180));
        boxOkCancel.add(buttonOK);
        boxOkCancel.add(Box.createHorizontalStrut(20));
        boxOkCancel.add(buttonCancel);

        dialog.getContentPane().add(boxOkCancel,BorderLayout.SOUTH);
        //dialog.setResizable(false);
    }

    private void selectColor() {
        int index = overlappableList.getSelectedIndex();

        if (index != -1) {
            Color newColor = JColorChooser.showDialog(Scene.getFrame(), "Choose a color", Color.black);
            if (newColor != null) {
                colorList.add(index+1,newColor);
                colorList.remove(index+2);
                modelList.removeAllElements();
                for (int i=0; i<candidates.size(); i++) {
                    if (colorList.get(i+1) == null)
                        modelList.addElement(((Visualization)candidates.get(i)).getWindow().getTitle());
                    else
                        modelList.addElement("<html><font color=#" + getColorHex((Color)colorList.get(i+1)) + ">"
                                             +((Visualization)candidates.get(i)).getWindow().getTitle() +" </font></html>" );
                }
                overlappableList.setSelectedIndex(index);
            } else {
                newColor = Color.black;
            }
        }
    }

    private void updateOverlappableList() {
        ArrayList arr = Scene.getVisualizationManager().getVisualizationList();
        overlappedVisualization = (Visualization)arr.get(availableList.getSelectedIndex());

        candidates.clear();
        colorList.clear();
        overlappableList.removeAll();
        modelList.removeAllElements();
        colorList.add(null);
        for (int i=0; i<arr.size(); i++) {
            Visualization visual =(Visualization)arr.get(i);
            if (visual.getVisualizationID() == overlappedVisualization.getVisualizationID()) continue;
            if (isOverlapable(visual)) {
                modelList.addElement(visual.getWindow().getTitle());
                colorList.add(null);
                candidates.add(visual);
            }
        }
    }

    private void onOK() {
        int[] indexes = overlappableList.getSelectedIndices();
        visualizationList.clear();

        if (indexes.length == 0) {
            Scene.showErrorMessage("Please select visualizations to be linked");
            return;
        }

        visualizationList.add(overlappedVisualization);
        for (int i=0; i<indexes.length; i++) {
            visualizationList.add(candidates.get(indexes[i]));
        }

        ArrayList newColorList = new ArrayList();
        newColorList.add(null);
        for (int i=0; i<modelList.size(); i++) {
            for (int j=0; j<indexes.length;j++) {
                if (i==indexes[j]) {
                    newColorList.add(colorList.get(i+1));
                    break;
                }
            }
        }
        colorList = newColorList;
        dialog.setVisible(false);
        noEntityAvailable = false;

        // visualize viz if the user has not visualize them yet
        ArrayList tobeViz = new ArrayList();

        for (int i=0; i<visualizationList.size(); i++) {
            Visualization visual = (Visualization)visualizationList.get(i);
            AutoImage image = visual.getImage();
            if (image == null) {
                Scene.showErrorMessage("Please visualize all visualizations you want to overlap first.");
                return;
                //tobeViz.add(visual);
            }
        }


        /*if (tobeViz.size() != 0) {
            int dataSourceId = Scene.getDataSourceManager().getCurrentDataSourceId();
            for (int i=0; i<tobeViz.size(); i++) {
                Visualization visual = (Visualization)visualizationList.get(i);
                if (visual.getDataSourceId() != dataSourceId) {
                    Scene.showErrorMessage("Visualizations belong to different data sources, please\n" +
                                           "try to first visualize them and then overlap them again.");
                    return;
                }
            }
            Scene.getVisualizationManager().prepareForLinkedViz(tobeViz,0);
            Scene.visualize();
        }*/

        overlappedVisualize();
    }

    private void onCancel() {
        dialog.setVisible(false);
    }

    public boolean isOverlapable(Visualization visualToBeOverlapped) {
        if (overlappedVisualization.getInterval() != visualToBeOverlapped.getInterval())
            return false;

        for (int i=0; i<2; i++) {
            if ((overlappedVisualization.getLinkableDimension(i) == null)||
               (visualToBeOverlapped.getLinkableDimension(i) == null))
            {
                if ((overlappedVisualization.getDimension()[i] instanceof ReferenceDimension) ||
                    visualToBeOverlapped.getDimension()[i] instanceof ReferenceDimension)
                    return false;

                if (!overlappedVisualization.getDimension()[i].getName().equals(visualToBeOverlapped.getDimension()[i].getName()))
                    return false;

            } else {
                if (overlappedVisualization.getDimension()[i].getDataFilter().getTargetType() !=
                    visualToBeOverlapped.getDimension()[i].getDataFilter().getTargetType())
                    return false;
            }
        }

        if ((visualToBeOverlapped.getSubjectDefinition().getName().equals(overlappedVisualization.getSubjectDefinition().getName())) &&
            (visualToBeOverlapped.getFactory().getName().equals(overlappedVisualization.getFactory().getName())) &&
            (visualToBeOverlapped.getDataSourceId() == overlappedVisualization.getDataSourceId()))
            return false;

        return true;
    }

    public void overlappedVisualize() {
        fullEntitySet = new HashSet[2];
        fullEntitySet[0] = new HashSet();
        fullEntitySet[1] = new HashSet();

        xMax = 0;

        if (((Visualization)visualizationList.get(0)).getDimension()[0] instanceof ReferenceDimension)
            Unification.getUnifiedEntity(fullEntitySet[0],visualizationList,0);
        if (((Visualization)visualizationList.get(0)).getDimension()[1] instanceof ReferenceDimension)
            Unification.getUnifiedEntity(fullEntitySet[1],visualizationList,1);


        if (window instanceof EVolve.Window) {
            Scene.getUIManager().removeWindow(window);
            window = null;
        }

        unifyVisualizations();

        newOverlappedVisualization(this);

        sort();
        enableSortMenu();
    }

    private void unifyVisualizations() {

        for (int i=0; i<visualizationList.size(); i++) {
            Visualization visual = (Visualization)visualizationList.get(i);

            Color newColor = (Color)colorList.get(i);
            Unification.changeColor(visual.getImage(),newColor);

            for (int j=0; j<fullEntitySet.length; j++) {
                Dimension dim = visual.getDimension()[j];
                if (fullEntitySet[j].size() > 0) {
                    ((ReferenceDimension)dim).linkEntities(fullEntitySet[j]);
                    try {
                        ((ReferenceDimension)dim).visualize();
                    } catch (NoDataPlotException e) {
                        Scene.showErrorMessage(e.getMessage());
                    }
                }
            }
        }
    }
}
