/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 28, 2002
 * Time: 10:30:02 AM
 */

package EVolve.util.painters;

import EVolve.Scene;
import EVolve.util.painters.shapes.Box;
import EVolve.util.ColorCache;
import EVolve.visualization.AutoImage;
import EVolve.visualization.AxesPanel;

import java.util.*;

public class BarChartPainter extends Painter{
    private HashMap bars; // value of the bars
    private long xMax;
    private int barHeight;
    private boolean useRandomColor;
    private ColorCache colorCache;
    private AxesPanel canvas;


    public BarChartPainter(AxesPanel canvas, int barHeight, boolean useRandomColor) {
        bars = new HashMap();
        xMax = 0;
        this.barHeight = barHeight;
        this.useRandomColor = useRandomColor;
        colorCache = new ColorCache();
        this.canvas = canvas;
    }

    public String getName() {
        return "Bar Chart Painter";
    }

    public void paint(AutoImage image, long x, long y, long z) {
        Box bar = null;
        if (bars.containsKey(new Long(y))) {
            bar = (Box)bars.get(new Long(y));
        } else {
            bar = new Box(0L,(int)y,0,barHeight,false,canvas);
            if (useRandomColor)
                bar.setColor(colorCache.getNextColor(y));
            else {
                bar.setColor(Scene.getColor());
            }
            bars.put(new Long(y), bar);
            image.setColor(0,(int)y,bar);
        }

        bar.setSize(bar.getRealWidth()+(int)x,barHeight);
        bar.move(0,(int)y*barHeight);

        if (xMax < bar.getRealWidth())
            xMax = bar.getRealWidth();

        image.setImageSize((int)(xMax > Box.maxLength ? Box.maxLength : xMax),barHeight*bars.size());
    }

    public long getxMax() {
        return xMax;
    }

    public int[] getValue() {
        int[] value = new int[bars.size()];
        Iterator it = bars.keySet().iterator();
        while (it.hasNext()) {
            Long key = (Long)it.next();
            value[key.intValue()] = ((Box)bars.get(key)).getRealWidth();
        }
        return value;
    }

    public Object clone() {
        BarChartPainter o = null;
        o = (BarChartPainter)super.clone();
        return o;
    }
}
