/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 10:19:18 PM
 */

package EVolve.util.painters;

import java.awt.Color;
import java.util.HashMap;

import EVolve.Scene;
import EVolve.data.Entity;
import EVolve.visualization.AutoImage;
import EVolve.visualization.Predictor;

public class PredictionPainter extends Painter{
    protected Color colorRed = new Color(255, 0, 0);
    protected Color colorBlue = new Color(120, 160, 255);
    protected Predictor[] predictor; // predictors
    protected int[] miss;
    protected HashMap targetCheckMap;
    protected int targetType;
    private boolean furtherCheckNeeded = true;

    public PredictionPainter() {
        targetCheckMap = new HashMap();
    }

    public PredictionPainter(Predictor[] predictor, int targetType) {
        this.predictor = predictor;
        targetCheckMap = new HashMap();
        this.targetType = targetType;
        miss = new int[predictor.length];
        for (int i = 0; i < predictor.length; i++) {
            miss[i] = 0;
        }
    }

    public String getName() {
        return "Prediction Painter";
    }

    public void paint(AutoImage image, long x, long y, long z) {
        predictor[(int)y].newTarget(z);

        if (validateTarget(y,z)) {
            if (predictor[(int)y].isCorrect()) {
                if (image.getColor((int)x,(int)y) == null) {
                    image.setColor((int)x,(int)y, colorBlue);
                }
            } else {
                image.setColor((int)x, (int)y, colorRed);
                miss[(int)y]++;
            }
        } else {
            if (image.getColor((int)x, (int)y) == null) {
                image.setColor((int)x, (int)y, colorBlue);
            }
        }
    }

    public int[] getMiss() {
        return miss;
    }

    public void setPredictor(Predictor[] predictor, int targetType) {
        this.predictor = predictor;
        this.targetType = targetType;
        miss = new int[predictor.length];
        for (int i = 0; i < predictor.length; i++) {
            miss[i] = 0;
        }
    }

    protected boolean validateTarget(long y, long z) {
        String newTarget = null;
        if (furtherCheckNeeded) {
            newTarget = ((Entity)Scene.getDataManager().getEntity()[targetType].get(new Long(z))).getName();
            if ((newTarget.indexOf('(') == -1) || // not a method
                (newTarget.indexOf('#') != -1) /*a location*/) {
                furtherCheckNeeded = false;
                return true;
            }
            String oldTarget = (String)targetCheckMap.get(new Long(y));
            int index1 = newTarget.indexOf('(');
            index1 = newTarget.substring(0,index1).lastIndexOf('.');
            newTarget = newTarget.substring(index1+1);
            targetCheckMap.put(new Long(y),newTarget);
            if ((oldTarget!=null)&&(!oldTarget.equals(newTarget)))
                return false;
        }
        return true;
    }

    public Object clone() {
        PredictionPainter o = (PredictionPainter)super.clone();
        o.colorRed = new Color(255,0,0);
        o.colorBlue = new Color(120,160,255);
        o.predictor = (predictor == null) ?  null : new Predictor[predictor.length];
        o.miss = null;
        if (miss != null) {
            o.miss = new int[miss.length];
            for (int i=0; i<miss.length; i++)
                o.miss[i] = miss[i];
        }
        return o;
    }
}
