/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 14, 2003
 * Time: 8:45:05 PM
 */

package EVolve.util.painters;

import java.util.*;
import EVolve.util.painters.shapes.*;
import EVolve.visualization.AutoImage;

public class RelationshipPainter extends Painter{
    private int[][] value;
    private HashMap[] axis;
    private int shapeSize;
    private int threshold;
    private long entity_type_1, entity_type_2;

    public RelationshipPainter() {
        axis = new HashMap[2];
    }

    public String getName() {
        return "Relationship Painter";
    }

    public void updatePainterParameters(int[][] value, int threshold) {
        this.value = value;
        this.threshold = threshold;
    }

    public void paint(AutoImage image, long x, long y, long z) {
        /*
            here x and y stands for the entity type of
            two axis, z stands for the edge length of the box
            or diameter of the ball
        */
        clearImage(image);
        entity_type_1 = x;
        entity_type_2 = y;
        axis[0] = new HashMap();
        axis[1] = new HashMap();
        shapeSize = (int)z;

        int w = value.length, h = value[0].length;

        for (int j=0; j<h; j++) {
            Shape consumer = null;
            for (int i=0; i<w; i++) {
                if (value[i][j]>threshold) {
                    consumer = new Ball(y,j,shapeSize);
                    image.setColor(i,j,consumer);
                    axis[1].put(new Integer(axis[1].size()),consumer);
                    break;
                }
            }
        }


        for (int i=0; i<w; i++) {
            Shape producer = null;
            for (int j=0; j<h; j++) {
                if (value[i][j] >= threshold) {
                    Shape consumer = findConsumer(j);
                    producer = new Box(x,i,shapeSize,shapeSize);
                    Iterator it = axis[0].keySet().iterator();
                    Shape found = null;
                    while (it.hasNext()) {
                        Object key = it.next();
                        Shape temp = (Shape)axis[0].get(key);
                        if (temp.getEntityID() == i) {
                            found = temp;
                            break;
                        }
                    }
                    if (found == null) {
                        found = producer;
                        image.setColor(i,j,found);
                        axis[0].put(new Integer(axis[0].size()),found);
                    }
                    if (consumer != null)
                        found.addConsumer(consumer,value[i][j]);
                }
            }
        }
    }

    public Shape getEntityShape(int x, int y) {
        int totalShapes = axis[0].size() + axis[1].size();

        int i = 0,producerSize = axis[0].size();
        while (i<totalShapes) {
            Iterator it = axis[i/producerSize].keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Shape object = (Shape)axis[i/producerSize].get(key);
                if (object.insideShape(x,y)) {
                    return object;
                }
            }
            i += axis[i/producerSize].size();
        }
        return null;
    }

    public HashMap getEntitiesInBox(int startX, int startY, int endX, int endY) {
        HashMap result = new HashMap();
        for (int i=0; i<axis.length; i++) {
            ArrayList ids = new ArrayList();
            int targetType = -1;
            for (int j=0; j<axis[i].size(); j++) {
                Shape object = (Shape)axis[i].get(new Integer(j));
                targetType = (int)object.getEntityType();
                if (inside(object,startX,startY,endX,endY))
                    ids.add(new Long(object.getEntityID()));
            }
            if (targetType!= -1) result.put(new Integer(targetType),ids);
        }
        return result;
    }

    private Shape findConsumer(int entityId) {
        Iterator it = axis[1].keySet().iterator();
        while (it.hasNext()) {
            Shape object = (Shape)axis[1].get(it.next());
            if (object.getEntityID() == entityId)
                return object;
        }
        return null;
    }

    private boolean inside(Shape object, int startX, int startY, int endX, int endY) {
        if ((startX<=object.x)&&(object.x<=endX)&&(startY<=object.y)&&(object.y<=endY))
            return true;
        return false;
    }

    private void clearImage(AutoImage image) {
        short[][] data = image.getImageDataArray();
        for (int i=0; i<data.length; i++)
            for (int j=0; j<data[i].length; j++)
                data[i][j] = 0;
    }

    public Object clone() {
        RelationshipPainter o = (RelationshipPainter)super.clone();
        o.axis = new HashMap[2];
        o.axis[0] = new HashMap();
        o.axis[1] = new HashMap();

        int w = value.length, h = value[0].length;
        for (int i=0; i<w; i++) {
            Shape producer = null;
            for (int j=0; j<h; j++) {
                if (value[i][j] >= threshold) {
                    Shape consumer = o.findConsumer(j);
                    if (producer == null) {
                        producer = new Box(entity_type_1,i,shapeSize,shapeSize);
                        o.axis[0].put(new Integer(o.axis[0].size()),producer);
                    }
                    if (consumer == null) {
                        consumer = new Ball(entity_type_2,j,shapeSize);
                        o.axis[1].put(new Integer(o.axis[1].size()),consumer);
                    }
                    producer.addConsumer(consumer,value[i][j]);
                }
            }
        }

        return o;
    }
}
