package EVolve.util.painters;

import EVolve.visualization.*;
import EVolve.util.HelperFuncs;

import java.util.*;
import java.awt.*;

/**
 * User: Wei Wang
 * Date: Jun 12, 2003
 * Time: 9:41:56 AM
 */
public class StatisticPainter extends RandomPainter{
    private HashMap intervalData;
    private ArrayList intervals, totals;
    private long current_interval;
    private float total;
    private long last_y;

    public StatisticPainter() {
        current_interval = 0;
        intervalData = new HashMap();
        intervals = new ArrayList();
        intervals.add(intervalData);
        totals = new ArrayList();
        total = 0;
    }

    //x: time, y:entity id, z: amount
    public void paint(AutoImage image, long x, long y, long z) {
        if (y < 0) return;

        while (x>current_interval) {
            totals.add(new Float(total));
            processData((int)current_interval);
            intervalData = new HashMap();
            intervals.add(intervalData);
            current_interval++;
            total = 0;
        }

        Color key = colorCache.getNextColor(y);
        total += (float)z;
        if (!intervalData.containsKey(key)) {
            intervalData.put(key, new Float(z));
        } else {
            Float old = (Float)intervalData.get(key);
            intervalData.put(key, new Float(old.floatValue()+z));
        }
        last_y = y;
    }

    public ArrayList getStatisticData() {
        return intervals;
    }

    public void postProcess() {
        totals.add(new Float(total));
        processData(intervals.size()-1);
    }

    private void processData(int index) {
        HashMap data = (HashMap)intervals.get(index);
        float total = ((Float)totals.get(index)).floatValue();
        if (data.size() == 0) {
            total = 100;
            data.put(colorCache.getNextColor(last_y),new Float(100));
        }

        Iterator it = data.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();

            Float Value = (Float)data.get(key);
            data.put(key,new Float(100F*Value.floatValue()/total));
        }
    }

    public Object clone() {
        StatisticPainter o = (StatisticPainter)super.clone();

        o.intervals = new ArrayList();
        for (int i=0; i<intervals.size(); i++) {
            o.intervals.add(HelperFuncs.cloneHashMap((HashMap)intervals.get(i)));
        }
        o.totals = HelperFuncs.cloneArrayList(totals);

        return o;
    }
}
