/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 14, 2003
 * Time: 9:35:11 PM
 */

package EVolve.util.painters.shapes;

import EVolve.visualization.AxesPanel;
import java.awt.Graphics2D;
import java.awt.*;

public class Box extends Shape{
    private int width, height;
    private boolean wrapped;
    public static final int maxLength = 600;
    private AxesPanel panel;

    public Box(long entity_type,long entity_id, int width, int height) {
        super(entity_type,entity_id);
        this.entity_type = entity_type;
        this.width = width;
        this.height = height;
        this.wrapped = false;
    }
    
	public Box(long entity_type,long entity_id, int width, int height, boolean wrapped) {
		this(entity_type,entity_id,width,height);
		this.wrapped = wrapped;		
	}

    public Box(long entity_type,long entity_id, int width, int height, boolean wrapped, AxesPanel panel) {
        this(entity_type, entity_id, width, height, wrapped);
        this.panel = panel;
    }

    public String getName() {
        return "Box";
    }

    public void draw(Graphics2D g) {
        boolean broken = false;

        if ((panel != null)&&((width > maxLength) || (height > maxLength))) {
            broken = true;
        }

        drawBox(g,x,y,width,height,broken);

        if (consumerMap == null) return;

        drawArrows(g);
    }

    public void fill(Graphics2D g) {
        if (color != null) 
        	g.fillRect(x,y,width,height);
        wrap(x,y,width,height,g);
    }

    public boolean insideShape(int x, int y) {
        if ((x<=this.x+width) && (x>=this.x) &&
            (y<=this.y+height) && (y>=this.y))
            return true;
        return false;
    }

    public int getWidth() {
        return (panel!=null)&&(width>maxLength) ? maxLength : width;
    }

    public int getRealWidth() {
        return width;
    }

    public int getHeight() {
        return (panel!=null)&&(height>maxLength) ? maxLength : height;
    }

    public int getRealHeight() {
        return height;
    }

    public void setSize(int width, int height) {
        this.width = width <=0 ? 1 : width;
        this.height = height;
    }

    private void wrap(int x, int y, int width, int height, Graphics2D g) {
        if (wrapped) {
			g.setColor(Color.black);
			g.drawRect(x+1,y+1,width-2,height-2);
        }
    }

    private void drawBox(Graphics2D g, int x, int y, int width, int height, boolean broken) {
        int[][][] points = new int[2][2][6];

        if (color != null)
            g.setColor(color);
        else
            g.setColor(defaultColor);

        if (broken) {
            if (width > maxLength) {
                points[0][0][0] = x; points[0][0][1] = x+maxLength/2-5; points[0][0][2] = x+maxLength/2-10;
                points[0][0][3] = x+maxLength/2-5; points[0][0][4] = x; points[0][0][5] = x;

                points[0][1][0] = y; points[0][1][1] = y; points[0][1][2] = y+height/2;
                points[0][1][3] = y+height; points[0][1][4] = y+height; points[0][1][5] = y;

                points[1][0][0] = x+maxLength/2; points[1][0][1] = x+maxLength; points[1][0][2] = x+maxLength;
                points[1][0][3] = x+maxLength/2; points[1][0][4] = x+maxLength/2-5; points[1][0][5] = x+maxLength/2;

                points[1][1][0] = y; points[1][1][1] = y; points[1][1][2] = y+height;
                points[1][1][3] = y+height; points[1][1][4] = y+height/2; points[1][1][5] = y;
            } else {
                points[0][0][0] = x; points[0][0][1] = x+width; points[0][0][2] = x+width;
                points[0][0][3] = x+width/2; points[0][0][4] = x; points[0][0][5] = x;

                points[0][1][0] = y; points[0][1][1] = y; points[0][1][2] = y+maxLength/2-5;
                points[0][1][3] = y+maxLength/2-10; points[0][1][4] = y+maxLength/2-5; points[0][1][5] = y;

                points[1][0][0] = x; points[1][0][1] = x+width/2; points[1][0][2] = x+width;
                points[1][0][3] = x+width; points[1][0][4] = x; points[1][0][5] = x;

                points[1][1][0] = y+maxLength/2; points[1][1][1] = y+maxLength/2-5; points[1][1][2] = y+maxLength/2;
                points[1][1][3] = y+maxLength; points[1][1][4] = y+maxLength; points[1][1][5] = y;
            }
            g.drawPolygon(points[0][0],points[0][1],points[0][0].length);
            g.drawPolygon(points[1][0],points[1][1],points[1][0].length);
        }

        if (color != null) {
            if (!broken) {
                g.fillRect(x,y,width,height);
                wrap(x, y, width, height, g);
            } else {
                g.fillPolygon(points[0][0],points[0][1],points[0][0].length);
                g.fillPolygon(points[1][0],points[1][1],points[1][0].length);

                wrap(x, y, maxLength, height, g);
            }
        } else {
            g.drawRect(x,y,width,height);
        }

        wrap(x, y, width, height, g);
    }

    public void move(int x, int y) {
        super.move(x,y);

        if (panel == null) return;

        if (width > maxLength) {
            panel.addString(x+maxLength/2-10,y+height/2-2,String.valueOf(width));
        } else if (height > maxLength){
            panel.addString(x+maxLength/2-10,y+height/2,String.valueOf(width));
        }
    }
}
