/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 14, 2003
 * Time: 9:34:32 PM
 */

package EVolve.util.painters.shapes;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Iterator;

public abstract class Shape implements Cloneable{
    public int x, y;
    protected long entity_id;
    protected long entity_type;
    protected HashMap consumerMap;
    protected Color color;
    protected Color defaultColor = Color.black;

    public Shape(long entity_type, long entity_id) {
        this.entity_id = entity_id;
        this.entity_type = entity_type;
        consumerMap = new HashMap();
        color = null;
    }

    public long getEntityID() {
        return entity_id;
    }

    public Rectangle getPosition() {
        return new Rectangle(x,y,1,1);
    }

    public void move(int x, int y) {
        this.x = x<0 ? 0:x;
        this.y = y<0 ? 0:y;
    }

    public void addConsumer(Shape consumer, int gravity) {
        consumerMap.put(consumer,new Integer(gravity));
    }

    public HashMap getConsumers() {
        return consumerMap;
    }

    public long getEntityType() {
        return entity_type;
    }

    public void setColor(Color color) {
        this.color = color;
    }

    public Color getColor() {
        return color;
    }

    protected void drawArrows(Graphics2D g2) {
        Iterator it2 = consumerMap.keySet().iterator();
        while (it2.hasNext()) {
            Shape consumer = (Shape)it2.next();
            int gravity = ((Integer)consumerMap.get(consumer)).intValue();
            // draw arrows
            int x1 = x+getWidth()/2, y1 = y+getHeight()/2;
            int x2 = consumer.x + consumer.getWidth()/2, y2 = consumer.y + consumer.getHeight()/2;
            g2.drawLine(x1,y1,x2,y2);
            double len = java.lang.Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
            double angle = java.lang.Math.asin(java.lang.Math.abs(y1-y2)/len);
            if (x1>x2) {
                if (y1 < y2)
                    angle = 2*java.lang.Math.PI - angle;
            } else {
                if (y1 > y2)
                    angle = java.lang.Math.PI - angle;
                else
                    angle = java.lang.Math.PI + angle;
            }
            g2.drawLine(x2,y2,(int)(x2+10*java.lang.Math.cos(angle-0.5236)),(int)(y2+10*java.lang.Math.sin(angle-0.5236)));
            g2.drawLine(x2,y2,(int)(x2+10*java.lang.Math.cos(angle+0.5236)),(int)(y2+10*java.lang.Math.sin(angle+0.5236)));
            g2.drawString(""+gravity+"",(x1+(x2-x1)/5),(y1-(y1-y2)/5));
        }
    }

    public Object clone() {
        Shape o = null;
        try {
            o = (Shape)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.consumerMap = new HashMap();
        return o;
    }

    public abstract void draw(Graphics2D g);

    public abstract void fill(Graphics2D g);

    public abstract String getName();

    public abstract boolean insideShape(int x, int y);

    public abstract int getWidth();

    public abstract int getHeight();

    public abstract void setSize(int width, int height);
}
