/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 27, 2003
 * Time: 9:18:07 PM
 */

package EVolve.util.phasedetectors;

import java.util.ArrayList;
import java.util.HashSet;

import EVolve.Scene;
import EVolve.util.HelperFuncs;
import EVolve.visualization.XYViz.XYVisualization;

public class PhaseClipboard {
    private ArrayList copiedPhase;
    private XYVisualization source;
    private HashSet targetList;
    private ArrayList sourceTimeMap, targetTimeMap;
    private int sourceInterval, targetInterval;
    private boolean isEmpty;

    public PhaseClipboard() {
        isEmpty = true;
        targetList = new HashSet();
    }

    public void copyPhase() {
        ArrayList phaseList = null;
        source = HelperFuncs.getActiveXYViz();

        if ((source == null)||(source.getPhaseDetector()==null)) {
            Scene.showErrorMessage("No phase information available.");
            return;
        }

        sourceTimeMap = source.getTimeMap();
        sourceInterval = source.getInterval();
        phaseList = source.getPhaseDetector().getPhase();
        if ((phaseList == null) || (phaseList.size() == 0)) {
            Scene.showErrorMessage("No phase information available.");
            return;
        }

        copiedPhase = HelperFuncs.cloneArrayList(phaseList);
        if (copiedPhase.size() > 0) isEmpty = false;
    }

    public void pastePhase() {
        XYVisualization target = null;

        if (copiedPhase == null) {
            Scene.showErrorMessage("No phase data available, please copy phase first.");
            return;
        }

        target = HelperFuncs.getActiveXYViz();
        if (target == null) {
            Scene.showErrorMessage("Can not paste phase to non-HotSpotVisualization.");
            return ;
        }

        if (source.getDataSourceId() != target.getDataSourceId()) {
            Scene.showErrorMessage("Can not paste phases to a visualization generated from a different data source");
            return;
        }

        PhaseDetector detector = target.getPhaseDetector();
        if (detector == null) return;

        if (!targetList.contains(target))
            targetList.add(target);

        targetTimeMap = target.getTimeMap();
        targetInterval = target.getInterval();
        ArrayList phases = translatePhase();

        detector.pastePhases(phases);
    }

    public boolean isEmpty() {
        return isEmpty;
    }

    private ArrayList translatePhase() {
        ArrayList translated = new ArrayList();
        ArrayList eventNoList = new ArrayList();

        for (int i=0; i<copiedPhase.size(); i++) {
            Integer phase = (Integer)copiedPhase.get(i);
            long event = findEventNo(sourceTimeMap, phase.intValue()+1,sourceInterval);
            eventNoList.add(new Long(event));
        }

        for (int i=0; i<eventNoList.size(); i++) {
            Long event = (Long)eventNoList.get(i);
            long time = findTime(targetTimeMap, event.longValue());
            translated.add(new Integer((int)(time/targetInterval)));
        }

        return translated;
    }

    private long findTime(ArrayList timeMap, long eventNo) {
        long prevEvent = 0, curEvent = 0;
        long time = 0;

        for (int i=0; i<timeMap.size(); i++) {
            long[] time2Event = (long[])timeMap.get(i);
            curEvent = time2Event[1];
            time = time2Event[0];
            if ((prevEvent == eventNo) || (curEvent == eventNo) ||
                ((prevEvent < eventNo) && (eventNo < curEvent))) {
                break;
            }
            prevEvent = curEvent;
        }

        return time;
    }

    private long findEventNo(ArrayList timeMap, int time, int interval) {
        long prevTime = 0, curTime = 0;
        long event = 0;

        for (int i=0; i<timeMap.size(); i++) {
            long[] time2Event = (long[])timeMap.get(i);
            curTime = time2Event[0];
            if ((prevTime/interval == time) || (curTime/interval == time) ||
                ((prevTime/interval < time) && (time < curTime/interval))) {
                break;
            }
            event = time2Event[1];
            prevTime = curTime;
        }

        return event;

    }
}
