/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 1, 2003
 * Time: 7:38:54 PM
 */

package EVolve.util.phasedetectors;

import java.awt.Component;
import java.util.ArrayList;

import javax.swing.JScrollPane;

import EVolve.Scene;
import EVolve.util.HelperFuncs;
import EVolve.util.phasedetectors.phasedetectorUI.PhaseDetectorToolBarState;
import EVolve.visualization.AxesPanel;
import EVolve.visualization.XYViz.XYVisualization;

public abstract class PhaseDetector implements Cloneable{
    protected int interval;
    protected ArrayList data;
    protected PhaseDetectorToolBarState toolbarState;
    protected PhaseUndoRedo undoRedo;
    protected AxesPanel panel;

    public PhaseDetector(XYVisualization visual) {
        interval = 1;
        data = new ArrayList();
        undoRedo = new PhaseUndoRedo((AxesPanel)((JScrollPane)visual.getPanel()).getViewport().getView());
        panel = (AxesPanel)((JScrollPane)visual.getPanel()).getViewport().getView();
        toolbarState = new PhaseDetectorToolBarState();
    }

    public PhaseDetector(XYVisualization visual, int interval) {
        this(visual);
        this.interval = interval;
    }

    public void reset() {
        data.clear();
        undoRedo.reset();
    }

    public void drawPhase(boolean append) {
        if (data.size() > 0) {
            ArrayList phase = autoDetectPhase();
            undoRedo.registerAction(phase, PhaseUndoRedo.detectedPhases, append);
        }

        panel.setPhases(undoRedo.getCurrentFramePhases());
    }

    public ArrayList getPhase() {
        return undoRedo.getCurrentFramePhases();
    }

    public void addPhaseManually(int added) {
        undoRedo.registerAction(new Integer(added),PhaseUndoRedo.singlePhase,true);
        undoRedo.update();
    }

    public void removePhaseManually(int removed) {
        ArrayList phases = undoRedo.getCurrentFramePhases();
        boolean found = false;
        for (int i=0; i<phases.size(); i++) {
            Integer aPhase = (Integer)phases.get(i);
            if (aPhase.intValue() == removed) {
                found = true;
                break;
            }
        }

        if (!found) return;

        undoRedo.registerAction(new Integer(-1*removed), PhaseUndoRedo.singlePhase, true);
        undoRedo.update();
    }

    public void pastePhases(ArrayList pasted) {
        undoRedo.registerAction(pasted, PhaseUndoRedo.pastedPhases, true);
        undoRedo.update();
    }

    public void undo() {
        undoRedo.undo();
    }

    public void redo() {
        undoRedo.redo();
    }

    public boolean undoable() {
        return undoRedo.undoable();
    }

    public boolean redoable() {
        return undoRedo.redoable();
    }

    public PhaseDetectorToolBarState getToolBarState() {
        return toolbarState;
    }

    public void updateToolBarState() {
        Scene.getUIManager().getPhaseDetectorToolBar().updateToolBarState(toolbarState);
    }

    protected abstract void refreshDetectorParameters();

    public abstract void saveSetting();

    public abstract String getName();

    public abstract void collectData(long xMappedId, long yMappedId);

    public abstract Component[] createDetectorParamsControls();

    public abstract void triggerPhases(int noiseTolerance, float threshold, boolean append);

    public abstract void entitySetPhases(int noiseTolerance, float threshold, boolean append);

    protected abstract ArrayList autoDetectPhase();

    public Object clone() {
        PhaseDetector o = null;
        try {
            o = (PhaseDetector)super.clone();
        } catch (CloneNotSupportedException e) {
            System.out.println(e.getStackTrace());
            return o;
        }
        o.data = HelperFuncs.cloneArrayList(data);
        o.toolbarState = (PhaseDetectorToolBarState)toolbarState.clone();
        o.undoRedo = (PhaseUndoRedo)undoRedo.clone();

        return o;
    }

    public void setPanel(AxesPanel panel) {
        this.panel = panel;
        undoRedo.setPanel(panel);
    }

}
