/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 25, 2003
 * Time: 12:37:08 AM
 */

package EVolve.util.phasedetectors;

import java.awt.Cursor;

import javax.swing.JScrollPane;

import EVolve.Scene;
import EVolve.util.HelperFuncs;
import EVolve.visualization.AxesPanel;
import EVolve.visualization.XYViz.XYVisualization;

public class PhaseOperation {
    private PhaseClipboard phaseClipboard;

    public PhaseOperation() {
        phaseClipboard = new PhaseClipboard();
    }

    public void add() {
        XYVisualization workingViz = HelperFuncs.getActiveXYViz();

        if (workingViz == null) return;

        workingViz.freeze(true);
        AxesPanel canvas = (AxesPanel)((JScrollPane)workingViz.getPanel()).getViewport().getView();
        canvas.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        new PhaseAddRemover().beginAdd();
    }

    public void remove() {
        XYVisualization workingViz = HelperFuncs.getActiveXYViz();

        if (workingViz == null) return;

        workingViz.freeze(true);
        AxesPanel canvas = (AxesPanel)((JScrollPane)workingViz.getPanel()).getViewport().getView();
        canvas.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        new PhaseAddRemover().beginRemove();
    }

    public void copyPhase() {
        phaseClipboard.copyPhase();
    }

    public void pastePhase() {
        phaseClipboard.pastePhase();
    }

    public boolean clipboardIsEmpty() {
        return phaseClipboard.isEmpty();
    }

    public void triggerPhases(int noiseTolerance, float threshold, boolean append) {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)&&(visual.getPhaseDetector() == null)) {
            Scene.showErrorMessage("No phase detector available.");
            return;
        }

        visual.getPhaseDetector().triggerPhases(noiseTolerance, threshold, append);
    }

    public void entitySetPhases(int noiseTolerance, float threshold, boolean append) {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)&&(visual.getPhaseDetector() == null)) {
            Scene.showErrorMessage("No phase detector available.");
            return;
        }

        visual.getPhaseDetector().entitySetPhases(noiseTolerance, threshold, append);
    }

    public void undo() {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)||(visual.getPhaseDetector() == null)) {
            Scene.showErrorMessage("No undoable actions available.");
            return;
        }

        visual.getPhaseDetector().undo();
    }

    public void redo() {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)||(visual.getPhaseDetector() == null)) {
            Scene.showErrorMessage("No redoable actions available.");
            return;
        }

        visual.getPhaseDetector().redo();
    }

    public boolean undoable() {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)||(visual.getPhaseDetector() == null)) {
            return false;
        }

        return visual.getPhaseDetector().undoable();
    }

    public boolean redoable() {
        XYVisualization visual = HelperFuncs.getActiveXYViz();

        if ((visual == null)||(visual.getPhaseDetector() == null)) {
            return false;
        }

        return visual.getPhaseDetector().redoable();
    }
}
