/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: May 7, 2003
 * Time: 1:09:09 PM
 */

package EVolve.util.phasedetectors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Stack;

import EVolve.Scene;
import EVolve.util.HelperFuncs;
import EVolve.util.phasedetectors.phasedetectorUI.PhaseDetectorToolBar;
import EVolve.visualization.AxesPanel;

public class PhaseUndoRedo implements Cloneable{
    public static final int singlePhase = 0x0, pastedPhases = 0x1;
    public static final int triggeredPhases = 0x2, detectedPhases = 0x03;
    public static final int entitySetPhases = 0x4;
    private Stack undoStack, redoStack;
    private PhaseFrame currentFrame;
    private AxesPanel panel;

    public PhaseUndoRedo(AxesPanel panel) {
        undoStack = new Stack();
        redoStack = new Stack();
        currentFrame = new PhaseFrame();
        this.panel = panel;
    }

    public void undo() {
        PhaseDetectorToolBar toolbar = Scene.getUIManager().getPhaseDetectorToolBar();

        if (currentFrame.undoList.isEmpty()) {
            redoStack.push(currentFrame);
            currentFrame = (PhaseFrame)undoStack.pop();
        } else {

            currentFrame.redoList.push(currentFrame.undoList.pop());
            currentFrame.redoParams.push(currentFrame.undoParams.pop());

            if (currentFrame.undoList.isEmpty()&&!undoStack.isEmpty()) {
                redoStack.push(currentFrame);
                currentFrame = (PhaseFrame)undoStack.pop();
            }
        }
        HashMap state = null;
        if (!currentFrame.undoParams.isEmpty())
            state = (HashMap)currentFrame.undoParams.peek();

        if (state != null)
            toolbar.setToolbarParams(state);
        update();
    }

    public void redo() {
        PhaseDetectorToolBar toolbar = Scene.getUIManager().getPhaseDetectorToolBar();
        if (currentFrame.redoList.isEmpty()) {
            undoStack.push(currentFrame);
            currentFrame = (PhaseFrame)redoStack.pop();
            if (currentFrame.undoList.isEmpty()&&!currentFrame.redoList.isEmpty()) {
                currentFrame.undoList.push(currentFrame.redoList.pop());
                currentFrame.undoParams.push(currentFrame.redoParams.pop());
            }
        } else {
            currentFrame.undoList.push(currentFrame.redoList.pop());
            currentFrame.undoParams.push(currentFrame.redoParams.pop());
        }
        HashMap state = null;
        if (!currentFrame.undoParams.isEmpty())
            state = (HashMap)currentFrame.undoParams.peek();

        if (state != null)
            toolbar.setToolbarParams(state);
        update();
    }

    public boolean undoable() {
        return !(currentFrame.undoList.isEmpty() && undoStack.isEmpty());
    }

    public boolean redoable() {
        return !(currentFrame.redoList.isEmpty() && redoStack.isEmpty());
    }

    public void registerAction(Object action, int desc, boolean append) {
        PhaseDetectorToolBar toolbar = Scene.getUIManager().getPhaseDetectorToolBar();
        HashMap state = toolbar.getToolbarParams();
        switch (desc) {
            case singlePhase:
            case pastedPhases:
                currentFrame.undoList.push(action);
                currentFrame.undoParams.push(state);
                currentFrame.redoList.clear();
                break;
            case triggeredPhases:
            case detectedPhases:
            case entitySetPhases:
                if (!append) {
                    undoStack.push(currentFrame);
                    currentFrame.redoList.clear();
                    currentFrame.redoParams.clear();
                    currentFrame = new PhaseFrame();
                } else { // append
                    currentFrame.redoList.clear();
                    currentFrame.redoParams.clear();
                }
                currentFrame.undoParams.push(state);
                currentFrame.undoList.push(action);
                break;
            default:
                Scene.showErrorMessage("Internal error occurred when register an action to Undo/Redo");
                break;
        }
        redoStack.clear();
        Scene.getUIManager().getPhaseDetectorToolBar().enableButton(true);
    }

    public void reset() {
        undoStack.clear();
        redoStack.clear();
        currentFrame = new PhaseFrame();
        panel.setPhases(null);
    }

    public void update() {
        ArrayList phases = getCurrentFramePhases();
        panel.setPhases(phases);
        panel.repaint();
    }

    public ArrayList getCurrentFramePhases() {
        HashSet tempPhases = new HashSet();
        ArrayList phases = new ArrayList();

        if (currentFrame.undoList.isEmpty()) return phases;

        for (int i=0; i<currentFrame.undoList.size(); i++) {
            Object action = currentFrame.undoList.get(i);
            if (action instanceof ArrayList) {
                ArrayList temp = (ArrayList)action;
                for (int j=0; j<temp.size(); j++) {
                    if (!tempPhases.contains(temp.get(j)))
                        tempPhases.add(temp.get(j));
                }
            } else {
                int aPhase = ((Integer)action).intValue();
                if (aPhase < 0) {// a removed phase
                    if (tempPhases.contains(new Integer(-1*aPhase)))
                        tempPhases.remove(new Integer(-1*aPhase));
                } else {
                    if (!tempPhases.contains(action))
                        tempPhases.add(action);
                }
            }
        }

        int list[] = new int[tempPhases.size()];
        Iterator it = tempPhases.iterator();
        int i=0;
        while (it.hasNext()) {
            list[i++] = ((Integer)it.next()).intValue();
        }
        // sort phases
        int j=1,temp;
        boolean done=false;
        while((j<list.length)&&(!done)) {
            done=true;
            for(i=0;i<list.length-j;i++){
                if(list[i]>list[i+1]){
                    done=false;
                    temp=list[i];
                    list[i]=list[i+1];
                    list[i+1]=temp;
                }
            }
            j++;
        }

        for (i=0; i<list.length; i++) {
            phases.add(new Integer(list[i]));
        }

        return phases;
    }

    public void setPanel(AxesPanel panel) {
        this.panel = panel;
    }

    public Object clone() {
        PhaseUndoRedo o = null;
        try {
            o = (PhaseUndoRedo)super.clone();
        } catch (CloneNotSupportedException e) {
            System.out.println(e.getStackTrace());
            return o;
        }

        o.undoStack = HelperFuncs.cloneStack(undoStack);
        o.redoStack = HelperFuncs.cloneStack(redoStack);
        o.currentFrame = (PhaseFrame)currentFrame.clone();
        return o;
    }
}
