/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Apr 27, 2003
 * Time: 5:48:17 PM
 */

package EVolve.util.phasedetectors.phasedetectorUI;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import EVolve.Scene;
import EVolve.util.phasedetectors.PhaseDetector;
import EVolve.visualization.AxesPanel;
import EVolve.visualization.Visualization;
import EVolve.visualization.XYViz.XYVisualization;

public class PhaseDetectorToolBar extends JToolBar{
    public static int NumberOfSharedOptions = 3;
    private JButton buttonRefreshAppend, buttonRefreshNew, buttonClose, buttonUndo, buttonRedo;
    private JButton buttonAddPhase, buttonRemovePhase;
    private JButton buttonCopyPhase, buttonPastePhase;
    private JComboBox comboOptions;

    private JButton buttonTriggerAppend, buttonTriggerNew;
    private JButton buttonSetAppend, buttonSetNew;
    private JLabel labelTriggerNoise, labelSetNoise;
    private JTextField noiseTriggerTolerance, noiseSetTolerance;
    private JLabel labelTriggerThreshold, labelSetThreshold;
    private JTextField textTriggerThreshold, textSetThreshold;
    private int noiseTriggerToler = 0, noiseSetToler = 2;
    private float triggerThreshold = 0.8F, setThreshold = 0.05F;

    private HashMap controlsMap;
    private PhaseDetector detector;
    private String detectorName, entityTrigger, entitySet;
    private Component commonControls[];
    private Component appendCommandButtons[];


    public PhaseDetectorToolBar() {
        super("Phase detector setting");
        controlsMap = new HashMap();
        entityTrigger = "Phase Entity Trigger";
        entitySet = "Phase Entity Set";
        detector = null;
        setPreferredSize(new java.awt.Dimension(850,28));
        FlowLayout layout = new FlowLayout(FlowLayout.LEFT,2,-2);
        layout.preferredLayoutSize(this);
        setLayout(layout);
        commonControls = createCommandButtons();
        appendCommandButtons = createRefreshButtons();
        createPhaseDetectorToolBar();
        layout.layoutContainer(this);
    }

    private void createPhaseDetectorToolBar() {
        setRollover(true);

        comboOptions = new JComboBox();

        controlsMap.put(entityTrigger, createEntityTriggerButtons());
        controlsMap.put(entitySet, createEntitySetButtons());
        comboOptions.addItem(entityTrigger);
        comboOptions.addItem(entitySet);

        comboOptions.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String item = (String)comboOptions.getSelectedItem();
                int number = getComponents().length;
                for (int i=number-1; i>=2; i--) {
                    remove(i);
                }
                Component[] controls = (Component[])controlsMap.get(item);
                for (int i=0; i<controls.length; i++) {
                    add(controls[i]);
                }
                for (int i=0; i<commonControls.length; i++) {
                    add(commonControls[i]);
                }
            }
        });
        comboOptions.setPreferredSize(new java.awt.Dimension(180,26));
        add(comboOptions);
        addSeparator();
        comboOptions.setSelectedIndex(0);
    }

    public void refresh() {
        ArrayList vizList = Scene.getVisualizationManager().getVisualizationList();
        for (int i=0; i<vizList.size(); i++) {
            Visualization visual = (Visualization)vizList.get(i);
            if (visual.isFreezed()) return;
        }

        if (detector != null)
            updateToolBarState(detector.getToolBarState());

        Visualization visual = Scene.getVisualizationManager().getActiveVisualization();
        if ((visual == null) || (!(visual instanceof XYVisualization))) {
            detector = null;
        } else
            detector = ((XYVisualization)visual).getPhaseDetector();

        controlsMap.remove(detectorName);

        comboOptions.removeAllItems();
        if (detector != null) {
            detectorName = detector.getName();
            Component[] detectorControls = detector.createDetectorParamsControls();
            Component[] all = new Component[detectorControls.length+appendCommandButtons.length];
            for ( int i = 0; i<detectorControls.length; i++) {
                all[i] = detectorControls[i];
            }
            for ( int i = 0; i<appendCommandButtons.length; i++) {
                all[i+detectorControls.length] = appendCommandButtons[i];
            }
            controlsMap.put(detectorName,all);
            comboOptions.addItem(detectorName);
        } else {
            detectorName = "";
        }

        comboOptions.addItem(entityTrigger);
        comboOptions.addItem(entitySet);

        if (detector != null) { //restore button/controls state
            PhaseDetectorToolBarState state = detector.getToolBarState();
            comboOptions.setSelectedIndex(state.selectedOption);
            HashMap optionControlsMap = state.optionsControlState;
            Iterator it = controlsMap.keySet().iterator();
            while (it.hasNext()) {
                String key = (String)it.next();
                if ((key.equals(detectorName)) || (!optionControlsMap.containsKey(key))) continue;

                Component[] controls = (Component[])controlsMap.get(key);
                boolean[] enableStates = (boolean[])optionControlsMap.get(key);
                for (int i=0; i<controls.length; i++) {
                    controls[i].setEnabled(enableStates[i]);
                }
            }
        }

        enableButton(detector != null);
        buttonClose.setEnabled(true);
    }

    public void enableButton(boolean flag) {
        comboOptions.setEnabled(flag);
        for (int i=0; i<commonControls.length-1; i++) {
            commonControls[i].setEnabled(flag);
        }
        Iterator it = controlsMap.keySet().iterator();
        while (it.hasNext()) {
            Component items[] = (Component[])controlsMap.get(it.next());
            for (int i=0; i<items.length; i++) {
                items[i].setEnabled(flag);
            }
        }
        boolean empty = Scene.getVisualizationManager().getPhaseOperation().clipboardIsEmpty();
        if (buttonPastePhase.isEnabled()) {
            buttonPastePhase.setEnabled(!empty);
        }
        if (flag == true) {
            boolean undoable = Scene.getVisualizationManager().getPhaseOperation().undoable();
            buttonUndo.setEnabled(undoable);
            boolean redoable = Scene.getVisualizationManager().getPhaseOperation().redoable();
            buttonRedo.setEnabled(redoable);
        }
    }

    public void updateVisualization(boolean cleanup, boolean append) {
        Visualization visual = Scene.getVisualizationManager().getActiveVisualization();
        if ((visual == null) || (!(visual instanceof XYVisualization)))
            return ;

        AxesPanel canvas = (AxesPanel)((JScrollPane)visual.getPanel()).getViewport().getView();
        PhaseDetector phaseDetector = ((XYVisualization)visual).getPhaseDetector();

        if (phaseDetector == null) return;

        phaseDetector.saveSetting();
        if (cleanup) {
            canvas.setPhases(null);
        } else
            phaseDetector.drawPhase(append);
        canvas.repaint();
    }

    public void updateToolBarState(PhaseDetectorToolBarState state) {
        state.selectedOption = comboOptions.getSelectedIndex();
        HashMap optionControlsMap = state.optionsControlState;
        Iterator it = controlsMap.keySet().iterator();
        while (it.hasNext()) {
            String key = (String)it.next();
            if (key.equals(detectorName)) continue;

            Component[] controls = (Component[])controlsMap.get(key);
            boolean[] enableStates = null;
            if (optionControlsMap.containsKey(key)) {
                enableStates = (boolean[])optionControlsMap.get(key);
            } else {
                enableStates = new boolean[controls.length];
                optionControlsMap.put(key,enableStates);
            }
            for (int i=0; i<controls.length; i++) {
                enableStates[i] = controls[i].isEnabled();
            }
        }
    }

    private Component[] createEntityTriggerButtons() {
        buttonTriggerAppend = new JButton(new ImageIcon(Scene.getGifURL("trigger_append.gif")));
        buttonTriggerAppend.setToolTipText("Append triggered phases to the current phase set");
        buttonTriggerAppend.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    noiseTriggerToler = Integer.parseInt(noiseTriggerTolerance.getText());
                    triggerThreshold = Float.parseFloat(textTriggerThreshold.getText());
                    Scene.getVisualizationManager().getPhaseOperation().triggerPhases(noiseTriggerToler, triggerThreshold, true);
                } catch (NumberFormatException exp) {
                    Scene.showErrorMessage("Noise Tolerance must be an integer!");
                }
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonTriggerAppend.setPreferredSize(new java.awt.Dimension(28,28));

        buttonTriggerNew = new JButton(new ImageIcon(Scene.getGifURL("trigger_new.gif")));
        buttonTriggerNew.setToolTipText("Trigger phases and use them as a new phase set");
        buttonTriggerNew.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    noiseTriggerToler = Integer.parseInt(noiseTriggerTolerance.getText());
                    triggerThreshold = Float.parseFloat(textTriggerThreshold.getText());
                    Scene.getVisualizationManager().getPhaseOperation().triggerPhases(noiseTriggerToler, triggerThreshold, false);
                } catch (NumberFormatException exp) {
                    Scene.showErrorMessage("Noise Tolerance must be an integer!");
                }
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonTriggerNew.setPreferredSize(new java.awt.Dimension(28,28));

        labelTriggerNoise = new JLabel("Noise tolerance (intervals):");
        noiseTriggerTolerance = new JTextField(String.valueOf(noiseTriggerToler));
        labelTriggerThreshold = new JLabel("Threshold");
        textTriggerThreshold = new JTextField(String.valueOf(triggerThreshold));
        noiseTriggerTolerance.setColumns(2);
        textTriggerThreshold.setColumns(3);

        Component[] returnVal = new Component[4];

        //returnVal[0] = labelTriggerNoise;
        //returnVal[1] = noiseTriggerTolerance;
        returnVal[0] = labelTriggerThreshold;
        returnVal[1] = textTriggerThreshold;
        returnVal[2] = buttonTriggerAppend;
        returnVal[3] = buttonTriggerNew;
        return returnVal;
    }

    private Component[] createEntitySetButtons() {
        Component[] returnVal = new Component[6];

        buttonSetAppend = new JButton(new ImageIcon(Scene.getGifURL("trigger_append.gif")));
        buttonSetAppend.setToolTipText("Append phases to the current phase set");
        buttonSetAppend.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    noiseSetToler = Integer.parseInt(noiseSetTolerance.getText());
                    setThreshold = Float.parseFloat(textSetThreshold.getText());
                    Scene.getVisualizationManager().getPhaseOperation().entitySetPhases(noiseSetToler, setThreshold, true);
                } catch (NumberFormatException exp) {
                    Scene.showErrorMessage("Noise Tolerance must be an integer!");
                }
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonSetAppend.setPreferredSize(new java.awt.Dimension(28,28));

        buttonSetNew = new JButton(new ImageIcon(Scene.getGifURL("trigger_new.gif")));
        buttonSetNew.setToolTipText("Set phases and use them as a new phase set");
        buttonSetNew.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    noiseSetToler = Integer.parseInt(noiseSetTolerance.getText());
                    setThreshold = Float.parseFloat(textSetThreshold.getText());
                    Scene.getVisualizationManager().getPhaseOperation().entitySetPhases(noiseSetToler, setThreshold, false);
                } catch (NumberFormatException exp) {
                    Scene.showErrorMessage("Noise Tolerance must be an integer!");
                }
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonSetNew.setPreferredSize(new java.awt.Dimension(28,28));

        labelSetNoise = new JLabel("Noise tolerance (intervals):");
        noiseSetTolerance = new JTextField(String.valueOf(noiseSetToler));
        labelSetThreshold = new JLabel("Threshold");
        textSetThreshold = new JTextField(String.valueOf(setThreshold));
        noiseSetTolerance.setColumns(2);
        textSetThreshold.setColumns(3);


        returnVal[0] = labelSetNoise;
        returnVal[1] = noiseSetTolerance;
        returnVal[2] = labelSetThreshold;
        returnVal[3] = textSetThreshold;
        returnVal[4] = buttonSetAppend;
        returnVal[5] = buttonSetNew;
        return returnVal;
    }

    private Component[] createCommandButtons() {
        Component returnVal[] = new Component[7];

        buttonClose = new JButton(new ImageIcon(Scene.getGifURL("close.gif")));
        buttonClose.setToolTipText("Disable Phase Detector");
        buttonClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().enablePhaseDetector(false);
                setVisible(false);
                updateVisualization(true, false);
                Scene.getUIManager().refreshToolBar();
            }
        });
        buttonClose.setPreferredSize(new java.awt.Dimension(28,28));

        buttonCopyPhase = new JButton(new ImageIcon(Scene.getGifURL("copy.gif")));
        buttonCopyPhase.setToolTipText("Copy phase in the current visualization");
        buttonCopyPhase.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().copyPhase();
                boolean empty = Scene.getVisualizationManager().getPhaseOperation().clipboardIsEmpty();
                buttonPastePhase.setEnabled(!empty);
            }
        });
        buttonCopyPhase.setPreferredSize(new java.awt.Dimension(28,28));

        buttonPastePhase = new JButton(new ImageIcon(Scene.getGifURL("paste.gif")));
        buttonPastePhase.setToolTipText("Paste phase to the current visualization");
        buttonPastePhase.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().pastePhase();
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonPastePhase.setPreferredSize(new java.awt.Dimension(28,28));
        buttonPastePhase.setEnabled(false);

        buttonUndo = new JButton(new ImageIcon(Scene.getGifURL("undo.gif")));
        buttonUndo.setToolTipText("Undo");
        buttonUndo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().undo();
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
                buttonRedo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().redoable());
            }
        });
        buttonUndo.setPreferredSize(new java.awt.Dimension(28,28));
        buttonUndo.setEnabled(false);

        buttonRedo = new JButton(new ImageIcon(Scene.getGifURL("redo.gif")));
        buttonRedo.setToolTipText("Redo");
        buttonRedo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().redo();
                buttonRedo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().redoable());
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonRedo.setPreferredSize(new java.awt.Dimension(28,28));
        buttonRedo.setEnabled(false);

        buttonAddPhase = new JButton(new ImageIcon(Scene.getGifURL("add.gif")));
        buttonAddPhase.setToolTipText("Add phase manually");
        buttonAddPhase.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().add();
            }
        });
        buttonAddPhase.setPreferredSize(new java.awt.Dimension(28,28));

        buttonRemovePhase = new JButton(new ImageIcon(Scene.getGifURL("remove.gif")));
        buttonRemovePhase.setToolTipText("Remove phase manually");
        buttonRemovePhase.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().getPhaseOperation().remove();
            }
        });
        buttonRemovePhase.setPreferredSize(new java.awt.Dimension(28,28));

        returnVal[0] = buttonCopyPhase;
        returnVal[1] = buttonPastePhase;
        returnVal[2] = buttonAddPhase;
        returnVal[3] = buttonRemovePhase;
        returnVal[4] = buttonUndo;
        returnVal[5] = buttonRedo;
        returnVal[6] = buttonClose;

        return returnVal;
    }

    private Component[] createRefreshButtons() {
        Component[] returnVal = new Component[2];

        buttonRefreshAppend = new JButton(new ImageIcon(Scene.getGifURL("refresh_append.gif")));
        buttonRefreshAppend.setToolTipText("Append detected phases to the current phase set");
        buttonRefreshAppend.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateVisualization(false, true);
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonRefreshAppend.setPreferredSize(new java.awt.Dimension(28,28));

        buttonRefreshNew = new JButton(new ImageIcon(Scene.getGifURL("refresh_new.gif")));
        buttonRefreshNew.setToolTipText("Use detected phases as a new phase set");
        buttonRefreshNew.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateVisualization(false, false);
                buttonUndo.setEnabled(Scene.getVisualizationManager().getPhaseOperation().undoable());
            }
        });
        buttonRefreshNew.setPreferredSize(new java.awt.Dimension(28,28));

        returnVal[0] = buttonRefreshAppend;
        returnVal[1] = buttonRefreshNew;

        return returnVal;
    }

    public HashMap getToolbarParams() {
        HashMap paramsState = new HashMap();
        Iterator it = controlsMap.keySet().iterator();
        while (it.hasNext()) {
            String params = "";
            Object key = it.next();
            Component[] value = (Component[])controlsMap.get(key);
            for (int i=0; i<value.length; i++) {
                if (value[i] instanceof JTextField) {
                    params = params + ((JTextField)value[i]).getText() + ";";
                }
            }
            paramsState.put(key,params);
        }
        return paramsState;
    }

    public void setToolbarParams(HashMap paramsState) {

        Iterator it = paramsState.keySet().iterator();

        while (it.hasNext()) {
            Object key = it.next();
            Component[] value = (Component[])controlsMap.get(key);
            StringTokenizer params = new StringTokenizer((String)paramsState.get(key),";");
            for (int i=0; i<value.length; i++) {
                if (value[i] instanceof JTextField) {
                    ((JTextField)value[i]).setText(params.nextToken());
                }
            }
        }
        revalidate();

    }
}
