/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * ACL Lab, School of Computer Science, Montreal, QC
 * Date: 2002-8-27
 * Time: 21:48:13
 */
package EVolve.util.predefinedutils;

import java.util.*;
import EVolve.exceptions.EVolveException;
import EVolve.util.xmlutils.datastructures.SerializedData;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.Scene;


public class PredefinedVisualization {

    private boolean translated;
    private String name;
    private String filename;
    private ArrayList vizInfoList;
    private int currentIndex;
    private boolean autoLoadTrace;
    private String dataSourceName;


    public PredefinedVisualization(String configFile,String name) {
        translated = false;
        filename = configFile;
        this.name = name;
        currentIndex = -1;
        autoLoadTrace = false;
        vizInfoList = null;
        try {
            SerializedData info = VisualizationSerializer.v().getVizInfoFromDisk(filename);
            autoLoadTrace = info.AutoLoadTrace.equals("Yes");
            dataSourceName = info.DataSourceName;
        } catch (EVolveException e) {

        }
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getFilename() {
        return filename;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }

    public boolean isTranslated() {
        return translated & (!autoLoadTrace);
    }

    public void setTranslated(boolean translated) {
        this.translated = false;
    }

    public SerializedVisualization getVizInfo(int index) {
        return (SerializedVisualization)vizInfoList.get(index);
    }

    public void translateVizInfo() throws EVolveException{
        currentIndex = -1;
        SerializedData info = VisualizationSerializer.v().getVizInfoFromDisk(filename);
        autoLoadTrace = info.AutoLoadTrace.equals("Yes");
        if (autoLoadTrace) {
            boolean success = Scene.getUIManager().addDataSource(info.DataSourceName);

            if (!success) return;

            String currentTrace = Scene.getCurrentDataFilename();
            if ((currentTrace==null)||(!currentTrace.equals(info.TraceFileName))) {
                Scene.setDataFilename(info.TraceFileName);
                Scene.autoLoadDataSource();
                Scene.setDataFilename(null);
            }
            Scene.getFilter().loadSelection(info.SerializedSelections,Integer.parseInt(info.SelectedSelection));
        }
        vizInfoList = info.SerializedVisualizations;
        setTranslated(true);
    }

    public boolean hasNext() {
        if (currentIndex == vizInfoList.size() -1) return false;
        return true;
    }

    public SerializedVisualization getNextVizInfo() {
        return (SerializedVisualization )vizInfoList.get(++currentIndex);
    }

    public void reset() {
        currentIndex = -1;
    }

    public boolean isAutoLoadTrace() {
        return autoLoadTrace;
    }

    public String getUsedDataSourceName() {
        return dataSourceName;
    }
}
