/*
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: 2002-8-29
 * Time: 13:31:24
 * To change template for new class use 
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package EVolve.util.predefinedutils;

import java.util.*;
import java.awt.*;
import java.io.*;

import EVolve.Scene;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.settings.SceneSetting;
import EVolve.exceptions.EVolveException;
import EVolve.visualization.Visualization;

public class PredefinedVisualizationRunner {
    private ArrayList predefinedVizCollection;
    private int selected;

    public PredefinedVisualizationRunner() {
        predefinedVizCollection = new ArrayList();
        selected = 0;
    }

    public int getSelected() {
        return this.selected;
    }

    public void setSelected(int selected) {
        this.selected = selected;
    }

    public int getPredefinedVisualizationNumber() {
        return predefinedVizCollection.size();
    }

    public String getConfigureName(int index) {
        return ((PredefinedVisualization)predefinedVizCollection.get(index)).getName();
    }

    public PredefinedVisualization getSelectedPredefinedViz() {
        return predefinedVizCollection.size() == 0 ? null : (PredefinedVisualization)predefinedVizCollection.get(selected);
    }

    public PredefinedVisualization getPredefinedVizByIndex(int index) {
        return (PredefinedVisualization)predefinedVizCollection.get(index);
    }

    public void addPredefinedVisualization(String filename, String configname) {
        for (int i=0; i<predefinedVizCollection.size(); i++) {
            PredefinedVisualization viz = (PredefinedVisualization)predefinedVizCollection.get(i);
            if (viz.getFilename().equals(filename)) {
                viz.setTranslated(false);
                viz.setFilename(filename);
                viz.setName(configname);
                return;
            }
        }
        PredefinedVisualization viz = new PredefinedVisualization(filename,configname);
        predefinedVizCollection.add(predefinedVizCollection.size(),viz);
    }

    public boolean containsConfigure(String name) {

        for (int i=0; i< predefinedVizCollection.size(); i++) {
            if (((PredefinedVisualization)predefinedVizCollection.get(i)).getName().equals(name)) return true;
        }

        return false;
    }

    public int getIdFromName(String name) {
        int retValue = -1;
        for (int i=0; i< predefinedVizCollection.size(); i++) {
            if (((PredefinedVisualization)predefinedVizCollection.get(i)).getName().equals(name)) retValue = i;
        }

        return retValue;
    }

    public void runPredefinedVisualization() {
        Visualization visual = null;
        PredefinedVisualization predefinedViz = getSelectedPredefinedViz();

        if (predefinedViz == null) return;
        try {
            if (!predefinedViz.isTranslated())
                predefinedViz.translateVizInfo();

        } catch (EVolveException e) {
            Scene.showErrorMessage(e.getMessage());
            return;
        }

        //Scene.getVisualizationManager().cleanup();
        while (predefinedViz.hasNext()) {
            SerializedVisualization viz = predefinedViz.getNextVizInfo();

            visual = Scene.getVisualizationManager().newVisualization(viz.FactoryName);

            visual.getWindow().setBounds(new Rectangle(Integer.parseInt(viz.WindowX), Integer.parseInt(viz.WindowY),
                                                       Integer.parseInt(viz.WindowWidth),Integer.parseInt(viz.WindowHeight)));
            visual.setName(viz.WindowTitle);
            visual.restoreConfiguration(viz);
        }
        Scene.getVisualizationManager().addAllVisualizations();
        Scene.getFilter().updateSelection();
        Scene.getVisualizationManager().prepareListToBeVisualized();
        Scene.visualize();
        predefinedViz.reset();
    }

    public ArrayList getPredefinedVizList() {
        return predefinedVizCollection;
    }

    public void loadPredefinedFiles() {
        String dirName = SceneSetting.v().getConfigurationPath();

        if ((dirName == null)||(dirName.length()==0))
            return ;


        File dir = new File(dirName);
        String files[] = dir.list(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.endsWith("xml");
            }
        });

        /* Were we able to read the directory? */
        if (files == null) {
            Scene.showErrorMessage("Directory "+ dirName + " does not exist!");
            return;
        }

        for (int i=0; i<files.length; i++) {
            String fname = dirName + File.separator + files[i];
            VisualizationSerializer.v().loadConfiguration(fname);
        }

    }
}
