/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Jan 16, 2003
 * Time: 8:39:52 PM
 */

package EVolve.util.predefinedutils;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.exceptions.*;
import EVolve.util.settings.SceneSetting;
import EVolve.util.xmlutils.*;
import EVolve.util.xmlutils.datastructures.SerializedData;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.xmlutils.datastructures.XMLWriteOrder;
import EVolve.util.xmlutils.datastructures.SerializedSelection;
import EVolve.visualization.Visualization;

public class VisualizationSerializer {
    private static VisualizationSerializer instance = null;
    private JDialog dialog;
    private JCheckBox checkSelection;
    private JTextField textConfigureName;
    private JTextField textConfigureFile;
    private JButton buttonConfigure;
    private JButton buttonSave, buttonCancel;

    private VisualizationSerializer() {
        dialog = null;
    }

    public static VisualizationSerializer v() {
        if (instance == null) instance = new VisualizationSerializer();
        return instance;
    }

    public void showSavingDialog() {
        if (dialog == null) {
            createDialog();
        } else {
            if (Scene.getFilter().getSelection().length == 0) {
                checkSelection.setSelected(false);
            }
            dialog.setVisible(true);
        }
    }

    private void createDialog() {
        Box boxMain = new Box(BoxLayout.Y_AXIS);

        dialog = new JDialog(Scene.getFrame(), "Setting Saving...", true);
        dialog.setBounds(new Rectangle(500,250));


        boxMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                "Save configuration and selections"));
        dialog.getContentPane().add(boxMain,BorderLayout.CENTER);

        //***********************************
        Box boxConfigureName = new Box(BoxLayout.X_AXIS);
        boxConfigureName.add(new JLabel("Configuration name:"));
        textConfigureName = new JTextField(12);
        boxConfigureName.add(Box.createHorizontalStrut(25));
        boxConfigureName.add(textConfigureName);

        Box boxConfig = new Box(BoxLayout.X_AXIS);
        boxConfig.add(new JLabel("Choose configure file:"));
        textConfigureFile = new JTextField(12);
        boxConfig.add(Box.createHorizontalStrut(20));
        boxConfig.add(textConfigureFile,BorderLayout.CENTER);
        buttonConfigure = new JButton("...");
        buttonConfigure.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = selectFile();
                if (fc == null) return;
                textConfigureFile.setText(fc.getSelectedFile().getPath());
            }
        });
        boxConfig.add(buttonConfigure,BorderLayout.EAST);

        //*****************************
        Box boxChk = new Box(BoxLayout.X_AXIS);
        checkSelection = new JCheckBox("Saving Filters & colors also");
        checkSelection.setMnemonic(KeyEvent.VK_F);
        checkSelection.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (Scene.getFilter().getSelection().length == 0) {
                    Scene.showErrorMessage("No active selection exists!");
                    checkSelection.setSelected(false);
                }
            }
        });
        boxChk.add(checkSelection);
        JLabel note = new JLabel("(If checked, the configuration file can ONLY be applied to " +
                      " the current data source.)");
        note.setFont(new Font("Dialog",Font.ITALIC|Font.BOLD,9));
        boxChk.add(note);

        //***********************************
        Box boxSaveCancel = new Box(BoxLayout.X_AXIS);
        buttonSave = new JButton("Save");
        buttonSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!validate()) return;
                dialogSave();
            }
        });

        buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
            }
        });
        boxMain.add(Box.createHorizontalStrut(40));
        boxSaveCancel.add(buttonSave);
        boxSaveCancel.add(Box.createHorizontalStrut(25));
        boxSaveCancel.add(buttonCancel);

        boxMain.add(boxConfigureName);
        boxMain.add(Box.createVerticalStrut(5));
        boxMain.add(boxConfig);
        boxMain.add(Box.createVerticalStrut(15));
        boxMain.add(boxChk);
        boxMain.add(Box.createVerticalStrut(8));
        boxMain.add(boxSaveCancel);

        dialog.getContentPane().add(boxMain);
        dialog.setResizable(false);
        dialog.pack();
        Scene.getUIManager().showDialog(dialog, dialog.getWidth(), dialog.getHeight());
    }

    private JFileChooser selectFile() {
        JFileChooser fc = new JFileChooser(SceneSetting.v().getConfigurationPath());
        if (fc.showSaveDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
            return fc;
        }
        return null;
    }

    private void dialogSave() {
        saveConfiguration();
        dialog.setVisible(false);
    }

    private boolean validate() {
        if ((textConfigureFile.getText() == null) || (textConfigureName.getText() == null)) {
            Scene.showErrorMessage("User must name the configuration to be saved and choose a file!");
            return false;
        }

        return true;
    }

    private void saveConfiguration() {
        ArrayList vizList = Scene.getVisualizationManager().getVisualizationList(Scene.getDataSourceManager().getCurrentDataSourceId());
        if (vizList.size() == 0) {
            Scene.showErrorMessage("No visualization available for the current data source.");
            return;
        }

        XMLWriter writer = new XMLWriter();
        writer.initialWriter(textConfigureFile.getText().trim(),false);
        SerializedData data = new SerializedData();
        data.EVolveVersion = Scene.VERSION;
        data.FileType = "Configuration File";
        data.AutoLoadTrace = checkSelection.isSelected() ? "Yes" : "No";
        data.ConfigurationName = textConfigureName.getText().trim();
        data.DataSourceName = Scene.getDataManager().getDataSource().getClass().getName();
        data.NumberOfVisualizations = String.valueOf(vizList.size());
        data.NumberOfSelections = String.valueOf(Scene.getFilter().getSelection().length);
        data.SelectedSelection = String.valueOf(Scene.getFilter().getSelectedIndex());
        data.PathForResult = null;
        data.TraceFileName = Scene.getCurrentDataFilename();
        data.SerializedVisualizations = new ArrayList();
        data.SerializedSelections = Scene.getFilter().getSelections();
        for (int i=0; i<vizList.size(); i++) {
            Visualization visual = (Visualization)vizList.get(i);
            data.SerializedVisualizations.add(visual.getCurrentConfiguration());
        }
        XMLWriteOrder orderMap = new XMLWriteOrder();
        writer.writeObject(data,orderMap.getOrder(data.getClass().getName()));
        writer.finalizeWriter();
        Scene.getToolsManager().getPredefinedVisualizationRunner().addPredefinedVisualization(textConfigureFile.getText(),data.ConfigurationName);
        //Scene.getUIManager().refreshMenu(textConfigureFile.getText().trim(),data.ConfigurationName);
    }

    public void loadConfiguration() {
        JFileChooser fc = new JFileChooser(Scene.getUIManager().getLastConfigDir());

        if(fc.showOpenDialog(Scene.getFrame()) != JFileChooser.APPROVE_OPTION) return;

        loadConfiguration(fc.getSelectedFile().getPath());
    }

    public void loadConfiguration(String filename) {
        String configureName = null;

        try {
            Scene.getUIManager().setLastConfigDir(filename);
            SerializedData data = getVizInfoFromDisk(filename);
            configureName = data.ConfigurationName;
        } catch (Exception e) {
            Scene.showErrorMessage(e.getMessage());
            return;
        }
        //Scene.getUIManager().refreshMenu(filename,configureName);
        Scene.getToolsManager().getPredefinedVisualizationRunner().addPredefinedVisualization(filename,configureName);
    }

    public SerializedData getVizInfoFromDisk(String filename) throws EVolveException {
        SerializedData data[] = null;

        try {
            data = new SerializedData[1];
            data[0] = new SerializedData();
            XMLLoader reader = new XMLLoader();
            reader.initialReader(filename, data[0]);
            reader.read(data);

            /*if (!data[0].EVolveVersion.equals(Scene.VERSION))
                throw (new WrongVersionException(data[0].EVolveVersion));*/
            if (data[0].FileType.charAt(0) != 'C')
                throw new FileTypeMismatchException("Configuration file", data[0].FileType + " file");

            // read in configurations
            int vizNumber = Integer.parseInt(data[0].NumberOfVisualizations);
            SerializedVisualization[] viz = new SerializedVisualization[vizNumber];
            for (int i=0; i<vizNumber; i++)
                viz[i] = new SerializedVisualization();
            reader = new XMLLoader();
            reader.initialReader(filename, viz[0]);
            reader.read(viz);
            data[0].SerializedVisualizations = new ArrayList();
            data[0].SerializedSelections = new ArrayList();
            for (int i=0; i<viz.length; i++) {
                data[0].SerializedVisualizations.add(viz[i]);
            }

            // read in selections
            int selectionNumber = Integer.parseInt(data[0].NumberOfSelections);
            if (selectionNumber != 0) {
                SerializedSelection[] selectionz = new SerializedSelection[selectionNumber];
                for (int i=0; i<selectionz.length; i++)
                    selectionz[i] = new SerializedSelection();
                reader = new XMLLoader();
                reader.initialReader(filename,selectionz[0]);
                reader.read(selectionz);
                for (int i=0; i<selectionz.length; i++) {
                    data[0].SerializedSelections.add(selectionz[i]);
                }
            }

            /*
            if (data[0].AutoLoadTrace.equals("Yes")) {
                if (!Scene.getCurrentDataFilename().equals(data[0].TraceFileName)) {
                    Scene.setDataFilename(data[0].TraceFileName);
                    Scene.autoLoadDataSource();
                    Scene.setDataFilename(null);
                }
                Scene.getFilter().loadSelection(data[0].SerializedSelections);
            }
            */

        }  catch (EVolveException e) {
            String detail = e.getMessage();
            throw new ReadConfigureFileException(filename, "N/A" ,"Detailed Information:\n"+detail);
        } catch (IOException e) {
            Scene.showErrorMessage(e.getMessage());
        } catch (Exception e) {
            throw new ReadConfigureFileException(filename, "N/A" ,"Detailed Information:N/A");
        }

        return data[0];
    }
}
