/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Jan 19, 2003
 * Time: 7:47:48 PM
 */

package EVolve.util.settings;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.exceptions.EVolveException;
import EVolve.data.ElementDefinition;
import EVolve.util.xmlutils.datastructures.*;
import EVolve.util.xmlutils.*;

public class SceneSetting extends Setting{
    private static SceneSetting instance = null;
    private JDialog settingDialog;
    private JTextField textClassPath, textSourcePath, textVizConfigPath, textVizResultPath, textDefaultDataPath ;
    private JButton buttonSave, buttonCancel;
    private ArrayList additionClassPath, sourcePath;
    private JButton[] buttonDir;


    private SceneSetting() {
        iniFilename = getFilenameWithPath("EVolve");

        contents = new JTextField[5];
        contents[0] = textDefaultDataPath = new JTextField(12);
        contents[1] = textVizResultPath = new JTextField(12);
        contents[2] = textVizConfigPath = new JTextField(12);
        contents[3] = textClassPath = new JTextField(12);
        contents[4] = textSourcePath = new JTextField(12);


        tags = new String[5];
        tags[0] = "[Default Data Path]";
        tags[1] = "[Default Viz result path]";
        tags[2] = "[Default Viz Config path]";
        tags[3] = "[Additional class path]";
        tags[4] = "[Source file path]";
        buttonDir = new JButton[contents.length];
        for (int i=0; i<buttonDir.length; i++) {
            buttonDir[i] = new JButton("...");
            buttonDir[i].setName(String.valueOf(i));
            buttonDir[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    addPath();
                }
            });
        }
        additionClassPath = new ArrayList();
        sourcePath = new ArrayList();
        readDataFromFile();
    }

    public static SceneSetting v() {
        if (instance == null)
            instance = new SceneSetting();
        return instance;
    }

    public void readDataFromFile() {
        try {
            EVolveSettingData data[] = new EVolveSettingData[1];
            data[0] = new EVolveSettingData();
            XMLLoader reader = new XMLLoader();

            File f = new File(iniFilename);
            if (!f.exists()) {
                f = null;
                save();
            }

            reader.initialReader(iniFilename,data[0]);

            reader.read(data);

            textDefaultDataPath.setText(data[0].DefaultDataPath);
            textVizResultPath.setText(data[0].DefaultVizResultPath);
            textVizConfigPath.setText(data[0].DefaultVizConfigurationPath);
            textClassPath.setText(data[0].AdditionalClassPath);
            textSourcePath.setText(data[0].SourceFilePath);

            additionClassPath.clear();
            StringTokenizer token = new StringTokenizer(data[0].AdditionalClassPath+File.pathSeparator,File.pathSeparator);
            while (token.hasMoreTokens()) {
                additionClassPath.add(token.nextToken());
            }

            sourcePath.clear();
            token = new StringTokenizer(data[0].SourceFilePath+File.pathSeparator,File.pathSeparator);
            while (token.hasMoreTokens()) {
                sourcePath.add(token.nextToken());
            }
        } catch (EVolveException e) {
            Scene.showErrorMessage(e.getMessage());
            return;
        } catch (IOException e) {
            Scene.showErrorMessage("Error occurred when access file "+iniFilename+".");
            return;
        }

        Scene.getUIManager().setLastConfigDir(textVizConfigPath.getText());
        Scene.getUIManager().setLastDataDir(textDefaultDataPath.getText());
        Scene.getUIManager().setLastResultDir(textVizResultPath.getText());
    }

    public void showSettingDialog() {
        dialog = new JDialog(Scene.getFrame(), "Set path...", true);
        dialog.setSize(650,230);

        JPanel panelName = new JPanel(new GridLayout(tags.length, 1, 5, 15));
        for (int i=0; i<tags.length; i++) {
            panelName.add(new JLabel(tags[i].substring(1,tags[i].length()-1) + ":"));
        }

        JPanel panelSetting = new JPanel(new GridLayout(contents.length, 1, 5, 12));
        for (int i=0; i<contents.length; i++) {
            contents[i].setColumns(40);
            panelSetting.add(contents[i]);
        }

        JPanel panelDirs = new JPanel(new GridLayout(buttonDir.length, 1, 5, 5));
        for (int i=0; i<buttonDir.length; i++) {
            panelDirs.add(buttonDir[i]);
        }

        Box boxSaveCancel = new Box(BoxLayout.X_AXIS);
        buttonSave = new JButton("Save");
        buttonSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save();
                dialog.setVisible(false);
            }
        });

        buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
            }
        });
        boxSaveCancel.add(Box.createHorizontalStrut(40));
        boxSaveCancel.add(buttonSave);
        boxSaveCancel.add(Box.createHorizontalStrut(25));
        boxSaveCancel.add(buttonCancel);


        JPanel panelMain = new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel panelAll = new JPanel(new BorderLayout());
        Box panelBottom = new Box(0);

        panelMain.add(panelName);
        panelMain.add(panelSetting);
        panelMain.add(panelDirs);
        panelBottom.add(Box.createHorizontalStrut(180));
        panelBottom.add(boxSaveCancel, BorderLayout.CENTER);
        panelAll.add(panelMain, BorderLayout.CENTER);
        panelAll.add(panelBottom, BorderLayout.SOUTH);
        dialog.getContentPane().add(panelAll, BorderLayout.CENTER);
        Scene.getUIManager().showDialog(dialog, dialog.getWidth(), dialog.getHeight());
    }

    private Box createPathItem(String prompt, JTextField item, JButton button) {
        Box itemBox = new Box(BoxLayout.X_AXIS);

        itemBox.add(new JLabel(prompt));
        itemBox.add(Box.createHorizontalStrut(15));
        itemBox.add(item);
        itemBox.add(Box.createHorizontalStrut(10));
        itemBox.add(button);
        return itemBox;
    }

    private void showCurrentSetting() {
        Box boxMain = new Box(BoxLayout.Y_AXIS);
        String setting;
        int index;

        settingDialog = new JDialog(Scene.getFrame(), "Current Setting", true);
        settingDialog.setSize(500,500);

        // show data source and its path
        setting = Scene.getCurrentDataFilename();
        index = setting.lastIndexOf(File.separatorChar);
        if (index == -1) { // have not loaded date yet
            boxMain.add(createInfoItem("Data source name:", "N/A"));
            boxMain.add(Box.createVerticalStrut(20));
            boxMain.add(createInfoItem("Data source path:", "N/A"));
        } else {
            boxMain.add(createInfoItem("Data source name:", setting.substring(index+1)));
            boxMain.add(Box.createVerticalStrut(20));
            boxMain.add(createInfoItem("Data source path:", setting.substring(0,index)));
        }
        boxMain.add(Box.createVerticalStrut(20));

        // show path info
        for (int i=0; i<tags.length; i++) {
            String prompt = tags[i].substring(1,tags[i].length()-1) + ":";
            boxMain.add(createInfoItem(prompt, contents[i].getText()));
            boxMain.add(Box.createVerticalStrut(20));
        }

        // show current visualization info
        HashMap config = readVizSetting();
        if (config != null) {
            boxMain.add(createInfoItem("Visualization: ", (String)config.get("Name")));
            boxMain.add(createInfoItem("Subject: ",((ElementDefinition)config.get("Subject")).getName()));
            EVolve.visualization.Dimension[] dims = (EVolve.visualization.Dimension[])config.get("Dimension");
            for (int i=0; i<dims.length; i++)
                boxMain.add(createInfoItem("Dimension "+(char)(88+i)+" ",dims[i].getName()));
            boxMain.add(createInfoItem("Interval",config.get("Interval").toString()));
        } else {
            boxMain.add(createInfoItem("Visualization: ", "N/A"));
            boxMain.add(createInfoItem("Subject: ","N/A"));
            for (int i=0; i<3; i++)
                boxMain.add(createInfoItem("Dimension "+(char)(88+i)+" ","N/A"));
            boxMain.add(createInfoItem("Interval","N/A"));
        }


        // create button box
        JButton buttonOK = new JButton("OK");
        buttonOK.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                settingDialog.setVisible(false);
            }
        });
        JPanel panelButton = new JPanel(new BorderLayout());
        panelButton.add(buttonOK,BorderLayout.CENTER);
        boxMain.add(panelButton);

        settingDialog.getContentPane().add(boxMain);
        Scene.getUIManager().showDialog(settingDialog, settingDialog.getWidth(), settingDialog.getHeight());
    }

    private Box createInfoItem(String prompt, String info) {
        Box item = new Box(BoxLayout.X_AXIS);
        item.add(new JLabel(prompt));
        item.add(Box.createHorizontalStrut(15));
        item.add(new JTextField(info));
        return item;
    }

    private void addPath() {
        int activeButtonId = 0;
        for (int i=0; i<buttonDir.length; i++) {
            if (buttonDir[i].isFocusOwner()) {
                activeButtonId = i;
                break;
            }
        }

        String path = contents[activeButtonId].getText();

        if (activeButtonId > 2) {
            path = path.substring(path.lastIndexOf(File.pathSeparatorChar)+1);
        }

        JFileChooser fc = new JFileChooser(path);
        String newPath;

        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        if(fc.showOpenDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
            newPath = fc.getSelectedFile().getAbsolutePath();
            switch (activeButtonId) {
                case 3:
                case 4:
                    String oldPath = contents[activeButtonId].getText();
                    if (oldPath.length() != 0)
                        newPath = oldPath + File.pathSeparator + newPath;
                case 0:
                case 1:
                case 2:
                    contents[activeButtonId].setText(newPath);
                    break;
            }
        }

    }

    public ArrayList getAdditionalClassPath() {
        return additionClassPath;
    }

    public ArrayList getSourcePath() {
        return sourcePath;
    }

    public String getConfigurationPath() {
        return textVizConfigPath.getText();
    }

    private HashMap readVizSetting() {
        /*ArrayList list = Scene.getVisualizationManager().getVisualizationList();

        Visualization visual = null;
        for (int i=0; i<list.size();i++) {
            visual = (Visualization)list.get(i);
            if (visual.getWindow().isSelected()) break;
        }

        if (visual != null)
            return visual.getCurrentConfiguration();
        else
            return null;*/
        return null;
   }

    public void save() {
        EVolveSettingData data = new EVolveSettingData();
        XMLWriter writer = new XMLWriter();
        writer.initialWriter(iniFilename,false);

        data.DefaultDataPath = textDefaultDataPath.getText();
        data.DefaultVizResultPath = textVizResultPath.getText();
        data.DefaultVizConfigurationPath = textVizConfigPath.getText();
        data.AdditionalClassPath = textClassPath.getText();
        data.SourceFilePath = textSourcePath.getText();
        XMLWriteOrder order = new XMLWriteOrder();
        String key = data.getClass().getName();
        key = key.substring(key.lastIndexOf(".")+1);
        writer.writeObject(data,order.getOrder(key));
        writer.finalizeWriter();
        Scene.getUIManager().setLastConfigDir(textVizConfigPath.getText());
        Scene.getUIManager().setLastDataDir(textDefaultDataPath.getText());
        Scene.getUIManager().setLastResultDir(textVizResultPath.getText());
    }

}

