/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Jan 15, 2003
 * Time: 1:19:37 PM
 * Some codes are attributed to ClassPathExplorer.java in Bruno Dufour's AdaptC toolkit.
 * please refer to http://www.sable.mcgill.ca/~bdufou1/ for more details
 */

package EVolve.util.sourcebrowser;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import EVolve.Scene;
import org.apache.bcel.classfile.*;
import EVolve.util.settings.SceneSetting;

public class ClassExplorer {
    private HashMap mapClassToSourcename;
    private ArrayList classPath,sourcePath;
    private static ClassExplorer instance = null;

    public ClassExplorer() {
        mapClassToSourcename = new HashMap();
        classPath = new ArrayList();
        sourcePath = new ArrayList();
    }

    public static ClassExplorer v() {
        if (instance == null) instance = new ClassExplorer();
        return instance;
    }

    public String getSourceFileFromClass(String className) {
        prepareClassPath();
        prepareSourcePath();

        if (!mapClassToSourcename.containsKey(className)) {

            JavaClass javaClass = loadClass(className);

            if (javaClass != null) {
                String sourceName = javaClass.getSourceFileName();
                boolean fileFound = false;

                for (int i=0; i<sourcePath.size(); i++) {
                    String path = (String)sourcePath.get(i)+File.separatorChar+sourceName;
                    File f = new File(path);
                    if (f.exists()) {
                        mapClassToSourcename.put(className, path);
                        fileFound = true;
                        break;
                    }
                }

                if (!fileFound) {
                    Scene.showErrorMessage("Can not find source file \""+sourceName+"\". \n");
                }
            }
        }

        return (String)mapClassToSourcename.get(className);
    }

    private void prepareClassPath() {
        String path = System.getProperty("java.class.path");
        StringTokenizer token = new StringTokenizer(path,System.getProperty("path.separator"));

        classPath.clear();
        while (token.hasMoreTokens()) {
            classPath.add(token.nextToken());
        }
        SceneSetting.v().readDataFromFile();
        ArrayList additional = SceneSetting.v().getAdditionalClassPath();
        for (int i=0; i<additional.size(); i++) {
            classPath.add(additional.get(i));
        }
    }

    private void prepareSourcePath() {
        String path = System.getProperty("java.library.path");
        StringTokenizer token = new StringTokenizer(path,System.getProperty("path.separator"));

        sourcePath.clear();
        while (token.hasMoreTokens()) {
            sourcePath.add(token.nextToken());
        }
        ArrayList additional = SceneSetting.v().getSourcePath();
        for (int i=0; i<additional.size(); i++) {
            sourcePath.add(additional.get(i));
        }
    }

    private JavaClass loadClass(String className) {
        JavaClass returnVal =  null;

        for (int i=0; i<classPath.size(); i++) {
            String path = (String)classPath.get(i);
            File f = new File(path);

            if (!f.exists()) continue;

            if (isArchive(f)) {
                returnVal = processArchive(className,f);
            } else if (isClassFile(f)) {
                returnVal = processClassFile(className, f);
            } else if (f.isDirectory()) {
                returnVal = processDirectory(className, f);
            }

            if (returnVal != null) break;
        }

        return returnVal;
    }

    private boolean isArchive(File f) {
        if (f.isFile() && f.canRead()) {
            String path;
            try {
                path = f.getCanonicalPath();
            } catch(IOException e) {
                return false;
            }

            if(path.endsWith("zip") || path.endsWith("jar")) {
                return true;
            }
        }
        return false;
    }

    private boolean isClassFile(File file) {
        try {
            if (!file.isFile()) {
                return false;
            }
            DataInputStream stream = new DataInputStream(new FileInputStream(file));

            String path = file.getCanonicalPath();

            if (path.endsWith(".class") && checkMagic(stream)) {
                return true;
            }

            return false;
        } catch (IOException e) {
            return false;
        }
    }

    private boolean checkMagic(DataInputStream classFileStream) {
        try {
            return ((classFileStream.readInt() & 0xFFFFFFFFL) == 0xCAFEBABEL);
        } catch (IOException e) {
            return false;
        }
    }

    private JavaClass processArchive(String className, File archive) {
        ZipFile zip;

        try {
            zip = new ZipFile(archive);
        } catch (ZipException e) {
            return null;
        } catch (IOException e) {
            return null;
        }

        Enumeration enum = zip.entries();
        while (enum.hasMoreElements()) {
            ZipEntry entry = (ZipEntry) enum.nextElement();
            String entryName = entry.getName(), path;

            /* make sure that the class name is valid */
            if (entryName != null && entryName.endsWith(".class")) {
                String tempClassName = entryName.substring(0, entryName.length() - 6).replace('/', '.');
                try {
                    path = archive.getCanonicalPath();
                    if (tempClassName.equals(className)) {
                        return new ClassParser(path, entryName).parse();
                    }
                } catch (IOException e) {
                    return null;
                }
            }
        }

        return null;
    }

    private JavaClass processClassFile(String className, File classFile) {
        JavaClass javaClass = getClass(classFile);

        if (javaClass == null) return null;
        /* This method is only invoked when the file exists and is a regular file,
           so these conditions are not checked again */
        String tempClassName = javaClass.getClassName();
        if (tempClassName.indexOf('.') >= 0) {
            /* This class file is part of a package, which does not match
               the location of the candidate file */
            return null;
        }
        if (tempClassName.equals(className)) {
            return javaClass;
        }
        return null;
    }

    private JavaClass processDirectory(String className, File dir) {
        JavaClass javaClass;

        /* If the class we are looking for is located in this directory,
           then theoreticalPath must represent its location */
        String theoreticalPath = dir.getPath() + File.separator + className.replace('.', File.separatorChar) + ".class";
        File theoreticalFile = new File(theoreticalPath);


        if (theoreticalFile.exists() && theoreticalFile.isFile()) {
            javaClass = getClass(theoreticalFile);

            if (javaClass == null) return null;
            /* This could be a match */
            String tempClassName = javaClass.getClassName();
            if (tempClassName != null && tempClassName.equals(className)) {
                /* This is a match */
                return javaClass;
            }
        }

        return null;
    }

    private JavaClass getClass(File classFile) {
        try {
            DataInputStream stream = new DataInputStream(new FileInputStream(classFile));

            ClassParser classParser = new ClassParser(stream, classFile.getName());
            return classParser.parse();
        } catch (FileNotFoundException e) {
            return null;
        } catch (IOException e) {
            return null;
        }
    }
}
