/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Jan 15, 2003
 * Time: 1:10:46 PM
 */

package EVolve.util.sourcebrowser;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import EVolve.Scene;

public class SourceBrowser implements Cloneable{
    private JInternalFrame window;
    private JDesktopPane desktop;
    private String target;
    private ArrayList textBuffer;
    private int line_no = 0;
    private int caretPosition = 0;
    private JPopupMenu popup;
    private JMenuItem itemFind, itemFindNext, itemFindPrevious;
    private JTextArea textArea;
    private String loadedFilename;

    public SourceBrowser() {
        window = null;
        desktop = Scene.getUIManager().getDesktop();
        textBuffer = new ArrayList();
        popup = new JPopupMenu();
        loadedFilename = null;

        itemFind = new JMenuItem("Find...");
        itemFind.setMnemonic(KeyEvent.VK_F);
        itemFind.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                target = (String)JOptionPane.showInputDialog(Scene.getFrame(), "Input string to be find:", "Find...", JOptionPane.QUESTION_MESSAGE, null, null, target);
                caretPosition = 0;
                find(target, 0, textBuffer.size(),false);
            }
        });
        popup.add(itemFind);

        itemFindNext = new JMenuItem("Find Next");
        itemFindNext.setMnemonic(KeyEvent.VK_N);
        itemFindNext.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                findNext(target);
            }
        });
        popup.add(itemFindNext);

        itemFindPrevious = new JMenuItem("Find Previous");
        itemFindPrevious.setMnemonic(KeyEvent.VK_P);
        itemFindPrevious.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                findPrevious(target);
            }
        });
        popup.add(itemFindPrevious);
    }

    public void showSourceFile(String entityName) {
        if (entityName == null) return;

        String className = extractClassName(entityName);
        String fileName = ClassExplorer.v().getSourceFileFromClass(className);

        if (fileName == null) {
            return;
        }

        if (window == null) {
            window = new JInternalFrame(entityName,true, true, true, true);
            window.addInternalFrameListener(new InternalFrameAdapter() {
                public void internalFrameClosing(InternalFrameEvent e) {
                    window = null;
                }
            });
            window.setClosable(true);
            window.setBounds(desktop.getSize().width - 600, 0, 600, 300);
            window.setResizable(true);
            desktop.add(window);
            textArea = new JTextArea();
        }
        window.moveToFront();
        window.setVisible(true);
        if (fileName.equals(loadedFilename)) {
            int position = find(target,0,textBuffer.size(),true);
            textArea.setCaretPosition(position);
        } else {
            window.getContentPane().removeAll();
            window.getContentPane().add(loadSourceFile(fileName));
        }
        window.setTitle(entityName);
    }

    private JScrollPane loadSourceFile(String fileName) {
        textArea = new JTextArea();
        line_no = 0;

        try {
            RandomAccessFile file = new RandomAccessFile(fileName,"r");
            textBuffer.clear();

            String line = file.readLine();
            while (line!=null) {
                textBuffer.add(line + "\n");
                textArea.append(line + "\n");
                line = file.readLine();
            }
            caretPosition  = 0;
            int position = find(target,0,textBuffer.size(),true);
            if (position!=-1) textArea.setCaretPosition(position);
            addPopupTrigger(textArea);
        } catch (IOException e) {
            Scene.showErrorMessage("Error occurred when accessing source file:\n" +
                                   "\""+fileName+"\".");
        }


        return new JScrollPane(textArea);
    }

    private String extractClassName(String entityName) {
        int index = entityName.lastIndexOf('(');
        target = "";

        if (index == -1) {
            /* no '(' found, this is not a method/location entity */
            return entityName;
        }
        String className = entityName.substring(0,index);
        index = className.lastIndexOf('.');

        if (index != -1) {
            className = entityName.substring(0,index);
            target = entityName.substring(index+1,entityName.indexOf('('));
        }

        return className;
    }

    private boolean isEntity(String line) {
        if ((line!=null)&&(line.indexOf('(')!=-1))
            //&&
            //((line.indexOf("public") != -1) ||
            //(line.indexOf("private") != -1) ||
            //(line.indexOf("protected") != -1))) {
        {
            return true;
        }

        return false;
    }

    private int find(String target, int start_line_no, int end_line_no, boolean target_is_entity) {
        String line;
        boolean found = false;

        for (int i=start_line_no; i<end_line_no; i++) {
            line = (String)textBuffer.get(i);
            if ((line.indexOf(target) != -1) && ((!target_is_entity) || isEntity(line))) {
                found = true;
                line_no = i;
                textArea.setCaretPosition(caretPosition);
                break;
            }
            caretPosition += line.length();
        }

        if (!found) {
            Scene.showErrorMessage("String \""+target+"\" is not found.");
        }

        return found ? caretPosition : -1;
    }

    private int findNext(String target) {
        caretPosition = 0;

        for (int i=0; i<=line_no; i++) {
            caretPosition += ((String)textBuffer.get(i)).length();
        }

        return find(target, line_no+1, textBuffer.size(), false);
    }

    private int findPrevious(String target) {
        String line;
        boolean found = false;

        caretPosition = 0;

        for (int i=0; i<line_no-1; i++) {
            caretPosition += ((String)textBuffer.get(i)).length();
        }

        for (int i=line_no - 1; i>=0; i--) {
            line = (String)textBuffer.get(i);
            if (line.indexOf(target) != -1) {
                found = true;
                textArea.setCaretPosition(caretPosition);
                line_no = i;
                break;
            }
            caretPosition -= line.length();
        }

        if (!found) {
            Scene.showErrorMessage("String \""+target+"\" is not found.");
        }

        return found ? caretPosition : -1;
    }

    protected void addPopupTrigger(Component component) {
        component.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }
        });
    }

    private void showPopup(MouseEvent e) {
        int mouseX = e.getX();
        int mouseY = e.getY();
        Rectangle rect = e.getComponent().getBounds();
        Rectangle rect2 = popup.getBounds();
        int posX= mouseX, posY = mouseY;
        if (mouseY+rect2.height > rect.height)
            posY = posY - rect2.height;
        if (mouseX+rect2.width>rect.width)
            posX = posX - rect2.width;
            popup.show(e.getComponent(), posX, posY);
    }

    public Object clone() {
        SourceBrowser o = null;
        try {
            o = (SourceBrowser)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.target = target;
        o.textBuffer = (ArrayList)textBuffer.clone();
        o.loadedFilename = loadedFilename;
        return o;
    }
}
