package EVolve.util.xmlutils;

import org.xml.sax.helpers.*;
import org.xml.sax.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.Field;
import EVolve.Scene;
import EVolve.exceptions.EVolveException;

/**
 * User: Wei Wang
 * Date: Jun 4, 2003
 * Time: 1:10:56 PM
 */
public class XMLLoader extends DefaultHandler{
    private XMLReader xmlReader;
    private HashSet tags;
    private String xmlFName, rootTag, currentTag, lastTag="";
    private Object[] storage;
    private boolean valueWanted,beginParse;
    private int numberOfValueRead;

    public XMLLoader () {
        tags = new HashSet();
    }

    public void initialReader(String xmlFName, Object rootData) throws EVolveException{
        tags.clear();
        Field[] fields = rootData.getClass().getFields();
        for (int i=0; i<fields.length; i++)
            tags.add(fields[i].getName());

        this.xmlFName = xmlFName;
        rootTag = null;
        storage = null;
        numberOfValueRead = 0;

        try {
            xmlReader = XMLReaderFactory.createXMLReader();
            xmlReader.setContentHandler(this);
            xmlReader.setContentHandler(this);
        } catch (SAXException e) {
            Scene.showErrorMessage("Failed to initialize XML Reader, reason:"+e.getMessage()+".");
            throw new EVolveException(e.getMessage());
        }
    }

    public void read(Object[] data) throws IOException{
        storage = data;
        valueWanted = false;
        beginParse = rootTag==null;
        try {
            BufferedReader input = new BufferedReader(new FileReader(xmlFName));
            xmlReader.parse(new InputSource(input));
            input.close();
        } catch (SAXException e) {
            Scene.showErrorMessage("Error occurred during processing XML file.");
            return;
        }
    }

    public void read(String rootTag, Object[] data) throws IOException{
        this.rootTag = rootTag;
        read(data);
    }

    public void startElement(String uri, String name, String qName, Attributes atts) {
        if ((rootTag!=null)&&(rootTag.equals(qName)))
            beginParse = true;

        currentTag = qName;

        if (beginParse&&tags.contains(qName))
            valueWanted = true;
    }

    public void endElement(String uri, String name, String qName) {
        valueWanted = false;
        if (beginParse&&tags.contains(name)) {
            numberOfValueRead++;
        }
    }

    public void characters(char ch[], int start, int length) {
        if (!valueWanted) return;

        Field[] fields = storage[numberOfValueRead/tags.size()].getClass().getFields();
        for (int i=0; i<fields.length; i++) {
            if (fields[i].getName().equals(currentTag)) {
                try {
                    if (fields[i].getType().getName().indexOf("String") != -1) {
                        String value = new String(ch,start,length);
                        if (lastTag.equals(currentTag)) {// the data is too long to be sent in one characters event
                            String oldValue = (String)fields[i].get(storage[numberOfValueRead/tags.size()]);
                            value = oldValue+value;
                        }
                        fields[i].set(storage[numberOfValueRead/tags.size()],value);
                    }
                    lastTag = currentTag;
                    break;
                } catch (IllegalAccessException e) {
                    Scene.showErrorMessage("Internal error, please check XMLLoader.characters(...)");
                }
            }
        }
    }
}
