package EVolve.util.xmlutils;

import EVolve.Scene;
import EVolve.util.xmlutils.datastructures.XMLWriteOrder;

import java.io.*;
import java.util.*;
import java.lang.reflect.Field;

/**
 * User: Wei Wang
 * Date: Jun 4, 2003
 * Time: 1:15:03 PM
 */

public class XMLWriter {
    private final String header = "<?xml version=\"1.0\" standalone=\"yes\"?>";
    private FileWriter output;
    private String xmlFName;

    public XMLWriter() {

    }

    public void initialWriter(String xmlFName, boolean append) {
        this.xmlFName = xmlFName;

        try {
            output = new FileWriter(this.xmlFName, append);
            output.write(header+"\n");
        } catch (IOException e) {
            Scene.showErrorMessage("Failed to initialize XML writer, reason:"+e.getMessage()+".");
        }
    }

    public void writeObject(Object data, String order[]) {
        try {
            String wrapTag = data.getClass().getName();
            output.write(generateTag(wrapTag.substring(wrapTag.lastIndexOf('.')+1),false));
            output.write("\n");
            Field[] fields = data.getClass().getFields();
            for (int i=0; i<order.length; i++) {
                if (fields[i].getType().getName().indexOf("String") == -1) {
                    XMLWriteOrder orders = new XMLWriteOrder();
                    try {
                        writeObjects(fields[i].getName(),(ArrayList)fields[i].get(data),orders.getOrder(fields[i].getName()));
                    } catch (IllegalAccessException e) {
                        Scene.showErrorMessage("Internal error in XMLWriter.writeObject(...)");
                    }
                } else {
                    String buf = generateTag(order[i],false);
                    for (int j=0; j<fields.length; j++) {
                        if (fields[i].getName().equals(order[i])) {
                            try {
                                String value = (String)fields[i].get(data);
                                if ((value!=null)&&(value.length()!=0))
                                    buf+=value.toString();
                                break;
                            } catch (IllegalAccessException e) {}
                        }
                    }
                    output.write(buf+generateTag(order[i],true)+"\n");
                }
            }
            output.write(generateTag(wrapTag.substring(wrapTag.lastIndexOf('.')+1),true));
            output.write("\n");
        } catch (IOException e) {
            Scene.showErrorMessage("IO Error!");
        }
    }

    public void writeObjects(String category, ArrayList data, String order[]) {
        for (int i=0; i<data.size(); i++) {
            writeObject(data.get(i),order);
        }
    }

    public void writeTag(String tagName, boolean isEnd) {
        try {
            output.write(generateTag(tagName,isEnd)+"\n");
        } catch (IOException e) {}
    }

    public void finalizeWriter() {
        try {
            output.flush();
            output.close();
        } catch (IOException e) {}
    }

    public String generateTag(String tagName, boolean end) {
        return !end ? "<"+tagName+">" : "</"+tagName+">";
    }
}
