/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 27, 2003
 * Time: 1:52:09 PM
 */

package EVolve.visualization;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import EVolve.exceptions.NoDataPlotException;
import EVolve.util.painters.shapes.Shape;

public class AutoShapeImage extends AutoImage{
    private Shape defaultShape;

    public AutoShapeImage() {
        w = h = 0;
        wMax = hMax = 200;
        image = new short[wMax][hMax];
        for (int i = 0; i < wMax; i++) {
            for (int j = 0; j < hMax; j++) {
                image[i][j] = 0;
            }
        }
        defaultShape = null;
    }

    public AutoShapeImage(Shape defaultShape) {
        this();
        this.defaultShape = defaultShape;
    }

    private void drawShapes(Graphics2D g2, int x, int y) {
        if (defaultShape == null) {
            ArrayList value = (ArrayList)Int2Object.get(new Integer(image[x][y]));
            for (int k=0; k<value.size(); k++)
                ((Shape)value.get(k)).draw(g2);
        } else { //when there is active default shape, image array contains color
            int width = defaultShape.getWidth(), height = defaultShape.getHeight();
            defaultShape.setColor((Color)Int2Object.get(new Integer(image[x][y])));
            defaultShape.move(x*width,y*height);
            defaultShape.draw(g2);
        }
    }

    public void setColor(int x, int y, Object shape) {

        if ((x >= wMax) || (y >= hMax)) {
            while (x >= wMax) {
                wMax += 200;
            }
            while (y >= hMax) {
                hMax += 200;
            }

            short[][] newImage = new short[wMax][hMax];
            for (int i = 0; i < wMax; i++) {
                for (int j = 0; j < hMax; j++) {
                    newImage[i][j] = 0;
                }
            }
            for (int i = 0; i < w; i++) {
                for (int j = 0; j < h; j++) {
                    newImage[i][j] = image[i][j];
                }
            }
            image = newImage;
        }

        if (defaultShape == null) {
            Shape aShape = (Shape)shape;
            if (image[x][y] == 0) {
                short size = (short)object2Int.size();
                ArrayList value = new ArrayList();
                object2Int.put(value, new Integer(size));
                Int2Object.put(new Integer(size),value);
                image[x][y] = size;
            }
            ((ArrayList)Int2Object.get(new Integer(image[x][y]))).add(aShape);
        } else {
            if (object2Int.containsKey(shape)) {
                image[x][y] = ((Integer)object2Int.get(shape)).shortValue(); //in this case, shape contains a color object
            } else {
                short size = (short)object2Int.size();
                object2Int.put(shape,new Integer(size));
                Int2Object.put(new Integer(size),shape);
                image[x][y] = size;
            }
        }

        if (x >= w) {
            w = x + 1;
        }
        if (y >= h) {
            h = y + 1;
        }
    }

    public BufferedImage getImage() throws NoDataPlotException{
        BufferedImage returnVal = null;

        if (defaultShape != null) {
            imageWidth = defaultShape.getWidth() * w;
            imageHeight = defaultShape.getHeight() * h;
        }

        if (imageWidth*imageHeight == 0) throw new NoDataPlotException();

        returnVal = new BufferedImage(imageWidth, imageHeight,BufferedImage.TYPE_INT_RGB);
        returnVal.getGraphics().setColor(Color.white);
        returnVal.getGraphics().fillRect(0, 0, imageWidth, imageHeight);
        Graphics2D g2 = (Graphics2D)returnVal.getGraphics();

        for (int i = 0; i < w; i++) {
            for (int j = 0; j < h; j++) {
                if (image[i][j] != 0)
                    drawShapes(g2,i,j);
            }
        }

        return returnVal;
    }

    public Object getColor(int x, int y) {
        if (((image!=null) && ((x < 0) || (y < 0) || (x >= image.length) || (y >= image[0].length))) ||(image[x][y]==0)) {
            return null;
        } else {
            if (defaultShape == null)
                return Int2Object.get(new Integer(image[x][y]));
            else
                return (Color)Int2Object.get(new Integer(image[x][y]));
        }
    }

    public Object getSortedColor(ReferenceDimension xAxis, ReferenceDimension yAxis,int x,int y) {
        int xMappedId, yMappedId;
        xMappedId = (xAxis == null) ? x : xAxis.getOriginMappedId(x);
        yMappedId = (yAxis == null) ? y : yAxis.getOriginMappedId(y);

        if ((xMappedId >= wMax) || (yMappedId >= hMax) || (xMappedId < 0) || (yMappedId < 0))
            return null;
        else
            if (defaultShape == null)
                return Int2Object.get(new Integer(image[xMappedId][yMappedId]));
            else
                return (Color)Int2Object.get(new Integer(image[x][y]));
    }

    public Shape getEntityShapes(int x,int y) {
        for (int i=0; i<w; i++) {
            for (int j=0; j<h; j++) {
                ArrayList data = (ArrayList)Int2Object.get(new Integer(image[i][j]));
                if (data != null) {
                    for (int k=0; k<data.size(); k++) {
                        Shape aShape = (Shape)data.get(k);
                        if (aShape.insideShape(x,y)) return aShape;
                    }
                }
            }
        }
        return null;
    }

    public AutoImage getSortedImage(ReferenceDimension xAxis, ReferenceDimension yAxis) {
        AutoShapeImage returnVal = new AutoShapeImage(defaultShape);
        for (int i = 0; i < w; i++) {
            for (int j = 0; j < h; j++) {
                int x, y;
                if (xAxis == null) {
                    x = i;
                } else {
                    x = xAxis.getSortedIndex(i);
                }
                if (yAxis == null) {
                    y = j;
                } else {
                    y = yAxis.getSortedIndex(j);
                }
                if ((x != -1) && (y != -1) && (image[i][j]!=0)) {
                    if (defaultShape == null) {
                        ArrayList value = (ArrayList)Int2Object.get(new Integer(image[i][j]));
                        for (int k=0; k< value.size(); k++) {
                            Shape aShape = (Shape)(value.get(k));

                            aShape.move(x*(xAxis == null ? 1 : aShape.getWidth()),
                                        y*(yAxis == null ? 1 : aShape.getHeight()));

                            returnVal.setColor(x, y, aShape);
                        }
                    } else {
                        returnVal.setColor(x,y,Int2Object.get(new Integer(image[i][j])));
                    }
                }
            }
        }
        returnVal.setImageSize(imageWidth,imageHeight);
        return returnVal;
    }

    public void setImageSize(int width, int height) {
        imageWidth = width;
        imageHeight = height;
    }

    public void reArrange(int width, int boxSize) {
        int newWidth = width/boxSize;
        int newHeight = ((w*h)%newWidth == 0) ? w*h/newWidth : w*h/newWidth + 1;
        short newImage[][] = new short[newWidth][newHeight];
        for (int i=0; i<w*h; i++) {
            newImage[i%newWidth][i/newWidth] = image[i%w][i/w];
        }

        w = newWidth;
        h = newHeight;
        image = null;
        image = newImage;
        defaultShape.setSize(boxSize, boxSize);
    }

    public Shape getDefaultShape() {
        return defaultShape;
    }

    public Object clone() {
        AutoShapeImage o = new AutoShapeImage();

        o.image = new short[wMax][hMax];
        o.imageWidth = imageWidth;
        o.imageHeight = imageHeight;
        o.Int2Object = new HashMap();
        o.object2Int = new HashMap();
        o.w = w;
        o.h = h;
        o.wMax = wMax;
        o.hMax = hMax;

        if (defaultShape == null) {
            o.object2Int.put(null,new Integer(0));
            o.Int2Object.put(new Integer(0),null);
            for (int i=0; i<image.length; i++) {
                for (int j=0; j<image[i].length; j++) {
                    o.image[i][j] = image[i][j];
                    if (image[i][j] != 0) {
                        ArrayList value = (ArrayList)Int2Object.get(new Integer(image[i][j]));
                        ArrayList newValue = new ArrayList();
                        o.Int2Object.put(new Integer(image[i][j]),newValue);
                        o.object2Int.put(newValue, new Integer(image[i][j]));
                        for (int k=0; k<value.size(); k++) {
                            newValue.add(((Shape)value.get(k)).clone());
                        }
                    } else
                        o.image[i][j] = 0;
                }
            }
        } else {
            o.defaultShape = (Shape)defaultShape.clone();
            for (int i=0; i<image.length; i++) {
                for (int j=0; j<image[i].length; j++)
                    o.image[i][j] = image[i][j];
            }

            Iterator it = object2Int.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                o.object2Int.put(key, object2Int.get(key));
            }
            it = Int2Object.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                o.Int2Object.put(key, Int2Object.get(key));
            }
        }

        return o;
    }
}
