/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Mar 2, 2003
 * Time: 9:05:11 PM
 */

package EVolve.visualization;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import EVolve.exceptions.NoDataPlotException;

public class AxesPanel extends JPanel{
    public final static int Axis_Enabled = 0x00001;
    public final static int Scale_Image = 0x11000000;
    public final static int Flip_Image = 0x00100;
    public final static int Pointer_Enabled = 0x01000;
    public final static int Box_Enabled = 0x10000;
    public final static int Null = 0x0;
    public final static int Scale_X = 0x10000000;
    public final static int Scale_Y = 0x01000000;

    private int flag;
    private String xName=" ", yName=" ";
    private Point origin, imageAdjustment, pointerPosition;
    private Point boxStart, boxEnd;
    private Point2D.Double mousePos;
    private int zoomingWidth, zoomingHeight, shapeSize;
    private BufferedImage image;
    private JScrollPane parent = null;
    private ArrayList phases, stringList;

    private AutoImage autoImage;
    private double factorX, factorY;

    public AxesPanel(int flag) {
        this.flag = flag;
        origin = new Point(0,0);
        imageAdjustment = new Point(0,0);
        pointerPosition = new Point(0,0);
        boxStart = new Point(0,0);
        boxEnd = new Point(0,0);
        mousePos = new Point2D.Double(0,0);
        zoomingWidth = zoomingHeight = 20;
        shapeSize = 1;
        stringList = new ArrayList();
        factorX = factorY = 1;
        if ((flag & 0x000f) == Axis_Enabled) {
            origin.setLocation(20,20);
            imageAdjustment.x = 60;
            imageAdjustment.y = 60;
        }
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if (e.getButton() == MouseEvent.BUTTON1) {
                    mPressed(e.getX(), e.getY());
                }
            }
        });

        addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.getButton() == MouseEvent.BUTTON1) {
                    mReleased();
                }
            }
        });

        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                mDragged(e.getX(), e.getY());
            }
        });
    }

    private void drawAxis(Graphics2D g2) {

        if ((flag & 0x000f)==Axis_Enabled) {//axis is enabled
            int w = getWidth(), h = getHeight();
            g2.setColor(Color.black);
            g2.drawLine(origin.x, h - origin.y, w - origin.x, h - origin.x);
            g2.drawLine(w - origin.x, h - origin.y, w - 30, h - 15);
            g2.drawLine(w - origin.x, h - origin.y, w - 30, h - 25);
            g2.drawLine(origin.x, h - origin.y, origin.x, origin.y);
            g2.drawLine(origin.x, origin.y, 15, 30);
            g2.drawLine(origin.x, origin.y, 25, 30);


            Font font = new Font("Arial", Font.PLAIN, 12);
            g2.setFont(font);
            g2.drawString(xName, w - g2.getFontMetrics().stringWidth(xName) - 40, h - 5);
            g2.rotate(-Math.PI / 2.0);
            g2.drawString(yName, -40 - g2.getFontMetrics().stringWidth(yName), 15);
            g2.rotate(Math.PI / 2.0);
        }
    }

    private void drawImage(Graphics2D g2) {
        int h = getHeight(), flip = 1;
        AffineTransform tf = g2.getTransform();

        if ((flag&0x0f00)==Flip_Image) {
            flip = -1;
            g2.translate(origin.x+1, h - origin.y);
        }

        calcScaleFactors();

        g2.scale(factorX,(double)flip*factorY);
        g2.drawImage(image, 0, 0, null);
        g2.setTransform(tf);
    }

    private void drawBox(Graphics2D g2) {
        g2.setColor(Color.blue);
        int x1 = (boxStart.x < boxEnd.x) ? boxStart.x : boxEnd.x;
        int x2 = (boxStart.x > boxEnd.x) ? boxStart.x : boxEnd.x;
        int y1 = (boxStart.y < boxEnd.y) ? boxStart.y : boxEnd.y;
        int y2 = (boxStart.y > boxEnd.y) ? boxStart.y : boxEnd.y;
        g2.drawRect(x1, y1, x2 - x1, y2 - y1);
    }

    private void drawPhases(Graphics2D g2) {
        int w = getWidth(), h = getHeight();

        if ((phases == null)||(phases.size() == 0)) return;

        calcScaleFactors();

        g2.setColor(Color.black);

        int lastX = 0, currentX = 0;
        for (int i=0; i<phases.size(); i++) {
            int phaseEnd = ((Integer)phases.get(i)).intValue();

            if (phaseEnd <= 0) continue;

            currentX = (int)(phaseEnd*factorX)+origin.x;
            g2.drawString(String.valueOf(phaseEnd),currentX,10+(i%2)*10);
            g2.drawString("("+i+")", (currentX + lastX)/2-5, 30);
            g2.drawLine(currentX, h-origin.y, (int)(phaseEnd*factorX)+origin.x, origin.y);
            lastX = currentX;
        }
        g2.drawString("("+phases.size()+")", (currentX + w)/2-5, 30);

    }

    private void drawPointer(Graphics2D g2) {
        if ((flag&0xf000)==Pointer_Enabled) {
            int x = pointerPosition.x, y = pointerPosition.y;
            g2.setColor(Color.black);
            g2.drawLine(x,y,x+20,y+20);
            g2.drawLine(x,y,x+10,y+5);
            g2.drawLine(x,y,x+5,y+10);
        }
    }

    private void setPreferredPanelSize() {
        if (parent == null) return;
        int width = image.getWidth()+imageAdjustment.x;
        int height = image.getHeight()+imageAdjustment.y;

        if ((flag&0xf0000000) == Scale_X)
            width = parent.getWidth() - 14;
        if ((flag&0x0f000000) == Scale_Y)
            height = parent.getHeight()-14;

        setPreferredSize(new java.awt.Dimension(width, height));
        /*if ((flag&0xff000000) != Scale_Image) {
            setPreferredSize(new java.awt.Dimension(image.getWidth()+imageAdjustment.x,image.getHeight()+imageAdjustment.y));
        } else {
            setPreferredSize(new java.awt.Dimension(parent.getWidth()-14, parent.getHeight()-14));
        }*/
        revalidate();
    }

    private void drawStrings(Graphics2D g2) {
        for (int i=0; i<stringList.size(); i++) {
            Object[] values = (Object[])stringList.get(i);
            int x = ((Integer)values[0]).intValue() - origin.x;
            int y = ((Integer)values[1]).intValue();
            String content = (String)values[2];

            if ((flag & 0xf0000000) == Scale_X)
                x = (int) ((float)x*(float)(getWidth()-imageAdjustment.x)/(float)image.getWidth()+origin.x);
            else
                x = x + origin.x;

            if ((flag & 0x0f000000) == Scale_Y) {
                if ((flag & 0x00f00) == Flip_Image)
                    y = (int)((float)getHeight()-(float)origin.y-(float)y*(float)(getHeight()-imageAdjustment.y)/(float)image.getHeight());
                else
                    y = (int)((float)y*(float)(getHeight()-imageAdjustment.y)/(float)image.getHeight()+(float)origin.y);
            }
            else {
                if ((flag & 0x00f00) == Flip_Image)
                    y = getHeight()-origin.y-y;
                else
                    y = y + origin.y;
            }
            g2.drawString(content,x,y);
        }
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D)g;

        int w = getWidth();
        int h = getHeight();

        g2.setColor(Color.white);
        g2.fillRect(0, 0, w, h);

        drawAxis(g2);

        if (image != null) {
            setPreferredPanelSize();
            drawImage(g2);
            drawPhases(g2);
            drawBox(g2);
            drawStrings(g2);
        }
        drawPointer(g2);

    }

    protected void mPressed(int x, int y) {
        if (image != null) {
            boxStart.x = x;
            boxStart.y = y;
            boxEnd.x = x;
            boxEnd.y = y;
            flag = flag | Box_Enabled;
            repaint();
        }
    }

    protected void mReleased() {
        if (image != null) {
            flag = flag & 0xff00ffff;
        }
    }

    protected void mDragged(int x, int y) {
        if ((image != null)&&((flag &0xf0000) == Box_Enabled)) {
            boxEnd.x = x;
            boxEnd.y = y;
            repaint();
        }
    }

    public void setName(String xName, String yName) {
        this.xName = xName;
        this.yName = yName;
    }

    public void setImage(BufferedImage image) {
        this.image = image;

        boxStart.x = 0;
        boxStart.y = 0;
        boxEnd.x = 0;
        boxEnd.y = 0;
        flag = flag & 0xff00ffff;
    }

    public void setAutoImage(AutoImage autoImage) throws NoDataPlotException{
        this.autoImage = null;
        this.autoImage = autoImage;
        this.autoImage.setPhases(phases);
        image = autoImage.getImage();
    }

    public int getImageX(int x) {
        if (image == null) return -1;

        if ((flag & 0xf0000000) == Scale_X)
            return (int)((float)(x-origin.x)*((float)image.getWidth()/(float)(getWidth()-imageAdjustment.x)));
        else
            return x - origin.x;
    }

    public int getImageY(int y) {
        if (image == null) return -1;

        if ((flag & 0x0f000000) == Scale_Y) {
            if ((flag & 0x00f00) == Flip_Image)
                return (int)((float)(getHeight()-origin.y-y)*((float)image.getHeight()/(float)(getHeight()-imageAdjustment.y)));
            else
                return (int)((float)(y-origin.y)*((float)image.getHeight()/(float)(getHeight()-imageAdjustment.y)));
        }
        else {
            if ((flag & 0x00f00) == Flip_Image)
                return getHeight()-origin.y-y;
            else
                return y - origin.y;
        }
    }

    public int getStartX() {
        int x = (boxStart.x < boxEnd.x) ? boxStart.x : boxEnd.x;
        return getImageX(x);
    }

    public int getEndX() {
        int x = (boxStart.x > boxEnd.x) ? boxStart.x : boxEnd.x;
        return getImageX(x);
    }

    public int getStartY() {
        int y = (boxStart.y < boxEnd.y) ? boxStart.y : boxEnd.y;
        return getImageY(y);
    }

    public int getEndY() {
        int y = (boxStart.y > boxEnd.y) ? boxStart.y : boxEnd.y;
        return getImageY(y);
    }

    public void setPointerPosition(int x, int y) {
        pointerPosition.x = x;
        pointerPosition.y = y;
    }

    public void enablePointer(boolean draw) {
        if (draw)
            flag = flag | Pointer_Enabled;
        else
            flag = flag & (~Pointer_Enabled);
    }

    private Rectangle2D.Double calcZoomingArea(int x, int y) {
        int w = image.getWidth(), h = image.getHeight();

        double x1=x,y1=y,w1,h1;

        x1 = x1 - (double)zoomingWidth*shapeSize/2;
        y1 = y1 - (double)zoomingHeight*shapeSize/2;

        w1 = zoomingWidth*shapeSize;
        h1 = zoomingHeight*shapeSize;

        mousePos.setLocation(shapeSize*zoomingWidth/2,shapeSize*zoomingHeight/2);
        if (x1 < 0) {
            mousePos.x+=x1;
            w1 = w1+x1;
            x1 = 0;
        }
        if (x1 + w1 > w) {
            w1 = w - x1;
        }

        if (y1 < 0) {
            mousePos.y+=y1;
            h1 = h1+y1;
            y1 = 0;
        }
        if (y1 + h1 > h) {
            h1 = h - y1;
        }

        Rectangle2D.Double ret = new Rectangle2D.Double();
        ret.x = x1;
        ret.y = y1;
        ret.width = w1;
        ret.height = h1;
        return ret;
    }

    public BufferedImage getSubImage(int x, int y) {
        int w = image.getWidth(), h = image.getHeight();
        Rectangle2D.Double zooming = calcZoomingArea(x,y);

        if ((image == null)||(zooming.x>w)||(zooming.y>h))
            return null;

        if ((zooming.width<=0) || (zooming.height<=0)) return null;

        return image.getSubimage((int)zooming.x,(int)zooming.y,(int)zooming.width,(int)zooming.height);
    }

    public void drawZoomingArea(int x, int y) {
        double coX = 1, coY = 1;

        if ((flag & 0xf0000000) == Scale_X) {
            coX = (double)(getWidth() - imageAdjustment.x)/(double)image.getWidth();
        }
        if ((flag & 0x0f0000000) == Scale_Y) {
            coY = (double)(getHeight() - imageAdjustment.y)/(double)image.getHeight();
        }
        int deltaX = (int)(coX*zoomingWidth*shapeSize/2);
        int deltaY = (int)(coY*zoomingHeight*shapeSize/2);

        boxStart.x = x - deltaX;
        boxStart.y = y - deltaY;
        boxEnd.x = x + deltaX;
        boxEnd.y = y + deltaY;
        flag = flag | Box_Enabled;
        repaint();
        flag = flag & (~Box_Enabled);
    }

    public void setShapeSize(int shapeSize) {
        this.shapeSize = shapeSize;
    }

    public void setZoomingSize(int w, int h) {
        zoomingWidth = w;
        zoomingHeight = h;
    }

    public Point2D.Double getMouseMovement() {
        return mousePos;
    }

    public void scaleImage(boolean scaled) {
        if (scaled)
            flag = flag | Scale_Image;
        else
            flag = flag & (~Scale_Image);
    }

    public void setParent(JScrollPane parent) {
        this.parent = parent;
    }

    public void setPanelFlag(int flag) {
        this.flag = flag;
        if ((flag & 0x000f) != Axis_Enabled) {
            origin.setLocation(0,0);
            imageAdjustment.x = 0;
            imageAdjustment.y = 0;
        }
    }

    public Point getOrigin() {
        return origin;
    }

    public void setPhases(ArrayList phases) {
        this.phases = phases;
        if (autoImage == null) return;
        autoImage.setPhases(phases);
        try {
            image = autoImage.getImage();
        } catch (NoDataPlotException e) {
            // fall through
        }
    }

    public void addString(int x, int y, String content) {
        Object[] aString = new Object[3];

        aString[0] = new Integer(x);
        aString[1] = new Integer(y);
        aString[2] = content;
        stringList.add(aString);
    }

    public void reset() {
        stringList.clear();
    }

    public int getFlag() {
        return flag;
    }

    public Object clone() {
        AxesPanel o = new AxesPanel(flag);

        o.xName = xName;
        o.yName = yName;
        o.zoomingWidth = zoomingWidth;
        o.zoomingHeight = zoomingHeight;
        o.shapeSize = shapeSize;
        return o;
    }

    private void calcScaleFactors() {
        int w = getWidth(), h = getHeight();
        factorX = factorY = 1;
        if ((flag&0xf0000000) == Scale_X)
            factorX = (double)(w - imageAdjustment.x) / (double)(image.getWidth());
        if ((flag&0x0f000000) == Scale_Y)
            factorY = (double)(h - imageAdjustment.y) / (double)(image.getHeight());
    }
}
