/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.visualization;

import EVolve.data.DataFilter;
import EVolve.data.Element;

public abstract class Dimension implements Cloneable{
    private String name; // name of the dimension
    protected DataFilter dataFilter; // data filter that gets data for the dimension

    /**
     * Gets the name of the dimension.
     *
     * @return  name of the dimension
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the data filter.
     *
     * @param  dataFilter  data filter
     */
    public void setDataFilter(DataFilter dataFilter) {
        this.name = dataFilter.getName();
        this.dataFilter = dataFilter;
    }

    /**
     * Gets the data filter.
     *
     * @return  data filter
     */
    public DataFilter getDataFilter() {
        return dataFilter;
    }

    /**
     * Gets the data from the element.
     *
     * @return  id of the entity if the field is a reference, or the value if the field is a value
     */
    public abstract long getField(Element element);

    public abstract void preVisualize();

    public int getEntityNumber() {
        return 0;
    }

    public Object clone() {
        Dimension o = null;
        try {
            o = (Dimension)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        o.name = name;
        o.dataFilter = (dataFilter == null) ? null : (DataFilter) dataFilter.clone();
        return o;
    }
}