/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import javax.swing.*;
import EVolve.data.Element;
import EVolve.data.ValueComparator;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;

/**
 * Two-column table.
 */
public class TableViz extends Visualization {
    private ReferenceDimension leftColumn; // dimension, left column
    private ValueDimension rightColumn; // dimension, right column

    private JTable table; // the table
    private TableVizModel model; // table model

    private int[] value; // value of the right column

    private JCheckBox selectionOptions[], chkSelectOccurredEntities;

    /**
     * Creates a table.
     */
    public TableViz() {
        model = new TableVizModel();
        table = new JTable(model);

        JScrollPane scrollPane = new JScrollPane(table);
        scrollPane.setBackground(Color.white);
        scrollPane.getViewport().setBackground(Color.white);
        ((JPanel)panel).add(scrollPane, BorderLayout.CENTER);

        addPopupTrigger(table);
        addPopupTrigger(scrollPane.getViewport());
    }

    public Dimension[] createDimension() {
        leftColumn = new ReferenceDimension(true);
        rightColumn = new ValueDimension();

        Dimension[] returnVal = new Dimension[2];
        returnVal[0] = leftColumn;
        returnVal[1] = rightColumn;

        leftColumn.addSortSchemes("Value");

        return returnVal;
    }

    protected Component createPanel() {
        JPanel returnVal = new JPanel(new BorderLayout());
        returnVal.setBackground(Color.white);
        return returnVal;
    }

    protected JPanel createConfigurationPanel() {
        return null;
    }

    protected void updateConfiguration() {
        model = new TableVizModel();
        model.setName(leftColumn.getName(), rightColumn.getName());
        table.setModel(model);
    }

    public void preVisualize() {
        value = new int[leftColumn.getMaxEntityNumber()];
        for (int i = 0; i < value.length; i++) {
            value[i] = 0;
        }
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        value[(int)leftColumn.getField(element)] += rightColumn.getField(element);
    }

    public void visualize() {
        leftColumn.addComparator(new ValueComparator("Value", false, value, leftColumn.getEntityName2IntMap()));
        leftColumn.selectComparator(comboSortSchemes[0].getSelectedItem().toString());
        sort();
    }

    public void sort() {
        String[] sortedName = new String[leftColumn.getEntityNumber()];
        int[] sortedValue = new int[leftColumn.getEntityNumber()];

        for (int i = 0; i < leftColumn.getMaxEntityNumber(); i++) {
            int sortedIndex = leftColumn.getSortedIndex(i);
            if (sortedIndex != -1) {
                sortedName[sortedIndex] = leftColumn.getEntity(sortedIndex).getName();
                sortedValue[sortedIndex] = value[i];
            }
        }

        model = new TableVizModel();
        model.setName(leftColumn.getName(), rightColumn.getName());
        model.setValue(sortedName, sortedValue);
        table.setModel(model);
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;
        leftColumn.makeSelection(selectionName,subjectDefinition.getType(),table.getSelectedRows());
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();
        data.xAxis = leftColumn.getName();
        data.yAxis = rightColumn.getName();
        data.xAxisSortScheme = leftColumn.getSelectedComparatorName();

        return data;
    }

    public ReferenceDimension getLinkableDimension(int dim) {
        return null;
    }

    public AutoImage getImage() {
        return null;
    }

    public long getxMax() {
        return -1;
    }

    public void clearMagnifier() {
        return;
    }

    public void setImage(AutoImage newImage) {
        return;
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectOccurredEntities = new JCheckBox("Occurred Entities");
        chkSelectOccurredEntities.setSelected(true);
        chkSelectOccurredEntities.setEnabled(false);

        selectionOptions = new JCheckBox[1];
        selectionOptions[0] = chkSelectOccurredEntities;

        return selectionOptions;
    }

    public Object clone() {
        TableViz o = (TableViz)super.clone();

        o.leftColumn = (ReferenceDimension)leftColumn.clone();
        o.rightColumn = (ValueDimension)rightColumn.clone();
        o.dimension[0] = o.leftColumn;
        o.dimension[1] = o.rightColumn;
        o.panelConfiguration = o.createConfigurationPanel();
        o.panel = new JPanel(new BorderLayout());
        o.panel.setBackground(Color.white);
        o.createDialog();
        o.createMenu();

        if (value!=null) {
            o.value = new int[value.length];
            for (int i=0; i<value.length; i++)
                o.value[i] = value[i];
        }

        o.model = (TableVizModel)model.clone();
        o.table = new JTable(o.model);

        JScrollPane scrollPane = new JScrollPane(o.table);
        scrollPane.setBackground(Color.white);
        scrollPane.getViewport().setBackground(Color.white);
        ((JPanel)o.panel).add(scrollPane, BorderLayout.CENTER);

        o.addPopupTrigger(o.table);
        o.addPopupTrigger(scrollPane.getViewport());
        return o;
    }
}