/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.visualization;

import javax.swing.table.AbstractTableModel;

/**
 * Model of a two-column table.
 */
public class TableVizModel extends AbstractTableModel implements Cloneable{
    private String[] name; // names of the columns
    private String[] left; // left column
    private int[] right; // right column

    /**
     * Creates a table model.
     */
    public TableVizModel() {
        name = new String[2];
        name[0] = "Left Column";
        name[1] = "Right Column";
        left = new String[0];
        right = new int[0];
    }

    /**
     * Gets the number of columns.
     *
     * @return  number of columns
     */
    public int getColumnCount() {
        return name.length;
    }

    /**
     * Gets the name of a column
     *
     * @param   col the column
     * @return  name of the column
     */
    public String getColumnName(int col) {
        return name[col];
    }

    /**
     * Gets the value of the table.
     *
     * @param   row the row
     * @param   col the column
     * @return  the value
     */
    public Object getValueAt(int row, int col) {
        if (col == 0) {
            return left[row];
        } else {
            return new Integer(right[row]);
        }
    }

    /**
     * Gets the number of rows.
     *
     * @return  number of rows
     */
    public int getRowCount() {
        return left.length;
    }

    /**
     * Whether the cell is editable.
     *
     * @param   row the row
     * @param   col the column
     * @return  false, not editable
     */
    public boolean isCellEditable(int row, int col) {
        return false;
    }

    /**
     * Sets the names of the columns.
     *
     * @param   nameLeft name of the left column
     * @param   nameRight name of the right column
     */
    public void setName(String nameLeft, String nameRight) {
        name[0] = nameLeft;
        name[1] = nameRight;
    }

    /**
     * Sets the value of the table.
     *
     * @param   left the left column
     * @param   right the right column
     */
    public void setValue(String[] left, int[] right) {
        this.left = left;
        this.right = right;
    }

    public Object clone() {
        TableVizModel o = null;
        try {
            o = (TableVizModel)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }
        if (name != null) {
            o.name = new String[name.length];
            for (int i=0; i<name.length; i++)
                o.name[i] = name[i];
        }
        if (left != null) {
            o.left = new String[left.length];
            for (int i=0; i<left.length; i++)
                o.left[i] = left[i];
        }
        if (right != null) {
            o.right = new int[right.length];
            for (int i=0; i<right.length; i++)
                o.right[i] = right[i];
        }

        return o;
    }
}