/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 3:50:11 PM
 */

package EVolve.visualization;

import EVolve.data.*;
import EVolve.Scene;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.VizFactory.VisualizationFactory;
import javax.swing.*;
import javax.imageio.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import java.util.*;
import java.io.*;

public abstract class Visualization implements Cloneable{
    public static final int SELECT_TIME_FRAME = 0x0001;
    public static final int SELECT_OCCURRED_ENTITIES = 0x0010;
    public static final int SELECT_ALL_ENTITIES = 0x0100;
    public static final int SELECT_X_AXIS = 0x1000;
    public static final int SELECT_Y_AXIS = 0x0100;
    public static final int SELECT_ALL_X_AXIS = 0x0010;
    public static final int SELECT_ALL_Y_AXIS = 0x0001;
    protected String selectionName;

    protected String name; // name of the visualization
    protected EVolve.Window window; // window where the visualization is displayed

    protected VisualizationDefinition definition; // definition of the visualization
    protected ElementDefinition subjectDefinition; // definition of the subject

    protected Dimension[] dimension; // dimension of the visualization
    protected Component panel; // panel to draw the visualization

    protected JDialog dialog; // configuration dialog
    protected JPanel panelConfiguration; // configuration panel
    protected JComboBox comboSubject; // subject selection
    protected JComboBox[] comboDimension; // dimension selection
    protected JComboBox[] comboSortSchemes; // dimension sort schemes
    protected JTextField textName; // name input

    protected JPopupMenu popup; // popup menu
    private JMenu menuSort;
    private JMenu[] menuDimension;
    protected JMenuItem[][] itemSort;
    private JMenuItem itemConfigure,itemRename;
    private JMenuItem itemSelect,itemCurrentSelection,itemSave;
    private JMenuItem itemClone;
    private ReferenceDimension[] referenceDimension;
    private int linkageId;
    private VisualizationFactory factory = null;
    private static int vizIDPool = 0;
    protected int dataSourceId;

    protected ElementDefinition[] elementDefinition; // definition of elements that can be used as subject
    protected DataFilter[][][] dataFilter; // data filters that can be used by the dimensions
    protected int mouseX, mouseY;

    protected boolean freezed;
    private JDialog addSelection;
    private JTextField textSelectionName;
    private int numberOfSortableDimension;

    public Visualization() {
        dimension = createDimension();
        panel = createPanel();
        panelConfiguration = null;
        dialog = null;
        linkageId = vizIDPool++;
        addPopupTrigger(panel);
        freezed = false;
    }

    public String getName() {
        return name;
    }

    /**
     * Sets the name of the visualization.
     *
     * @param  name  name of the visualization
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Sets the window of the visualization.
     *
     * @param  window  window of the visualization
     */
    public void setWindow(EVolve.Window window) {
        this.window = window;
    }

    /**
     * Gets the panel of the visualization.
     *
     * @return  panel of the visualization
     */
    public Component getPanel() {
        return panel;
    }

    /**
     * Gets the additional configuration panel.
     *
     * @return  the additional configuration panel, null if not necessary
     */
    protected JPanel createConfigurationPanel() {
        return null; // by default, return null, each visualization can override this
    }

    protected void createMenu() {
        popup = new JPopupMenu();

        itemConfigure = new JMenuItem("Configure...");
        itemConfigure.setMnemonic(KeyEvent.VK_C);
        itemConfigure.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                configure();
            }
        });
        popup.add(itemConfigure);

        itemRename = new JMenuItem("Rename...");
        itemRename.setMnemonic(KeyEvent.VK_R);
        itemRename.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String newName = JOptionPane.showInputDialog(Scene.getFrame(),"Enter a new name:",name);
                if (newName != null) {
                    name = newName;
                    window.setTitle(name+" - " + subjectDefinition.getName());
                }
            }
        });
        popup.add(itemRename);

        itemSelect = new JMenuItem("Make Selection");
        itemSelect.setMnemonic(KeyEvent.VK_M);
        itemSelect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                makeSelection();
            }
        });
        popup.add(itemSelect);

        itemCurrentSelection = new JMenuItem("Current Selection...");
        itemCurrentSelection.setMnemonic(KeyEvent.VK_C);
        itemCurrentSelection.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getFilter().load();
            }
        });
        popup.add(itemCurrentSelection);

        menuSort = new JMenu("Sort");
        menuSort.setMnemonic(KeyEvent.VK_S);
        menuSort.setEnabled(false);
        popup.add(menuSort);

        ArrayList tempList = new ArrayList();
        for (int i = 0; i < dimension.length; i++) {
            if ((dimension[i] instanceof ReferenceDimension)&&((ReferenceDimension)dimension[i]).isSortable()) {
                tempList.add(new Integer(i));
            }
        }

        referenceDimension = new ReferenceDimension[tempList.size()];
        menuDimension = new JMenu[tempList.size()];
        itemSort = new JMenuItem[tempList.size()][];
        for (int i = 0; i < tempList.size(); i++) {
            int j = ((Integer)(tempList.get(i))).intValue();
            referenceDimension[i] = (ReferenceDimension)(dimension[j]);
            menuDimension[i] = new JMenu(definition.getDimensionDefinition()[j].getName());
            menuSort.add(menuDimension[i]);
        }

        itemClone = new JMenuItem("Clone");
        itemClone.setMnemonic(KeyEvent.VK_C);
        itemClone.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scene.getVisualizationManager().cloneVisualization();
            }
        });
        popup.add(itemClone);

        itemSave = new JMenuItem("Save Image...");
        itemSave.setMnemonic(KeyEvent.VK_V);
        itemSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        });
        popup.add(itemSave);
    }

    private void showPopup(MouseEvent e) {
        mouseX = e.getX();
        mouseY = e.getY();
        Rectangle rect = e.getComponent().getBounds();
        Rectangle rect2 = popup.getBounds();
        int posX = mouseX, posY = mouseY;
        if (mouseY+rect2.height > rect.height)
            posY = posY - rect2.height;
        if (mouseX+rect2.width>rect.width)
            posX = posX - rect2.width;
            popup.show(e.getComponent(), posX, posY);
    }

    /**
     * Sets the definition of the visualization, called by visualization manager after the visualization is created.
     *
     * @param  definition  definition of the visualization
     */
    public void setDefinition(VisualizationDefinition definition) {
        panelConfiguration = createConfigurationPanel();

        this.definition = definition;

        elementDefinition = Scene.getDataManager().getElementDefinition(definition);

        dataFilter = new DataFilter[elementDefinition.length][definition.getDimensionDefinition().length][];
        for (int i = 0; i < dataFilter.length; i++) {
            for (int j = 0; j < dataFilter[i].length; j++) {
                dataFilter[i][j] = Scene.getDataManager().getDataFilter(elementDefinition[i],definition.getDimensionDefinition()[j].getProperty());
                for (int k = 0; k < dataFilter[i][j].length; k++) {
                    assert (((dataFilter[i][j][k].getTargetType() != -1) && (dimension[j] instanceof ReferenceDimension)) || ((dataFilter[i][j][k].getTargetType() == -1) && (dimension[j] instanceof ValueDimension))) : "Incompatible dimension type.";
                }
            }
        }

        subjectDefinition = elementDefinition[0];
        for (int i = 0; i < dimension.length; i++) {
            dimension[i].setDataFilter(dataFilter[0][i][0]);
        }

        createDialog();
        updateConfiguration();

        createMenu();
    }

    /**
     * Gets the definition of the subject.
     *
     * @return  definition of the subject
     */
    public ElementDefinition getSubjectDefinition() {
        return subjectDefinition;
    }

    /**
     * Configures the visualization.
     */
    public void configure() {
        if (dialog == null) {
            createDialog();
        }
        dialog.pack();
        Scene.getUIManager().showDialog(dialog, dialog.getWidth(), dialog.getHeight());
    }

    /**
     * Save visualizations as disk file
     */
    public void save() {
        Component target = panel;
        if (target instanceof JScrollPane)
            target = ((JScrollPane)panel).getViewport().getView();

        BufferedImage image = new BufferedImage(target.getWidth(), target.getHeight(), BufferedImage.TYPE_INT_RGB);
        target.paint(image.getGraphics());
        ImageWriter writer = (ImageWriter)(ImageIO.getImageWritersByFormatName("png").next());

        JFileChooser fc = new JFileChooser(Scene.getUIManager().getLastResultDir());
        fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
            public String getDescription() {
                return "PNG Files";
            }

            public boolean accept(File f) {
                return f.getName().substring(f.getName().lastIndexOf('.') + 1).toLowerCase().equals("png");
            }
        });

        if (fc.showSaveDialog(Scene.getFrame()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            Scene.getUIManager().setLastResultDir(file.getPath());
            try {
                writer.setOutput(ImageIO.createImageOutputStream(file));
                writer.write(image);
            } catch (IOException e) {}
        }
    }

    /**
     * Creates the configuration dialog.
     */
    protected void createDialog() {
        dialog = new JDialog(Scene.getFrame(), "Configure", true);

        JPanel panelTitle = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelTitle, BorderLayout.NORTH);

        panelTitle.add(new JLabel("Title: "));

        textName = new JTextField(name, 12);
        panelTitle.add(textName);

        JPanel panelMain = new JPanel(new BorderLayout());
        panelMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Choose subject & dimensions"));
        dialog.getContentPane().add(panelMain, BorderLayout.CENTER);

        numberOfSortableDimension = 0;
        for (int i=0; i<dimension.length; i++) {
            if ((dimension[i] instanceof ReferenceDimension)&&((ReferenceDimension)dimension[i]).isSortable())
                numberOfSortableDimension++;
        }

        JPanel panelName = new JPanel(new GridLayout(definition.getDimensionDefinition().length + 1 + numberOfSortableDimension, 1, 5, 5));
        panelMain.add(panelName, BorderLayout.WEST);

        panelName.add(new JLabel(" Subject:      "));
        for (int i = 0; i < definition.getDimensionDefinition().length; i++) {
            panelName.add(new JLabel(" " + definition.getDimensionDefinition()[i].getName() + ":      "));
            if ((dimension[i] instanceof ReferenceDimension)&&(((ReferenceDimension)dimension[i]).isSortable()))
                panelName.add(new JLabel(" Sort "+ definition.getDimensionDefinition()[i].getName() + " By :  "));
        }

        JPanel panelCombo = new JPanel(new GridLayout(definition.getDimensionDefinition().length + 1 + numberOfSortableDimension, 1, 5, 5));
        panelMain.add(panelCombo, BorderLayout.CENTER);

        comboSubject = new JComboBox();
        for (int i = 0; i < elementDefinition.length; i++) {
            comboSubject.addItem(elementDefinition[i].getName());
        }
        comboSubject.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateComboSubject();
            }
        });
        panelCombo.add(comboSubject);

        comboDimension = new JComboBox[definition.getDimensionDefinition().length];
        int k = 0;
        comboSortSchemes = new JComboBox[numberOfSortableDimension];
        for (int i = 0; i < comboDimension.length; i++) {
            comboDimension[i] = new JComboBox();
            comboDimension[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    updateComboDimension();
                }
            });
            panelCombo.add(comboDimension[i]);
            if ((dimension[i] instanceof ReferenceDimension)&&(k<numberOfSortableDimension)) {
                comboSortSchemes[k] = new JComboBox();
                panelCombo.add(comboSortSchemes[k]);
                k++;
            }
        }

        updateComboSubject();

        if (panelConfiguration != null) {
            panelMain.add(panelConfiguration, BorderLayout.SOUTH);
        }

        JPanel panelButton = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelButton, BorderLayout.SOUTH);

        JButton buttonApply = new JButton("Apply");
        buttonApply.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogApply();
            }
        });
        panelButton.add(buttonApply);

        JButton buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogCancel();
            }
        });
        panelButton.add(buttonCancel);

        //*************
        for (int i=0; i<comboSubject.getItemCount(); i++) {
            if ((comboSubject.getItemAt(i)).equals(subjectDefinition.getName())) {
                comboSubject.setSelectedIndex(i);
                break;
            }
        }
        for (int i=0;i<comboDimension.length;i++) {
            for (int j =0 ; j<comboDimension[i].getItemCount();j++) {
                if (comboDimension[i].getItemAt(j).equals(dimension[i].getDataFilter().getName())) {
                    comboDimension[i].setSelectedIndex(j);
                    break;
                }
            }
        }
    }

    /**
     * Updates the subject combo-box.
     */
    protected void updateComboSubject() {
        comboSubject.setToolTipText(elementDefinition[comboSubject.getSelectedIndex()].getDescription());
        for (int i = 0; i < comboDimension.length; i++) {
            comboDimension[i].removeAllItems();
            for (int j = 0; j < dataFilter[comboSubject.getSelectedIndex()][i].length; j++) {
                comboDimension[i].addItem(dataFilter[comboSubject.getSelectedIndex()][i][j].getName());
            }
        }
        updateComboDimension();

    }

    /**
     * Updates the dimension combo-boxes.
     */
    protected void updateComboDimension() {
        for (int i = 0; i < comboDimension.length; i++) {
            if (comboDimension[i].getSelectedIndex() != -1) {
                comboDimension[i].setToolTipText(dataFilter[comboSubject.getSelectedIndex()][i][comboDimension[i].getSelectedIndex()].getDescription());
                dimension[i].setDataFilter(dataFilter[comboSubject.getSelectedIndex()][i][comboDimension[i].getSelectedIndex()]);
            }
        }

        if (numberOfSortableDimension == 0)  return;

        int k=0;
        for (int i=0; i<dimension.length; i++) {
            if ((dimension[i] instanceof ReferenceDimension)&&(k<numberOfSortableDimension)) {
                ArrayList schemes = ((ReferenceDimension)dimension[i]).getAvailableSortSchemes();
                String selected ="";
                if (comboSortSchemes[k].getSelectedIndex()!= -1)
                    selected = comboSortSchemes[k].getSelectedItem().toString();
                int selIndex = 0;
                comboSortSchemes[k].removeAllItems();
                for (int n=0; n<schemes.size(); n++) {
                    comboSortSchemes[k].addItem(schemes.get(n));
                    if (schemes.get(n).equals(selected)) {
                        selIndex = n;
                    }
                }
                comboSortSchemes[k].setSelectedIndex(selIndex);
                k++;
            }
        }
    }

    /**
     * Button "Apply" is clicked.
     */
    protected void dialogApply() {
        dialog.setVisible(false);
        subjectDefinition = elementDefinition[comboSubject.getSelectedIndex()];
        name = textName.getText();
        if (name.indexOf(subjectDefinition.getName()) == -1)
            window.setTitle(name + " - " + subjectDefinition.getName());
        else
            window.setTitle(name);
        reset();
        updateConfiguration();
    }

    /**
     * Button "Cancel" is clicked.
     */
    protected void dialogCancel() {
        dialog.setVisible(false);
    }

    public void updateMenu() {
        if (referenceDimension.length == 0) {
            menuSort.setVisible(false);
            return;
        }
        for (int i = 0; i < referenceDimension.length; i++) {
            ArrayList comparatorList = referenceDimension[i].getComparator();
            itemSort[i] = new JMenuItem[comparatorList.size()];
            menuDimension[i].removeAll();
            for (int j = 0; j < itemSort[i].length; j++) {
                itemSort[i][j] = new JMenuItem(((EntityComparator)(comparatorList.get(j))).getName());
                itemSort[i][j].addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        selectComparator(e);
                    }
                });
                menuDimension[i].add(itemSort[i][j]);
            }
        }

        ArrayList menuOptional = createOptionalMenu();
        if (menuOptional == null ) return;
        for (int i=0; i<menuOptional.size(); i++)
            popup.add((JMenuItem)menuOptional.get(i));
    }

    protected ArrayList createOptionalMenu() {
        return null;
    }

    public void selectComparator(ActionEvent e) {
        for (int i = 0; i < itemSort.length; i++) {
            for (int j = 0; j < itemSort[i].length; j++) {
                if (itemSort[i][j] == e.getSource()) {
                    referenceDimension[i].selectComparator(j);
                    sort();
                    i = itemSort.length+1;
                    break;
                }
            }
        }

        //synchronize sort combox here:
        int k=0;
        for (int i=0; i<dimension.length; i++) {
            if ((dimension[i] instanceof ReferenceDimension)&&(k<numberOfSortableDimension)) {
                for (int j=0; j<comboSortSchemes[k].getItemCount(); j++) {
                    if (comboSortSchemes[k].getItemAt(j).equals(((ReferenceDimension)dimension[i]).getSelectedComparatorName())) {
                        comboSortSchemes[k].setSelectedIndex(j);
                        break;
                    }
                }
                k++;
            }
        }
    }

    protected void addPopupTrigger(Component component) {
        Component target = component;
        if (target instanceof JScrollPane)
            target = ((JScrollPane)target).getViewport().getView();

        target.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    showPopup(e);
                }
            }
        });
    }

    public void enableSortMenu() {
        menuSort.setEnabled(true);
    }

    /**
     * Creates the dimensions.
     *
     * @return  the dimensions
     */
    public abstract Dimension[] createDimension();

    /**
     * Creates available selection menu items.
     *
     * @return  the array of selection option checkboxes
     */
    public abstract JCheckBox[] createSelectionOptions();

    /**
     * Creates the panel.
     *
     * @return  the panel
     */
    protected abstract Component createPanel();

    /**
     * Updates the configuration.
     */
    protected abstract void updateConfiguration();

    /**
     * Gets ready for receiving data.
     */
    public abstract void preVisualize();

    /**
     * Receives data.
     *
     * @param  data  the data
     */
    protected abstract void receiveElement(Element element);

    /**
     * Visualizes after receiving data.
     */
    public abstract void visualize();

    /**
     * sorting the visualization with selected comparator
     */
    public abstract void sort();

    /**
     * according to user selectiong, get a data subset
     */
    public abstract void makeSelection();

    /**
     *
     * @return a data structure contains all current visualization's configure
     */
    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = new SerializedVisualization();
        Rectangle rect = window.getBounds();

        data.FactoryName = getFactory().getName();
        data.SubjectName = subjectDefinition.getName();
        data.VisualizationName = name;
        data.WindowTitle = window.getTitle();
        data.WindowX = String.valueOf(rect.x);
        data.WindowY = String.valueOf(rect.y);
        data.WindowWidth = String.valueOf(rect.width);
        data.WindowHeight = String.valueOf(rect.height);
        data.Interval = "-1";

        return data;
    }

    /**
     * following are all interfaces needed for predefined viz
     */
    public void restoreConfiguration(SerializedVisualization config) {
        int selectedSubjectId = -1;

        for (int i=0; i< elementDefinition.length; i++) {
            if (elementDefinition[i].getName().equals(config.SubjectName)) {
                subjectDefinition = elementDefinition[i];
                comboSubject.setSelectedIndex(i);
                selectedSubjectId = i;
                break;
            }
        }

        String dimDefinition[] = new String[3];
        dimDefinition[0] = config.xAxis;
        dimDefinition[1] = config.yAxis;
        dimDefinition[2] = config.zAxis;
        for (int i=0; i<dimension.length; i++) {
            for (int j=0; j< dataFilter[selectedSubjectId][i].length;j++) {
                if (dataFilter[selectedSubjectId][i][j].getName().equals(dimDefinition[i])) {
                    dimension[i].setDataFilter(dataFilter[selectedSubjectId][i][j]);
                    break;
                }
            }
        }

        for (int i=0; i<comboDimension.length; i++) {
            for (int j=0; j<comboDimension[i].getItemCount(); j++) {
                if (comboDimension[i].getItemAt(j).equals(dimDefinition[i])) {
                    comboDimension[i].setSelectedIndex(j);
                }
            }
        }
        updateComboDimension();

        String sortScheme[] = new String[3];
        sortScheme[0] = config.xAxisSortScheme;
        sortScheme[1] = config.yAxisSortScheme;
        sortScheme[2] = config.zAxisSortScheme;
        int k = 0;
        for (int i=0; i<dimension.length; i++) {
            if ((sortScheme[i] != null)&&(sortScheme[i].length() > 0)&&(k<numberOfSortableDimension)) {
                for (int j=0; j<comboSortSchemes[k].getItemCount(); j++) {
                    if (sortScheme[i].equals(comboSortSchemes[k].getItemAt(j).toString())) {
                        comboSortSchemes[k].setSelectedIndex(j);

                    }
                }
                k++;
            }
        }


        window.setTitle(config.WindowTitle);
        updateConfiguration();
        menuSort.setEnabled(false);
    }

    public EVolve.Window getWindow() {
        return window;
    }

    public void setFactory(VisualizationFactory factory) {
        this.factory = factory;
    }

    public VisualizationFactory getFactory() {
        return factory;
    }

    public void autoSave(String path, String dataFn) {
        Component target = panel;
        if (target instanceof JScrollPane)
            target = ((JScrollPane)panel).getViewport().getView();

        BufferedImage image = new BufferedImage(target.getWidth(), target.getHeight(), BufferedImage.TYPE_INT_RGB);
        target.paint(image.getGraphics());
        ImageWriter writer = (ImageWriter)(ImageIO.getImageWritersByFormatName("png").next());

        try {
            RandomAccessFile file = new RandomAccessFile((path+File.separator+
                    dataFn.substring(dataFn.lastIndexOf(File.separator)+1,dataFn.length()-4)+".png"),"rw");
            writer.setOutput(ImageIO.createImageOutputStream(file));
            writer.write(image);
        } catch (IOException e) {
            System.out.println("writing file error.");
        }

    }

    /**
     * following methods are used in unify/overlap viz
     */
    public abstract ReferenceDimension getLinkableDimension(int dim);

    public int getVisualizationID() {
        return linkageId;
    }

    public Dimension[] getDimension() {
        return dimension;
    }

    public VisualizationDefinition getDefinition() {
        return definition;
    }

    public long getxMax() {
        return -1;
    }

    public AutoImage getImage() {
        return null;
    }

    public void setImage(AutoImage image) {
        return;
    }

    public void cleanup() {
        dimension = null;
        comboDimension = null;
        menuDimension = null;
        itemSort = null;
        referenceDimension = null;
        elementDefinition = null;
        dataFilter = null;
    }

    public int switchOption(boolean turn_on, int option, int add_on_option) {
        int returnVal;
        if (turn_on)
            returnVal = option | add_on_option;
        else
            returnVal = option & ~add_on_option;
        return returnVal;
    }

    public void setDataSourceId(int dataSourceId) {
        this.dataSourceId = dataSourceId;
    }

    public int getDataSourceId() {
        return dataSourceId;
    }

    protected void reset() {
        for (int i = 0; i < dimension.length; i++) {
            int selected = comboDimension[i].getSelectedIndex();
            if (selected < dataFilter[comboSubject.getSelectedIndex()][i].length)
                dimension[i].setDataFilter(dataFilter[comboSubject.getSelectedIndex()][i][selected]);
            if (dimension[i] instanceof ReferenceDimension) ((ReferenceDimension)dimension[i]).clearEntityMap();
        }
        menuSort.setEnabled(false);
    }

    public boolean isFreezed() {
        return freezed;
    }

    protected void preMakeSelection() {
        selectionName = null;
        addSelection = new JDialog(Scene.getUIManager().getFrame(),"Add Selection",true);

        JCheckBox[] options = createSelectionOptions();

        JPanel panelMain = new JPanel(new BorderLayout());
        addSelection.getContentPane().add(panelMain, BorderLayout.CENTER);

        JPanel panelOptions = new JPanel(new GridLayout(options.length, 1, 1, 1));
        JPanel panelName = new JPanel(new GridLayout(2,1,1,1));
        JPanel panelButton = new JPanel(new FlowLayout());

        for (int i=0; i<options.length; i++)
            panelOptions.add(options[i]);
        panelMain.add(panelOptions,BorderLayout.NORTH);

        textSelectionName = new JTextField("Selection"+Selection.total);
        panelName.add(new JLabel("Input selection name:"));
        panelName.add(textSelectionName);
        panelMain.add(panelName,BorderLayout.CENTER);

        JButton buttonOk = new JButton("Ok");
        buttonOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectionName = textSelectionName.getText();
                if (selectionName.length()==0) {
                    Scene.showErrorMessage("You must name this selection.");
                    return;
                }
                addSelection.setVisible(false);
                addSelection.dispose();
            }
        });
        panelButton.add(buttonOk);

        JButton buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	selectionName = null;
                addSelection.setVisible(false);
                addSelection.dispose();
            }
        });
        panelButton.add(buttonCancel);
        panelMain.add(panelButton,BorderLayout.SOUTH);

        addSelection.pack();
        Scene.getUIManager().showDialog(addSelection, addSelection.getWidth(), addSelection.getHeight());
    }

    public int getInterval() {
        return -1;
    }

    public Object clone() {
       Visualization o = null;

       try {
           o = (Visualization)super.clone();
       } catch (CloneNotSupportedException e) {
           e.printStackTrace();
           return null;
       }

       o.name = name + "- cloned";
       o.definition = (VisualizationDefinition)definition.clone();
       o.subjectDefinition = (ElementDefinition) subjectDefinition.clone();
       o.dimension = new Dimension[dimension.length];
       o.factory = (VisualizationFactory)factory.clone();
       o.elementDefinition = new ElementDefinition[elementDefinition.length];
       for (int i=0; i<elementDefinition.length; i++)
           o.elementDefinition[i] = (ElementDefinition)elementDefinition[i].clone();
       o.dataFilter = new DataFilter[dataFilter.length][][];
       o.linkageId = vizIDPool++;
       for (int i=0; i<dataFilter.length; i++) {
           o.dataFilter[i] = new DataFilter[dataFilter[i].length][];
           for (int j=0; j<dataFilter[i].length; j++) {
               o.dataFilter[i][j] = new DataFilter[dataFilter[i][j].length];
               for (int k=0; k<dataFilter[i][j].length; k++)
                   o.dataFilter[i][j][k] = (DataFilter)dataFilter[i][j][k].clone();
           }
       }
       return o;
   }
}
