/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.visualization.VizFactory;

import java.util.ArrayList;

import EVolve.visualization.DimensionDefinition;
import EVolve.visualization.PredictorFactory;
import EVolve.visualization.Visualization;
import EVolve.visualization.VisualizationDefinition;
import EVolve.visualization.XYViz.ValRefViz.HotSpotViz.PredictionViz;

/**
 * Factory of prediction visualization.
 */
public class PredictionVizFactory extends VisualizationFactory {
    protected ArrayList factoryList; // predictor factories

    /**
     * Creates a prediction visualization factory.
     */
    public PredictionVizFactory() {
        factoryList = new ArrayList();
    }

    /**
     * Adds a predictor factory.
     *
     * @param   factory the predictor factory
     */
    public void addPredictorFactory(PredictorFactory factory) {
        factoryList.add(factory);
    }

    public String getName() {
        return "Prediction Visualization";
    }

    public String getFactoryName() {
        return "PredictionVizFactory";
    }

    public VisualizationDefinition createDefinition() {
        DimensionDefinition[] dimensionDefinition = new DimensionDefinition[3];
        dimensionDefinition[0] = new DimensionDefinition("X-axis", "time");
        dimensionDefinition[1] = new DimensionDefinition("Y-axis", "reference");
        dimensionDefinition[2] = new DimensionDefinition("Prediction", "reference");
        return new VisualizationDefinition(dimensionDefinition);
    }

    public Visualization createVisualization() {
        PredictorFactory[] factory = new PredictorFactory[factoryList.size()];
        for (int i = 0; i < factory.length; i++) {
            factory[i] = (PredictorFactory)(factoryList.get(i));
        }
        return new PredictionViz(factory);
    }
}