/* EVolve - an Extensible Software Visualization Framework
 * Copyright (C) 2001-2002 Qin Wang
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * EVolve is distributed at http://www.sable.mcgill.ca/EVolve/
 */

package EVolve.visualization.VizFactory;

import EVolve.visualization.Visualization;
import EVolve.visualization.VisualizationDefinition;

/**
 * Factory of visualizations.
 */
public abstract class VisualizationFactory implements Cloneable {
    private VisualizationDefinition definition; // definition of the visualizations

    /**
     * Creates a visualization factory.
     */
    public VisualizationFactory() {
        definition = createDefinition();
    }

    /**
     * Gets the definition of the visualizations.
     *
     * @return  definition of the visualizations
     */
    public VisualizationDefinition getDefinition() {
        return definition;
    }

    /**
     * Gets the name of the visualizations.
     *
     * @return  name of the visualizations
     */
    public abstract String getName();

    /**
     * Gets the name of the visualization factory.
     *
     * @return  name of the visualization factory
     */
    public abstract String getFactoryName();

    /**
     * Creates the definition of the visualizations.
     *
     * @return  definition of the visualizations
     */
    public abstract VisualizationDefinition createDefinition();

    /**
     * Creates a visualization.
     *
     * @return  the visualization
     */
    public abstract Visualization createVisualization();

    public Object clone() {
        VisualizationFactory o = null;
        try {
            o = (VisualizationFactory)super.clone();
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
            return null;
        }

        o.definition = (VisualizationDefinition)definition.clone();
        return o;
    }
}