/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 28, 2002
 * Time: 11:50:25 PM
 */

package EVolve.visualization.XYViz.RefRefViz;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBox;

import EVolve.data.Element;
import EVolve.util.painters.CorrelationPainter;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoImage;
import EVolve.visualization.Dimension;
import EVolve.visualization.ValueDimension;

public class CorrelationViz extends ReferenceReferenceVisualization{
    private int valueMax;
    private JCheckBox chkSelectX, chkSelectY, chkSelectAllX, chkSelectAllY;
    private static JCheckBox selectionOptions[] = null;

    public Dimension[] createDimension() {
        Dimension[] returnVal = new Dimension[3];
        Dimension[] temp = super.createDimension();

        zAxis = new ValueDimension();

        returnVal[0] = temp[0];
        returnVal[1] = temp[1];
        returnVal[2] = zAxis;

        return returnVal;
    }

    public void preVisualize() {
        super.preVisualize();
        valueMax = 0;
        image = new AutoImage();
        value = new int[xAxis.getMaxEntityNumber()][yAxis.getMaxEntityNumber()];
        for (int i = 0; i < value.length; i++) {
            for (int j = 0; j < value[i].length; j++) {
                value[i][j] = 0;
            }
        }
        installPainter();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        value[(int)xAxis.getField(element)][(int)yAxis.getField(element)] += zAxis.getField(element);

        if (value[(int)xAxis.getField(element)][(int)yAxis.getField(element)] > valueMax) {
            valueMax = value[(int)xAxis.getField(element)][(int)yAxis.getField(element)];
        }
    }

    public void visualize() {
        painter.paint(image,-1,-1,valueMax);
        xAxis.selectComparator(comboSortSchemes[0].getSelectedItem().toString());
        yAxis.selectComparator(comboSortSchemes[1].getSelectedItem().toString());
        sort();
    }

    public SerializedVisualization getCurrentConfiguration() {

        SerializedVisualization data = super.getCurrentConfiguration();
        data.xAxis = xAxis.getName();
        data.xAxisSortScheme = comboSortSchemes[0].getSelectedItem().toString();
        //data.xAxisSortScheme = xAxis.getSelectedComparatorName();
        data.yAxis = yAxis.getName();
        data.yAxisSortScheme = comboSortSchemes[1].getSelectedItem().toString();
        //data.yAxisSortScheme = yAxis.getSelectedComparatorName();
        data.zAxis = zAxis.getName();
        return data;
    }

    protected void updateConfiguration() {
        info[0] = xAxis.getName() +": ";
        info[1] = "    " + yAxis.getName() +": ";
        info[2] = " Correlation: ";
        
        super.updateConfiguration();
    }

    public long getxMax() {
        return xAxis.getEntityNumber();
    }

    protected void changeOrientation() {
        int newValue[][] = new int[value[0].length][value.length];
        for (int i = 0; i < newValue.length; i++) {
            for (int j = 0; j < newValue[i].length ; j++) {
                newValue[i][j] = value[newValue[i].length - j - 1][newValue.length - i - 1];
            }
        }
        value = newValue;
        super.changeOrientation();
    }

    protected void installPainter() {
        painter = new CorrelationPainter(value);
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectX = new JCheckBox("Occurred Entities on X Axis");
        chkSelectX.setMnemonic(KeyEvent.VK_X);
        chkSelectX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectX.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_X_AXIS);

                if (chkSelectAllX.isSelected() && selected) {
                    chkSelectAllX.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION,SELECT_ALL_X_AXIS);
                }
            }
        });
        chkSelectX.setSelected(false);

        chkSelectAllX = new JCheckBox("All Entities on X Axis");
        chkSelectAllX.setMnemonic(KeyEvent.VK_A);
        chkSelectAllX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectAllX.isSelected();
                SELECT_OPTION = switchOption(selected,SELECT_OPTION,SELECT_ALL_X_AXIS);

                if (chkSelectX.isSelected() && selected) {
                    chkSelectX.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION,SELECT_X_AXIS);
                }
            }
        });
        chkSelectAllX.setSelected(true);

        chkSelectY = new JCheckBox("Occurred Entities on Y Axis");
        chkSelectY.setMnemonic(KeyEvent.VK_Y);
        chkSelectY.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectY.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_Y_AXIS);

                if (chkSelectAllY.isSelected() && selected) {
                    chkSelectAllY.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION, SELECT_ALL_Y_AXIS);
                }
            }
        });
        chkSelectY.setSelected(false);

        chkSelectAllY = new JCheckBox("All Entities on Y Axis");
        chkSelectAllY.setMnemonic(KeyEvent.VK_E);
        chkSelectAllY.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectAllY.isSelected();
                SELECT_OPTION = switchOption(selected,SELECT_OPTION,SELECT_ALL_Y_AXIS);

                if (chkSelectY.isSelected() && selected) {
                    chkSelectY.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION,SELECT_Y_AXIS);
                }
            }
        });
        chkSelectAllY.setSelected(true);

        selectionOptions = new JCheckBox[4];
        selectionOptions[0] = chkSelectX;
        selectionOptions[1] = chkSelectAllX;
        selectionOptions[2] = chkSelectY;
        selectionOptions[3] = chkSelectAllY;

        return selectionOptions;
    }


}
