/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 28, 2002
 * Time: 12:45:22 AM
 */

package EVolve.visualization.XYViz.RefRefViz;

import java.util.HashMap;

import EVolve.Scene;
import EVolve.exceptions.NoDataPlotException;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;
import EVolve.visualization.XYViz.XYVisualization;

public abstract class ReferenceReferenceVisualization extends XYVisualization{
    protected ReferenceDimension xAxis, yAxis;
    protected ValueDimension zAxis; // used as correlation
    protected int[][] value; // value of the points
    protected String[] info;
    protected static int SELECT_OPTION;


    public Dimension[] createDimension() {
        value = null;
        xAxis = new ReferenceDimension(true);
        yAxis = new ReferenceDimension(true);
        zAxis = null;
        info = new String[3];

        Dimension[] returnVal = new Dimension[2];
        returnVal[0] = xAxis;
        returnVal[1] = yAxis;

        SELECT_OPTION = 0x0011;
        return returnVal;
    }

    protected void updateConfiguration() {
        canvas.setName(xAxis.getName(), yAxis.getName());
        super.updateConfiguration();
    }

    public void preVisualize() {
        timeMap.clear();
    }

    public ReferenceDimension getLinkableDimension(int dim) {
        if (dim > 1)
            return null;
        return (ReferenceDimension)dimension[dim];
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        if (SELECT_OPTION == 0) {
            Scene.showErrorMessage("Nothing will be selected, please check Selection menu.");
            return;
        }

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();
        int y1 = canvas.getEndY();
        int y2 = canvas.getStartY();

        if (((x1<0)&&(x2<0)) || ((x1>=xAxis.getEntityNumber()))&&(x2>=xAxis.getEntityNumber()) ||
            ((y1<0)&&(y2<0)) || ((y1>=yAxis.getEntityNumber())&&(y2>=yAxis.getEntityNumber())))
           return;

        if (x1 < 0) {
            x1 = 0;
        }

        if (x2 > (xAxis.getEntityNumber() - 1)) {
            x2 = xAxis.getEntityNumber() - 1;
        }

        if (y1 < 0) {
            y1 = 0;
        }

        if (y2 > (yAxis.getEntityNumber() - 1)) {
            y2 = yAxis.getEntityNumber() - 1;
        }

        int[] selection = null;
        switch (SELECT_OPTION & 0xf0f0) {
            case 0x1000: // select occurred entities on x axis
                HashMap idList = new HashMap();
                for (int i=x1; i<=x2; i++) {
                    for (int j=y1; j<=y2; j++) {
                        if ((image.getSortedColor(xAxis,yAxis,i,j) != null)&&(!idList.containsValue(new Integer(i))))
                            idList.put(new Integer(idList.size()),new Integer(i));
                    }
                }
                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++)
                    selection[i] = ((Integer)idList.get(new Integer(i))).intValue();
                xAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0010: // select all entities on x axis
                selection = new int[x2-x1+1];
                for (int i=x1; i<=x2; i++)
                    selection[i-x1] = i;
                xAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0000: // x axis is not seletcted
                /*x1 = 0;
                x2 = xAxis.getEntityNumber() - 1;
                selection = new int[x2-x1+1];
                for (int i=0; i<=x2; i++)
                    selection[i] = i;*/
                break;
        }

        switch (SELECT_OPTION & 0x0f0f) {
            case 0x0100: //select occurred entities
                HashMap idList = new HashMap();
                for (int i=x1; i<=x2; i++) {
                    for (int j=y1; j<=y2; j++) {
                        if ((image.getSortedColor(xAxis,yAxis,i,j) != null)&&(!idList.containsValue(new Integer(i))))
                            idList.put(new Integer(idList.size()),new Integer(i));
                    }
                }
                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++)
                    selection[i] = ((Integer)idList.get(new Integer(i))).intValue();
                yAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0001: // select all entities
                selection = new int[y2-y1+1];
                for (int i=y1; i<=y2; i++)
                    selection[i-y1] = i;
                yAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0000:
                /*y1 = 0;
                y2 = yAxis.getEntityNumber() - 1;
                selection = new int[y2-y1+1];
                for (int i=0; i<=y2; i++)
                    selection[i] = i;*/
                break;
        }

    }

    /**
     * Mouse moved.
     *
     * @param   x position on X-axis
     * @param   y position on Y-axis
     */
    protected String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);

        if ((X >= 0) && (X < xAxis.getEntityNumber()) && (Y >= 0) && (Y < yAxis.getEntityNumber())) {

            if (shift_pressed && (image.getSortedColor(xAxis,yAxis,X,Y)==null))
                Scene.setStatus("  ");
            else {
                if (value == null)
                    Scene.setStatus(info[0] + xAxis.getEntity(X).getName() + info[1] +
                                     yAxis.getEntity(Y).getName() + info[2]);
                else {
                    HashMap mapX = xAxis.getEntityName2IntMap();
                    HashMap mapY = yAxis.getEntityName2IntMap();
                    String xName = xAxis.getEntity(X).getName(), yName = yAxis.getEntity(Y).getName();
                    Scene.setStatus(info[0] + xName + info[1] + yName +
                                    info[2] + value[((Integer)mapX.get(xName)).intValue()][((Integer)mapY.get(yName)).intValue()]);
                }
            }
        } else {
            Scene.setStatus(" ");
        }

        return null;
    }

    public void sort() {
        try {
            int before1 = xAxis.getEntityNumberBeforeLink();
            int before2 = xAxis.getEntityNumberBeforeLink();
            canvas.setName(xAxis.getName() + " (" + ((before1 == -1) ? "" : (""+before1+" / "))
                           + xAxis.getEntityNumber() +
                           ", " + xAxis.getSelectedComparatorName() + ")",
                           yAxis.getName() + " (" + ((before2 == -1) ? "" : (""+before2+" / "))
                           + yAxis.getEntityNumber() +
                           ", " + yAxis.getSelectedComparatorName() + ")");
            canvas.setImage(image.getSortedImage(xAxis, yAxis).getImage());
            canvas.repaint();
            enableBrowseSourceMenu();
        } catch (NoDataPlotException e) {
            Scene.showErrorMessage(e.getMessage());
        }
    }

    protected void changeOrientation() {
        super.changeOrientation();
        ReferenceDimension temp = xAxis;
        xAxis = yAxis;
        yAxis = temp;
        String tempInfo = info[0];
        info[0] = info[1];
        info[1] = tempInfo;
        sort();
    }

    public int[][] getValue() {
        return value;
    }

    public void setValue(int[][] newValue) {
        value = newValue;
    }

    protected String getEntityUnderMouse() {
        return null;
    }

    public Object clone() {
        ReferenceReferenceVisualization o = null;

        o = (ReferenceReferenceVisualization)super.clone();
        o.xAxis = (ReferenceDimension)xAxis.clone();
        o.yAxis = (ReferenceDimension)yAxis.clone();
        o.zAxis = (ValueDimension)zAxis.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.yAxis;
        o.dimension[2] = o.zAxis;
        o.createMenu();
        if (value != null) {
            o.value = new int[value.length][];
            for (int i=0; i<value.length; i++) {
                o.value[i] = new int[value[i].length];
                for (int j=0; j<value[i].length; j++)
                    o.value[i][j] = value[i][j];
            }
        }
        if (info != null) {
            o.info = new String[info.length];
            for (int i=0; i<info.length; i++)
                o.info[i] = info[i];
        }
        o.panelConfiguration = o.createConfigurationPanel();
        o.createDialog();
        return o;
    }
}
