/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 14, 2003
 * Time: 7:44:18 PM
 */

package EVolve.visualization.XYViz.RefRefViz;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Entity;
import EVolve.exceptions.NoDataPlotException;
import EVolve.util.painters.RelationshipPainter;
import EVolve.util.painters.placements.*;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoImage;
import EVolve.visualization.AutoShapeImage;
import EVolve.visualization.AxesPanel;

public class RelationshipViz extends CorrelationViz{
    protected static int SELECT_OPTION = SELECT_X_AXIS|SELECT_Y_AXIS;
    private static JMenuItem selectionMenu[] = null;
    private JTextField textThreshold;
    private int threshold=10;
    private ArrayList menuList;
    private JMenuItem itemReArrange;
    private JMenu menuChangePlacement;
    private JMenuItem itemSelectX, itemSelectY;
    private JComboBox comboPlacement;
    private Placement selectedPlacement;
    private Placement[] placementList;

    public RelationshipViz() {
        super();
        menuList = new ArrayList();
    }

    private void createPlacements() {
        placementList = new Placement[2];
        placementList[0] = new RectPlacement(10);
        placementList[1] = new RoundPlacement(10);
        selectedPlacement = placementList[0];
    }

    protected Component createPanel() {
        Component returnVal = super.createPanel();
        canvas.setPanelFlag(AxesPanel.Null);
        canvas.setZoomingSize(50,50);
        flipMagnifier = false;
        addMagnifier(canvas,flipMagnifier);

        return returnVal;
    }

    protected JPanel createConfigurationPanel() {
        JPanel panelBottom = new JPanel(new GridLayout(2,2));

        panelBottom.add(new JLabel("Threshold: "));

        textThreshold = new JTextField(String.valueOf(threshold), 10);
        panelBottom.add(textThreshold);

        createPlacements();
        panelBottom.add(new JLabel("Placement: "));
        comboPlacement = new JComboBox();
        for (int i = 0; i < placementList.length; i++) {
            comboPlacement.addItem(placementList[i].getName());
        }
        panelBottom.add(comboPlacement);

        JPanel configurationPanel = new JPanel(new FlowLayout());
        configurationPanel.add(panelBottom);

        return configurationPanel;
    }

    protected void updateConfiguration() {
        try {
            threshold = Integer.parseInt(textThreshold.getText());
            selectedPlacement = placementList[comboPlacement.getSelectedIndex()];
            super.updateConfiguration();

        } catch (Exception e) {
            Scene.showErrorMessage("Threshold must be an integer");
            configure();
        }
    }

    public void restoreConfiguration(SerializedVisualization config) {
        super.restoreConfiguration(config);
        threshold = Integer.parseInt(config.Threshold);
        textThreshold.setText(String.valueOf(threshold));
        autoInterval = -1;
        interval = -1;

        for (int i=0; i<placementList.length; i++) {
            if (placementList[i].getName().equals(config.PlacementName)) {
                selectedPlacement = placementList[i];
                comboPlacement.setSelectedIndex(i);
                break;
            }
        }
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.Threshold = String.valueOf(threshold);
        data.PlacementName = selectedPlacement.getName();

        return data;
    }

    protected void installPainter() {
        painter = new RelationshipPainter();
    }

    public void preVisualize() {
        super.preVisualize();
        image = new AutoShapeImage();
    }

    public void visualize() {
        ((RelationshipPainter)painter).updatePainterParameters(value,threshold);
        painter.paint(image,xAxis.getDataFilter().getTargetType(),yAxis.getDataFilter().getTargetType(),10);
        xAxis.selectComparator(comboSortSchemes[0].getSelectedItem().toString());
        yAxis.selectComparator(comboSortSchemes[1].getSelectedItem().toString());
        sort();
    }

    public void sort() {
        try {
            AutoImage sorted = image.getSortedImage(xAxis,yAxis);
            selectedPlacement.categorize(sorted, yAxis.getDataFilter().getTargetType());
            selectedPlacement.initialPlacement(sorted);
            canvas.setImage(sorted.getImage());
            canvas.repaint();
        } catch (NoDataPlotException e) {
            Scene.showErrorMessage(e.getMessage());
        }
    }

    public String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);
        if (image != null) {
            EVolve.util.painters.shapes.Shape entityShape = ((AutoShapeImage)image).getEntityShapes(X,Y);
            if (entityShape != null) {
                Entity entity;
                if (entityShape.getEntityType() == xAxis.getDataFilter().getTargetType()) {
                    entity = xAxis.getEntityFromInt((int)entityShape.getEntityID());
                } else {
                    entity = yAxis.getEntityFromInt((int)entityShape.getEntityID());
                }
                Scene.setStatus(entity.getName());
            } else
                Scene.setStatus(" ");
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    public Object clone() {
        RelationshipViz o = (RelationshipViz)super.clone();

        o.menuList = new ArrayList();

        return o;
    }

    public JMenuItem[] createSelectionMenuItem() {
        if (selectionMenu != null) return selectionMenu;

        itemSelectX = new JCheckBoxMenuItem("Occurred Entities of Object One");
        itemSelectX.setMnemonic(KeyEvent.VK_O);
        itemSelectX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = itemSelectX.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_X_AXIS);
            }
        });
        itemSelectX.setSelected(true);

        itemSelectY = new JCheckBoxMenuItem("Occurred Entities of Object Two");
        itemSelectY.setMnemonic(KeyEvent.VK_T);
        itemSelectY.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = itemSelectY.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_Y_AXIS);
            }
        });
        itemSelectY.setSelected(true);

        selectionMenu = new JMenuItem[2];
        selectionMenu[0] = itemSelectX;
        selectionMenu[1] = itemSelectY;

        return selectionMenu;
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        if (SELECT_OPTION == 0) {
            Scene.showErrorMessage("Nothing will be selected, please check Selection menu.");
            return;
        }

        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();
        int y1 = canvas.getStartY();
        int y2 = canvas.getEndY();

        int[] selection = null;
        HashMap entities = ((RelationshipPainter)painter).getEntitiesInBox(x1,y1,x2,y2);
        int xComparator = xAxis.getSelectedComparatorIndex(), yComparator = yAxis.getSelectedComparatorIndex();
        xAxis.selectComparator(1);
        yAxis.selectComparator(1);
        switch (SELECT_OPTION & 0xf0f0) {
            case SELECT_X_AXIS: // select occurred entities of object one
                ArrayList idList = (ArrayList)entities.get(new Integer(xAxis.getDataFilter().getTargetType())) ;
                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++) {
                    selection[i] = ((Long)idList.get(i)).intValue();
                }
                xAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0000: // x axis is not seletcted
                break;
        }

        switch (SELECT_OPTION & 0x0f0f) {
            case SELECT_Y_AXIS: //select occurred entities of object two
                ArrayList idList = (ArrayList)entities.get(new Integer(yAxis.getDataFilter().getTargetType())) ;
                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++) {
                    selection[i] = ((Long)idList.get(i)).intValue();
                }
                yAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
                break;
            case 0x0000:
                break;
        }
        xAxis.selectComparator(xComparator);
        yAxis.selectComparator(yComparator);
    }

    protected ArrayList createOptionalMenu() {
        if (menuList.size() > 0) return null;

        menuList.clear();
        itemReArrange = new JMenuItem("Rearrange");
        itemReArrange.setMnemonic(KeyEvent.VK_R);
        itemReArrange.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String newSize = (String)JOptionPane.showInputDialog(Scene.getFrame(),
                        "Input new threshold:",
                        "Rearrange",
                        JOptionPane.QUESTION_MESSAGE, null, null, String.valueOf(threshold));
                threshold = Integer.parseInt(newSize);
                canvas.scaleImage(false);
                sortValues();
                //((RelationshipPainter)painter).updatePainterParameters(value,threshold);
                painter.paint(image,xAxis.getDataFilter().getTargetType(),yAxis.getDataFilter().getTargetType(),10);
                sort();
            }
        });

        menuChangePlacement = new JMenu("Change Placement");
        menuChangePlacement.setMnemonic(KeyEvent.VK_F);
        for (int i = 0; i < placementList.length; i++) {
            JMenuItem item = new JMenuItem(placementList[i].getName());
            item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    for (int j=0; j<placementList.length; j++) {
                        if (placementList[j].getName().equals(((JMenuItem)e.getSource()).getActionCommand())) {
                            selectedPlacement = placementList[j];
                            sort();
                            break;
                        }
                    }
                }
            });
            menuChangePlacement.add(item);
        }
        menuList.add(itemReArrange);
        menuList.add(menuChangePlacement);
        return menuList;
    }

    private void sortValues() {
        int newValues[][] = new int[value.length][value[0].length];

        for (int i=0; i<value.length; i++) {
            for (int j=0; j<value[i].length; j++) {
                int x, y;
                x = xAxis.getSortedIndex(i);
                y = yAxis.getSortedIndex(j);
                if ((x!=-1)&&(y!=-1)) {
                    newValues[x][y] = value[i][j];
                }
            }
        }

        ((RelationshipPainter)painter).updatePainterParameters(newValues,threshold);
    }

    public void updateMenu() {
        super.updateMenu();
        itemScale.setSelected(false);
    }
}
