/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 28, 2002
 * Time: 10:23:35 AM
 */

package EVolve.visualization.XYViz.ValRefViz;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import java.util.ArrayList;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Element;
import EVolve.data.ValueComparator;
import EVolve.data.Entity;
import EVolve.util.painters.BarChartPainter;
import EVolve.util.painters.shapes.Box;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoShapeImage;
import EVolve.visualization.Dimension;

public class BarChartViz extends ValueReferenceVisualization{
    private static JCheckBox selectionOptions[] = null;
    private JCheckBox chkSelectOccurredEntities,chkSelectAllEntities;
    protected static int SELECT_OPTION = 0x100;
    private int barHeight = 5;
    private boolean useRandomColor;
    private JComboBox comboPainter;
    private final int randomPainter = 1;


    public BarChartViz() {
        super();
        useRandomColor = false;
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = super.createDimension();

        yAxis.addSortSchemes("Value");
        return returnDimension;
    }

    protected JPanel createConfigurationPanel() {
        JPanel panelBottom = new JPanel(new GridLayout(1,2));

        comboPainter = new JComboBox();
        comboPainter.addItem("Default Painter");
        comboPainter.addItem("Random Color Painter");
        panelBottom.add(new JLabel("Painter used:  "));
        panelBottom.add(comboPainter);


        JPanel configurationPanel = new JPanel(new FlowLayout());
        configurationPanel.add(panelBottom);

        return configurationPanel;
    }

    public void restoreConfiguration(SerializedVisualization config) {
        useRandomColor = config.PainterName.equals("Random Color Painter");
        comboPainter.setSelectedIndex(useRandomColor ? 1 : 0);
        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.PainterName = useRandomColor ? "Random Color Painter" : "Default Painter";
        return data;
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        int y1 = canvas.getEndY();
        int y2 = canvas.getStartY();
        int x1 = canvas.getStartX();

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        if ((y1<0)&&(y2<0))
           return;

        if (y1 < 0) {
            y1 = 0;
        }

        int selection[] = null;
        switch (SELECT_OPTION & 0x0f0f) {
            case 0x0001: // select entities in the draw box
                selection = new int[y2/barHeight - y1/barHeight + 1];
                for (int i = y1/barHeight; i <= y2/barHeight; i++) {
                    selection[i - y1/barHeight] = i;
                }
                break;
            case 0x0100: // select occurred entities
                int w = image.getW(), h = image.getH();
                ArrayList idList = new ArrayList();
                for (int i=0; i<w; i++) {
                    for (int j=0; j<h; j++) {
                        ArrayList value = (ArrayList)image.getColor(i,j);
                        if (value == null) continue;
                        Box box = (Box)value.get(0);
                        if ((box.getWidth()>x1) &&
                           (((box.y<y2)&&(box.y>y1)) || ((box.y+box.getHeight()<y2)&&(box.y+box.getHeight()>y1))) &&
                           !idList.contains(new Long(box.getEntityID())))
                            idList.add(new Long(box.getEntityID()));
                    }
                }

                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++) {
                    selection[i] = yAxis.getSortedIndex(((Long)idList.get(i)) .intValue());
                }
                break;
        }

        yAxis.makeSelection(selectionName,subjectDefinition.getType(),selection);
    }

    public void preVisualize() {
        image = new AutoShapeImage();
        adjustTimeAxis();
        installPainter();
        timeMap.clear();
        canvas.reset();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        long y = yAxis.getField(element);
        long v = xAxis.getField(element);
        painter.paint(image,v,y,0);

    }

    public void visualize() {
        xMax = ((BarChartPainter)painter).getxMax();

        yAxis.addComparator(new ValueComparator("Value", false, ((BarChartPainter)painter).getValue(), yAxis.getEntityName2IntMap()));
        yAxis.selectComparator(comboSortSchemes[0].getSelectedItem().toString());
        sort();
    }

    public String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);

        if (image != null) {
            EVolve.util.painters.shapes.Shape entityShape = ((AutoShapeImage)image).getEntityShapes(X,Y);
            if (entityShape != null) {
                Entity entity = yAxis.getEntityFromInt((int)entityShape.getEntityID());
                Scene.setStatus(entity.getName()+", "+((Box)entityShape).getRealWidth());
            } else
                Scene.setStatus(" ");
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    public void makeSelection2() {
        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();
        int y1 = canvas.getEndY();
        int y2 = canvas.getStartY();

        if (((x1<0)&&(x2<0)) || ((x1>=timeMap.size()))&&(x2>=timeMap.size()))
           return;

        if (x1 < 0) {
            x1 = 0;
        }

        if (x1 > (timeMap.size() - 1)) {
            x1 = timeMap.size() - 1;
        }

        if (x2 > (timeMap.size() - 1)) {
            x2 = timeMap.size() - 1;
        }

        if (y1 < 0) {
            y1 = 0;
        }

        preMakeSelection();
        long start = ((long[])timeMap.get(x1))[1];
        long end = ((long[])timeMap.get(x2))[1];

        if (((SELECT_OPTION & 0x000f) != SELECT_TIME_FRAME)||((x2==x1)&&(x1==0))) { // do not select time frame
            start = 0;
            end = Long.MAX_VALUE;
        } else {
            if (x1==x2) {
                if (x2+1<timeMap.size()) {
                    end = ((long[])timeMap.get(x2+1))[1];
                } else
                    end = Long.MAX_VALUE;
            }
        }

        int[] selection = null;
        switch (SELECT_OPTION & 0x0ff0) {
            case 0x0100: // select all entities in the draw box
                selection = new int[y2/barHeight - y1/barHeight + 1];
                for (int i = y1/barHeight; i <= y2/barHeight; i++) {
                    selection[i - y1/barHeight] = i;
                }
                break;
            case 0x0010: // select occurred entities
                int w = image.getW(), h = image.getH();
                ArrayList idList = new ArrayList();
                for (int i=0; i<w; i++) {
                    for (int j=0; j<h; j++) {
                        ArrayList value = (ArrayList)image.getColor(i,j);
                        if (value == null) continue;
                        Box box = (Box)value.get(0);
                        if (((x2-box.x < x2-x1+box.getWidth())||(box.x-x1 < x2-x1)) &&
                           ((box.y-y2 <0) || (y1-box.y>box.getHeight())) &&
                           !idList.contains(new Long(box.getEntityID())) )
                            idList.add(new Long(box.getEntityID()));
                    }
                }

                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++) {
                    selection[i] = ((Long)idList.get(i)).intValue();
                }
                break;
            case 0x0000: // no selection on entities
                y1 = 0;
                y2 = yAxis.getEntityNumber() - 1;
                selection = new int[y2 - y1 + 1];
                for (int i = y1; i <= y2; i++) {
                    selection[i - y1] = i;
                }
                break;
        }
        yAxis.makeSelection(selectionName, subjectDefinition.getType(),selection,start,end,timeMap);
    }

    protected void installPainter() {
        useRandomColor = (comboPainter.getSelectedIndex() == randomPainter);
        painter = new BarChartPainter(canvas, barHeight,useRandomColor);
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectOccurredEntities = new JCheckBox("Occurred Entities in the box");
        chkSelectOccurredEntities.setSelected(true);
        chkSelectOccurredEntities.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectOccurredEntities.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_Y_AXIS);

                if (chkSelectAllEntities.isSelected() && selected) {
                    chkSelectAllEntities.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION,SELECT_ALL_Y_AXIS);
                }
            }
        });

        chkSelectAllEntities = new JCheckBox("All Entities in the Y-Axis range");
        chkSelectAllEntities.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectAllEntities.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_ALL_Y_AXIS);

                if (chkSelectOccurredEntities.isSelected() && selected) {
                    chkSelectOccurredEntities.setSelected(false);
                    SELECT_OPTION = switchOption(false,SELECT_OPTION,SELECT_Y_AXIS);
                }
            }
        });
        chkSelectAllEntities.setSelected(false);

        selectionOptions = new JCheckBox[2];
        selectionOptions[0] = chkSelectOccurredEntities;
        selectionOptions[1] = chkSelectAllEntities;

        return selectionOptions;
    }

    public Object clone() {
        BarChartViz o = (BarChartViz) super.clone();
        o.panelConfiguration = o.createConfigurationPanel();
        o.createDialog();
        return o;
    }
}
