/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 5:00:08 PM
 */

package EVolve.visualization.XYViz.ValRefViz.HotSpotViz;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;

import EVolve.Scene;
import EVolve.data.Element;
import EVolve.util.painters.DefaultPainter;
import EVolve.util.painters.Painter;
import EVolve.util.painters.RandomPainter;
import EVolve.util.phasedetectors.HotspotPhaseDetector;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoImage;
import EVolve.visualization.XYViz.ValRefViz.ValueReferenceVisualization;

public class HotSpotVisualization extends ValueReferenceVisualization {
    protected JTextField textInterval; // input for interval
    protected JComboBox comboPainter;
    protected Painter painters[];
    protected int selectedPainter;
    protected JPanel configurationPanel;
    private JCheckBox chkSelectTimeFrame, chkSelectOccurredEntities, chkSelectAllEntities;
    private JMenuItem itemSelectPhase;
    protected static JCheckBox selectionOptions[] = null;
    protected static int SELECT_OPTION = 0x0011;


    public HotSpotVisualization() {
        interval = 10000;
        painters = new Painter[2];
        painters[0] = new DefaultPainter();
        painters[1] = new RandomPainter();
        selectedPainter = 0;
        phaseDetector = new HotspotPhaseDetector(this);
    }

    protected JPanel createConfigurationPanel() {
        JPanel panelBottom = new JPanel(new GridLayout(2,2));
        panelBottom.add(new JLabel("Interval: "));

        textInterval = new JTextField(String.valueOf(interval), 10);
        panelBottom.add(textInterval);

        comboPainter = new JComboBox();
        for (int i = 0; i < painters.length; i++) {
            comboPainter.addItem(painters[i].getName());
        }
        comboPainter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectedPainter = comboPainter.getSelectedIndex();
            }
        });
        panelBottom.add(new JLabel("Painter used:  "));
        panelBottom.add(comboPainter);


        if (configurationPanel == null)
            configurationPanel = new JPanel(new FlowLayout());
        configurationPanel.add(panelBottom);

        return configurationPanel;
    }

    public void restoreConfiguration(SerializedVisualization config) {
        for (int i=0; i<painters.length; i++) {
            if (painters[i].getName().equals(config.PainterName)) {
                comboPainter.setSelectedIndex(i);
                selectedPainter = i;
                break;
            }
        }
        textInterval.setText(config.Interval);
        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.PainterName = painters[selectedPainter].getName();
        return data;
    }

    public void preVisualize() {
        xMax = 0;
        imageMap.clear();
        currentThread = -1;
        image = new AutoImage();
        xOffset = -1;
        adjustTimeAxis();
        installPainter();
        timeMap.clear();
        phaseDetector.reset();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        long x = xAxis.getField(element);
        long y = yAxis.getField(element);

        paint(x,y,y);
    }

    public void visualize() {
        yAxis.selectComparator(comboSortSchemes[0].getSelectedItem().toString());
        sort();
    }

    public void makeSelection() {
        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();
        int y1 = canvas.getEndY();
        int y2 = canvas.getStartY();
        select(x1,y1,x2,y2);
    }

    public void paint(long x, long y, long z) {
        long temp = x;
        if (x > xMax) {
            xMax = x;
        }

        if (xOffset == -1) xOffset = x/interval;

        phaseDetector.collectData(x/interval-xOffset,y);

        x = x/interval;
        painter.paint(image,x-xOffset,y,z);

        countEvents(temp);
    }

    protected void updateConfiguration() {
        try {
            if (autoInterval == -1)
                interval = Integer.parseInt(textInterval.getText());
            else {
                interval = autoInterval;
                autoInterval = -1;
                textInterval.setText(String.valueOf(interval));
            }
            super.updateConfiguration();

        } catch (Exception e) {
            Scene.showErrorMessage("Interval must be an integer");
            configure();
        }
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectTimeFrame.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_TIME_FRAME);
            }
        });
        chkSelectTimeFrame.setSelected(true);

        chkSelectOccurredEntities = new JCheckBox("Occurred Entities");
        chkSelectOccurredEntities.setMnemonic(KeyEvent.VK_O);
        chkSelectOccurredEntities.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectOccurredEntities.isSelected();
                SELECT_OPTION = switchOption(selected,SELECT_OPTION, SELECT_OCCURRED_ENTITIES);
            }
        });
        chkSelectOccurredEntities.setSelected(true);

        chkSelectAllEntities = new JCheckBox("All Entities");
        chkSelectAllEntities.setMnemonic(KeyEvent.VK_A);
        chkSelectAllEntities.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectAllEntities.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_ALL_ENTITIES);

                if (chkSelectOccurredEntities.isSelected() && selected) {
                    SELECT_OPTION = switchOption(false,SELECT_OPTION, SELECT_OCCURRED_ENTITIES);
                    chkSelectOccurredEntities.setSelected(false);
                }
            }
        });
        chkSelectAllEntities.setSelected(true);

        selectionOptions = new JCheckBox[2];
        selectionOptions[0] = chkSelectTimeFrame;
        selectionOptions[1] = chkSelectOccurredEntities;

        return selectionOptions;
    }

    protected void installPainter() {
        painter = painters[selectedPainter];
    }

    protected void createMenu() {
        super.createMenu();

        itemSelectPhase = new JMenuItem("Select current phase ...");
        itemSelectPhase.setMnemonic(KeyEvent.VK_S);
        itemSelectPhase.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ArrayList phases = phaseDetector.getPhase();
                int mousePosition = canvas.getImageX(mouseX);
                int lastPhase = 0, currentPhase = 0;
                if (phases.size() == 0) {
                    Scene.showErrorMessage("No phase information available.");
                    return;
                }
                for (int i=0; i<phases.size(); i++) {
                    currentPhase = ((Integer)phases.get(i)).intValue();
                    if (mousePosition < currentPhase) break;
                    lastPhase = currentPhase;
                }
                if (lastPhase==currentPhase) {
                    currentPhase = Integer.MAX_VALUE;
                }
                select(lastPhase,0,currentPhase,yAxis.getEntityNumber()-1);
            }
        });

        popup.add(itemSelectPhase);
    }

    private void select(int X1, int Y1, int X2, int Y2) {
        preMakeSelection();
        if (selectionName == null) return;

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        int x1 = X1;//canvas.getStartX();
        int x2 = X2;//canvas.getEndX();
        int y1 = Y1;//canvas.getEndY();
        int y2 = Y2;//canvas.getStartY();

        if (!normalOrientation) {
            int temp;
            temp = x1;
            x1 = y1;
            y1 = temp;
            temp = x2;
            x2 = y2;
            y2 = temp;
        }

        if (((x1<0)&&(x2<0)) || ((x1>=timeMap.size()))&&(x2>=timeMap.size()) ||
            ((y1<0)&&(y2<0)) || ((y1>=yAxis.getEntityNumber())&&(y2>=yAxis.getEntityNumber())))
           return;

        /*if (x1==x2) {
            x2++;
        }*/

        if (x1 < 0) {
            x1 = 0;
        }

        if (x1 > (timeMap.size() - 1)) {
            x1 = timeMap.size() - 1;
        }

        if (x2 > (timeMap.size() - 1)) {
            x2 = timeMap.size() - 1;
        }

        if (y1 < 0) {
            y1 = 0;
        }

        if (y1 > (yAxis.getEntityNumber() - 1)) {
            y1 = yAxis.getEntityNumber() - 1;
        }

        if (y2 > (yAxis.getEntityNumber() - 1)) {
            y2 = yAxis.getEntityNumber() - 1;
        }

        long start = ((long[])timeMap.get(x1))[1];
        long end = ((long[])timeMap.get(x2))[1];

        if (((SELECT_OPTION & 0x000f) != SELECT_TIME_FRAME)||((x2==x1)&&(x1==0))) { // do not select time frame
            start = 0;
            end = Long.MAX_VALUE;
        } else {
            /*if (x2==x1) {
                x1--;
            }
            start = ((long[])timeMap.get(x1))[1];*/
            if (x1==x2) {
                if (x2+1<timeMap.size()) {
                    end = ((long[])timeMap.get(x2+1))[1];
                } else
                    end = Long.MAX_VALUE;
            }
        }

        int[] selection = null;
        switch (SELECT_OPTION & 0x0ff0) {
            case 0x0100: // select all entities in the draw box
                selection = new int[y2 - y1 + 1];
                for (int i = y1; i <= y2; i++) {
                    selection[i - y1] = i;
                }
                break;
            case 0x0010: // select occurred entities
                ArrayList idList = new ArrayList();
                for (int i=y1; i<=y2; i++) {
                    for (int j=x1; j<=x2; j++) {
                        Color color = (Color)image.getSortedColor(null,yAxis,j,i);
                        if (color != null) {
                            idList.add(new Integer(i));
                            break;
                        }
                    }
                }

                selection = new int[idList.size()];
                for (int i=0; i<idList.size(); i++) {
                    selection[i] = ((Integer)idList.get(i)).intValue();
                }
                break;
            case 0x0000: // no selection on entities
                y1 = 0;
                y2 = yAxis.getEntityNumber() - 1;
                selection = new int[y2 - y1 + 1];
                for (int i = y1; i <= y2; i++) {
                    selection[i - y1] = i;
                }
                break;
        }

        yAxis.makeSelection(selectionName, subjectDefinition.getType(),selection,start,end,timeMap);
    }

    public Object clone() {
        HotSpotVisualization o = (HotSpotVisualization) super.clone();
        o.configurationPanel = null;
        o.panelConfiguration = o.createConfigurationPanel();
        o.createDialog();
        return o;
    }
}
