/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 9:11:28 PM
 */

package EVolve.visualization.XYViz.ValRefViz.HotSpotViz;

import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import EVolve.data.Element;
import EVolve.data.ValueComparator;
import EVolve.util.painters.PredictionPainter;
import EVolve.util.painters.Painter;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.Dimension;
import EVolve.visualization.Predictor;
import EVolve.visualization.PredictorFactory;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;

public class PredictionViz extends HotSpotVisualization{
    private PredictorFactory[] factory; // predictor factories
    private PredictorFactory selectedFactory; // predictor factory in use
    private Predictor[] predictor; // predictors
    private JComboBox comboPredictor; // combobox for selecting predictor factory

    public PredictionViz(PredictorFactory[] factory) {
        super();
        this.factory = factory;
        painters = new Painter[1];
        painters[0] = new PredictionPainter();
        //painters[0] = new PredictionPainter();
        //painters[1] = painters[0];
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[3];

        xAxis = new ValueDimension();
        yAxis = new ReferenceDimension(true);
        zAxis = new ReferenceDimension(true);
        returnDimension[0] = xAxis;
        returnDimension[1] = yAxis;
        returnDimension[2] = zAxis;
        yAxis.addSortSchemes("Miss");

        return returnDimension;
    }

    protected JPanel createConfigurationPanel() {
        configurationPanel = new JPanel(new GridLayout(2, 1, 5, 5));

        selectedFactory = factory[0];

        JPanel panelTop = new JPanel(new FlowLayout());
        configurationPanel.add(panelTop);

        panelTop.add(new JLabel("Predictor: "));

        comboPredictor = new JComboBox();
        for (int i = 0; i < factory.length; i++) {
            comboPredictor.addItem(factory[i].getName());
        }
        panelTop.add(comboPredictor);

        JPanel returnValue = super.createConfigurationPanel();

        return returnValue;
    }

    public void restoreConfiguration(SerializedVisualization config) {

        for (int i=0; i<factory.length; i++) {
            if (factory[i].getName().equals(config.PredictorName)) {
                comboPredictor.setSelectedIndex(i);
                selectedFactory = factory[i];
                break;
            }
        }

        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();
        data.PredictorName = selectedFactory.getName();

        return data;
    }

    public void preVisualize() {
        predictor = new Predictor[yAxis.getMaxEntityNumber()];
        for (int i = 0; i < predictor.length; i++) {
            predictor[i] = selectedFactory.createPredictor();
        }
        super.preVisualize();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        long x = xAxis.getField(element);
        long y = yAxis.getField(element);
        long z = zAxis.getField(element);

        paint(x,y,zAxis.getEntityFromInt((int)z).getId());
    }

    public void visualize() {
        yAxis.addComparator(new ValueComparator("Miss", false, ((PredictionPainter)painter).getMiss(),yAxis.getEntityName2IntMap()));
        super.visualize();
    }

    protected void installPainter() {
        ((PredictionPainter)painters[0]).setPredictor(predictor,zAxis.getDataFilter().getTargetType());
        super.installPainter();
    }

    public Object clone() {
        PredictionViz o = (PredictionViz) super.clone();
        o.factory = new PredictorFactory[factory.length];
        for (int i=0; i<factory.length; i++) {
            o.factory[i] = (PredictorFactory)factory[i].clone();
        }
        o.selectedFactory = (PredictorFactory)selectedFactory.clone();
        if (predictor != null) {
            o.predictor = new Predictor[predictor.length];
            for (int i=0; i<predictor.length; i++)
                o.predictor[i] = (Predictor)predictor[i].clone();
        }
        return o;
    }
}
