/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 28, 2002
 * Time: 2:37:43 AM
 */

package EVolve.visualization.XYViz.ValRefViz.HotSpotViz;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;

import javax.swing.JMenuItem;

import EVolve.data.Element;
import EVolve.util.ThreadChooser;
import EVolve.util.painters.StackHotspotPainter;
import EVolve.util.painters.Painter;
import EVolve.visualization.AutoImage;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;

public class StackHotspotViz extends HotSpotVisualization{
    private ArrayList menuList;
    private String dataFilterName[];
    private JMenuItem itemChangThread;
    private HashMap methodStacks;
    private ThreadChooser td;
    private Stack stack;

    public StackHotspotViz() {
        super();
        dataFilterName = new String[2];

        dataFilterName[0] = "Bytecode";
        dataFilterName[1] = "Method";

        menuList = new ArrayList();
        td = new ThreadChooser(imageMap,false);
        painters = new Painter[1];
        painters[0] = new StackHotspotPainter();
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[3];

        xAxis = new ValueDimension();
        yAxis = new ReferenceDimension(true);
        zAxis = new ReferenceDimension();

        returnDimension[0] = xAxis;
        returnDimension[1] = yAxis;
        returnDimension[2] = zAxis;

        return returnDimension;
    }

    protected ArrayList createOptionalMenu() {
        if (menuList.size() > 0) return null;

        menuList.clear();
        itemChangThread = new JMenuItem("Switch thread...");
        itemChangThread.setMnemonic(KeyEvent.VK_T);
        itemChangThread.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                visualize();
            }
        });

        menuList.add(itemChangThread);
        return menuList;
    }

    public void preVisualize() {
        super.preVisualize();
        image = null;
        methodStacks = new HashMap();
    }

    public void receiveElement(Element element) {
        long threadId = zAxis.getField(element);

        switchThread(threadId);
        long x = xAxis.getField(element);
        long y = yAxis.getField(element);
        long z = element.isOptional() ? Integer.MAX_VALUE : Integer.MIN_VALUE;

        ((StackHotspotPainter)painter).setStack(stack);
        paint(x,y,z);
    }

    public void visualize() {
        String selected = null;
        if (imageMap.size() == 0 ) return;
        while (selected==null) {
            selected = td.showDialog();
            if (selected != null) {
                image = (AutoImage)imageMap.get(new Long(selected.substring(7,selected.length())));
                selected = null;
                break;
            }
        }
        super.visualize();
    }

    protected void switchThread(long threadId) {
        if (currentThread != threadId) {
            stack = (Stack)methodStacks.get(new Long(threadId));
            if (stack == null) {
                stack = new Stack();
                methodStacks.put(new Long(threadId),stack);
            }
            super.switchThread(threadId);
        }
    }

    public Object clone() {
        StackHotspotViz o = (StackHotspotViz)super.clone();
        o.createOptionalMenu();
        o.dataFilterName = new String[dataFilterName.length];
        for (int i=0; i<dataFilterName.length; i++)
            o.dataFilterName[i] = dataFilterName[i];
        if (methodStacks == null) {
            o.methodStacks = (HashMap)methodStacks.clone();
            Iterator it = methodStacks.keySet().iterator();
            while(it.hasNext()) {
                Object key = it.next();
                o.methodStacks.put(key, methodStacks.get(key));
            }
        }
        o.td = new ThreadChooser(o.imageMap,false);
        o.stack = (Stack)stack.clone();
        o.menuList.clear();
        return o;
    }
}
