/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 3:57:05 PM
 */

package EVolve.visualization.XYViz.ValRefViz;

import EVolve.Scene;
import EVolve.data.DataFilter;
import EVolve.data.Entity;
import EVolve.data.Selection;
import EVolve.exceptions.NoDataPlotException;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;
import EVolve.visualization.XYViz.XYVisualization;


public abstract class ValueReferenceVisualization extends XYVisualization{
    protected ValueDimension xAxis;
    protected ReferenceDimension yAxis;
    protected ReferenceDimension zAxis;
    private long start=0;

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[2];

        xAxis = new ValueDimension();
        yAxis = new ReferenceDimension(true);
        zAxis = null;

        returnDimension[0] = xAxis;
        returnDimension[1] = yAxis;

        return returnDimension;
    }

    public ReferenceDimension getLinkableDimension(int dim) {
        if (dim!=1) return null;
        return (ReferenceDimension)dimension[dim];
    }

    protected void adjustTimeAxis() {
        Selection activeSelection = Scene.getFilter().getActiveSelection();
        start = ((activeSelection == null)) ? 0 : activeSelection.getStartTime();
        if (start < 0) start = 0;
    }

    public void sort() {
        if (image == null) return;

        canvas.reset();
        try {
            String comparatorName = "";
            try {
                comparatorName = yAxis.getSelectedComparatorName();
            } catch (IndexOutOfBoundsException e) {
                // this exception happens when user click fresh button before visulize the topmost viz
                return;
            }
            int beforeLink = yAxis.getEntityNumberBeforeLink();

            if (normalOrientation) {
                canvas.setName(timeHeader() + xAxis.getName() + " (" +
                              ((interval == -1) ? ""+xMax+"" : ((""+start  +" - ")) + (xMax > start ? xMax : start+xMax)) +
                              (interval == -1 ? "" : (",interval " + String.valueOf(interval))) +")",
                              yAxis.getName() + " (" + ((beforeLink == -1) ? "" : (""+beforeLink+" / ")) + yAxis.getEntityNumber()
                              + ", " + comparatorName +")");
                canvas.setAutoImage(image.getSortedImage(null, yAxis));
            } else {
                canvas.setName(yAxis.getName() + " (" + ((beforeLink == -1) ? "" : (""+beforeLink+" / ")) +
                               yAxis.getEntityNumber() + ", " + comparatorName + ")",
                               timeHeader() + xAxis.getName() + " (" +
                               ((interval == -1) ? ""+xMax+"" : ((""+start  +" - ")) + (xMax > start ? xMax : start+xMax))
                               + ")");
                canvas.setAutoImage(image.getSortedImage(yAxis, null));
            }
            canvas.repaint();
            enableBrowseSourceMenu();
        } catch (NoDataPlotException e) {
            Scene.showErrorMessage(e.getMessage());
        }
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();
        data.xAxis = xAxis.getName();
        data.yAxis = yAxis.getName();
        data.yAxisSortScheme = comboSortSchemes[0].getSelectedItem().toString();
        data.zAxis = (zAxis == null) ? null : zAxis.getName();

        return data;
    }

    protected void updateConfiguration() {
        canvas.setName(xAxis.getName(), yAxis.getName());
        super.updateConfiguration();
    }

    /**
     * Mouse moved.
     *
     * @param   x position on X-axis
     * @param   y position on Y-axis
     */
    protected String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);
        String time = xAxis.getName();
        String returnVal = null;

        if ((Y >= 0) && (Y < yAxis.getEntityNumber())) {
            if (shift_pressed && (image.getSortedColor(null,yAxis,X,Y)==null))
                Scene.setStatus(" ");
            else {
                Entity entity = yAxis.getEntity(Y);
                time = ((interval == -1)||(X>=timeMap.size())||(X<0)) ? "" : time + ":" + ((long[])timeMap.get(X))[0];
                if (entity != null) {
                    if (((interval == -1)||(X>=timeMap.size())||(X<0)))
                        return null;
                    returnVal = entity.getName() + ":" + ((long[])timeMap.get(X))[0];
                    Scene.setStatus(returnVal);
                } else
                    Scene.setStatus(" ");
            }

        } else {
            Scene.setStatus(" ");
        }
        return returnVal;
    }

    protected void changeOrientation() {
        super.changeOrientation();
        sort();
    }

    private String timeHeader() {
        DataFilter dataFilter = xAxis.getDataFilter();
        String[] property = dataFilter.getProperty();
        for (int i=0; i< property.length; i++) {
            if (property[i].equals("time"))
                return "Time - ";
        }
        return "";
    }

    protected String getEntityUnderMouse() {
        int X = canvas.getImageX(mouseX);
        int Y = canvas.getImageY(mouseY);

        if ((Y >= 0) && (Y < yAxis.getEntityNumber())) {
            if (normalOrientation) {
                if ((image.getSortedColor(null,yAxis,X,Y)!=null))
                    return yAxis.getEntity(Y).getName();
            } else {
                if (image.getSortedColor(yAxis,null,X,Y)!=null)
                    return yAxis.getEntity(X).getName();
            }
        }

        return null;
    }

    public Object clone(){
        ValueReferenceVisualization o = null;

        o = (ValueReferenceVisualization)super.clone();

        o.xAxis =(ValueDimension)xAxis.clone();
        o.yAxis =(ReferenceDimension) yAxis.clone();
        o.zAxis =(zAxis == null) ? null : (ReferenceDimension)zAxis.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.yAxis;
        if (o.zAxis != null) o.dimension[2] = o.zAxis;
        o.createMenu();

        return o;
    }
}
