/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 30, 2002
 * Time: 7:49:41 PM
 */

package EVolve.visualization.XYViz.ValValViz;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Element;
import EVolve.data.Entity;
import EVolve.util.DataTrace;
import EVolve.util.painters.RandomPainter;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.equators.*;
import EVolve.util.equators.Set;
import EVolve.visualization.*;
import EVolve.visualization.Dimension;

public class DotplotViz extends ValueValueVisualization {
    private JTextField textInterval,textMatch, textBegin,textEnd;
    private DataTrace trace;
    private long event;
    private JComboBox comboEquator;
    private Equator[] equatorList;
    private float percent;
    private Equator selectedEquator;
    private JCheckBox chkSelectX, chkSelectTimeFrame;
    private static JCheckBox selectionOptions[] = null;
    private static int SELECT_OPTION = 0x0011;
    private JLabel toEventNo;


    public DotplotViz() {
        super();

        beginCall = 0;
        endCall = 300;
        interval = 10;
        percent = 100;


        equatorList = new Equator[2];
        equatorList[0] = new OrderedEquator();
        equatorList[1] = new UnorderedEquator();
        selectedEquator = null;
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[2];

        xAxis = new ValueDimension();
        entityIdFilter = new ReferenceDimension();

        returnDimension[0] = xAxis;
        returnDimension[1] = entityIdFilter;

        return returnDimension;
    }

    protected void updateConfiguration() {

        selectedEquator = equatorList[comboEquator.getSelectedIndex()];

        try {
            if (autoInterval != -1) {
                interval = autoInterval;
                autoInterval = -1;
            } else {
                interval = Integer.parseInt(textInterval.getText());
                percent = Float.parseFloat(textMatch.getText());
                if ((percent>100)||(percent < 0)) {
                    Scene.showErrorMessage("Match percent must be between 0 and 100.");
                    configure();
                }
                textInterval.setText(String.valueOf(interval));
                textMatch.setText(String.valueOf(percent));
            }

            beginCall = Integer.parseInt(textBegin.getText());
            endCall = Integer.parseInt(textEnd.getText());
            selectedEquator.setInterval(interval);
            selectedEquator.setMatch(percent);

            canvas.setName(entityIdFilter.getName(), entityIdFilter.getName());
            super.updateConfiguration();
        } catch (NumberFormatException e) {
            Scene.showErrorMessage("Interval, begin call and end call must be an integer and smaller than "+Integer.MAX_VALUE+".");
            configure();
        };

    }

    public void restoreConfiguration(SerializedVisualization config) {
        super.restoreConfiguration(config);

        beginCall = Long.parseLong(config.BeginEvent);
        endCall = Long.parseLong(config.EndEvent);
        percent = Float.parseFloat(config.Threshold);

        for (int i=0; i<equatorList.length; i++) {
            if (equatorList[i].getName().equals(config.EqualizerName)) {
                selectedEquator = equatorList[i];
                comboEquator.setSelectedIndex(i);
                selectedEquator.setMatch(percent);
                selectedEquator.setInterval(Integer.parseInt(config.EqualizerSetSize));
                break;
            }
        }

        textBegin.setText(config.BeginEvent);
        textEnd.setText(config.EndEvent);
        textMatch.setText(config.Threshold);
        interval = Integer.parseInt(config.EqualizerSetSize);
        textInterval.setText(String.valueOf(interval));
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.BeginEvent = String.valueOf(beginCall);
        data.EndEvent = String.valueOf(endCall);
        data.Threshold = String.valueOf(percent);
        data.EqualizerName = selectedEquator.getName();
        data.EqualizerSetSize = String.valueOf(interval);
        data.xAxis = xAxis.getName();
        data.yAxis = entityIdFilter.getName();

        return data;
    }

    public void preVisualize() {
        image = new AutoImage();
        trace = selectedEquator.initialDataTrace(interval, Scene.getDataManager().getEntity()[entityIdFilter.getDataFilter().getTargetType()].size());
        event = 0;
        getSelection();
        installPainter();
        super.preVisualize();
    }

    protected void receiveElement(Element element) {
        if (element.isOptional()) return;

        long traceId = entityIdFilter.getField(element);
        long no = xAxis.getField(element) - 1;

        countEvents(no);
        if ((no < beginCall)||(no > endCall)) {
            return;
        }

        event++;
        trace.updateTrace(traceId);

        ArrayList data = trace.getTrace();

        if ((event%interval == 0)||(event-endCall==1)) {
            int size = data.size() - 1;
            for (int i=size+1; i>1; i--) {
                Set set1 = (Set)data.get(size);
                Set set2 = (Set)data.get(i-2);
                if (selectedEquator.isEqual(set1,set2)) {
                    painter.paint(image,i-2,size,set1.getHashValue());
                    painter.paint(image,size,i-2,set1.getHashValue());
                }
            }
            if (size>=0) {
                painter.paint(image,size,size,((Set)data.get(size)).getHashValue());
                image.setColor(size,size,Color.black);
            }
        }
    }

    public void visualize() {
        canvas.setName("Time - " + xAxis.getDataFilter().getName() + " (" + event +" events)", "Time - " + xAxis.getDataFilter().getName() + " (" + event + " events)");
        sort();
    }

    protected String mouseMove(int x,int y) {
        if (interval > 1) return null;

        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);
        int index = X < Y ? X : Y;
        if ((index >= 0) && (trace!=null) && (index < trace.size())) {
            int mappedId = (int)((Set)trace.getTrace().get(index)).getElement(0);
            Entity entity = entityIdFilter.getEntityFromInt(mappedId);
            if (shift_pressed && (image.getSortedColor(null,null,X,Y)==null))
                Scene.setStatus("  ");
            else
                Scene.setStatus(entity.getName());
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        ArrayList idList = new ArrayList();

        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        if (((x1<0)&&(x2<0)) || ((x1>=trace.getTrace().size())&&(x2>=trace.getTrace().size())))
           return;

        if (x1 < 0) x1 = 0;
        if (x2 >= trace.getTrace().size()) x2 = trace.getTrace().size()-1;

        Set unionSet = ((Set)(trace.getTrace().get(x1)));
        for (int i=x1+1; i<=x2; i++) {
            Set next = ((Set)(trace.getTrace().get(i)));
            unionSet = unionSet.union(next);
        }

        for (int i=0; i<unionSet.size(); i++) {
            if (unionSet.getEntityId(i)!=-1)
                idList.add(new Long(unionSet.getEntityId(i)));
        }

        int selection[] = null;
        if ((SELECT_OPTION & 0x00f0) == SELECT_OCCURRED_ENTITIES) {
            selection = new int[idList.size()];
            for (int i=0; i<selection.length; i++)
                selection[i] = ((Long)idList.get(i)).intValue();
        } else {
            selection = new int[entityIdFilter.getEntityNumber()];
            for (int i=0; i<selection.length; i++)
                selection[i] = i;
        }


        long start = ((long[])timeMap.get(((int)(x1+beginCall/interval))))[1];
        long end;
        if (x1==x2) {
            if (x1+1>trace.getTrace().size()-1)
                end = Long.MAX_VALUE;
            else {
                end = ((long[])timeMap.get((int)(x1+1+beginCall/interval)))[1]-1;
            }
        } else {
            end = ((long[])timeMap.get(((int)(x2+beginCall/interval))))[1];
        }
        if ((SELECT_OPTION & 0x000f) == 0x0000) {
            start = 0;
            end = Long.MAX_VALUE;
        }
        entityIdFilter.selectComparator(1);
        entityIdFilter.makeSelection(selectionName,subjectDefinition.getType(),selection,start,end, timeMap);
    }

    protected JPanel createConfigurationPanel() {
        JPanel returnVal = new JPanel(new GridLayout(1, 1, 5, 5));
        JPanel panelEvent = new JPanel(new GridLayout(2,2));
        JPanel equatorBox = new JPanel(new GridLayout(1,2));

        Box panelBottom = new Box(BoxLayout.Y_AXIS);
        returnVal.add(panelBottom);

        textBegin = new JTextField(String.valueOf(beginCall));
        textEnd = new JTextField(String.valueOf(endCall));
        panelEvent.add(new JLabel("From Event No:"));
        panelEvent.add(textBegin);

        toEventNo = new JLabel("To Event No:");
        panelEvent.add(toEventNo);
        panelEvent.add(textEnd);
        panelBottom.add(panelEvent);

        comboEquator = new JComboBox();
        for (int i=0; i<equatorList.length; i++)
            comboEquator.addItem(equatorList[i].getName());
        comboEquator.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                textInterval.setText(String.valueOf(equatorList[comboEquator.getSelectedIndex()].getInterval()));
                textMatch.setText(String.valueOf(equatorList[comboEquator.getSelectedIndex()].getMatch()));
            }
        });

        equatorBox.add(new JLabel("Equator:"));
        equatorBox.add(comboEquator);
        panelBottom.add(equatorBox);

        JPanel panelEquator = new JPanel(new GridLayout(2,2));

        textInterval = new JTextField(String.valueOf(interval));
        textMatch = new JTextField(String.valueOf(percent));

        panelEquator.add(new JLabel("Interval:"));
        panelEquator.add(textInterval);
        panelEquator.add(new JLabel("Match:"));
        panelEquator.add(textMatch);

        panelBottom.add(panelEquator);

        return returnVal;
    }

    protected void installPainter() {
        painter = new RandomPainter();
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectTimeFrame.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_TIME_FRAME);
            }
        });
        chkSelectTimeFrame.setSelected(true);

        chkSelectX = new JCheckBox("Occurred Entitiies");
        chkSelectX.setMnemonic(KeyEvent.VK_O);
        chkSelectX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectX.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_OCCURRED_ENTITIES);
            }
        });
        chkSelectX.setSelected(true);

        selectionOptions = new JCheckBox[2];
        selectionOptions[0] = chkSelectTimeFrame;
        selectionOptions[1] = chkSelectX;

        return selectionOptions;
    }

    protected void updateComboSubject() {
        super.updateComboSubject();
        String range = String.valueOf(Scene.getDataManager().getDataSource().getNumberOfEvents(elementDefinition[comboSubject.getSelectedIndex()].getName()));
        toEventNo.setText("To Event No:("+range+")");
    }

    public Object clone() {
        DotplotViz o = (DotplotViz)super.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.entityIdFilter;
        o.trace = (trace == null) ? null : (DataTrace)trace.clone();
        o.createDialog();
        o.createMenu();

        return o;
    }
}
