/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Feb 9, 2003
 * Time: 5:21:17 PM
 */

package EVolve.visualization.XYViz.ValValViz;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Element;
import EVolve.data.Entity;
import EVolve.exceptions.NoDataPlotException;
import EVolve.util.painters.*;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.*;
import EVolve.visualization.Dimension;

public class EventViz extends ValueValueVisualization{
    private JTextField textBegin,textEnd,textBoxSize;
    private int boxSize = 10;
    private ArrayList menuList;
    private JCheckBox chkSelectX, chkSelectTimeFrame, selectionOptions[] = null;
    private JMenuItem itemReArrange;
    private static int SELECT_OPTION = 0x0011;
    private JComboBox comboPainter,comboPredictor;
    private Painter[] painters;
    private int selectedPainter;
    private PredictorFactory[] factory; // predictor factories
    private Predictor[] predictor; // predictors
    private AutoImage entityMapImage;
    private final int PredictionPainter = 0, RandomColorPainter = 1, DefaultPainter = 2;
    private JLabel toEventNo;

	public EventViz() {
		super();
		menuList = new ArrayList();

		beginCall = 0;
		endCall = 300;
		interval = 1;
		selectedPainter = 0;
		flipMagnifier = false;
	}

    public EventViz(PredictorFactory[] factory) {
        super();
        menuList = new ArrayList();

        beginCall = 0;
        endCall = 300;
        interval = 1;
        selectedPainter = 0;
        painters = createPainters();
        this.factory = factory;
        flipMagnifier = false;
    }

    private Painter[] createPainters() {
        Painter[] painters = new Painter[3];
        painters[0] = new EventPredictionPainter();
        painters[1] = new RandomPainter();
        painters[2] = new DefaultPainter();
        return painters;
    }

    public void restoreConfiguration(SerializedVisualization config) {

        beginCall = Long.parseLong(config.BeginEvent);
        endCall = Long.parseLong(config.EndEvent);
        boxSize = Integer.parseInt(config.BlockSize);
        for (int i=0; i<painters.length; i++) {
            if (painters[i].getName().equals(config.PainterName)) {
                selectedPainter = i;
                comboPainter.setSelectedIndex(i);
                break;
            }
        }

        textBoxSize.setText(config.BlockSize);
        textBegin.setText(config.BeginEvent);
        textEnd.setText(config.EndEvent);
        super.restoreConfiguration(config);
    }

    protected void updateConfiguration() {

        try {
            beginCall = Integer.parseInt(textBegin.getText());
            endCall = Integer.parseInt(textEnd.getText());
            boxSize = Integer.parseInt(textBoxSize.getText());

            super.updateConfiguration();
        } catch (NumberFormatException e) {
            Scene.showErrorMessage("Begin call and end call must be an integer and smaller than "+Integer.MAX_VALUE+".");
            configure();
        };

    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();
        data.xAxis = xAxis.getName();
        data.yAxis = entityIdFilter.getName();
        data.zAxis = filter2.getName();
        data.PredictorName = comboPredictor != null ? factory[comboPredictor.getSelectedIndex()].getName() :
                                                      factory[0].getName();
        data.BeginEvent = String.valueOf(beginCall);
        data.EndEvent = String.valueOf(endCall);
        data.BlockSize = String.valueOf(boxSize);
        data.PainterName = painters[selectedPainter].getName();

        return data;
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[3];

        xAxis = new ValueDimension();
        entityIdFilter = new ReferenceDimension();
        filter2 = new ReferenceDimension();

        returnDimension[0] = xAxis;
        returnDimension[1] = entityIdFilter;
        returnDimension[2] = filter2;

        return returnDimension;
    }

    protected Component createPanel() {
        Component returnVal = super.createPanel();
        canvas.setPanelFlag(AxesPanel.Null);
        canvas.setZoomingSize(50,50);
        flipMagnifier = false;
        addMagnifier(canvas,flipMagnifier);

        return returnVal;
    }

    public void preVisualize() {
        super.preVisualize();
        image = new AutoShapeImage(new EVolve.util.painters.shapes.Box(0,0,boxSize,boxSize));
        entityMapImage = new AutoShapeImage(new EVolve.util.painters.shapes.Box(0,0,boxSize,boxSize));
        canvas.setShapeSize(boxSize);
        if (selectedPainter == 0) {
            predictor = new Predictor[entityIdFilter.getMaxEntityNumber()];
            for (int i = 0; i < predictor.length; i++) {
                if (comboPredictor == null)
                    predictor[i] = factory[0].createPredictor();
                else
                    predictor[i] = factory[comboPredictor.getSelectedIndex()].createPredictor();
            }
        }
        getSelection();
        installPainter();
    }

    protected void receiveElement(Element element) {
        if (element.isOptional()) return;

        long no = xAxis.getField(element) - 1;
        long traceId = entityIdFilter.getField(element);
        long z = filter2.getField(element);

        countEvents(no);
        if ((no < beginCall)||(no > endCall)) {
            return;
        }
        if (selectedPainter != RandomColorPainter)
            painters[1].paint(entityMapImage,(no-beginCall)%(panel.getWidth()/boxSize),
                             (no-beginCall)/(panel.getWidth()/boxSize),traceId);

        if (selectedPainter == PredictionPainter) {// prediction color painter
            painter.paint(image,no-beginCall,traceId,filter2.getEntityFromInt((int)z).getId());
        } else {
            painter.paint(image,(no-beginCall)%(panel.getWidth()/boxSize),
                         (no-beginCall)/(panel.getWidth()/boxSize),traceId);
        }
    }

    public void visualize() {
        sort();
    }

    public void sort() {
        try {
            int width = panel.getWidth() - ((JScrollPane)panel).getVerticalScrollBar().getWidth();
            canvas.setShapeSize(boxSize);
            ((AutoShapeImage)image).reArrange(width,boxSize);
            if (selectedPainter != 1) ((AutoShapeImage)entityMapImage).reArrange(width,boxSize);
            BufferedImage result = image.getImage();
            canvas.setImage(result);
            canvas.setPreferredSize(new java.awt.Dimension(result.getWidth(),result.getHeight()));
            canvas.revalidate();
            canvas.repaint();
        } catch (NoDataPlotException e) {
            Scene.showErrorMessage(e.getMessage());
        }
    }

    protected String mouseMove(int x,int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);
        AutoShapeImage aImage = (AutoShapeImage)image;
        if (selectedPainter != 1)
            aImage = (AutoShapeImage)entityMapImage;
        Color color= (aImage == null) ? null : (Color)aImage.getColor(X/boxSize,Y/boxSize);
        if (color != null) {
            int mappedId = (int)((RandomPainter)painters[1]).getKeyFromColor(color);
            Entity entity = entityIdFilter.getEntityFromInt(mappedId);
            if (entity != null) {
                Scene.setStatus(entity.getName()
                                + " " + xAxis.getDataFilter().getName()+":"+ (x/boxSize + aImage.getW() * (y/boxSize)));
            }
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        int w = image.getW();
        int startX = canvas.getStartX()/boxSize;
        int endX = canvas.getEndX()/boxSize;
        int startY = canvas.getStartY()/boxSize;
        int endY = canvas.getEndY()/boxSize;

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        long start = startX+startY*w;
        long end = endX+endY*w;
        long eventInterval[] = findEventInterval(start+beginCall, end+beginCall);
        start = eventInterval[0];
        end = eventInterval[1];

        ArrayList idList = new ArrayList();

        AutoShapeImage aImage = (AutoShapeImage)image;
        if (selectedPainter != 1)
            aImage = (AutoShapeImage)entityMapImage;

        for (int i=startX; i<=endX; i++) {
            for (int j=startY; j<=endY; j++) {
                Long id = new Long(((RandomPainter)painters[RandomColorPainter]).getKeyFromColor((Color)aImage.getColor(i,j)));
                if (!idList.contains(id))
                    idList.add(id);
            }
        }

        int selection[] = null;
        if ((SELECT_OPTION & 0x00f0) == SELECT_OCCURRED_ENTITIES) {
            selection = new int[idList.size()];
            for (int i=0; i<selection.length; i++)
                selection[i] = ((Long)idList.get(i)).intValue();
        } else {
            selection = new int[entityIdFilter.getEntityNumber()];
            for (int i=0; i<selection.length; i++)
                selection[i] = i;
        }
        if ((SELECT_OPTION & 0x000f) == 0x0000) {
            start = 0;
            end = Long.MAX_VALUE;
        }
        entityIdFilter.selectComparator(1);
        entityIdFilter.makeSelection(selectionName,subjectDefinition.getType(),selection,start,end,timeMap);
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectTimeFrame.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_TIME_FRAME);
            }
        });
        chkSelectTimeFrame.setSelected(true);

        chkSelectX = new JCheckBox("Occurred Entitiies");
        chkSelectX.setMnemonic(KeyEvent.VK_O);
        chkSelectX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectX.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_OCCURRED_ENTITIES);
            }
        });
        chkSelectX.setSelected(true);

        selectionOptions = new JCheckBox[2];
        selectionOptions[0] = chkSelectTimeFrame;
        selectionOptions[1] = chkSelectX;

        return selectionOptions;
    }

    public int getBoxSize() {
        return boxSize;
    }

    protected void installPainter() {
        painters = createPainters();
        painter = painters[selectedPainter];
        if (selectedPainter == 0)
            ((EventPredictionPainter)painter).setParameters(predictor,filter2.getDataFilter().getTargetType(),panel.getWidth()/boxSize);
    }

    protected ArrayList createOptionalMenu() {
        if (menuList.size() > 0) return null;

        menuList.clear();
        itemReArrange = new JMenuItem("Rearrange");
        itemReArrange.setMnemonic(KeyEvent.VK_R);
        itemReArrange.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String newSize = (String)JOptionPane.showInputDialog(Scene.getFrame(),
                        "Input new size for rectangles:",
                        "Rearrange",
                        JOptionPane.QUESTION_MESSAGE, null, null, String.valueOf(boxSize));
                if (newSize!=null) {
                    boxSize = Integer.parseInt(newSize);
                    if (selectedPainter != 1)
                        ((AutoShapeImage)entityMapImage).reArrange(panel.getWidth(),boxSize);
                    sort();
                }
            }
        });

        menuList.add(itemReArrange);
        return menuList;
    }

    public Object clone() {
        EventViz o = (EventViz)super.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.entityIdFilter;
        o.dimension[2] = o.filter2;
        o.createDialog();
        o.createMenu();
        o.menuList = new ArrayList();
        o.updateMenu();
        o.painters = new Painter[3];
        for (int i=0; i<o.painters.length; i++)
            o.painters[i] = (Painter)painters[i].clone();
        o.entityMapImage = (AutoShapeImage)entityMapImage.clone();

        return o;
    }

 /**
 *  customize UI
 */
    protected void createDialog() {
        dialog = new JDialog(Scene.getFrame(), "Configure", true);

        JPanel panelTitle = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelTitle, BorderLayout.NORTH);

        panelTitle.add(new JLabel("Title: "));

        textName = new JTextField(name, 12);
        panelTitle.add(textName);

        JPanel panelMain = new JPanel(new BorderLayout());
        panelMain.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Choose subject & dimensions"));
        dialog.getContentPane().add(panelMain, BorderLayout.CENTER);

        JPanel panelName = new JPanel(new GridLayout(definition.getDimensionDefinition().length , 1, 5, 5));
        panelMain.add(panelName, BorderLayout.WEST);

        panelName.add(new JLabel(" Subject:      "));
        for (int i = 0; i < definition.getDimensionDefinition().length - 1; i++) {
            panelName.add(new JLabel(" " + definition.getDimensionDefinition()[i].getName() + ":      "));
        }

        JPanel panelCombo = new JPanel(new GridLayout(definition.getDimensionDefinition().length, 1, 5, 5));
        panelMain.add(panelCombo, BorderLayout.CENTER);

        comboSubject = new JComboBox();
        for (int i = 0; i < elementDefinition.length; i++) {
            comboSubject.addItem(elementDefinition[i].getName());
        }
        comboSubject.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateComboSubject();
            }
        });
        panelCombo.add(comboSubject);

        comboDimension = new JComboBox[definition.getDimensionDefinition().length];
        for (int i = 0; i < comboDimension.length; i++) {
            comboDimension[i] = new JComboBox();
            comboDimension[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    updateComboDimension();
                }
            });
        }
        for (int i = 0; i < comboDimension.length - 1; i++)
            panelCombo.add(comboDimension[i]);

        JPanel returnVal = new JPanel(new GridLayout(1, 1, 5, 5));
        JPanel panelEvent = new JPanel(new GridLayout(6,2));

        Box panelBottom = new Box(BoxLayout.Y_AXIS);
        returnVal.add(panelBottom);

        comboPredictor = new JComboBox();
        for (int i = 0; i < factory.length; i++) {
            comboPredictor.addItem(factory[i].getName());
        }

        comboPainter = new JComboBox();
        for (int i = 0; i < painters.length; i++) {
            comboPainter.addItem(painters[i].getName());
        }
        comboPainter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectedPainter = comboPainter.getSelectedIndex();
                if (selectedPainter == 0) {
                    comboPredictor.setEnabled(true);
                    comboDimension[comboDimension.length-1].setEnabled(true);
                } else {
                    comboPredictor.setEnabled(false);
                    comboDimension[comboDimension.length-1].setEnabled(false);
                }
            }
        });
        comboPainter.setSelectedIndex(selectedPainter);

        textBegin = new JTextField(String.valueOf(beginCall));
        textEnd = new JTextField(String.valueOf(endCall));
        textBoxSize = new JTextField(String.valueOf(boxSize));

        panelEvent.add(new JLabel("Painter used:  "));
        panelEvent.add(comboPainter);
        panelEvent.add(new JLabel(definition.getDimensionDefinition()[comboDimension.length -1].getName() + ":   "));
        panelEvent.add(comboDimension[comboDimension.length -1]);
        panelEvent.add(new JLabel("Predictor: "));
        panelEvent.add(comboPredictor);
        panelEvent.add(new JLabel("From Event No:"));
        panelEvent.add(textBegin);

        String range = String.valueOf(Scene.getDataManager().getDataSource().getNumberOfEvents(elementDefinition[comboSubject.getSelectedIndex()].getName()));
        toEventNo = new JLabel("To Event No:("+range+")");
        panelEvent.add(toEventNo);
        panelEvent.add(textEnd);
        panelEvent.add(new JLabel("Box Size:"));
        panelEvent.add(textBoxSize);

        panelBottom.add(panelEvent);

        updateComboSubject();

        panelMain.add(returnVal, BorderLayout.SOUTH);

        JPanel panelButton = new JPanel(new FlowLayout());
        dialog.getContentPane().add(panelButton, BorderLayout.SOUTH);

        JButton buttonApply = new JButton("Apply");
        buttonApply.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogApply();
            }
        });
        panelButton.add(buttonApply);

        JButton buttonCancel = new JButton("Cancel");
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogCancel();
            }
        });
        panelButton.add(buttonCancel);

        //*************
        for (int i=0; i<comboSubject.getItemCount(); i++) {
            if ((comboSubject.getItemAt(i)).equals(subjectDefinition.getName())) {
                comboSubject.setSelectedIndex(i);
                break;
            }
        }
        for (int i=0;i<comboDimension.length;i++) {
            for (int j =0 ; j<comboDimension[i].getItemCount();j++) {
                if (comboDimension[i].getItemAt(j).equals(dimension[i].getDataFilter().getName())) {
                    comboDimension[i].setSelectedIndex(j);
                    break;
                }
            }
        }
    }

}
