/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Dec 4, 2002
 * Time: 8:51:17 PM
 */

package EVolve.visualization.XYViz.ValValViz;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import EVolve.Scene;
import EVolve.data.Element;
import EVolve.util.painters.PriorityRandomPainter;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoImage;
import EVolve.visualization.Dimension;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;

public class HardwareCounterViz extends ValueValueVisualization{
    private JCheckBox chkSelectTimeFrame, selectionOptions[];
    private JTextField textIntervalX, textIntervalY;
    private int intervalY;

    public HardwareCounterViz() {
        super();
        interval = 1;
        intervalY = 1000000;
    }

    public Dimension[] createDimension() {
        Dimension[] returnVal = new Dimension[3];
        xAxis = new ValueDimension();
        yAxis = new ValueDimension();
        entityIdFilter = new ReferenceDimension();

        returnVal[0] = xAxis;
        returnVal[1] = yAxis;
        returnVal[2] = entityIdFilter;

        return returnVal;
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions != null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.setSelected(true);
        chkSelectTimeFrame.setEnabled(false);


        selectionOptions = new JCheckBox[1];
        selectionOptions[0] = chkSelectTimeFrame;

        return selectionOptions;
    }

    protected JPanel createConfigurationPanel() {
        JPanel configurationPanel = new JPanel(new GridLayout(2, 1, 5, 5));

        JPanel panelIntervalX = new JPanel(new FlowLayout());
        panelIntervalX.add(new JLabel("Interval X: "));

        textIntervalX = new JTextField(String.valueOf(interval), 10);
        panelIntervalX.add(textIntervalX);

        JPanel panelIntervalY = new JPanel(new FlowLayout());
        panelIntervalY.add(new JLabel("Interval Y: "));

        textIntervalY = new JTextField(String.valueOf(intervalY), 10);
        panelIntervalY.add(textIntervalY);

        configurationPanel.add(panelIntervalX);
        configurationPanel.add(panelIntervalY);

        return configurationPanel;
    }

    public void restoreConfiguration(SerializedVisualization config) {

        intervalY = Integer.parseInt(config.BeginEvent);
        textIntervalY.setText(config.BeginEvent);
        super.restoreConfiguration(config);
        textIntervalX.setText(config.Interval);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.BeginEvent = String.valueOf(intervalY);
        data.EndEvent = String.valueOf(intervalY);
        data.xAxis = xAxis.getName();
        data.yAxis = yAxis.getName();
        data.zAxis = entityIdFilter.getName();

        return data;
    }

    protected void updateConfiguration() {
        try {
            if (autoInterval != -1) {
                interval = autoInterval;
                autoInterval = -1;
            } else {
                interval = Integer.parseInt(textIntervalX.getText());
                intervalY = Integer.parseInt(textIntervalY.getText());
            }

            super.updateConfiguration();
            canvas.setName("Bytecode", "Counter Value");
        } catch (NumberFormatException e) {
            Scene.showErrorMessage("Interval must be an integer");
            configure();
        }
    }

    public void preVisualize() {
        super.preVisualize();
        image = new AutoImage();
        xOffset = -1;
        installPainter();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        long x = xAxis.getField(element);
        long y = yAxis.getField(element);
        long z = entityIdFilter.getField(element);

        if (xOffset == -1) xOffset = x;


        painter.paint(image,(x-xOffset)/interval,y/intervalY,z);
    }

    public void visualize() {
        sort();
    }

    public String mouseMove(int x, int y) {
        return null;
    }

    public void makeSelection() { // nothing to be selected in this viz
    }

    public void installPainter() {
        painter = new PriorityRandomPainter();
    }

    public Object clone() {
        HardwareCounterViz o = (HardwareCounterViz)super.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.yAxis;
        o.dimension[2] = o.entityIdFilter;

        return o;
    }
}
