/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Dec 1, 2002
 * Time: 1:16:41 AM
 */

package EVolve.visualization.XYViz.ValValViz;

import java.awt.*;
import java.awt.event.KeyEvent;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Element;
import EVolve.util.painters.MissRatePainter;
import EVolve.util.painters.shapes.Line;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.visualization.AutoShapeImage;
import EVolve.visualization.Dimension;
import EVolve.visualization.Predictor;
import EVolve.visualization.PredictorFactory;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.ValueDimension;

public class MissPredictionMetricViz extends ValueValueVisualization {
    private PredictorFactory[] factory; // predictor factories
    private Predictor[] predictor; // predictors
    private JComboBox comboPredictor; // combobox for selecting predictor factory
    private PredictorFactory selectedFactory; // predictor factory in use
    private JCheckBox chkSelectTimeFrame;
    protected static JCheckBox selectionOptions[] = null;
    private static int SELECT_OPTION;
    protected JTextField textInterval;

    public MissPredictionMetricViz(PredictorFactory[] factory) {
        super();
        interval = 1000;
        this.factory = factory;
        SELECT_OPTION = 0x0001;
    }

    public Dimension[] createDimension() {
        Dimension[] returnVal = new Dimension[3];
        xAxis = new ValueDimension();
        entityIdFilter = new ReferenceDimension();
        filter2 = new ReferenceDimension();

        returnVal[0] = xAxis;
        returnVal[1] = entityIdFilter;
        returnVal[2] = filter2;

        return returnVal;
    }

    protected void updateConfiguration() {
        try {
            if (autoInterval != -1) {
                interval = autoInterval;
                autoInterval = -1;
            } else
                interval = Integer.parseInt(textInterval.getText());

            super.updateConfiguration();
            canvas.setName("Bytecode", "Miss rate");
        } catch (NumberFormatException e) {
            Scene.showErrorMessage("Interval must be an integer");
            configure();
        }
    }

    protected JPanel createConfigurationPanel() {
        JPanel configurationPanel = new JPanel(new GridLayout(2, 1, 5, 5));

        selectedFactory = factory[0];

        JPanel panelTop = new JPanel(new FlowLayout());
        configurationPanel.add(panelTop);

        panelTop.add(new JLabel("Predictor: "));

        comboPredictor = new JComboBox();
        for (int i = 0; i < factory.length; i++) {
            comboPredictor.addItem(factory[i].getName());
        }
        panelTop.add(comboPredictor);


        JPanel panelBottom = new JPanel(new FlowLayout());
        panelBottom.add(new JLabel("Interval: "));

        textInterval = new JTextField(String.valueOf(interval), 10);
        panelBottom.add(textInterval);

        configurationPanel.add(panelBottom);

        return configurationPanel;
    }

    public void preVisualize() {
        predictor = new Predictor[entityIdFilter.getMaxEntityNumber()];
        for (int i = 0; i < predictor.length; i++) {
            predictor[i] = selectedFactory.createPredictor();
        }

        image = new AutoShapeImage(new Line(0,0,1));
        xMax = 0;
        installPainter();
        super.preVisualize();
    }

    public void receiveElement(Element element) {
        if (element.isOptional()) return;

        long temp = xAxis.getField(element);
        long x = temp / interval;
        long y = entityIdFilter.getField(element);
        long z = filter2.getField(element);

        countEvents(temp);

        if (xOffset == -1) xOffset = x;
        painter.paint(image,x-xOffset,y,filter2.getEntityFromInt((int)z).getId());

        if (temp > xMax) {
            xMax = temp;
        }
    }

    public void visualize() {
        canvas.setName(xAxis.getName() + " (" + xMax + ")","Miss Prediction rate (" +
                      ((MissRatePainter)painter).getMaxMiss()*100 + "%)");
        sort();
    }

    protected String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        if ((X >= 0) && (X < image.getW())) {
            Scene.setStatus(((MissRatePainter)painter).getMissrate(X)+"%");
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    protected void installPainter() {
        painter = new MissRatePainter(predictor,filter2.getDataFilter().getTargetType());
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        if (((x1<0)&&(x2<0)) || ((x1>=timeMap.size()))&&(x2>=timeMap.size()))
           return;

        if (x1 < 0) {
            x1 = 0;
        }

        if (x2 > (timeMap.size() - 1)) {
            x2 = timeMap.size() - 1;
        }

        int[] selection = new int[entityIdFilter.getEntityNumber()];
        for (int i = 0; i < selection.length; i++) {
            selection[i] = i;
        }


        entityIdFilter.makeSelection(selectionName,subjectDefinition.getType(),selection,((long[])timeMap.get(x1))[1],
                                     ((long[])timeMap.get(x2))[1],timeMap);

    }

    public void restoreConfiguration(SerializedVisualization config) {

        for (int i=0; i<factory.length; i++) {
            if (factory[i].getName().equals(config.PredictorName)) {
                comboPredictor.setSelectedIndex(i);
                selectedFactory = factory[i];
                break;
            }
        }

        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();
        data.xAxis = xAxis.getName();
        data.yAxis = entityIdFilter.getName();
        data.zAxis = filter2.getName();
        data.PredictorName = selectedFactory.getName();

        return data;
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions!=null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.setSelected(true);
        chkSelectTimeFrame.setEnabled(false);


        selectionOptions = new JCheckBox[1];
        selectionOptions[0] = chkSelectTimeFrame;

        return selectionOptions;
    }

    public Object clone() {
        MissPredictionMetricViz o = (MissPredictionMetricViz) super.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.entityIdFilter;
        o.dimension[2] = o.filter2;
        o.factory = new PredictorFactory[factory.length];
        for (int i=0; i<factory.length; i++) {
            o.factory[i] = (PredictorFactory)factory[i].clone();
        }
        o.selectedFactory = (PredictorFactory)selectedFactory.clone();
        if (predictor != null) {
            o.predictor = new Predictor[predictor.length];
            for (int i=0; i<predictor.length; i++)
                o.predictor[i] = (Predictor)predictor[i].clone();
        }
        o.createDialog();
        return o;
    }

}
