/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 30, 2002
 * Time: 2:13:56 PM
 */

package EVolve.visualization.XYViz.ValValViz;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Element;
import EVolve.data.Entity;
import EVolve.util.ThreadChooser;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.painters.RandomPainter;
import EVolve.visualization.Dimension;
import EVolve.visualization.*;

public class StackViz extends ValueValueVisualization{
    private ArrayList menuList;
    private JMenuItem itemChangeThread;
    private HashMap methodStacks;
    private ThreadChooser td;
    private JTextField textBegin, textEnd;
    private Stack stack;
    private int event;
    private JCheckBox chkSelectTimeFrame, chkSelectOccurredEntities;
    private static JCheckBox selectionOptions[] = null;
    private String selected;
    protected static int SELECT_OPTION = 0x0011;
    private JLabel toEventNo;


    public StackViz() {
        super();

        menuList = new ArrayList();
        imageMap = new HashMap();
        methodStacks = new HashMap();
        td = new ThreadChooser(imageMap,false);

        interval = 1;
        beginCall = 0;
        endCall = 300;
    }

    public Dimension[] createDimension() {
        Dimension [] returnDimension = new Dimension[3];

        xAxis = new ValueDimension();
        entityIdFilter = new ReferenceDimension();
        filter2 = new ReferenceDimension();

        returnDimension[0] = xAxis;
        returnDimension[1] = entityIdFilter;
        returnDimension[2] = filter2;

        return returnDimension;
    }

    protected void updateConfiguration() {
        try {
            beginCall = Integer.parseInt(textBegin.getText());
            endCall = Integer.parseInt(textEnd.getText());

            canvas.setName("Time - Invocations", "Stack");
            super.updateConfiguration();
        } catch (NumberFormatException e) {
            Scene.showErrorMessage("Begin call and end call must be an integer and smaller than "+Integer.MAX_VALUE+".");
            configure();
        };
    }

    public void restoreConfiguration(SerializedVisualization config) {

        beginCall = Long.parseLong(config.BeginEvent);
        endCall = Long.parseLong(config.EndEvent);
        textBegin.setText(config.BeginEvent);
        textEnd.setText(config.EndEvent);
        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.BeginEvent = String.valueOf(beginCall);
        data.EndEvent = String.valueOf(endCall);
        data.xAxis = xAxis.getName();
        data.yAxis = entityIdFilter.getName();
        data.zAxis = filter2.getName();

        return data;
    }

    public void preVisualize() {
        xMax = 0;
        event = 0;
        stack = null;
        image = null;
        imageMap.clear();
        methodStacks.clear();
        currentThread = -1;
        getSelection();
        installPainter();
        super.preVisualize();
    }

    public void receiveElement(Element element) {
        long methodId = entityIdFilter.getField(element);
        long threadId = filter2.getField(element);
        long callno = xAxis.getField(element) - 1;

        countEvents(callno);

        switchThread(threadId);

        if ((callno*2<beginCall*2) || (callno*2>endCall*2+1)) {
            if (element.isOptional()) {// method return;
                if (!stack.empty()) stack.pop();
            } else
                stack.push(new Long(methodId));
            return;
        }

        event++;
        if (!element.isOptional()) // method enter;
            stack.push(new Long(methodId));

        for (int j=0;j<stack.size();j++) {
            painter.paint(image,callno-beginCall,j,((Long)stack.get(j)).intValue());
        }

        if (element.isOptional()) {// method return;
            if (!stack.empty()) stack.pop();
        }
    }

    public void visualize() {
        if (imageMap.size() == 0 ) return;
        while (selected==null) {
            selected = td.showDialog();
            if (selected != null) {
                image = (AutoImage)imageMap.get(new Long(selected.substring(7,selected.length())));
                canvas.setName("Time - "+ xAxis.getDataFilter().getName() + "(" + event/2 +" events)", "Stack");
                sort();
                selected = null;
                break;
            }
        }
    }

    public void makeSelection() {
        preMakeSelection();
        if (selectionName==null) return;

        /*if (SELECT_OPTION == 0) {
            Scene.showErrorMessage("No data is to be selected.");
            return;
        }*/

        if (dataSourceId != Scene.getDataSourceManager().getCurrentDataSourceId()) {
            Scene.showErrorMessage("The active data source used currently is different from \n" +
                                   "this visualization, please choose \"" +
                                   Scene.getDataSourceManager().getUsedDataSourceName(dataSourceId)+"\".");
            return;
        }

        int x1 = canvas.getStartX();
        int x2 = canvas.getEndX();
        int y1 = canvas.getEndY();
        int y2 = canvas.getStartY();

        if (!normalOrientation) {
            int temp;
            temp = x1;
            x1 = y1;
            y1 = temp;
            temp = x2;
            x2 = y2;
            y2 = temp;
        }

        if (((x1<0)&&(x2<0)) || ((x1>=timeMap.size()))&&(x2>=timeMap.size()))
           return;

        if (x1 < 0) x1 = 0;

        if (x2 > timeMap.size())
            x2 = (timeMap.size()-1);


        long start, end;
        if ((SELECT_OPTION & 0x000f) == 0) {
            start = 0;
            end = Long.MAX_VALUE;
        } else {
            long eventInterval[] = (long[])timeMap.get((int)(x1+beginCall));
            start = eventInterval[1];
            eventInterval = (long[])timeMap.get((int)(x2+beginCall));
            end = eventInterval[1];
        }

        if (y1 < 0) y1 = 0;

        int methodId;
        ArrayList idList = new ArrayList();
        for (int i = x1; i <= x2; i++) {
            for (int j = y1; j <= y2; j++) {
                if (normalOrientation)
                    methodId = (int)getEntityId(i,j);
                else
                    methodId = (int)getEntityId(j,i);
                if ((methodId == -1)||(idList.contains(new Integer(methodId)))) continue;
                idList.add(new Integer(methodId));
            }
        }


        int selection[] = null;

        if ((SELECT_OPTION & 0x00f0) == 0x0010) {
            selection = new int[idList.size()];
            for (int i=0; i<selection.length; i++)
                selection[i] = ((Integer)idList.get(i)).intValue();
        } else {
            selection = new int[entityIdFilter.getEntityNumber()];
            for (int i=0; i<selection.length; i++)
                selection[i] = i;
        }

        entityIdFilter.makeSelection(selectionName,subjectDefinition.getType(),selection,start,end,timeMap);
    }

    protected String mouseMove(int x, int y) {
        int X = canvas.getImageX(x);
        int Y = canvas.getImageY(y);

        if ((X>=0) && (Y >= 0) && (image != null)) {
            if (image.getSortedColor(null,null,X,Y)==null)
                Scene.setStatus("  ");
            else
                Scene.setStatus(getEntityName(X,Y)+" "+X+":"+Y);
        } else {
            Scene.setStatus(" ");
        }
        return null;
    }

    protected ArrayList createOptionalMenu() {
        if (menuList.size() > 0) return null;

        menuList.clear();
        itemChangeThread = new JMenuItem("Switch thread...");
        itemChangeThread.setMnemonic(KeyEvent.VK_T);
        itemChangeThread.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                visualize();
            }
        });

        menuList.add(itemChangeThread);
        return menuList;
    }

    private String getEntityName(int x, int y) {
        String returnVal = "";
        int mappedId = (int)getEntityId(x,y);
        Entity entity = entityIdFilter.getEntityFromInt(mappedId);

        if (entity != null) returnVal = entity.getName();
        return returnVal;
    }

    private long getEntityId(int x, int y) {
        Color color = (Color)image.getSortedColor(null,null,x,y);
        if ((color == null)||(painter == null)) return -1;
        return ((RandomPainter)painter).getKeyFromColor(color);
    }

    protected void switchThread(long threadId) {
        if (currentThread != threadId) {
            stack = (Stack)methodStacks.get(new Long(threadId));
            if (stack == null) {
                stack = new Stack();
                methodStacks.put(new Long(threadId),stack);
            }
            super.switchThread(threadId);
        }
    }

    protected void installPainter() {
        painter = new RandomPainter();
    }

    public JCheckBox[] createSelectionOptions() {
        if (selectionOptions!=null) return selectionOptions;

        chkSelectTimeFrame = new JCheckBox("Time Frame");
        chkSelectTimeFrame.setMnemonic(KeyEvent.VK_T);
        chkSelectTimeFrame.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectTimeFrame.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_TIME_FRAME);
            }
        });
        chkSelectTimeFrame.setSelected(true);

        chkSelectOccurredEntities = new JCheckBox("Occurred Entities");
        chkSelectOccurredEntities.setMnemonic(KeyEvent.VK_O);
        chkSelectOccurredEntities.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean selected = chkSelectOccurredEntities.isSelected();
                SELECT_OPTION = switchOption(selected, SELECT_OPTION, SELECT_OCCURRED_ENTITIES);
            }
        });
        chkSelectOccurredEntities.setSelected(true);

        selectionOptions = new JCheckBox[2];
        selectionOptions[0] = chkSelectTimeFrame;
        selectionOptions[1] = chkSelectOccurredEntities;

        return selectionOptions;
    }

    protected JPanel createConfigurationPanel() {
        JPanel returnVal = new JPanel(new GridLayout(1, 1, 5, 5));
        JPanel panelEvent = new JPanel(new GridLayout(2,2));

        Box panelBottom = new Box(BoxLayout.Y_AXIS);
        returnVal.add(panelBottom);

        textBegin = new JTextField(String.valueOf(beginCall));
        textEnd = new JTextField(String.valueOf(endCall));
        panelEvent.add(new JLabel("From Call No:"));
        panelEvent.add(textBegin);

        toEventNo = new JLabel("To Call No:");
        panelEvent.add(toEventNo);
        panelEvent.add(textEnd);
        panelBottom.add(panelEvent);

        return returnVal;
    }

    protected void updateComboSubject() {
        super.updateComboSubject();
        String range = String.valueOf(Scene.getDataManager().getDataSource().getNumberOfEvents(elementDefinition[comboSubject.getSelectedIndex()].getName()));
        toEventNo.setText("To Call No:("+range+")");
    }

    public Object clone() {
        StackViz o = (StackViz)super.clone();
        o.dimension[0] = o.xAxis;
        o.dimension[1] = o.entityIdFilter;
        o.dimension[2] = o.filter2;


        o.td = new ThreadChooser(o.imageMap,false);

        o.methodStacks = new HashMap();
        Iterator it = methodStacks.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Stack newStack = new Stack(), oldStack = (Stack)methodStacks.get(key);
            o.methodStacks.put(key,newStack);
            for (int i=0; i<oldStack.size(); i++)
                newStack.add(oldStack.get(i));
        }

        o.menuList = new ArrayList();
        o.createDialog();
        return o;
    }

}
