/**
 * Created by IntelliJ IDEA.
 * User: Wei Wang
 * Date: Nov 27, 2002
 * Time: 3:50:38 PM
 */

package EVolve.visualization.XYViz;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import EVolve.Scene;
import EVolve.data.Entity;
import EVolve.data.Selection;
import EVolve.util.HelperFuncs;
import EVolve.util.Magnifier;
import EVolve.util.xmlutils.datastructures.SerializedVisualization;
import EVolve.util.painters.DefaultPainter;
import EVolve.util.painters.Painter;
import EVolve.util.phasedetectors.PhaseDetector;
import EVolve.util.sourcebrowser.SourceBrowser;
import EVolve.visualization.AutoImage;
import EVolve.visualization.AutoShapeImage;
import EVolve.visualization.AxesPanel;
import EVolve.visualization.ReferenceDimension;
import EVolve.visualization.Visualization;
import EVolve.visualization.VisualizationDefinition;

public abstract class XYVisualization extends Visualization{
    protected int autoInterval,interval;
    protected long xMax, xOffset, currentThread;
    protected boolean shift_pressed, flipMagnifier = true;
    protected Painter painter;
    protected Magnifier magnifier;
    protected JMenuItem itemChangeOrientation,itemScale,itemBrowseSource,itemRecolor;
    protected JMenuItem itemScaleX, itemScaleY;
    protected boolean normalOrientation;
    protected HashMap imageMap,threadDataSet;
    protected SourceBrowser sourceBrowser;
    protected AxesPanel canvas;
    protected AutoImage image; // the image
    protected ArrayList timeMap;
    protected PhaseDetector phaseDetector;
    protected boolean phaseDetectorEnabled;

    protected abstract String mouseMove(int x, int y);

    public XYVisualization() {
        super();
        shift_pressed = false;
        autoInterval = -1;
        interval = -1;
        normalOrientation = true;
        currentThread = -1;
        imageMap = new HashMap();
        threadDataSet = new HashMap();
        sourceBrowser = new SourceBrowser();
        timeMap = new ArrayList();
        phaseDetectorEnabled = Scene.getVisualizationManager().isPhaseDetectorEnabled();
    }

    protected Component createPanel() {
        canvas = new AxesPanel(AxesPanel.Axis_Enabled | AxesPanel.Flip_Image | AxesPanel.Scale_Image);
        flipMagnifier = true;
        addMagnifier(canvas,flipMagnifier);
        JScrollPane returnVal = new JScrollPane(canvas);
        canvas.setParent(returnVal);

        return returnVal;
    }

    protected void addMagnifier(Component aPanel, boolean flipImage) {
        magnifier = new Magnifier(flipImage);
        aPanel.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent e) {

                if (e.isShiftDown()) shift_pressed = true;
                else shift_pressed = false;

                if (e.isAltDown()) {
                    int x = e.getX(), y = e.getY();
                    x = canvas.getImageX(x);
                    y = canvas.getImageY(y);
                    magnifier.convertMousePosition(canvas);
                    magnifier.showWindow(canvas.getSubImage(x,y));
                    canvas.drawZoomingArea(e.getX(),e.getY());
                }
                mouseMove(e.getX(), e.getY());
            }

            public void mouseDragged(MouseEvent e) {
                if (e.isShiftDown()) shift_pressed = true;
                else shift_pressed = false;
                mouseMove(e.getX(), e.getY());
            }
        });


        aPanel.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
                mouseX = e.getX();
                mouseY = e.getY();
                freezed = false;
            }

            public void mouseEntered(MouseEvent e) {}

            public void mouseExited(MouseEvent e) {}

            public void mousePressed(MouseEvent e) {}

            public void mouseReleased(MouseEvent e) {}
        });

    }

    protected void updateConfiguration() {
        if (panel instanceof JScrollPane)
            ((AxesPanel)(((JScrollPane)panel)).getViewport().getView()).setImage(null);
        else
            ((AxesPanel)panel).setImage(null);
        panel.repaint();

        normalOrientation = true;
    }

    protected void createMenu() {
        super.createMenu();
        int flag = canvas.getFlag();

        if ((flag & 0xff000000) == 0) {
            itemScale = new JMenuItem("Fit Visualization to Window");
            itemScale.setMnemonic(KeyEvent.VK_F);
        } else {
            itemScale = new JMenuItem("Restore Visualization");
            itemScale.setMnemonic(KeyEvent.VK_R);
        }
        itemScale.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().charAt(0) == 'F') {
                    canvas.scaleImage(true);
                    itemScale.setText("Restore Visualization");
                    itemScale.setActionCommand("Restore Visualization");
                    itemScale.setMnemonic(KeyEvent.VK_R);
                } else {
                    canvas.scaleImage(false);
                    itemScale.setText("Fit Visualization to Window");
                    itemScale.setActionCommand("Fit Visualization to Window");
                    itemScale.setMnemonic(KeyEvent.VK_F);
                }
                canvas.repaint();
            }
        });


        if ((flag & 0xf0000000) == AxesPanel.Scale_X) {
            itemScaleX = new JMenuItem("RestoreX axis");
        } else {
            itemScaleX = new JMenuItem("Fit X axis to Window");
        }
        itemScaleX.setMnemonic(KeyEvent.VK_X);
        itemScaleX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().charAt(0) == 'F') {
                    int flag = canvas.getFlag();
                    canvas.setPanelFlag(flag | AxesPanel.Scale_X);
                    itemScaleX.setText("Restore X axis");
                    itemScaleX.setActionCommand("Restore X axis");
                    itemScaleX.setMnemonic(KeyEvent.VK_X);
                } else {
                    int flag = canvas.getFlag();
                    canvas.setPanelFlag(flag & ~AxesPanel.Scale_X);
                    itemScaleX.setText("Fit X axis to Window");
                    itemScaleX.setActionCommand("Fit X axis to Window");
                    itemScaleX.setMnemonic(KeyEvent.VK_X);
                }
                canvas.repaint();
            }
        });

        if ((flag & 0x0f000000) == AxesPanel.Scale_Y) {
            itemScaleY = new JMenuItem("Restore Y axis");
        } else {
            itemScaleY = new JMenuItem("Fit Y axis to Window");
        }
        itemScaleY.setMnemonic(KeyEvent.VK_Y);
        itemScaleY.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().charAt(0) == 'F') {
                    int flag = canvas.getFlag();
                    canvas.setPanelFlag(flag | AxesPanel.Scale_Y);
                    itemScaleY.setText("Restore Y axis");
                    itemScaleY.setActionCommand("Restore Y axis");
                    itemScaleY.setMnemonic(KeyEvent.VK_X);
                } else {
                    int flag = canvas.getFlag();
                    canvas.setPanelFlag(flag & ~AxesPanel.Scale_Y);
                    itemScaleY.setText("Fit Y axis to Window");
                    itemScaleY.setActionCommand("Fit Y axis to Window");
                    itemScaleY.setMnemonic(KeyEvent.VK_Y);
                }
                canvas.repaint();
            }
        });

        itemBrowseSource = new JMenuItem("Browse source code...");
        itemBrowseSource.setMnemonic(KeyEvent.VK_B);
        itemBrowseSource.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                sourceBrowser.showSourceFile(getEntityUnderMouse());
            }
        });
        
        itemRecolor = new JMenuItem("Recolor");
        itemRecolor.setMnemonic(KeyEvent.VK_R);
        itemRecolor.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Selection[] selections = Scene.getFilter().getSelection();
                boolean found = false;
                for (int i=0; i<selections.length; i++) {
                    if (selections[i].getColor()!=null) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    Scene.showErrorMessage("No selection is colored, can not recolor current visualization.");
                    return;
                }
                Visualization visual = Scene.getVisualizationManager().getActiveVisualization();
                if (visual == null) return;
                if (image instanceof AutoShapeImage) {
                    //reColorShapes();
                    Scene.showErrorMessage("Can not recolor image with shapes.");
                    return;
                } else
                    reColor();
            }
        });

        popup.add(itemBrowseSource);
        popup.add(itemScale);
        popup.add(itemScaleX);
        popup.add(itemScaleY);
        popup.add(itemRecolor);
        itemBrowseSource.setEnabled(false);
    }

    public VisualizationDefinition getDefinition() {
        return definition;
    }

    public void restoreConfiguration(SerializedVisualization config) {
        autoInterval = Integer.parseInt(config.Interval);
        interval = autoInterval;
        super.restoreConfiguration(config);
    }

    public SerializedVisualization getCurrentConfiguration() {
        SerializedVisualization data = super.getCurrentConfiguration();

        data.Interval = String.valueOf(interval);

        return data;
    }

    public long getxMax() {
        return xMax;
    }

    public AutoImage getImage() {
        return image;
    }

    public void setImage(AutoImage image) {
        this.image = image;
    }

    public void disablePopupMenu() {
        MenuElement[] menus = popup.getSubElements();
        for (int i=0; i<menus.length; i++) {
            if (((JMenuItem)menus[i]).getText().equals("Sort") ||
                ((JMenuItem)menus[i]).getText().equals("Save...") ||
                ((JMenuItem)menus[i]).getText().equals("Restore") ||
                ((JMenuItem)menus[i]).getText().equals("Fit Window"))
                continue;
            ((JMenuItem)menus[i]).setEnabled(false);
        }
        Scene.getUIManager().disableFunctionMenus();
    }

    protected void changeOrientation() {
        AutoImage newImage = new AutoImage();
        int wMax = image.getW();
        int hMax = image.getH();
        Color color;

        for (int i=0; i<wMax; i++) {
            for (int j=0; j<hMax; j++) {
                color = (Color)image.getColor(i,j);
                if (color != null) {
                    newImage.setColor(j,i,color);
                }
            }
        }
        image = newImage;

        normalOrientation = ! normalOrientation;
    }

    protected void switchThread(long threadId) {
        if (currentThread != threadId) {
            currentThread = threadId;
            image = (AutoImage)imageMap.get(new Long(threadId));
            if (image == null) {
                image = new AutoImage();
                imageMap.put(new Long(threadId),image);
            }
        }
    }

    protected void installPainter() {
        painter = new DefaultPainter();
    }

    protected void enableBrowseSourceMenu() {
        itemBrowseSource.setEnabled(true);
    }

    protected void countEvents(long x) {
        /*while (x/Math.abs(interval) >= eventCounter.size()) {
            eventCounter.add(new Long(Scene.getEventCounter()));
        }

        while (x/Math.abs(interval) >= timeCounter.size()) {
            timeCounter.add(new Long(x));
        }*/
        long eventNo = Scene.getEventCounter();
        long[] time2event = new long[2];
        time2event[0] = x;
        time2event[1] = eventNo;
        while (x/Math.abs(interval) - xOffset >= timeMap.size()) {
            timeMap.add(time2event);
        }
    }

    public void cleanup() {
        super.cleanup();
        magnifier.cleanup();
    }

    public void enablePhaseDetector(boolean enabled) {
        phaseDetectorEnabled = enabled;
    }

    protected void reset() {
        super.reset();
        image = null;
    }

    protected void dialogApply() {
        super.dialogApply();
        image = null;
    }

    public PhaseDetector getPhaseDetector() {
        return phaseDetector;
    }

    public Point getMousePosition() {
        return new Point(mouseX, mouseY);
    }

    public void freeze(boolean flag) {
        freezed = flag;
    }

    public Object clone() {
        XYVisualization o = null;

        o = (XYVisualization) super.clone();
        o.image = (image == null) ? null : (AutoImage)image.clone();

        o.imageMap = HelperFuncs.cloneHashMap(imageMap);
        o.itemChangeOrientation = null;
        o.threadDataSet = HelperFuncs.cloneHashMap(threadDataSet);
        o.sourceBrowser = (SourceBrowser)sourceBrowser.clone();
        o.painter = (painter == null) ?null : (Painter)painter.clone();
        o.itemBrowseSource = null;
        o.itemChangeOrientation = null;
        o.canvas = (AxesPanel)canvas.clone();
        o.panel = new JScrollPane(o.canvas);
        o.canvas.setParent((JScrollPane)o.panel);
        o.addMagnifier(o.canvas,flipMagnifier);
        o.addPopupTrigger(o.panel);
        o.timeMap = new ArrayList();
        for (int i=0; i<timeMap.size(); i++) {
            long[] newValue = new long[2];
            long[] oldValue = (long[])timeMap.get(i);
            newValue[0] = oldValue[0];
            newValue[1] = oldValue[1];
            o.timeMap.add(newValue);
        }
        o.phaseDetector = null;
        if (phaseDetector != null) {
            o.phaseDetector = (PhaseDetector)phaseDetector.clone();
            o.phaseDetector.setPanel((AxesPanel)((JScrollPane)o.getPanel()).getViewport().getView());
        }
        return o;
    }
    
    public ArrayList getTimeMap() {
        return timeMap;    
    }
    
    public int getInterval() {
        return interval;
    }

    private void reColor() {
        Selection[] selections = Scene.getFilter().getSelection();

        for (int i=0; i<2; i++) {
            if (!(dimension[i] instanceof ReferenceDimension)) continue;
            ReferenceDimension reference = (ReferenceDimension)dimension[i];

            HashMap entityName2Int = reference.getEntityName2IntMap();
            for (int j=0; j<selections.length; j++) {
                Color color = selections[j].getColor();
                if (color == null) continue;

                HashSet set = new HashSet();
                Entity[] selected = selections[j].getSelected();
                for (int k=0; k<selected.length; k++) {
                    String entityName = selected[k].getName();
                    if (entityName2Int.containsKey(entityName)) {
                        set.add(entityName2Int.get(entityName));
                    }
                }

                fillColor(image.getW(), image.getH(), set, color, i==0);
            }
        }
        sort();
    }

    private void reColorShapes() {
        Selection[] selections = Scene.getFilter().getSelection();
        int w = image.getW();
        int h = image.getH();

        for (int i=0; i<2; i++) {
            if (!(dimension[i] instanceof ReferenceDimension)) continue;
            ReferenceDimension reference = (ReferenceDimension)dimension[i];

            HashMap entityName2Int = reference.getEntityName2IntMap();
            for (int j=0; j<selections.length; j++) {
                Color color = selections[j].getColor();
                if (color == null) continue;

                HashSet set = new HashSet();
                Entity[] selected = selections[j].getSelected();
                for (int k=0; k<selected.length; k++) {
                    String entityName = selected[k].getName();
                    if (entityName2Int.containsKey(entityName)) {
                        set.add(entityName2Int.get(entityName));
                    }
                }
                for (int a=0; a<w; a++) {
                    for (int b=0; b<h; b++) {
                        ArrayList value = (ArrayList)image.getColor(a,b);
                        if (value == null) continue;
                        EVolve.util.painters.shapes.Shape shape = (EVolve.util.painters.shapes.Shape)value.get(0);
                        if (set.contains(new Integer((int)shape.getEntityID())))
                            shape.setColor(color);
                    }
                }
            }
        }

        sort();
    }

    private void fillColor(int width, int height, HashSet tobeFilled, Color color, boolean fillColumn) {
        int loop1 = width, loop2 = height;
        if (!fillColumn) {
            loop1 = height;
            loop2 = width;
        }

        for (int w = 0; w<loop1; w++) {
            if (!tobeFilled.contains(new Integer(w))) continue;

            for (int h = 0; h<loop2; h++) {
                if (fillColumn) {
                    if (image.getColor(w,h)!=null)
                        image.setColor(w,h,color);
                }
                else {
                    if (image.getColor(h,w)!=null)
                        image.setColor(h,w,color);
                }
            }
        }
    }

    protected abstract String getEntityUnderMouse();
}
